@@ -44,6 +44,7 @@ import org.apache.yetus.audience.InterfaceStability;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import org.apache.kudu.Schema;
 import org.apache.kudu.client.AsyncKuduClient.LookupType;
 import org.apache.kudu.util.AsyncUtil;
 import org.apache.kudu.util.LogThrottler;
@@ -183,6 +184,11 @@ public class AsyncKuduSession implements SessionConfiguration {
   private boolean ignoreAllDuplicateRows = false;
   private boolean ignoreAllNotFoundRows = false;
 
+  /**
+   * Cumulative operation metrics since the beginning of the session.
+   */
+  private final ResourceMetrics writeOpMetrics = new ResourceMetrics();
+
   /**
    * Package-private constructor meant to be used via AsyncKuduClient
    * @param client client that creates this session
@@ -317,6 +323,11 @@ public class AsyncKuduSession implements SessionConfiguration {
     return errorCollector.getErrors();
   }
 
+  @Override
+  public ResourceMetrics getWriteOpMetrics() {
+    return this.writeOpMetrics;
+  }
+
   /**
    * Flushes the buffered operations and marks this session as closed.
    * See the javadoc on {@link #flush()} on how to deal with exceptions coming out of this method.
@@ -331,17 +342,6 @@ public class AsyncKuduSession implements SessionConfiguration {
     return flush();
   }
 
-  /**
-   * Returns a buffer to the inactive queue after flushing.
-   * @param buffer the buffer to return to the inactive queue.
-   */
-  private void queueBuffer(Buffer buffer) {
-    inactiveBuffers.add(buffer);
-    buffer.callbackFlushNotification();
-    Deferred<Void> localFlushNotification = flushNotification.getAndSet(new Deferred<>());
-    localFlushNotification.callback(null);
-  }
-
   /**
    * Callback which waits for all tablet location lookups to complete, groups all operations into
    * batches by tablet, and dispatches them. When all of the batches are complete, a deferred is
@@ -381,7 +381,8 @@ public class AsyncKuduSession implements SessionConfiguration {
           if (failure instanceof NonCoveredRangeException) {
             // TODO: this should be something different than NotFound so that
             // applications can distinguish from updates on missing rows.
-            error = new RowError(Status.NotFound(failure.getMessage()), operation);
+            error = new RowError(Status.NotFound(String.format(
+                    "%s: %s", failure.getMessage(), operation.getTable().getName())), operation);
           } else {
             LOG.warn("unexpected tablet lookup failure for operation {}", operation, failure);
             error = new RowError(Status.RuntimeError(failure.getMessage()), operation);
@@ -440,6 +441,92 @@ public class AsyncKuduSession implements SessionConfiguration {
 
       return null;
     }
+
+    /**
+     * Creates callbacks to handle a multi-put and adds them to the request.
+     * @param request the request for which we must handle the response
+     */
+    private void addBatchCallbacks(final Batch request) {
+      final class BatchCallback implements Callback<BatchResponse, BatchResponse> {
+        @Override
+        public BatchResponse call(final BatchResponse response) {
+          LOG.trace("Got a Batch response for {} rows", request.operations.size());
+          AsyncKuduSession.this.client.updateLastPropagatedTimestamp(response.getWriteTimestamp());
+
+          // Send individualized responses to all the operations in this batch.
+          for (OperationResponse operationResponse : response.getIndividualResponses()) {
+            if (flushMode == FlushMode.AUTO_FLUSH_BACKGROUND && operationResponse.hasRowError()) {
+              errorCollector.addError(operationResponse.getRowError());
+            }
+
+            // Fire the callback after collecting the errors so that the errors
+            // are visible should the callback interrogate the error collector.
+            operationResponse.getOperation().callback(operationResponse);
+          }
+          writeOpMetrics.update(response.getWriteOpMetrics());
+
+          return response;
+        }
+
+        @Override
+        public String toString() {
+          return "apply batch response";
+        }
+      }
+
+      final class BatchErrCallback implements Callback<Object, Exception> {
+        @Override
+        public Object call(Exception e) {
+          // If the exception we receive is a KuduException we're going to build OperationResponses.
+          Status status = null;
+          List<OperationResponse> responses = null;
+          boolean handleKuduException = e instanceof KuduException;
+          if (handleKuduException) {
+            status = ((KuduException) e).getStatus();
+            responses = new ArrayList<>(request.operations.size());
+          }
+
+          for (Operation operation : request.operations) {
+            // Same comment as in BatchCallback regarding the ordering of when to callback.
+            if (handleKuduException) {
+              RowError rowError = new RowError(status, operation);
+              OperationResponse response = new OperationResponse(0, null, 0, operation, rowError);
+              errorCollector.addError(rowError);
+              responses.add(response);
+
+              operation.callback(response);
+            } else {
+              // We have no idea what the exception is so we'll just send it up.
+              operation.errback(e);
+            }
+          }
+
+          // Note that returning an object that's not an exception will make us leave the
+          // errback chain. Effectively, the BatchResponse below will end up as part of the list
+          // passed to ConvertBatchToListOfResponsesCB.
+          return handleKuduException ? new BatchResponse(responses, request.operationIndexes) : e;
+        }
+
+        @Override
+        public String toString() {
+          return "apply batch error response";
+        }
+      }
+
+      request.getDeferred().addCallbacks(new BatchCallback(), new BatchErrCallback());
+    }
+
+    /**
+     * Returns a buffer to the inactive queue after flushing.
+     * @param buffer the buffer to return to the inactive queue.
+     */
+    private void queueBuffer(Buffer buffer) {
+      if (buffer.callbackFlushNotification()) {
+        inactiveBuffers.add(buffer);
+      }
+      Deferred<Void> localFlushNotification = flushNotification.getAndSet(new Deferred<>());
+      localFlushNotification.callback(null);
+    }
   }
 
   /**
@@ -531,7 +618,7 @@ public class AsyncKuduSession implements SessionConfiguration {
   public boolean hasPendingOperations() {
     synchronized (monitor) {
       return activeBuffer == null ? inactiveBuffers.size() < 2 :
-             activeBuffer.getOperations().size() > 0 || !inactiveBufferAvailable();
+             !activeBuffer.getOperations().isEmpty() || !inactiveBufferAvailable();
     }
   }
 
@@ -549,6 +636,7 @@ public class AsyncKuduSession implements SessionConfiguration {
     return client.sendRpcToTablet(operation)
         .addCallbackDeferring(resp -> {
           client.updateLastPropagatedTimestamp(resp.getWriteTimestampRaw());
+          writeOpMetrics.update(resp.getWriteOpMetrics());
           return Deferred.fromResult(resp);
         })
         .addErrback(new SingleOperationErrCallback(operation));
@@ -571,6 +659,14 @@ public class AsyncKuduSession implements SessionConfiguration {
     Preconditions.checkArgument(operation.getTable().getAsyncClient() == client,
         "Applied operations must be created from a KuduTable instance opened " +
         "from the same client that opened this KuduSession");
+    // We do not want to have auto-incrementing column set for INSERT operations.
+    if (operation.getRow().getSchema().hasAutoIncrementingColumn() &&
+            operation.getRow().isSet(Schema.getAutoIncrementingColumnName()) &&
+            (operation.getChangeType() == Operation.ChangeType.INSERT ||
+                 operation.getChangeType() == Operation.ChangeType.INSERT_IGNORE)) {
+      throw new IllegalArgumentException("Auto-Incrementing column should not " +
+              "be specified for INSERT operation");
+    }
     if (closed) {
       // Ideally this would be a precondition, but that may break existing
       // clients who have grown to rely on this unsafe behavior.
@@ -718,78 +814,6 @@ public class AsyncKuduSession implements SessionConfiguration {
     }
   }
 
-  /**
-   * Creates callbacks to handle a multi-put and adds them to the request.
-   * @param request the request for which we must handle the response
-   */
-  private void addBatchCallbacks(final Batch request) {
-    final class BatchCallback implements Callback<BatchResponse, BatchResponse> {
-      @Override
-      public BatchResponse call(final BatchResponse response) {
-        LOG.trace("Got a Batch response for {} rows", request.operations.size());
-        AsyncKuduSession.this.client.updateLastPropagatedTimestamp(response.getWriteTimestamp());
-
-        // Send individualized responses to all the operations in this batch.
-        for (OperationResponse operationResponse : response.getIndividualResponses()) {
-          if (flushMode == FlushMode.AUTO_FLUSH_BACKGROUND && operationResponse.hasRowError()) {
-            errorCollector.addError(operationResponse.getRowError());
-          }
-
-          // Fire the callback after collecting the errors so that the errors are visible should the
-          // callback interrogate the error collector.
-          operationResponse.getOperation().callback(operationResponse);
-        }
-
-        return response;
-      }
-
-      @Override
-      public String toString() {
-        return "apply batch response";
-      }
-    }
-
-    final class BatchErrCallback implements Callback<Object, Exception> {
-      @Override
-      public Object call(Exception e) {
-        // If the exception we receive is a KuduException we're going to build OperationResponses.
-        Status status = null;
-        List<OperationResponse> responses = null;
-        boolean handleKuduException = e instanceof KuduException;
-        if (handleKuduException) {
-          status = ((KuduException) e).getStatus();
-          responses = new ArrayList<>(request.operations.size());
-        }
-
-        for (Operation operation : request.operations) {
-          // Same comment as in BatchCallback regarding the ordering of when to callback.
-          if (handleKuduException) {
-            RowError rowError = new RowError(status, operation);
-            OperationResponse response = new OperationResponse(0, null, 0, operation, rowError);
-            errorCollector.addError(rowError);
-            responses.add(response);
-
-            operation.callback(response);
-          } else {
-            // We have no idea what the exception is so we'll just send it up.
-            operation.errback(e);
-          }
-        }
-
-        // Note that returning an object that's not an exception will make us leave the
-        // errback chain. Effectively, the BatchResponse below will end up as part of the list
-        // passed to ConvertBatchToListOfResponsesCB.
-        return handleKuduException ? new BatchResponse(responses, request.operationIndexes) : e;
-      }
-
-      @Override
-      public String toString() {
-        return "apply batch error response";
-      }
-    }
-
-    request.getDeferred().addCallbacks(new BatchCallback(), new BatchErrCallback());
-  }
 
   /**
    * Analogous to BatchErrCallback above but for AUTO_FLUSH_SYNC which doesn't handle lists of
@@ -806,9 +830,14 @@ public class AsyncKuduSession implements SessionConfiguration {
     @Override
     public Object call(Exception e) throws Exception {
       if (e instanceof KuduException) {
-        Status status = ((KuduException) e).getStatus();
-        RowError rowError = new RowError(status, operation);
-        return new OperationResponse(0, null, 0, operation, rowError);
+        Status status;
+        if (e instanceof NonCoveredRangeException) {
+          status = Status.NotFound(String.format(
+                  "%s: %s", e.getMessage(), operation.getTable().getName()));
+        } else {
+          status = ((KuduException) e).getStatus();
+        }
+        return new OperationResponse(0, null, 0, operation, new RowError(status, operation));
       }
       return e;
     }
@@ -860,6 +889,7 @@ public class AsyncKuduSession implements SessionConfiguration {
     private FlusherTask flusherTask = null;
 
     private Deferred<Void> flushNotification = Deferred.fromResult(null);
+    private boolean flushNotificationFired = false;
 
     public List<BufferedOperation> getOperations() {
       return operations;
@@ -883,12 +913,27 @@ public class AsyncKuduSession implements SessionConfiguration {
     }
 
     /**
-     * Completes the buffer's flush notification. Should be called when the buffer has been
-     * successfully flushed.
+     * Completes the buffer's flush notification. Should be called when
+     * the buffer has been successfully flushed.
      */
-    void callbackFlushNotification() {
+    boolean callbackFlushNotification() {
       LOG.trace("buffer flush notification fired: {}", this);
-      flushNotification.callback(null);
+      if (injectLatencyBufferFlushCb) {
+        try {
+          Thread.sleep(randomizer.nextInt(16));
+        } catch (InterruptedException e) {
+          Thread.currentThread().interrupt();
+        }
+      }
+      synchronized (monitor) {
+        if (flushNotificationFired) {
+          // Do nothing: the callback has been called already.
+          return false;
+        }
+        flushNotificationFired = true;
+        flushNotification.callback(null);
+      }
+      return true;
     }
 
     /**
@@ -900,6 +945,7 @@ public class AsyncKuduSession implements SessionConfiguration {
       LOG.trace("buffer resetUnlocked: {}", this);
       operations.clear();
       flushNotification = new Deferred<>();
+      flushNotificationFired = false;
       flusherTask = null;
     }
 
@@ -975,4 +1021,16 @@ public class AsyncKuduSession implements SessionConfiguration {
                         .toString();
     }
   }
-}
+
+  private static boolean injectLatencyBufferFlushCb = false;
+
+  /**
+   * Inject latency into {@link Buffer#callbackFlushNotification}.
+   */
+  @InterfaceAudience.LimitedPrivate("Test")
+  static void injectLatencyBufferFlushCb(boolean injectLatency) {
+    injectLatencyBufferFlushCb = injectLatency;
+    LOG.warn("latency injection for Buffer flush notification is {}",
+        injectLatency ? "enabled" : "disabled");
+  }
+}
\ No newline at end of file
