@@ -18,6 +18,7 @@
 #include "kudu/client/client-internal.h"
 
 #include <algorithm>
+#include <cstddef>
 #include <cstdint>
 #include <functional>
 #include <limits>
@@ -27,6 +28,7 @@
 #include <ostream>
 #include <random>
 #include <string>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
@@ -55,9 +57,7 @@
 #include "kudu/master/master.proxy.h"
 #include "kudu/master/txn_manager.proxy.h"
 #include "kudu/rpc/connection.h"
-#include "kudu/rpc/messenger.h"
 #include "kudu/rpc/request_tracker.h"
-#include "kudu/rpc/response_callback.h"
 #include "kudu/rpc/rpc.h"
 #include "kudu/rpc/rpc_controller.h"
 #include "kudu/security/cert.h"
@@ -99,6 +99,8 @@ using kudu::master::ListTabletServersRequestPB;
 using kudu::master::ListTabletServersResponsePB;
 using kudu::master::MasterFeatures;
 using kudu::master::MasterServiceProxy;
+using kudu::master::RecallDeletedTableRequestPB;
+using kudu::master::RecallDeletedTableResponsePB;
 using kudu::master::TableIdentifierPB;
 using kudu::rpc::BackoffType;
 using kudu::rpc::CredentialsPolicy;
@@ -187,17 +189,22 @@ KuduClient::Data::~Data() {
 
 namespace {
 
-// This random integer is used when making any random choice for replica
-// selection. It is static to provide a deterministic selection for any given
-// process and therefore also better cache affinity while ensuring that we can
+// This utility function returns an integer used when making any random choice
+// for replica selection. It has the sematics of a per-process constant to
+// provide deterministic selection for any given process that creates Kudu
+// clients and therefore also better cache affinity while ensuring that we can
 // still benefit from spreading the load across replicas for other processes
 // and applications.
-int InitRandomSelectionInt() {
-  std::random_device rdev;
-  std::mt19937 gen(rdev());
-  return gen();
+uint32_t GetReplicaRandomSelection() {
+  // Initialization of function-local statics is guaranteed to occur only once
+  // even when called from multiple threads, and may be more efficient than
+  // the equivalent code using std::call_once [1].
+  //
+  // [1] 'Notes' section at https://en.cppreference.com/w/cpp/thread/call_once
+  static const uint32_t kRandomSelection =
+      std::mt19937 {std::random_device {}()}();
+  return kRandomSelection;
 }
-static const int kRandomSelectionInt = InitRandomSelectionInt();
 
 } // anonymous namespace
 
@@ -266,12 +273,13 @@ RemoteTabletServer* KuduClient::Data::SelectTServer(
           same_location.push_back(rts);
         }
       }
+      static const size_t kRandomSelection = GetReplicaRandomSelection();
       if (!local.empty()) {
-        ret = local[kRandomSelectionInt % local.size()];
+        ret = local[kRandomSelection % local.size()];
       } else if (!same_location.empty()) {
-        ret = same_location[kRandomSelectionInt % same_location.size()];
+        ret = same_location[kRandomSelection % same_location.size()];
       } else if (!filtered.empty()) {
-        ret = filtered[kRandomSelectionInt % filtered.size()];
+        ret = filtered[kRandomSelection % filtered.size()];
       }
       break;
     }
@@ -316,11 +324,23 @@ Status KuduClient::Data::CreateTable(KuduClient* client,
                                      const CreateTableRequestPB& req,
                                      CreateTableResponsePB* resp,
                                      const MonoTime& deadline,
-                                     bool has_range_partition_bounds) {
+                                     bool has_range_partition_bounds,
+                                     bool has_range_specific_hash_schema,
+                                     bool has_immutable_column_schema,
+                                     bool has_auto_incrementing_column) {
   vector<uint32_t> features;
   if (has_range_partition_bounds) {
     features.push_back(MasterFeatures::RANGE_PARTITION_BOUNDS);
   }
+  if (has_range_specific_hash_schema) {
+    features.push_back(MasterFeatures::RANGE_SPECIFIC_HASH_SCHEMA);
+  }
+  if (has_immutable_column_schema) {
+    features.push_back(MasterFeatures::IMMUTABLE_COLUMN_ATTRIBUTE);
+  }
+  if (has_auto_incrementing_column) {
+    features.push_back(MasterFeatures::AUTO_INCREMENTING_COLUMN);
+  }
   Synchronizer sync;
   AsyncLeaderMasterRpc<CreateTableRequestPB, CreateTableResponsePB> rpc(
       deadline, client, BackoffType::EXPONENTIAL, req, resp,
@@ -369,12 +389,16 @@ Status KuduClient::Data::WaitForCreateTableToFinish(
 Status KuduClient::Data::DeleteTable(KuduClient* client,
                                      const string& table_name,
                                      const MonoTime& deadline,
-                                     bool modify_external_catalogs) {
+                                     bool modify_external_catalogs,
+                                     std::optional<uint32_t> reserve_seconds) {
   DeleteTableRequestPB req;
   DeleteTableResponsePB resp;
 
   req.mutable_table()->set_table_name(table_name);
   req.set_modify_external_catalogs(modify_external_catalogs);
+  if (reserve_seconds) {
+    req.set_reserve_seconds(*reserve_seconds);
+  }
   Synchronizer sync;
   AsyncLeaderMasterRpc<DeleteTableRequestPB, DeleteTableResponsePB> rpc(
       deadline, client, BackoffType::EXPONENTIAL, req, &resp,
@@ -383,15 +407,43 @@ Status KuduClient::Data::DeleteTable(KuduClient* client,
   return sync.Wait();
 }
 
+Status KuduClient::Data::RecallTable(KuduClient* client,
+                                     const std::string& table_id,
+                                     const MonoTime& deadline,
+                                     const std::string& new_table_name) {
+  RecallDeletedTableRequestPB req;
+  RecallDeletedTableResponsePB resp;
+
+  req.mutable_table()->set_table_id(table_id);
+  if (!new_table_name.empty()) {
+    req.set_new_table_name(new_table_name);
+  }
+  Synchronizer sync;
+  AsyncLeaderMasterRpc<RecallDeletedTableRequestPB, RecallDeletedTableResponsePB> rpc(
+      deadline, client, BackoffType::EXPONENTIAL, req, &resp,
+      &MasterServiceProxy::RecallDeletedTableAsync, "RecallDeletedTable", sync.AsStatusCallback(),
+      {});
+  rpc.SendRpc();
+  return sync.Wait();
+}
+
 Status KuduClient::Data::AlterTable(KuduClient* client,
                                     const AlterTableRequestPB& req,
                                     AlterTableResponsePB* resp,
                                     const MonoTime& deadline,
-                                    bool has_add_drop_partition) {
+                                    bool has_add_drop_partition,
+                                    bool adding_range_with_custom_hash_schema,
+                                    bool has_immutable_column_schema) {
   vector<uint32_t> required_feature_flags;
   if (has_add_drop_partition) {
     required_feature_flags.push_back(MasterFeatures::ADD_DROP_RANGE_PARTITIONS);
   }
+  if (adding_range_with_custom_hash_schema) {
+    required_feature_flags.push_back(MasterFeatures::RANGE_SPECIFIC_HASH_SCHEMA);
+  }
+  if (has_immutable_column_schema) {
+    required_feature_flags.push_back(MasterFeatures::IMMUTABLE_COLUMN_ATTRIBUTE);
+  }
   Synchronizer sync;
   AsyncLeaderMasterRpc<AlterTableRequestPB, AlterTableResponsePB> rpc(
       deadline, client, BackoffType::EXPONENTIAL, req, resp,
@@ -436,11 +488,15 @@ Status KuduClient::Data::WaitForAlterTableToFinish(
 
 Status KuduClient::Data::ListTablesWithInfo(KuduClient* client,
                                             vector<TableInfo>* tables_info,
-                                            const string& filter) {
+                                            const string& filter,
+                                            bool list_tablet_with_partition,
+                                            bool show_soft_deleted) {
   ListTablesRequestPB req;
   if (!filter.empty()) {
     req.set_name_filter(filter);
   }
+  req.set_show_soft_deleted(show_soft_deleted);
+  req.set_list_tablet_with_partition(list_tablet_with_partition);
 
   auto deadline = MonoTime::Now() + client->default_admin_operation_timeout();
   Synchronizer sync;
@@ -460,8 +516,26 @@ Status KuduClient::Data::ListTablesWithInfo(KuduClient* client,
     info.live_row_count = table.has_live_row_count() ? table.live_row_count() : 0;
     info.num_tablets = table.has_num_tablets() ? table.num_tablets() : 0;
     info.num_replicas = table.has_num_replicas() ? table.num_replicas() : 0;
+    if (list_tablet_with_partition) {
+      for (const auto& pt : table.tablet_with_partition()) {
+        Partition partition;
+        Partition::FromPB(pt.partition(), &partition);
+        PartitionWithTabletId partition_with_tablet_id = {pt.tablet_id(), std::move(partition)};
+        info.partition_with_tablet_info.emplace_back(std::move(partition_with_tablet_id));
+      }
+    }
+
+    // sort by range key
+    // Different range partitions of the same table do not intersect.
+    // So we just compare the start/begin key of two ranges.
+    std::sort(info.partition_with_tablet_info.begin(), info.partition_with_tablet_info.end(),
+              [] (const PartitionWithTabletId& lhs, const PartitionWithTabletId& rhs) {
+      return lhs.partition.begin().range_key() < rhs.partition.begin().range_key();
+    });
+
     tables_info->emplace_back(std::move(info));
   }
+
   return Status::OK();
 }
 
