@@ -18,20 +18,23 @@
 // This module is internal to the client and not a public API.
 #pragma once
 
+#include <cstdint>
+
 #include <atomic>
 #include <map>
 #include <memory>
+#include <optional>
 #include <set>
 #include <string>
 #include <unordered_map>
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <glog/logging.h>
 #include <gtest/gtest_prod.h>
 
 #include "kudu/client/replica_controller-internal.h"
+#include "kudu/common/key_range.h"
 #include "kudu/common/partition.h"
 #include "kudu/consensus/metadata.pb.h"
 #include "kudu/gutil/macros.h"
@@ -132,7 +135,7 @@ class RemoteTabletServer {
   // The path on which this server is listening for unix domain socket connections.
   // This should only be used in the case that it can be determined that the tablet
   // server is local to the client.
-  boost::optional<std::string> unix_domain_socket_path_;
+  std::optional<std::string> unix_domain_socket_path_;
 
   std::shared_ptr<tserver::TabletServerServiceProxy> proxy_;
   std::shared_ptr<tserver::TabletServerAdminServiceProxy> admin_proxy_;
@@ -406,6 +409,18 @@ class MetaCache : public RefCountedThreadSafe<MetaCache> {
     kLowerBound
   };
 
+  struct RangeWithRemoteTablet {
+    RangeWithRemoteTablet(KeyRange krange,
+                          const scoped_refptr<internal::RemoteTablet>& rtablet)
+        : key_range(std::move(krange)),
+          remote_tablet(rtablet) {}
+
+    KeyRange key_range;
+    scoped_refptr<internal::RemoteTablet> remote_tablet;
+  };
+
+  typedef std::map<PartitionKey, MetaCacheEntry> TabletMap;
+
   // Look up which tablet hosts the given partition key for a table. When it is
   // available, the tablet is stored in 'remote_tablet' (if not NULL) and the
   // callback is fired. Only tablets with non-failed LEADERs are considered.
@@ -422,6 +437,17 @@ class MetaCache : public RefCountedThreadSafe<MetaCache> {
                          scoped_refptr<RemoteTablet>* remote_tablet,
                          const StatusCallback& callback);
 
+  // Look up which tablet hosts the given partition key for a table.
+  // If @split_size_bytes set nonzero, send SplitKeyRangeRPC to remote tserver,
+  // otherwise search only occurs locally.
+  // The result stored in 'range_tablets'.
+  Status GetTableKeyRanges(const KuduTable* table,
+                           const PartitionKey& partition_key,
+                           LookupType lookup_type,
+                           uint64_t split_size_bytes,
+                           const MonoDelta& timeout,
+                           std::vector<RangeWithRemoteTablet>* range_tablets);
+
   // Look up the locations of the given tablet, storing the result in
   // 'remote_tablet' if not null, and calling 'lookup_complete_cb' once the
   // lookup is complete. Only tablets with non-failed LEADERs are considered.
@@ -551,7 +577,6 @@ class MetaCache : public RefCountedThreadSafe<MetaCache> {
   // for key-based lookups.
   //
   // Protected by lock_.
-  typedef std::map<PartitionKey, MetaCacheEntry> TabletMap;
   std::unordered_map<std::string, TabletMap> tablets_by_table_and_key_;
 
   // Cache entries for tablets, keyed by tablet ID, used for ID-based lookups.
