@@ -19,11 +19,11 @@
 #include <cstdint>
 #include <map>
 #include <memory>
+#include <optional>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <glog/logging.h>
 
 #include "kudu/client/client.h"
@@ -71,17 +71,17 @@ class KuduTableCreator::Data {
 
   PartitionSchemaPB partition_schema_;
 
-  boost::optional<std::string> owner_;
+  std::optional<std::string> owner_;
 
-  boost::optional<std::string> comment_;
+  std::optional<std::string> comment_;
 
-  boost::optional<int> num_replicas_;
+  std::optional<int> num_replicas_;
 
-  boost::optional<std::string> dimension_label_;
+  std::optional<std::string> dimension_label_;
 
-  boost::optional<std::map<std::string, std::string>> extra_configs_;
+  std::optional<std::map<std::string, std::string>> extra_configs_;
 
-  boost::optional<TableTypePB> table_type_;
+  std::optional<TableTypePB> table_type_;
 
   MonoDelta timeout_;
 
@@ -91,20 +91,20 @@ class KuduTableCreator::Data {
   DISALLOW_COPY_AND_ASSIGN(Data);
 };
 
-class KuduTableCreator::KuduRangePartition::Data {
+class KuduRangePartition::Data {
  public:
   Data(KuduPartialRow* lower_bound,
        KuduPartialRow* upper_bound,
-       RangePartitionBound lower_bound_type,
-       RangePartitionBound upper_bound_type);
+       KuduTableCreator::RangePartitionBound lower_bound_type,
+       KuduTableCreator::RangePartitionBound upper_bound_type);
   ~Data() = default;
 
   Status add_hash_partitions(const std::vector<std::string>& column_names,
                              int32_t num_buckets,
                              uint32_t seed);
 
-  const RangePartitionBound lower_bound_type_;
-  const RangePartitionBound upper_bound_type_;
+  const KuduTableCreator::RangePartitionBound lower_bound_type_;
+  const KuduTableCreator::RangePartitionBound upper_bound_type_;
 
   std::unique_ptr<KuduPartialRow> lower_bound_;
   std::unique_ptr<KuduPartialRow> upper_bound_;
