@@ -35,6 +35,7 @@
 #include "kudu/clock/time_service.h"
 #include "kudu/common/timestamp.h"
 #include "kudu/gutil/casts.h"
+#include "kudu/gutil/port.h"
 #include "kudu/gutil/ref_counted.h"
 #include "kudu/gutil/strings/join.h"
 #include "kudu/gutil/strings/substitute.h"
@@ -52,13 +53,16 @@
 #include "kudu/util/random_util.h"
 #include "kudu/util/scoped_cleanup.h"
 #include "kudu/util/status.h"
+#include "kudu/util/stopwatch.h"
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
 
 DECLARE_bool(inject_unsync_time_errors);
 DECLARE_string(builtin_ntp_servers);
+DECLARE_string(cloud_curl_dns_servers_for_testing);
 DECLARE_string(time_source);
 DECLARE_uint32(cloud_metadata_server_request_timeout_ms);
+
 METRIC_DECLARE_entity(server);
 
 using kudu::cloud::InstanceDetector;
@@ -67,6 +71,7 @@ using kudu::cluster::ExternalMiniClusterOptions;
 using std::string;
 using std::thread;
 using std::vector;
+using strings::Substitute;
 
 namespace kudu {
 namespace clock {
@@ -392,8 +397,14 @@ TEST_F(HybridClockTest, TestRideOverNtpInterruption) {
 TEST_F(HybridClockTest, SlowClockInitialisation) {
   SKIP_IF_SLOW_NOT_ALLOWED();
 
-  const vector<string> kExtraFlags =
-      { "--hybrid_clock_inject_init_delay_ms=100" };
+  const vector<string> kExtraFlags = {
+    // Inject delay to emulate slow initialization of the hybrid clock.
+    "--hybrid_clock_inject_init_delay_ms=100",
+
+    // Switching from the default "auto" to "enabled" for faster startup times:
+    // this test scenario is sensitive to start up timing.
+    "--wall_clock_jump_detection=enabled",
+  };
 
   ExternalMiniClusterOptions opts;
   opts.num_masters = 3;
@@ -453,7 +464,7 @@ TEST_F(HybridClockTest, SlowClockInitialisation) {
   EasyCurl c;
   for (const auto& addr : addresses) {
     faststring buf;
-    ASSERT_OK(c.FetchURL(strings::Substitute("http://$0/metrics", addr), &buf));
+    ASSERT_OK(c.FetchURL(Substitute("http://$0/metrics", addr), &buf));
     const auto str = buf.ToString();
     ASSERT_STR_CONTAINS(str, "builtin_ntp_error");
     ASSERT_STR_CONTAINS(str, "builtin_ntp_local_clock_delta");
@@ -486,8 +497,15 @@ TEST_F(HybridClockTest, TimeSourceAutoSelection) {
       ASSERT_STR_CONTAINS(s.ToString(), "Operation not permitted");
       preconditions_satisfied = false;
     } else if (s.IsServiceUnavailable()) {
-      ASSERT_STR_CONTAINS(
-          s.ToString(), "Error reading clock. Clock considered unsynchronized");
+      // For the effective time source auto-selected, the pattern for the error
+      // message below covers the 'builtin' and the 'system' cases
+      // correspondingly. The only other possible case left is 'system_unsync':
+      // if the 'system_unsync' time source is auto-selected, there shouldn't be
+      // any clock synchronization error unless the --inject_unsync_time_errors
+      // flag is set to 'true', which isn't the case in this test scenario.
+      static const string kErrMsgPattern = "(wallclock is not synchronized|"
+          "Error reading clock\\. Clock considered unsynchronized)";
+      ASSERT_STR_MATCHES(s.ToString(), kErrMsgPattern);
       preconditions_satisfied = false;
     }
   }
@@ -514,6 +532,10 @@ TEST_F(HybridClockTest, AutoTimeSourceNoDedicatedNtpServer) {
   // Set very short interval for the instance detection timeout.
   FLAGS_cloud_metadata_server_request_timeout_ms = 1;
 
+  // Configure a bad DNS server to ensure a timeout,
+  // even when run on fast cloud instances.
+  FLAGS_cloud_curl_dns_servers_for_testing = "192.0.2.0";
+
   {
     // Check for the pre-condition: the auto-detection fails as expected due
     // to the super-short timeout setting for the cloud metadata requests and
@@ -551,6 +573,57 @@ TEST_F(HybridClockTest, AutoTimeSourceNoDedicatedNtpServer) {
   }
 }
 
+// Imitate the clock jumping well other the threshold.
+TEST_F(HybridClockTest, ClockJumpDetection) {
+  static constexpr uint64_t kThresholdSeconds = 100;
+  FLAGS_time_source = "mock";
+  HybridClock clock(metric_entity_, kThresholdSeconds * 1000 * 1000);
+
+  ASSERT_OK(clock.Init());
+
+  const auto time_setter = [&](MonoDelta delta) {
+    uint64_t now = HybridClock::GetPhysicalValueMicros(clock.Now());
+    uint64_t to_update = now + delta.ToMicroseconds();
+    auto* hybrid_clock = down_cast<HybridClock*>(&clock);
+    auto* mock_ntp = down_cast<clock::MockNtp*>(hybrid_clock->time_service());
+    mock_ntp->SetMockClockWallTimeForTests(to_update);
+  };
+
+  // Move the 'now' timestamp of the mock clock to make sure the proper 'if'
+  // branch is taken in HybridClock::NowWithErrorUnlocked().
+  time_setter(MonoDelta::FromMicroseconds(1));
+  {
+    Timestamp ts;
+    uint64_t max_error_usec;
+    ASSERT_OK(clock.NowWithError(&ts, &max_error_usec));
+    ASSERT_EQ(1, HybridClock::GetPhysicalValueMicros(ts));
+  }
+
+  // Get closer to the threshold. It should still work since the threshold
+  // isn't crossed yet.
+  time_setter(MonoDelta::FromSeconds(kThresholdSeconds));
+  {
+    Timestamp ts;
+    uint64_t max_error_usec;
+    ASSERT_OK(clock.NowWithError(&ts, &max_error_usec));
+    ASSERT_EQ(kThresholdSeconds * 1000 * 1000 + 1,
+              HybridClock::GetPhysicalValueMicros(ts));
+  }
+
+  // Now make the clock jumping well over the threshold: with previous jump
+  // it should be about two times over the threshold.
+  time_setter(MonoDelta::FromSeconds(2 * kThresholdSeconds));
+  {
+    Timestamp ts;
+    uint64_t max_error_usec;
+    const auto s = clock.NowWithError(&ts, &max_error_usec);
+    ASSERT_TRUE(s.IsServiceUnavailable()) << s.ToString();
+    ASSERT_STR_MATCHES(s.ToString(),
+        "wall \\(200000000 us\\) and monotonic \\([0-9]+ us\\) "
+        "clock deltas diverged too much \\(threshold 100000000 us\\)");
+  }
+}
+
 #if defined(KUDU_HAS_SYSTEM_TIME_SOURCE)
 TEST_F(HybridClockTest, TestNtpDiagnostics) {
   FLAGS_time_source = "system";
@@ -571,12 +644,59 @@ TEST_F(HybridClockTest, TestNtpDiagnostics) {
   // error messages that corresponding binaries cannot be found. In either case
   // it proves the tools were attempted to run to collect NTP-related diagnostic
   // information.
+  ASSERT_STR_CONTAINS(s, "ntpstat");
   ASSERT_STR_CONTAINS(s, "ntptime");
   ASSERT_STR_CONTAINS(s, "ntpq");
   ASSERT_STR_CONTAINS(s, "ntpdc");
   ASSERT_STR_CONTAINS(s, "chronyc");
+
+  // Verify the output contains relevant metrics.
+  ASSERT_STR_CONTAINS(s, "clock_ntp_status");
+  ASSERT_STR_CONTAINS(s, "hybrid_clock_extrapolating");
+  ASSERT_STR_CONTAINS(s, "hybrid_clock_error");
+  ASSERT_STR_CONTAINS(s, "hybrid_clock_timestamp");
+  ASSERT_STR_CONTAINS(s, "hybrid_clock_max_errors");
+  ASSERT_STR_CONTAINS(s, "hybrid_clock_extrapolation_intervals");
 }
 #endif // #if defined(KUDU_HAS_SYSTEM_TIME_SOURCE) ...
 
+// The boolean parameter is to specify whether the wall clock protection is
+// enabled or not ('true' -- enabled, 'false' -- disabled).
+class HybridClockJumpProtectionTest : public ClockTest,
+                            public ::testing::WithParamInterface<bool> {
+};
+INSTANTIATE_TEST_SUITE_P(Perf, HybridClockJumpProtectionTest, ::testing::Bool());
+
+// This is a scenario for basic peformance assessment of the
+// HybridClock::NowWithError() method with clock jump check enabled/disabled.
+TEST_P(HybridClockJumpProtectionTest, BasicPerf) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+
+  constexpr const int kNumIterations = 1000000;
+  FLAGS_time_source = "system_unsync";
+  const auto enable_protection = GetParam();
+  HybridClock clock(metric_entity_, enable_protection ? 10000000UL : 0UL);
+  ASSERT_OK(clock.Init());
+
+  Stopwatch sw;
+  sw.start();
+  Status res_status;
+  for (auto i = 0; i < kNumIterations; ++i) {
+    Timestamp timestamp;
+    uint64_t max_error_usec;
+    auto s = clock.NowWithError(&timestamp, &max_error_usec);
+    if (PREDICT_FALSE(!s.ok())) {
+      res_status = s;
+      break;
+    }
+  }
+  sw.stop();
+
+  ASSERT_OK(res_status);
+
+  LOG(INFO) << Substitute("$0 iterations in $1 ms",
+                          kNumIterations, sw.elapsed().wall_millis());
+}
+
 }  // namespace clock
 }  // namespace kudu
