@@ -27,8 +27,10 @@
 #include <memory>
 #include <mutex>
 #include <numeric>
+#include <optional>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <typeinfo>
 #include <unordered_map>
 #include <utility>
@@ -237,7 +239,7 @@ class MergeIterState : public boost::intrusive::list_base_hook<> {
     RowBlockRow lower;
     RowBlockRow upper;
   };
-  boost::optional<DecodedBounds> decoded_bounds_;
+  std::optional<DecodedBounds> decoded_bounds_;
 
   // Current block of buffered rows from the iterator.
   //
@@ -616,7 +618,7 @@ Status MergeIterator::Init(ScanSpec *spec) {
   finished_iter_stats_by_col_.resize(schema_->num_columns());
 #ifndef NDEBUG
   for (const auto& s : states_) {
-    if (!s.schema().Equals(*schema_)) {
+    if (s.schema() != *schema_) {
       return Status::InvalidArgument(
           Substitute("Schemas do not match: $0 vs. $1",
                      schema_->ToString(), s.schema().ToString()));
@@ -1008,7 +1010,7 @@ Status UnionIterator::Init(ScanSpec *spec) {
   finished_iter_stats_by_col_.resize(schema_->num_columns());
 #ifndef NDEBUG
   for (const auto& i : iters_) {
-    if (!i.iter->schema().Equals(*schema_)) {
+    if (i.iter->schema() != *schema_) {
       return Status::InvalidArgument(
           Substitute("Schemas do not match: $0 vs. $1",
                      schema_->ToString(), i.iter->schema().ToString()));
@@ -1173,7 +1175,8 @@ Status MaterializingIterator::Init(ScanSpec *spec) {
   const int32_t num_columns = schema().num_columns();
   col_idx_predicates_.clear();
   non_predicate_column_indexes_.clear();
-  if (PREDICT_FALSE(!disallow_pushdown_for_tests_) && spec != nullptr) {
+  if (PREDICT_TRUE(!disallow_pushdown_for_tests_) && spec != nullptr &&
+      !spec->predicates().empty()) {
     col_idx_predicates_.reserve(spec->predicates().size());
     DCHECK_GE(num_columns, spec->predicates().size());
     non_predicate_column_indexes_.reserve(num_columns - spec->predicates().size());
