@@ -20,6 +20,7 @@
 #include <cstring>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <utility>
 
 #include <gflags/gflags.h>
@@ -69,6 +70,8 @@ string DecodedRowOperation::ToString(const Schema& schema) const {
       return "INSERT IGNORE " + schema.DebugRow(ConstContiguousRow(&schema, row_data));
     case RowOperationsPB::UPSERT:
       return "UPSERT " + schema.DebugRow(ConstContiguousRow(&schema, row_data));
+    case RowOperationsPB::UPSERT_IGNORE:
+      return "UPSERT IGNORE " + schema.DebugRow(ConstContiguousRow(&schema, row_data));
     case RowOperationsPB::UPDATE:
     case RowOperationsPB::UPDATE_IGNORE:
     case RowOperationsPB::DELETE:
@@ -176,8 +179,8 @@ size_t RowOperationsPBEncoder::Add(RowOperationsPB::Type op_type,
 }
 
 void RowOperationsPBEncoder::RemoveLast() {
-  CHECK_NE(string::npos, prev_indirect_data_size_);
-  CHECK_NE(string::npos, prev_rows_size_);
+  DCHECK_NE(string::npos, prev_indirect_data_size_);
+  DCHECK_NE(string::npos, prev_rows_size_);
   pb_->mutable_indirect_data()->resize(prev_indirect_data_size_);
   pb_->mutable_rows()->resize(prev_rows_size_);
   prev_indirect_data_size_ = string::npos;
@@ -228,7 +231,7 @@ Status RowOperationsPBDecoder::ReadOpType(RowOperationsPB::Type* type) {
     return Status::Corruption("Cannot find operation type");
   }
   if (PREDICT_FALSE(!RowOperationsPB_Type_IsValid(src_[0]))) {
-    return Status::Corruption(Substitute("Unknown operation type: $0", src_[0]));
+    return Status::NotSupported(Substitute("Unknown operation type: $0", src_[0]));
   }
   *type = static_cast<RowOperationsPB::Type>(src_[0]);
   src_.remove_prefix(1);
@@ -306,8 +309,7 @@ Status RowOperationsPBDecoder::ReadColumn(const ColumnSchema& col,
 
 Status RowOperationsPBDecoder::ReadColumnAndDiscard(const ColumnSchema& col) {
   uint8_t scratch[kLargestTypeSize];
-  RETURN_NOT_OK(ReadColumn(col, scratch, nullptr));
-  return Status::OK();
+  return ReadColumn(col, scratch, nullptr);
 }
 
 bool RowOperationsPBDecoder::HasNext() const {
@@ -345,7 +347,7 @@ class ClientServerMapping {
                       const Schema* tablet_schema)
     : client_schema_(client_schema),
       tablet_schema_(tablet_schema),
-      saw_tablet_col_(tablet_schema->num_columns()) {
+      saw_tablet_col_(tablet_schema->num_columns(), false) {
   }
 
   Status ProjectBaseColumn(size_t client_col_idx, size_t tablet_col_idx) {
@@ -418,7 +420,8 @@ size_t RowOperationsPBDecoder::GetTabletColIdx(const ClientServerMapping& mappin
 
 Status RowOperationsPBDecoder::DecodeInsertOrUpsert(const uint8_t* prototype_row_storage,
                                                     const ClientServerMapping& mapping,
-                                                    DecodedRowOperation* op) {
+                                                    DecodedRowOperation* op,
+                                                    int64_t* auto_incrementing_counter) {
   const uint8_t* client_isset_map = nullptr;
   const uint8_t* client_null_map = nullptr;
 
@@ -429,9 +432,9 @@ Status RowOperationsPBDecoder::DecodeInsertOrUpsert(const uint8_t* prototype_row
   }
 
   // Allocate a row with the tablet's layout.
-  auto tablet_row_storage = reinterpret_cast<uint8_t*>(
+  auto* tablet_row_storage = reinterpret_cast<uint8_t*>(
       dst_arena_->AllocateBytesAligned(tablet_row_size_, 8));
-  auto tablet_isset_bitmap = reinterpret_cast<uint8_t*>(
+  auto* tablet_isset_bitmap = reinterpret_cast<uint8_t*>(
       dst_arena_->AllocateBytes(BitmapSize(tablet_schema_->num_columns())));
   if (PREDICT_FALSE(!tablet_row_storage || !tablet_isset_bitmap)) {
     return Status::RuntimeError("Out of memory");
@@ -447,7 +450,7 @@ Status RowOperationsPBDecoder::DecodeInsertOrUpsert(const uint8_t* prototype_row
 
   // Now handle each of the columns passed by the user, replacing the defaults
   // from the prototype.
-  Status row_status;
+  const auto auto_incrementing_col_idx = tablet_schema_->auto_incrementing_col_idx();
   for (size_t client_col_idx = 0;
        client_col_idx < client_schema_->num_columns();
        client_col_idx++) {
@@ -455,32 +458,102 @@ Status RowOperationsPBDecoder::DecodeInsertOrUpsert(const uint8_t* prototype_row
     // ColumnSchema object since it has the most up-to-date default, nullability,
     // etc.
     size_t tablet_col_idx = GetTabletColIdx(mapping, client_col_idx);
+    DCHECK_NE(tablet_col_idx, Schema::kColumnNotFound);
     const ColumnSchema& col = tablet_schema_->column(tablet_col_idx);
 
     bool isset = BitmapTest(client_isset_map, client_col_idx);
     BitmapChange(tablet_isset_bitmap, tablet_col_idx, isset);
     if (isset) {
       // If the client provided a value for this column, copy it.
-
       // Copy null-ness, if the server side column is nullable.
-      bool client_set_to_null = client_schema_->has_nullables() &&
-        BitmapTest(client_null_map, client_col_idx);
+      const bool client_set_to_null = client_schema_->has_nullables() &&
+          BitmapTest(client_null_map, client_col_idx);
       if (col.is_nullable()) {
         tablet_row.set_null(tablet_col_idx, client_set_to_null);
       }
       if (!client_set_to_null) {
-        // Copy the value if it's not null.
-        RETURN_NOT_OK(ReadColumn(col, tablet_row.mutable_cell_ptr(tablet_col_idx), &row_status));
-        if (PREDICT_FALSE(!row_status.ok())) op->SetFailureStatusOnce(row_status);
+        // Check if the non-null value is present for auto-incrementing column. For UPSERT or
+        // UPSERT_IGNORE operations we allow the user to specify the auto-incrementing value.
+        if (tablet_col_idx != auto_incrementing_col_idx) {
+          // Copy the non-null value.
+          Status row_status;
+          RETURN_NOT_OK(ReadColumn(
+              col, tablet_row.mutable_cell_ptr(tablet_col_idx), &row_status));
+          if (PREDICT_FALSE(!row_status.ok())) {
+            op->SetFailureStatusOnce(row_status);
+          }
+        } else {
+          if (op->type == RowOperationsPB_Type_INSERT ||
+              op->type == RowOperationsPB_Type_INSERT_IGNORE) {
+            // auto-incrementing column values not to be set for INSERT/INSERT_IGNORE operations.
+            static const Status kErrFieldIncorrectlySet = Status::InvalidArgument(
+                "auto-incrementing column should not be set for INSERT/INSERT_IGNORE operations");
+            op->SetFailureStatusOnce(kErrFieldIncorrectlySet);
+            RETURN_NOT_OK(ReadColumnAndDiscard(col));
+            return kErrFieldIncorrectlySet;
+          }
+          // Fetch the auto-incrementing counter from the request.
+          Status row_status;
+          int64_t counter = 0;
+          RETURN_NOT_OK(ReadColumn(
+              col, reinterpret_cast<uint8_t*>(&counter), &row_status));
+          if (PREDICT_FALSE(!row_status.ok())) {
+            op->SetFailureStatusOnce(row_status);
+          } else {
+            // Make sure it is positive.
+            if (counter < 0) {
+              static const Status kErrorValue = Status::InvalidArgument(
+                  "auto-incrementing column value must be greater than zero");
+              op->SetFailureStatusOnce(kErrorValue);
+              return kErrorValue;
+            }
+            // Check if the provided counter value is less than what is in memory
+            // and update the counter in memory.
+            if (counter > *auto_incrementing_counter) {
+              *auto_incrementing_counter = counter;
+            }
+            memcpy(tablet_row.mutable_cell_ptr(tablet_col_idx), &counter, 8);
+          }
+        }
       } else if (PREDICT_FALSE(!col.is_nullable())) {
         op->SetFailureStatusOnce(Status::InvalidArgument(
             "NULL values not allowed for non-nullable column", col.ToString()));
         RETURN_NOT_OK(ReadColumnAndDiscard(col));
       }
     } else {
-      // If the client didn't provide a value, then the column must either be nullable or
-      // have a default (which was already set in the prototype row).
-      if (PREDICT_FALSE(!(col.is_nullable() || col.has_write_default()))) {
+      // If the client didn't provide a value, check if it's an auto-incrementing
+      // field. If so, populate the field as appropriate.
+      if (tablet_col_idx == auto_incrementing_col_idx) {
+        if (op->type == RowOperationsPB_Type_UPSERT ||
+            op->type == RowOperationsPB_Type_UPSERT_IGNORE) {
+          static const Status kErrMaxValue = Status::InvalidArgument("auto-incrementing column "
+                                                                     "should be set for "
+                                                                     "UPSERT/UPSERT_IGNORE "
+                                                                     "operations");
+          op->SetFailureStatusOnce(kErrMaxValue);
+          return kErrMaxValue;
+        }
+        if (*DCHECK_NOTNULL(auto_incrementing_counter) == INT64_MAX) {
+          static const Status kErrMaxValue = Status::IllegalState("max auto-incrementing column "
+                                                                   "value reached");
+          op->SetFailureStatusOnce(kErrMaxValue);
+          return kErrMaxValue;
+        }
+        if (*DCHECK_NOTNULL(auto_incrementing_counter) < 0) {
+          static const Status kErrValue = Status::IllegalState("invalid auto-incrementing "
+                                                               "column value");
+          op->SetFailureStatusOnce(kErrValue);
+          return kErrValue;
+        }
+        // We increment the auto incrementing counter at this point regardless of future failures
+        // in the op for simplicity. The auto-incrementing column key space is large enough to
+        // not run of values for any realistic workloads.
+        (*auto_incrementing_counter)++;
+        memcpy(tablet_row.mutable_cell_ptr(tablet_col_idx), auto_incrementing_counter, 8);
+        BitmapChange(tablet_isset_bitmap, client_col_idx, true);
+      } else if (PREDICT_FALSE(!(col.is_nullable() || col.has_write_default()))) {
+        // Otherwise, the column must either be nullable or have a default (which
+        // was already set in the prototype row).
         op->SetFailureStatusOnce(Status::InvalidArgument("No value provided for required column",
                                                          col.ToString()));
       }
@@ -494,8 +567,6 @@ Status RowOperationsPBDecoder::DecodeInsertOrUpsert(const uint8_t* prototype_row
 
 Status RowOperationsPBDecoder::DecodeUpdateOrDelete(const ClientServerMapping& mapping,
                                                     DecodedRowOperation* op) {
-  size_t rowkey_size = tablet_schema_->key_byte_size();
-
   const uint8_t* client_isset_map = nullptr;
   const uint8_t* client_null_map = nullptr;
 
@@ -506,8 +577,8 @@ Status RowOperationsPBDecoder::DecodeUpdateOrDelete(const ClientServerMapping& m
   }
 
   // Allocate space for the row key.
-  auto rowkey_storage = reinterpret_cast<uint8_t*>(
-    dst_arena_->AllocateBytesAligned(rowkey_size, 8));
+  auto* rowkey_storage = reinterpret_cast<uint8_t*>(
+      dst_arena_->AllocateBytesAligned(tablet_schema_->key_byte_size(), 8));
   if (PREDICT_FALSE(!rowkey_storage)) {
     return Status::RuntimeError("Out of memory");
   }
@@ -537,7 +608,7 @@ Status RowOperationsPBDecoder::DecodeUpdateOrDelete(const ClientServerMapping& m
     }
 
     bool client_set_to_null = client_schema_->has_nullables() &&
-      BitmapTest(client_null_map, client_col_idx);
+        BitmapTest(client_null_map, client_col_idx);
     if (PREDICT_FALSE(client_set_to_null)) {
       op->SetFailureStatusOnce(Status::InvalidArgument("NULL values not allowed for key column",
                                                        col.ToString()));
@@ -559,6 +630,7 @@ Status RowOperationsPBDecoder::DecodeUpdateOrDelete(const ClientServerMapping& m
     RowChangeListEncoder rcl_encoder(&buf);
 
     // Now process the rest of columns as updates.
+    DCHECK_EQ(client_schema_->num_key_columns(), client_col_idx);
     for (; client_col_idx < client_schema_->num_columns(); client_col_idx++) {
       size_t tablet_col_idx = GetTabletColIdx(mapping, client_col_idx);
       const ColumnSchema& col = tablet_schema_->column(tablet_col_idx);
@@ -566,11 +638,29 @@ Status RowOperationsPBDecoder::DecodeUpdateOrDelete(const ClientServerMapping& m
       if (BitmapTest(client_isset_map, client_col_idx)) {
         bool client_set_to_null = client_schema_->has_nullables() &&
           BitmapTest(client_null_map, client_col_idx);
+
+        if (col.is_immutable()) {
+          if (op->type == RowOperationsPB::UPDATE) {
+            op->SetFailureStatusOnce(
+                Status::Immutable("UPDATE not allowed for immutable column", col.ToString()));
+          } else {
+            DCHECK_EQ(RowOperationsPB::UPDATE_IGNORE, op->type);
+            op->error_ignored = true;
+          }
+          if (!client_set_to_null) {
+            RETURN_NOT_OK(ReadColumnAndDiscard(col));
+          }
+          // Use 'continue' not 'break' to consume the rest row data.
+          continue;
+        }
+
         uint8_t scratch[kLargestTypeSize];
         uint8_t* val_to_add = nullptr;
         if (!client_set_to_null) {
           RETURN_NOT_OK(ReadColumn(col, scratch, &row_status));
-          if (PREDICT_FALSE(!row_status.ok())) op->SetFailureStatusOnce(row_status);
+          if (PREDICT_FALSE(!row_status.ok())) {
+            op->SetFailureStatusOnce(row_status);
+          }
           val_to_add = scratch;
         } else if (PREDICT_FALSE(!col.is_nullable())) {
           op->SetFailureStatusOnce(Status::InvalidArgument(
@@ -578,6 +668,7 @@ Status RowOperationsPBDecoder::DecodeUpdateOrDelete(const ClientServerMapping& m
           RETURN_NOT_OK(ReadColumnAndDiscard(col));
           continue;
         }
+
         rcl_encoder.AddColumnUpdate(col, tablet_schema_->column_id(tablet_col_idx), val_to_add);
       }
     }
@@ -590,8 +681,8 @@ Status RowOperationsPBDecoder::DecodeUpdateOrDelete(const ClientServerMapping& m
 
     if (PREDICT_TRUE(op->result.ok())) {
       // Copy the row-changelist to the arena.
-      auto rcl_in_arena = reinterpret_cast<uint8_t*>(
-        dst_arena_->AllocateBytesAligned(buf.size(), 8));
+      auto* rcl_in_arena = reinterpret_cast<uint8_t*>(
+          dst_arena_->AllocateBytesAligned(buf.size(), 8));
       if (PREDICT_FALSE(rcl_in_arena == nullptr)) {
         return Status::RuntimeError("Out of memory allocating RCL");
       }
@@ -608,7 +699,7 @@ Status RowOperationsPBDecoder::DecodeUpdateOrDelete(const ClientServerMapping& m
             "DELETE should not have a value for column", col.ToString()));
 
         bool client_set_to_null = client_schema_->has_nullables() &&
-          BitmapTest(client_null_map, client_col_idx);
+            BitmapTest(client_null_map, client_col_idx);
         if (!client_set_to_null || !col.is_nullable()) {
           RETURN_NOT_OK(ReadColumnAndDiscard(col));
         }
@@ -626,7 +717,7 @@ Status RowOperationsPBDecoder::DecodeUpdateOrDelete(const ClientServerMapping& m
 
 Status RowOperationsPBDecoder::DecodeSplitRow(const ClientServerMapping& mapping,
                                               DecodedRowOperation* op) {
-  op->split_row.reset(new KuduPartialRow(tablet_schema_));
+  op->split_row = std::make_shared<KuduPartialRow>(tablet_schema_);
 
   const uint8_t* client_isset_map;
   const uint8_t* client_null_map;
@@ -651,12 +742,9 @@ Status RowOperationsPBDecoder::DecodeSplitRow(const ClientServerMapping& mapping
       // If the client provided a value for this column, copy it.
       Slice column_slice;
       RETURN_NOT_OK(GetColumnSlice(col, &column_slice, nullptr));
-      const uint8_t* data;
-      if (col.type_info()->physical_type() == BINARY) {
-        data = reinterpret_cast<const uint8_t*>(&column_slice);
-      } else {
-        data = column_slice.data();
-      }
+      const uint8_t* data =  (col.type_info()->physical_type() == BINARY)
+          ? reinterpret_cast<const uint8_t*>(&column_slice)
+          : column_slice.data();
       RETURN_NOT_OK(op->split_row->Set(static_cast<int32_t>(tablet_col_idx), data));
     }
   }
@@ -664,7 +752,8 @@ Status RowOperationsPBDecoder::DecodeSplitRow(const ClientServerMapping& mapping
 }
 
 template <DecoderMode mode>
-Status RowOperationsPBDecoder::DecodeOperations(vector<DecodedRowOperation>* ops) {
+Status RowOperationsPBDecoder::DecodeOperations(vector<DecodedRowOperation>* ops,
+                                                int64_t* auto_incrementing_counter) {
   // TODO(todd): there's a bug here, in that if a client passes some column in
   // its schema that has been deleted on the server, it will fail even if the
   // client never actually specified any values for it.  For example, a DBA
@@ -687,15 +776,18 @@ Status RowOperationsPBDecoder::DecodeOperations(vector<DecodedRowOperation>* ops
   uint8_t prototype_row_storage[tablet_row_size_];
   ContiguousRow prototype_row(tablet_schema_, prototype_row_storage);
   SetupPrototypeRow(*tablet_schema_, &prototype_row);
+  int64_t counter = -1;
+  auto_incrementing_counter = auto_incrementing_counter ? auto_incrementing_counter : &counter;
 
   while (HasNext()) {
     RowOperationsPB::Type type = RowOperationsPB::UNKNOWN;
     RETURN_NOT_OK(ReadOpType(&type));
+
     DecodedRowOperation op;
     op.type = type;
-
-    RETURN_NOT_OK(DecodeOp<mode>(type, prototype_row_storage, mapping, &op));
-    ops->push_back(op);
+    RETURN_NOT_OK(DecodeOp<mode>(type, prototype_row_storage, mapping, &op,
+                                 auto_incrementing_counter));
+    ops->emplace_back(std::move(op));
   }
 
   return Status::OK();
@@ -704,54 +796,52 @@ Status RowOperationsPBDecoder::DecodeOperations(vector<DecodedRowOperation>* ops
 template<>
 Status RowOperationsPBDecoder::DecodeOp<DecoderMode::WRITE_OPS>(
     RowOperationsPB::Type type, const uint8_t* prototype_row_storage,
-    const ClientServerMapping& mapping, DecodedRowOperation* op) {
+    const ClientServerMapping& mapping, DecodedRowOperation* op,
+    int64_t* auto_incrementing_counter) {
   switch (type) {
-    case RowOperationsPB::UNKNOWN:
-      return Status::NotSupported("Unknown row operation type");
+    case RowOperationsPB::UPSERT:
+    case RowOperationsPB::UPSERT_IGNORE:
     case RowOperationsPB::INSERT:
     case RowOperationsPB::INSERT_IGNORE:
-    case RowOperationsPB::UPSERT:
-      RETURN_NOT_OK(DecodeInsertOrUpsert(prototype_row_storage, mapping, op));
-      break;
+      return DecodeInsertOrUpsert(prototype_row_storage, mapping, op,
+                                  auto_incrementing_counter);
     case RowOperationsPB::UPDATE:
     case RowOperationsPB::UPDATE_IGNORE:
     case RowOperationsPB::DELETE:
     case RowOperationsPB::DELETE_IGNORE:
-      RETURN_NOT_OK(DecodeUpdateOrDelete(mapping, op));
-      break;
+      return DecodeUpdateOrDelete(mapping, op);
     default:
-      return Status::InvalidArgument(Substitute("Invalid write operation type $0",
-                                                RowOperationsPB_Type_Name(type)));
+      break;
   }
-  return Status::OK();
+  return Status::InvalidArgument(Substitute("Invalid write operation type $0",
+                                            RowOperationsPB_Type_Name(type)));
 }
 
 template<>
 Status RowOperationsPBDecoder::DecodeOp<DecoderMode::SPLIT_ROWS>(
     RowOperationsPB::Type type, const uint8_t* /*prototype_row_storage*/,
-    const ClientServerMapping& mapping, DecodedRowOperation* op) {
+    const ClientServerMapping& mapping, DecodedRowOperation* op,
+    int64_t* /*auto_incrementing_counter*/) {
   switch (type) {
-    case RowOperationsPB::UNKNOWN:
-      return Status::NotSupported("Unknown row operation type");
     case RowOperationsPB::SPLIT_ROW:
     case RowOperationsPB::RANGE_LOWER_BOUND:
     case RowOperationsPB::RANGE_UPPER_BOUND:
     case RowOperationsPB::EXCLUSIVE_RANGE_LOWER_BOUND:
     case RowOperationsPB::INCLUSIVE_RANGE_UPPER_BOUND:
-      RETURN_NOT_OK(DecodeSplitRow(mapping, op));
-      break;
+      return DecodeSplitRow(mapping, op);
     default:
-      return Status::InvalidArgument(Substitute("Invalid split row type $0",
-                                                RowOperationsPB_Type_Name(type)));
+      break;
   }
-  return Status::OK();
+  return Status::InvalidArgument(Substitute("Invalid split row type $0",
+                                            RowOperationsPB_Type_Name(type)));
 }
 
 template
 Status RowOperationsPBDecoder::DecodeOperations<DecoderMode::SPLIT_ROWS>(
-    vector<DecodedRowOperation>* ops);
+    vector<DecodedRowOperation>* ops, int64_t* auto_incrementing_counter);
+
 template
 Status RowOperationsPBDecoder::DecodeOperations<DecoderMode::WRITE_OPS>(
-    vector<DecodedRowOperation>* ops);
+    vector<DecodedRowOperation>* ops, int64_t* auto_incrementing_counter);
 
 } // namespace kudu
