@@ -50,6 +50,8 @@ DEFINE_bool(cmeta_force_fsync, false,
             "Whether fsync() should be called when consensus metadata files are updated");
 TAG_FLAG(cmeta_force_fsync, advanced);
 
+DECLARE_bool(cmeta_fsync_override_on_xfs);
+
 using std::string;
 using strings::Substitute;
 
@@ -302,6 +304,8 @@ Status ConsensusMetadata::Flush(FlushMode flush_mode) {
                           "Unable to fsync consensus parent dir " + parent_dir);
   }
 
+  const bool cmeta_force_fsync =
+      FLAGS_cmeta_force_fsync || (FLAGS_cmeta_fsync_override_on_xfs && fs_manager_->meta_on_xfs());
   string meta_file_path = fs_manager_->GetConsensusMetadataPath(tablet_id_);
   RETURN_NOT_OK_PREPEND(pb_util::WritePBContainerToPath(
       fs_manager_->env(), meta_file_path, pb_,
@@ -315,7 +319,8 @@ Status ConsensusMetadata::Flush(FlushMode flush_mode) {
       // fsync() due to periodic commit with default settings, whereas other
       // filesystems such as XFS will not commit as often and need the fsync to
       // avoid significant data loss when a crash happens.
-      FLAGS_log_force_fsync_all || FLAGS_cmeta_force_fsync ? pb_util::SYNC : pb_util::NO_SYNC),
+      FLAGS_log_force_fsync_all || cmeta_force_fsync ? pb_util::SYNC : pb_util::NO_SYNC,
+      pb_util::SENSITIVE),
           Substitute("Unable to write consensus meta file for tablet $0 to path $1",
                      tablet_id_, meta_file_path));
   return UpdateOnDiskSize();
@@ -372,7 +377,8 @@ Status ConsensusMetadata::Load(FsManager* fs_manager,
   scoped_refptr<ConsensusMetadata> cmeta(new ConsensusMetadata(fs_manager, tablet_id, peer_uuid));
   RETURN_NOT_OK(pb_util::ReadPBContainerFromPath(fs_manager->env(),
                                                  fs_manager->GetConsensusMetadataPath(tablet_id),
-                                                 &cmeta->pb_));
+                                                 &cmeta->pb_,
+                                                 pb_util::SENSITIVE));
   cmeta->UpdateActiveRole(); // Needs to happen here as we sidestep the accessor APIs.
 
   RETURN_NOT_OK(cmeta->UpdateOnDiskSize());
