@@ -48,9 +48,7 @@
 #include "kudu/util/flag_tags.h"
 #include "kudu/util/logging.h"
 #include "kudu/util/monotime.h"
-#include "kudu/util/net/dns_resolver.h"
 #include "kudu/util/net/net_util.h"
-#include "kudu/util/net/sockaddr.h"
 #include "kudu/util/pb_util.h"
 #include "kudu/util/status.h"
 #include "kudu/util/threadpool.h"
@@ -103,26 +101,22 @@ namespace consensus {
 // The number of retries between failed requests whose failure is logged.
 constexpr auto kNumRetriesBetweenLoggingFailedRequest = 5;
 
-Status Peer::NewRemotePeer(RaftPeerPB peer_pb,
-                           string tablet_id,
-                           string leader_uuid,
-                           PeerMessageQueue* queue,
-                           ThreadPoolToken* raft_pool_token,
-                           unique_ptr<PeerProxy> proxy,
-                           shared_ptr<Messenger> messenger,
-                           shared_ptr<Peer>* peer) {
-
+void Peer::NewRemotePeer(RaftPeerPB peer_pb,
+                         string tablet_id,
+                         string leader_uuid,
+                         PeerMessageQueue* queue,
+                         ThreadPoolToken* raft_pool_token,
+                         PeerProxyFactory* peer_proxy_factory,
+                         shared_ptr<Peer>* peer) {
   auto new_peer(Peer::make_shared(
       std::move(peer_pb),
       std::move(tablet_id),
       std::move(leader_uuid),
       queue,
       raft_pool_token,
-      std::move(proxy),
-      std::move(messenger)));
-  RETURN_NOT_OK(new_peer->Init());
+      peer_proxy_factory));
+  new_peer->Init();
   *peer = std::move(new_peer);
-  return Status::OK();
 }
 
 Peer::Peer(RaftPeerPB peer_pb,
@@ -130,25 +124,25 @@ Peer::Peer(RaftPeerPB peer_pb,
            string leader_uuid,
            PeerMessageQueue* queue,
            ThreadPoolToken* raft_pool_token,
-           unique_ptr<PeerProxy> proxy,
-           shared_ptr<Messenger> messenger)
+           PeerProxyFactory* peer_proxy_factory)
     : tablet_id_(std::move(tablet_id)),
       leader_uuid_(std::move(leader_uuid)),
       peer_pb_(std::move(peer_pb)),
       log_prefix_(Substitute("T $0 P $1 -> Peer $2 ($3:$4): ",
                   tablet_id_, leader_uuid_, peer_pb_.permanent_uuid(),
                   peer_pb_.last_known_addr().host(), peer_pb_.last_known_addr().port())),
-      proxy_(std::move(proxy)),
+      peer_proxy_factory_(peer_proxy_factory),
       queue_(queue),
       failed_attempts_(0),
-      messenger_(std::move(messenger)),
+      messenger_(peer_proxy_factory_->messenger()),
       raft_pool_token_(raft_pool_token),
       request_pending_(false),
       closed_(false),
       has_sent_first_request_(false) {
+  CreateProxyIfNeeded();
 }
 
-Status Peer::Init() {
+void Peer::Init() {
   {
     std::lock_guard<simple_spinlock> l(peer_lock_);
     queue_->TrackPeer(peer_pb_);
@@ -166,7 +160,6 @@ Status Peer::Init() {
       },
       MonoDelta::FromMilliseconds(FLAGS_raft_heartbeat_interval_ms));
   heartbeater_->Start();
-  return Status::OK();
 }
 
 Status Peer::SignalRequest(bool even_if_queue_empty) {
@@ -241,6 +234,14 @@ void Peer::SendNextRequest(bool even_if_queue_empty) {
     return;
   }
 
+  // NOTE: we only perform this check after creating the RequestForPeer() call
+  // to ensure any peer health updates that happen therein associated with this
+  // peer actually happen. E.g. if we haven't been able to create a proxy in a
+  // long enough time, the peer should be considered failed.
+  if (PREDICT_FALSE(!CreateProxyIfNeeded())) {
+    return;
+  }
+
   if (PREDICT_FALSE(needs_tablet_copy)) {
     Status s = PrepareTabletCopyRequest();
     if (s.ok()) {
@@ -300,6 +301,9 @@ void Peer::SendNextRequest(bool even_if_queue_empty) {
 }
 
 void Peer::StartElection() {
+  if (PREDICT_FALSE(!CreateProxyIfNeeded())) {
+    return;
+  }
   // The async proxy contract is such that the response and RPC controller must
   // stay in scope until the callback is invoked. Unlike other Peer methods, we
   // can't guarantee that there's only one outstanding StartElection call at a
@@ -315,7 +319,8 @@ void Peer::StartElection() {
   // TODO(adar): lack of C++14 move capture makes for ugly code.
   RunLeaderElectionResponsePB* resp = resp_uniq.release();
   RpcController* controller = controller_uniq.release();
-  proxy_->StartElectionAsync(req, resp, controller, [resp, controller, peer_uuid]() {
+  auto s_this = shared_from_this();
+  proxy_->StartElectionAsync(req, resp, controller, [resp, controller, peer_uuid, s_this]() {
       unique_ptr<RunLeaderElectionResponsePB> r(resp);
       unique_ptr<RpcController> c(controller);
       string error_msg = Substitute("unable to start election on peer $0", peer_uuid);
@@ -489,6 +494,23 @@ void Peer::ProcessResponseErrorUnlocked(const Status& status) {
   request_pending_ = false;
 }
 
+bool Peer::CreateProxyIfNeeded() {
+  std::lock_guard<simple_spinlock> l(proxy_lock_);
+  if (!proxy_) {
+    unique_ptr<PeerProxy> peer_proxy;
+    Status s = DCHECK_NOTNULL(peer_proxy_factory_)->NewProxy(peer_pb_, &peer_proxy);
+    if (!s.ok()) {
+      HostPort hostport = HostPortFromPB(peer_pb_.last_known_addr());
+      KLOG_EVERY_N_SECS(WARNING, 1)
+          << Substitute("Unable to create proxy for $0 ($1)",
+                        peer_pb_.permanent_uuid(), hostport.ToString());
+      return false;
+    }
+    proxy_ = std::move(peer_proxy);
+  }
+  return true;
+}
+
 const string& Peer::LogPrefixUnlocked() const {
   return log_prefix_;
 }
@@ -565,14 +587,8 @@ Status CreateConsensusServiceProxyForHost(
     const shared_ptr<Messenger>& messenger,
     DnsResolver* dns_resolver,
     unique_ptr<ConsensusServiceProxy>* new_proxy) {
-  vector<Sockaddr> addrs;
-  RETURN_NOT_OK(dns_resolver->ResolveAddresses(hostport, &addrs));
-  if (addrs.size() > 1) {
-    LOG(WARNING) << Substitute(
-        "Peer address '$0' resolves to $1 different addresses. "
-        "Using $2", hostport.ToString(), addrs.size(), addrs[0].ToString());
-  }
-  new_proxy->reset(new ConsensusServiceProxy(messenger, addrs[0], hostport.host()));
+  new_proxy->reset(new ConsensusServiceProxy(messenger, hostport, dns_resolver));
+  (*new_proxy)->Init();
   return Status::OK();
 }
 
