@@ -77,10 +77,19 @@ TAG_FLAG(fs_data_dirs_reserved_bytes, runtime);
 TAG_FLAG(fs_data_dirs_reserved_bytes, evolving);
 
 DEFINE_int32(fs_data_dirs_available_space_cache_seconds, 10,
-             "Number of seconds we cache the available disk space in the block manager. ");
+             "Number of seconds we cache the available disk space in the block manager.");
+DEFINE_validator(fs_data_dirs_available_space_cache_seconds,
+                 [](const char* /*n*/, int32_t v) { return v >= 0; });
 TAG_FLAG(fs_data_dirs_available_space_cache_seconds, advanced);
 TAG_FLAG(fs_data_dirs_available_space_cache_seconds, evolving);
 
+DEFINE_int32(fs_wal_dir_available_space_cache_seconds, 10,
+             "Number of seconds we cache the available disk space the WAL directory.");
+DEFINE_validator(fs_wal_dir_available_space_cache_seconds,
+                 [](const char* /*n*/, int32_t v) { return v >= 0; });
+TAG_FLAG(fs_wal_dir_available_space_cache_seconds, advanced);
+TAG_FLAG(fs_wal_dir_available_space_cache_seconds, evolving);
+
 DEFINE_bool(fs_lock_data_dirs, true,
             "Lock the data directories to prevent concurrent usage. "
             "Note that read-only concurrent usage is still allowed.");
