@@ -25,16 +25,17 @@
 #include <functional>
 #include <initializer_list>
 #include <memory>
+#include <optional>
 #include <ostream>
 #include <random>
 #include <set>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <unordered_set>
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 #include <gtest/gtest.h>
@@ -65,6 +66,7 @@
 #include "kudu/util/stopwatch.h" // IWYU pragma: keep
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
+#include "kudu/util/threadpool.h"
 
 using kudu::pb_util::ReadablePBContainerFile;
 using std::set;
@@ -88,6 +90,8 @@ DECLARE_string(env_inject_eio_globs);
 DECLARE_uint64(log_container_preallocate_bytes);
 DECLARE_uint64(log_container_max_size);
 DECLARE_uint64(log_container_metadata_max_size);
+DECLARE_bool(log_container_metadata_runtime_compact);
+DECLARE_double(log_container_metadata_size_before_compact_ratio);
 DEFINE_int32(startup_benchmark_block_count_for_testing, 1000000,
              "Block count to do startup benchmark.");
 DEFINE_int32(startup_benchmark_data_dir_count_for_testing, 8,
@@ -134,7 +138,7 @@ class LogBlockManagerTest : public KuduTest, public ::testing::WithParamInterfac
   void SetUp() override {
     // Pass in a report to prevent the block manager from logging unnecessarily.
     FsReport report;
-    ASSERT_OK(bm_->Open(&report));
+    ASSERT_OK(bm_->Open(&report, nullptr, nullptr));
     ASSERT_OK(dd_manager_->CreateDataDirGroup(test_tablet_name_));
     ASSERT_OK(dd_manager_->GetDataDirGroupPB(test_tablet_name_, &test_group_pb_));
   }
@@ -180,7 +184,7 @@ class LogBlockManagerTest : public KuduTest, public ::testing::WithParamInterfac
     }
 
     bm_.reset(CreateBlockManager(metric_entity, test_data_dirs));
-    RETURN_NOT_OK(bm_->Open(report));
+    RETURN_NOT_OK(bm_->Open(report, nullptr, nullptr));
     return Status::OK();
   }
 
@@ -202,6 +206,10 @@ class LogBlockManagerTest : public KuduTest, public ::testing::WithParamInterfac
     *metadata_file = metadata_files[0];
   }
 
+  void GetContainerMetadataFiles(vector<string>* metadata_files) {
+    DoGetContainers(METADATA_FILES, metadata_files);
+  }
+
   // Like GetOnlyContainerDataFile(), but returns a container name (i.e. data
   // or metadata file with the file suffix removed).
   void GetOnlyContainer(string* container) {
@@ -232,10 +240,6 @@ class LogBlockManagerTest : public KuduTest, public ::testing::WithParamInterfac
     ASSERT_TRUE(report.partial_record_check->entries.empty());
   }
 
-  void EnableEncryption(bool enable) {
-    FLAGS_encrypt_data_at_rest = enable;
-  }
-
   DataDirGroupPB test_group_pb_;
   string test_tablet_name_;
   CreateBlockOptions test_block_opts_;
@@ -335,7 +339,7 @@ static void CheckLogMetrics(const scoped_refptr<MetricEntity>& entity,
 INSTANTIATE_TEST_SUITE_P(EncryptionEnabled, LogBlockManagerTest, ::testing::Values(false, true));
 
 TEST_P(LogBlockManagerTest, MetricsTest) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   MetricRegistry registry;
   scoped_refptr<MetricEntity> entity = METRIC_ENTITY_server.Instantiate(&registry, "test");
   ASSERT_OK(ReopenBlockManager(entity));
@@ -449,10 +453,7 @@ TEST_P(LogBlockManagerTest, MetricsTest) {
           {1, &METRIC_block_manager_total_blocks_deleted},
           {0, &METRIC_log_block_manager_dead_containers_deleted} }));
   }
-  // Wait for the actual hole punching to take place.
-  for (const auto& data_dir : dd_manager_->dirs()) {
-    data_dir->WaitOnClosures();
-  }
+  dd_manager_->WaitOnClosures();
   NO_FATALS(CheckLogMetrics(new_entity,
       { {9 * 1024, &METRIC_log_block_manager_bytes_under_management},
         {10, &METRIC_log_block_manager_blocks_under_management},
@@ -511,7 +512,7 @@ TEST_P(LogBlockManagerTest, MetricsTest) {
 }
 
 TEST_P(LogBlockManagerTest, ContainerPreallocationTest) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   string kTestData = "test data";
 
   // For this test to work properly, the preallocation window has to be at
@@ -556,7 +557,7 @@ TEST_P(LogBlockManagerTest, ContainerPreallocationTest) {
 // Test for KUDU-2202 to ensure that once the block manager has been notified
 // of a block ID, it will not reuse it.
 TEST_P(LogBlockManagerTest, TestBumpBlockIds) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   const int kNumBlocks = 10;
   vector<BlockId> block_ids;
   unique_ptr<WritableBlock> writer;
@@ -589,7 +590,7 @@ TEST_P(LogBlockManagerTest, TestBumpBlockIds) {
 // Regression test for KUDU-1190, a crash at startup when a block ID has been
 // reused.
 TEST_P(LogBlockManagerTest, TestReuseBlockIds) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   // Typically, the LBM starts with a random block ID when running as a
   // gtest. In this test, we want to control the block IDs.
   bm_->next_block_id_.Store(1);
@@ -664,7 +665,7 @@ TEST_P(LogBlockManagerTest, TestReuseBlockIds) {
 // Note that we rely on filesystem integrity to ensure that we do not lose
 // trailing, fsync()ed metadata.
 TEST_P(LogBlockManagerTest, TestMetadataTruncation) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   // Create several blocks.
   vector<BlockId> created_blocks;
   BlockId last_block_id;
@@ -843,7 +844,7 @@ TEST_P(LogBlockManagerTest, TestMetadataTruncation) {
   // Now try to reopen the container.
   // This should look like a bad checksum, and it's not recoverable.
   s = ReopenBlockManager();
-  ASSERT_TRUE(s.IsCorruption());
+  ASSERT_TRUE(s.IsCorruption()) << s.ToString();
   ASSERT_STR_CONTAINS(s.ToString(), "Incorrect checksum");
 
   // Now truncate both the data and metadata files.
@@ -859,7 +860,7 @@ TEST_P(LogBlockManagerTest, TestMetadataTruncation) {
 // Regression test for a crash when a container's append offset exceeded its
 // preallocation offset.
 TEST_P(LogBlockManagerTest, TestAppendExceedsPreallocation) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   FLAGS_log_container_preallocate_bytes = 1;
 
   // Create a container, preallocate it by one byte, and append more than one.
@@ -875,7 +876,7 @@ TEST_P(LogBlockManagerTest, TestAppendExceedsPreallocation) {
 }
 
 TEST_P(LogBlockManagerTest, TestPreallocationAndTruncation) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   // Ensure preallocation window is greater than the container size itself.
   FLAGS_log_container_max_size = 1024 * 1024;
   FLAGS_log_container_preallocate_bytes = 32 * 1024 * 1024;
@@ -939,7 +940,7 @@ TEST_P(LogBlockManagerTest, TestPreallocationAndTruncation) {
 }
 
 TEST_P(LogBlockManagerTest, TestContainerWithManyHoles) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   // This is a regression test of sorts for KUDU-1508, though it doesn't
   // actually fail if the fix is missing; it just corrupts the filesystem.
 
@@ -998,7 +999,7 @@ TEST_P(LogBlockManagerTest, TestContainerWithManyHoles) {
 }
 
 TEST_P(LogBlockManagerTest, TestParseKernelRelease) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   ASSERT_TRUE(LogBlockManager::IsBuggyEl6Kernel("1.7.0.0.el6.x86_64"));
 
   // no el6 infix
@@ -1122,7 +1123,7 @@ TEST_P(LogBlockManagerStartupBenchmarkTest, StartupBenchmark) {
 #endif
 
 TEST_P(LogBlockManagerTest, TestFailMultipleTransactionsPerContainer) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   // Create multiple transactions that will share a container.
   const int kNumTransactions = 3;
   vector<unique_ptr<BlockCreationTransaction>> block_transactions;
@@ -1151,7 +1152,7 @@ TEST_P(LogBlockManagerTest, TestFailMultipleTransactionsPerContainer) {
     FLAGS_crash_on_eio = false;
     FLAGS_env_inject_eio = 1.0;
     Status s = block_transactions[0]->CommitCreatedBlocks();
-    ASSERT_TRUE(s.IsIOError());
+    ASSERT_TRUE(s.IsIOError()) << s.ToString();
   }
 
   // Now try to add some more blocks.
@@ -1164,7 +1165,7 @@ TEST_P(LogBlockManagerTest, TestFailMultipleTransactionsPerContainer) {
     // new container.
     Status s = block->Append("x");
     if (i == 0) {
-      ASSERT_TRUE(s.IsIOError());
+      ASSERT_TRUE(s.IsIOError()) << s.ToString();
     } else {
       ASSERT_OK_FAST(s);
       ASSERT_OK_FAST(block->Finalize());
@@ -1181,7 +1182,7 @@ TEST_P(LogBlockManagerTest, TestFailMultipleTransactionsPerContainer) {
 }
 
 TEST_P(LogBlockManagerTest, TestLookupBlockLimit) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   int64_t limit_1024 = LogBlockManager::LookupBlockLimit(1024);
   int64_t limit_2048 = LogBlockManager::LookupBlockLimit(2048);
   int64_t limit_4096 = LogBlockManager::LookupBlockLimit(4096);
@@ -1199,7 +1200,7 @@ TEST_P(LogBlockManagerTest, TestLookupBlockLimit) {
 }
 
 TEST_P(LogBlockManagerTest, TestContainerBlockLimitingByBlockNum) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   const int kNumBlocks = 1000;
 
   // Creates 'kNumBlocks' blocks with minimal data.
@@ -1234,7 +1235,7 @@ TEST_P(LogBlockManagerTest, TestContainerBlockLimitingByBlockNum) {
 }
 
 TEST_P(LogBlockManagerTest, TestContainerBlockLimitingByMetadataSize) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   const int kNumBlocks = 1000;
 
   // Creates 'kNumBlocks' blocks with minimal data.
@@ -1270,8 +1271,98 @@ TEST_P(LogBlockManagerTest, TestContainerBlockLimitingByMetadataSize) {
   NO_FATALS(AssertNumContainers(4));
 }
 
+TEST_F(LogBlockManagerTest, TestContainerBlockLimitingByMetadataSizeWithCompaction) {
+  const int kNumBlocks = 2000;
+  const int kNumThreads = 10;
+  const double kLiveBlockRatio = 0.1;
+
+  // Creates and deletes some blocks.
+  auto create_and_delete_blocks = [&]() {
+    vector<BlockId> ids;
+    // Creates 'kNumBlocks' blocks.
+    for (int i = 0; i < kNumBlocks; i++) {
+      unique_ptr<WritableBlock> block;
+      RETURN_NOT_OK(bm_->CreateBlock(test_block_opts_, &block));
+      RETURN_NOT_OK(block->Append("aaaa"));
+      RETURN_NOT_OK(block->Close());
+      ids.push_back(block->id());
+    }
+
+    // Deletes 'kNumBlocks * (1 - kLiveBlockRatio)' blocks.
+    shared_ptr<BlockDeletionTransaction> deletion_transaction =
+        bm_->NewDeletionTransaction();
+    for (const auto& id : ids) {
+      if (rand() % 100 < 100 * kLiveBlockRatio) {
+        continue;
+      }
+      deletion_transaction->AddDeletedBlock(id);
+    }
+    vector<BlockId> deleted;
+    RETURN_NOT_OK(deletion_transaction->CommitDeletedBlocks(&deleted));
+
+    return Status::OK();
+  };
+
+  // Create a thread pool to create and delete blocks.
+  unique_ptr<ThreadPool> pool;
+  ASSERT_OK(ThreadPoolBuilder("test-metadata-compact-pool")
+                .set_max_threads(kNumThreads)
+                .Build(&pool));
+  auto mt_create_and_delete_blocks = [&]() {
+    for (int i = 0; i < kNumThreads; ++i) {
+      ASSERT_OK(pool->Submit(create_and_delete_blocks));
+    }
+    pool->Wait();
+    dd_manager_->WaitOnClosures();
+  };
+
+  FLAGS_log_container_metadata_runtime_compact = true;
+  // Define a small value to make metadata easy to be full.
+  FLAGS_log_container_metadata_max_size = 32 * 1024;
+  NO_FATALS(mt_create_and_delete_blocks());
+  vector<string> metadata_files;
+  NO_FATALS(GetContainerMetadataFiles(&metadata_files));
+  for (const auto& metadata_file : metadata_files) {
+    uint64_t file_size;
+    NO_FATALS(env_->GetFileSize(metadata_file, &file_size));
+    ASSERT_GE(FLAGS_log_container_metadata_max_size *
+                  FLAGS_log_container_metadata_size_before_compact_ratio,
+              file_size);
+  }
+
+  // Reopen and test again.
+  ASSERT_OK(ReopenBlockManager());
+  NO_FATALS(mt_create_and_delete_blocks());
+  NO_FATALS(GetContainerMetadataFiles(&metadata_files));
+  for (const auto& metadata_file : metadata_files) {
+    uint64_t file_size;
+    NO_FATALS(env_->GetFileSize(metadata_file, &file_size));
+    ASSERT_GE(FLAGS_log_container_metadata_max_size *
+                  FLAGS_log_container_metadata_size_before_compact_ratio,
+              file_size);
+  }
+
+  // Now remove the limit and create more blocks. They should go into existing
+  // containers, which are now no longer full.
+  FLAGS_log_container_metadata_runtime_compact = false;
+  ASSERT_OK(ReopenBlockManager());
+  NO_FATALS(mt_create_and_delete_blocks());
+  NO_FATALS(GetContainerMetadataFiles(&metadata_files));
+  bool exist_larger_one = false;
+  for (const auto& metadata_file : metadata_files) {
+    uint64_t file_size;
+    NO_FATALS(env_->GetFileSize(metadata_file, &file_size));
+    if (file_size > FLAGS_log_container_metadata_max_size *
+                         FLAGS_log_container_metadata_size_before_compact_ratio) {
+      exist_larger_one = true;
+      break;
+    }
+  }
+  ASSERT_TRUE(exist_larger_one);
+}
+
 TEST_P(LogBlockManagerTest, TestMisalignedBlocksFuzz) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   FLAGS_log_container_preallocate_bytes = 0;
   const int kNumBlocks = 100;
 
@@ -1374,7 +1465,7 @@ TEST_P(LogBlockManagerTest, TestMisalignedBlocksFuzz) {
 }
 
 TEST_P(LogBlockManagerTest, TestRepairPreallocateExcessSpace) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   // Enforce that the container's actual size is strictly upper-bounded by the
   // calculated size so we can more easily trigger repairs.
   FLAGS_log_container_excess_space_before_cleanup_fraction = 0.0;
@@ -1423,7 +1514,7 @@ TEST_P(LogBlockManagerTest, TestRepairPreallocateExcessSpace) {
 }
 
 TEST_P(LogBlockManagerTest, TestRepairUnpunchedBlocks) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   const int kNumBlocks = 100;
 
   // Enforce that the container's actual size is strictly upper-bounded by the
@@ -1484,7 +1575,7 @@ TEST_P(LogBlockManagerTest, TestRepairUnpunchedBlocks) {
 }
 
 TEST_P(LogBlockManagerTest, TestRepairIncompleteContainer) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   const int kNumContainers = 20;
 
   // Create some incomplete containers. The corruptor will select between
@@ -1515,7 +1606,7 @@ TEST_P(LogBlockManagerTest, TestRepairIncompleteContainer) {
 }
 
 TEST_P(LogBlockManagerTest, TestDetectMalformedRecords) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   const int kNumRecords = 50;
 
   // Create one container.
@@ -1548,7 +1639,7 @@ TEST_P(LogBlockManagerTest, TestDetectMalformedRecords) {
 }
 
 TEST_P(LogBlockManagerTest, TestDetectMisalignedBlocks) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   const int kNumBlocks = 50;
 
   // Create one container.
@@ -1581,7 +1672,7 @@ TEST_P(LogBlockManagerTest, TestDetectMisalignedBlocks) {
 }
 
 TEST_P(LogBlockManagerTest, TestRepairPartialRecords) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   const int kNumContainers = 50;
   const int kNumRecords = 10;
 
@@ -1623,7 +1714,7 @@ TEST_P(LogBlockManagerTest, TestRepairPartialRecords) {
 }
 
 TEST_P(LogBlockManagerTest, TestDeleteDeadContainersAtStartup) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   // Force our single container to become full once created.
   FLAGS_log_container_max_size = 0;
 
@@ -1661,7 +1752,7 @@ TEST_P(LogBlockManagerTest, TestDeleteDeadContainersAtStartup) {
 }
 
 TEST_P(LogBlockManagerTest, TestCompactFullContainerMetadataAtStartup) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   // With this ratio, the metadata of a full container comprised of half dead
   // blocks will be compacted at startup.
   FLAGS_log_container_live_metadata_before_compact_ratio = 0.50;
@@ -1727,7 +1818,7 @@ TEST_P(LogBlockManagerTest, TestCompactFullContainerMetadataAtStartup) {
 // The bug was related to a stale file descriptor left in the file_cache, so
 // this test explicitly targets that scenario.
 TEST_P(LogBlockManagerTest, TestDeleteFromContainerAfterMetadataCompaction) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   // Compact aggressively.
   FLAGS_log_container_live_metadata_before_compact_ratio = 0.99;
   // Use a single shard so that we have an accurate max cache capacity
@@ -1791,7 +1882,7 @@ TEST_P(LogBlockManagerTest, TestDeleteFromContainerAfterMetadataCompaction) {
 // will run smoothly. The directory manager will note the failed directories
 // and only healthy ones are reported.
 TEST_P(LogBlockManagerTest, TestOpenWithFailedDirectories) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   // Initialize a new directory manager with multiple directories.
   bm_.reset();
   vector<string> test_dirs;
@@ -1823,7 +1914,7 @@ TEST_P(LogBlockManagerTest, TestOpenWithFailedDirectories) {
 
   // Check the report, ensuring the correct directory has failed.
   FsReport report;
-  ASSERT_OK(bm_->Open(&report));
+  ASSERT_OK(bm_->Open(&report, nullptr, nullptr));
   ASSERT_EQ(kNumDirs - 1, report.data_dirs.size());
   for (const string& data_dir : report.data_dirs) {
     ASSERT_NE(data_dir, test_dirs[failed_idx]);
@@ -1841,7 +1932,7 @@ TEST_P(LogBlockManagerTest, TestOpenWithFailedDirectories) {
 // 2) the block cannot be opened/found until close it.
 // 3) the same container is not marked as available twice.
 TEST_P(LogBlockManagerTest, TestFinalizeBlock) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   // Create 4 blocks.
   vector<unique_ptr<WritableBlock>> blocks;
   for (int i = 0; i < 4; i++) {
@@ -1866,7 +1957,7 @@ TEST_P(LogBlockManagerTest, TestFinalizeBlock) {
 
 // Test available log container selection is LIFO.
 TEST_P(LogBlockManagerTest, TestLIFOContainerSelection) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   // Create 4 blocks and 4 opened containers that are not full.
   vector<unique_ptr<WritableBlock>> blocks;
   for (int i = 0; i < 4; i++) {
@@ -1919,7 +2010,7 @@ TEST_P(LogBlockManagerTest, TestAbortBlock) {
 }
 
 TEST_P(LogBlockManagerTest, TestDeleteDeadContainersByDeletionTransaction) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   const auto TestProcess = [&] (int block_num) {
     ASSERT_GT(block_num, 0);
     MetricRegistry registry;
@@ -2039,7 +2130,7 @@ TEST_P(LogBlockManagerTest, TestDeleteDeadContainersByDeletionTransaction) {
 // Test for KUDU-2665 to ensure that once the container is full and has no live
 // blocks but with a reference by WritableBlock, it will not be deleted.
 TEST_P(LogBlockManagerTest, TestDoNotDeleteFakeDeadContainer) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   // Lower the max container size.
   FLAGS_log_container_max_size = 64 * 1024;
 
@@ -2103,7 +2194,7 @@ TEST_P(LogBlockManagerTest, TestDoNotDeleteFakeDeadContainer) {
 }
 
 TEST_P(LogBlockManagerTest, TestHalfPresentContainer) {
-  EnableEncryption(GetParam());
+  SetEncryptionFlags(GetParam());
   BlockId block_id;
   string data_file_name;
   string metadata_file_name;
