@@ -20,11 +20,11 @@
 #include <algorithm>
 #include <cstdint>
 #include <map>
+#include <optional>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <glog/stl_logging.h>
 #include <gtest/gtest.h>
 
@@ -42,7 +42,7 @@
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
 
-using boost::optional;
+using std::optional;
 using kudu::rpc::SaslProtection;
 using std::make_pair;
 using std::string;
@@ -72,6 +72,9 @@ class HmsClientTest : public KuduTest,
         make_pair(HmsClient::kKuduMasterAddrsKey, string("TODO")),
         make_pair(HmsClient::kStorageHandlerKey, HmsClient::kKuduStorageHandler)
     });
+    table.sd.inputFormat = HmsClient::kKuduInputFormat;
+    table.sd.outputFormat = HmsClient::kKuduOutputFormat;
+    table.sd.serdeInfo.serializationLib = HmsClient::kKuduSerDeLib;
 
     hive::EnvironmentContext env_ctx;
     env_ctx.__set_properties({ make_pair(HmsClient::kKuduMasterEventKey, "true") });
@@ -91,7 +94,7 @@ class HmsClientTest : public KuduTest,
 
 INSTANTIATE_TEST_SUITE_P(ProtectionTypes,
                          HmsClientTest,
-                         ::testing::Values(boost::none
+                         ::testing::Values(std::nullopt
                                         , SaslProtection::kIntegrity
 // On macos, krb5 has issues repeatedly spinning up new KDCs ('unable to reach
 // any KDC in realm KRBTEST.COM, tried 1 KDC'). Integrity protection gives us
@@ -103,8 +106,7 @@ INSTANTIATE_TEST_SUITE_P(ProtectionTypes,
                                           ));
 
 TEST_P(HmsClientTest, TestHmsOperations) {
-  auto protection = boost::make_optional<SaslProtection::Type>(false, SaslProtection::kIntegrity);
-  protection = GetParam();
+  const auto protection = GetParam();
 
   MiniKdc kdc;
   MiniHms hms;
@@ -182,6 +184,9 @@ TEST_P(HmsClientTest, TestHmsOperations) {
   EXPECT_EQ(cluster_id, my_table.parameters[HmsClient::kKuduClusterIdKey]);
   EXPECT_EQ(HmsClient::kKuduStorageHandler, my_table.parameters[HmsClient::kStorageHandlerKey]);
   EXPECT_EQ(HmsClient::kManagedTable, my_table.tableType);
+  EXPECT_EQ(HmsClient::kKuduInputFormat, my_table.sd.inputFormat);
+  EXPECT_EQ(HmsClient::kKuduOutputFormat, my_table.sd.outputFormat);
+  EXPECT_EQ(HmsClient::kKuduSerDeLib, my_table.sd.serdeInfo.serializationLib);
 
   string new_table_name = "my_altered_table";
 
@@ -207,6 +212,9 @@ TEST_P(HmsClientTest, TestHmsOperations) {
   EXPECT_EQ(HmsClient::kKuduStorageHandler,
             renamed_table.parameters[HmsClient::kStorageHandlerKey]);
   EXPECT_EQ(HmsClient::kManagedTable, renamed_table.tableType);
+  EXPECT_EQ(HmsClient::kKuduInputFormat, renamed_table.sd.inputFormat);
+  EXPECT_EQ(HmsClient::kKuduOutputFormat, renamed_table.sd.outputFormat);
+  EXPECT_EQ(HmsClient::kKuduSerDeLib, renamed_table.sd.serdeInfo.serializationLib);
 
   // Create a table with an uppercase name.
   string uppercase_table_name = "my_UPPERCASE_Table";
@@ -291,8 +299,7 @@ TEST_P(HmsClientTest, TestHmsOperations) {
 }
 
 TEST_P(HmsClientTest, TestLargeObjects) {
-  auto protection = boost::make_optional<SaslProtection::Type>(false, SaslProtection::kIntegrity);
-  protection = GetParam();
+  const auto protection = GetParam();
 
   MiniKdc kdc;
   MiniHms hms;
@@ -335,6 +342,9 @@ TEST_P(HmsClientTest, TestLargeObjects) {
   partition_key.name = "c1";
   partition_key.type = "int";
   table.partitionKeys.emplace_back(std::move(partition_key));
+  table.sd.inputFormat = HmsClient::kKuduInputFormat;
+  table.sd.outputFormat = HmsClient::kKuduOutputFormat;
+  table.sd.serdeInfo.serializationLib = HmsClient::kKuduSerDeLib;
 
   ASSERT_OK(client.CreateTable(table));
 
