@@ -22,14 +22,15 @@
 #include <functional>
 #include <map>
 #include <memory>
+#include <optional>
 #include <ostream>
 #include <string>
 #include <thread>
+#include <type_traits>
 #include <unordered_map>
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags_declare.h>
 #include <glog/logging.h>
 #include <gtest/gtest.h>
@@ -121,6 +122,8 @@ using kudu::tablet::TabletReplica;
 using kudu::tablet::Tablet;
 using std::atomic;
 using std::map;
+using std::nullopt;
+using std::optional;
 using std::pair;
 using std::string;
 using std::thread;
@@ -307,8 +310,8 @@ class AlterTableTest : public KuduTest {
         } else {
           ASSERT_EQ(first_node_replica->tablet()->tablet_id(),
             cur_node_replica->tablet()->tablet_id());
-          ASSERT_TRUE(first_node_replica->tablet()->schema()->Equals(
-            *(cur_node_replica->tablet()->schema())));
+          ASSERT_EQ(*(first_node_replica->tablet()->schema()),
+                    *(cur_node_replica->tablet()->schema()));
           if (verify_row_count == VerifyRowCount::kEnable) {
             uint64_t cur_count = 0;
             ASSERT_OK(cur_node_replica->CountLiveRows(&cur_count));
@@ -329,7 +332,7 @@ class AlterTableTest : public KuduTest {
 
   void VerifyRows(int start_row, int num_rows, VerifyPattern pattern);
 
-  void InsertRows(int start_row, int num_rows);
+  void InsertRows(int start_row, int num_rows, bool only_set_first_col = true);
   void DeleteRow(int row_key);
 
   Status InsertRowsSequential(const string& table_name, int start_row, int num_rows);
@@ -367,12 +370,12 @@ class AlterTableTest : public KuduTest {
                      vector<unique_ptr<KuduPartialRow>> split_rows,
                      vector<pair<unique_ptr<KuduPartialRow>, unique_ptr<KuduPartialRow>>> bounds);
 
-  void CheckMaintenancePriority(int32_t expect_priority) {
-    for (auto op : tablet_replica_->maintenance_ops_) {
-      ASSERT_EQ(op->priority(), expect_priority);
+  void CheckMaintenancePriority(int32_t expected_priority) {
+    for (auto& op : tablet_replica_->maintenance_ops_) {
+      ASSERT_EQ(expected_priority, op->priority());
     }
-    for (auto op : tablet_replica_->tablet()->maintenance_ops_) {
-      ASSERT_EQ(op->priority(), expect_priority);
+    for (auto& op : tablet_replica_->tablet()->maintenance_ops_) {
+      ASSERT_EQ(expected_priority, op->priority());
     }
   }
 
@@ -427,7 +430,7 @@ TEST_F(AlterTableTest, TestAddExistingColumn) {
 
   {
     Status s = AddNewI32Column(kTableName, "c1", 0);
-    ASSERT_TRUE(s.IsAlreadyPresent());
+    ASSERT_TRUE(s.IsAlreadyPresent()) << s.ToString();
     ASSERT_STR_CONTAINS(s.ToString(), "The column already exists: c1");
   }
 
@@ -457,7 +460,7 @@ TEST_F(AlterTableTest, TestAddNotNullableColumnWithoutDefaults) {
     master::CatalogManager::ScopedLeaderSharedLock l(catalog);
     ASSERT_OK(l.first_failed_status());
     Status s = catalog->AlterTableRpc(req, &resp, /*rpc=*/nullptr);
-    ASSERT_TRUE(s.IsInvalidArgument());
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
     ASSERT_STR_CONTAINS(s.ToString(), "column `c2`: NOT NULL columns must have a default");
   }
 
@@ -643,7 +646,7 @@ TEST_F(AlterTableTest, TestAlterOnTSRestart) {
   {
     Status s = AddNewI32Column(kTableName, "new-32", 10,
                                MonoDelta::FromMilliseconds(500));
-    ASSERT_TRUE(s.IsTimedOut());
+    ASSERT_TRUE(s.IsTimedOut()) << s.ToString();
   }
 
   // Verify that the Schema is the old one
@@ -670,7 +673,7 @@ TEST_F(AlterTableTest, TestShutdownWithPendingTasks) {
   {
     Status s = AddNewI32Column(kTableName, "new-i32", 10,
                                MonoDelta::FromMilliseconds(500));
-    ASSERT_TRUE(s.IsTimedOut());
+    ASSERT_TRUE(s.IsTimedOut()) << s.ToString();
   }
 }
 
@@ -686,7 +689,7 @@ TEST_F(AlterTableTest, TestRestartTSDuringAlter) {
 
   Status s = AddNewI32Column(kTableName, "new-i32", 10,
                              MonoDelta::FromMilliseconds(1));
-  ASSERT_TRUE(s.IsTimedOut());
+  ASSERT_TRUE(s.IsTimedOut()) << s.ToString();
 
   // Restart the TS while alter is running
   for (int i = 0; i < 3; i++) {
@@ -706,7 +709,8 @@ TEST_F(AlterTableTest, TestGetSchemaAfterAlterTable) {
   ASSERT_OK(client_->GetTableSchema(kTableName, &s));
 }
 
-void AlterTableTest::InsertRows(int start_row, int num_rows) {
+void AlterTableTest::InsertRows(int start_row, int num_rows,
+                                bool only_set_first_col) {
   shared_ptr<KuduSession> session = client_->NewSession();
   shared_ptr<KuduTable> table;
   CHECK_OK(session->SetFlushMode(KuduSession::AUTO_FLUSH_BACKGROUND));
@@ -722,8 +726,13 @@ void AlterTableTest::InsertRows(int start_row, int num_rows) {
     int32_t key = bswap_32(i);
     CHECK_OK(insert->mutable_row()->SetInt32(0, key));
 
-    if (table->schema().num_columns() > 1) {
-      CHECK_OK(insert->mutable_row()->SetInt32(1, i));
+    for (int cid = 1; cid < table->schema().num_columns(); cid++) {
+      if (table->schema().Column(cid).type() == KuduColumnSchema::INT32) {
+        CHECK_OK(insert->mutable_row()->SetInt32(cid, i));
+        if (only_set_first_col) {
+          break;
+        }
+      }
     }
 
     CHECK_OK(session->Apply(insert.release()));
@@ -1026,7 +1035,7 @@ TEST_F(AlterTableTest, TestMajorCompactDeltasAfterUpdatingRemovedColumn) {
   UpdateRow(0, { {"c1", 54321} });
 
   // Make sure the delta is in a delta-file.
-  ASSERT_OK(tablet_replica_->tablet()->FlushBiggestDMS());
+  ASSERT_OK(tablet_replica_->tablet()->FlushBiggestDMSForTests());
 
   // Drop c1.
   LOG(INFO) << "Dropping c1";
@@ -1076,7 +1085,7 @@ TEST_F(AlterTableTest, TestMajorCompactDeltasIntoMissingBaseData) {
   UpdateRow(0, { {"c2", 54321} });
 
   // Make sure the delta is in a delta-file.
-  ASSERT_OK(tablet_replica_->tablet()->FlushBiggestDMS());
+  ASSERT_OK(tablet_replica_->tablet()->FlushBiggestDMSForTests());
 
   NO_FATALS(ScanToStrings(&rows));
   ASSERT_EQ(2, rows.size());
@@ -1126,7 +1135,7 @@ TEST_F(AlterTableTest, TestMajorCompactDeltasAfterAddUpdateRemoveColumn) {
   UpdateRow(0, { {"c2", 54321} });
 
   // Make sure the delta is in a delta-file.
-  ASSERT_OK(tablet_replica_->tablet()->FlushBiggestDMS());
+  ASSERT_OK(tablet_replica_->tablet()->FlushBiggestDMSForTests());
 
   NO_FATALS(ScanToStrings(&rows));
   ASSERT_EQ(1, rows.size());
@@ -1871,7 +1880,7 @@ TEST_F(AlterTableTest, TestAddRangePartitionConflictExhaustive) {
                          ->wait(true)->Alter());
 
   // Turns an optional value into a row for the table.
-  auto fill_row = [&] (boost::optional<int32_t> value) {
+  auto fill_row = [&] (optional<int32_t> value) {
     unique_ptr<KuduPartialRow> row(schema_.NewRow());
     if (value) {
       CHECK_OK(row->SetInt32("c0", *value));
@@ -1880,8 +1889,8 @@ TEST_F(AlterTableTest, TestAddRangePartitionConflictExhaustive) {
   };
 
   // Attempts to add a range partition to the table with the specified bounds.
-  auto add_range_partition = [&] (boost::optional<int32_t> lower_bound,
-                                  boost::optional<int32_t> upper_bound) {
+  auto add_range_partition = [&] (optional<int32_t> lower_bound,
+                                  optional<int32_t> upper_bound) {
     table_alterer.reset(client_->NewTableAlterer(table_name));
     return table_alterer->AddRangePartition(fill_row(lower_bound).release(),
                                             fill_row(upper_bound).release())
@@ -1890,8 +1899,8 @@ TEST_F(AlterTableTest, TestAddRangePartitionConflictExhaustive) {
   };
 
   // Attempts to drop a range partition to the table with the specified bounds.
-  auto drop_range_partition = [&] (boost::optional<int32_t> lower_bound,
-                                   boost::optional<int32_t> upper_bound) {
+  auto drop_range_partition = [&] (optional<int32_t> lower_bound,
+                                   optional<int32_t> upper_bound) {
     table_alterer.reset(client_->NewTableAlterer(table_name));
     return table_alterer->DropRangePartition(fill_row(lower_bound).release(),
                                              fill_row(upper_bound).release())
@@ -1900,10 +1909,10 @@ TEST_F(AlterTableTest, TestAddRangePartitionConflictExhaustive) {
   };
 
   // Attempts to add two range partitions to the table in a single op.
-  auto add_range_partitions = [&] (boost::optional<int32_t> a_lower_bound,
-                                   boost::optional<int32_t> a_upper_bound,
-                                   boost::optional<int32_t> b_lower_bound,
-                                   boost::optional<int32_t> b_upper_bound) {
+  auto add_range_partitions = [&] (optional<int32_t> a_lower_bound,
+                                   optional<int32_t> a_upper_bound,
+                                   optional<int32_t> b_lower_bound,
+                                   optional<int32_t> b_upper_bound) {
     table_alterer.reset(client_->NewTableAlterer(table_name));
     return table_alterer->AddRangePartition(fill_row(a_lower_bound).release(),
                                             fill_row(a_upper_bound).release())
@@ -1914,10 +1923,10 @@ TEST_F(AlterTableTest, TestAddRangePartitionConflictExhaustive) {
   };
 
   // Attempts to add and drop two range partitions in a single op.
-  auto add_drop_range_partitions = [&] (boost::optional<int32_t> a_lower_bound,
-                                        boost::optional<int32_t> a_upper_bound,
-                                        boost::optional<int32_t> b_lower_bound,
-                                        boost::optional<int32_t> b_upper_bound) {
+  auto add_drop_range_partitions = [&] (optional<int32_t> a_lower_bound,
+                                        optional<int32_t> a_upper_bound,
+                                        optional<int32_t> b_lower_bound,
+                                        optional<int32_t> b_upper_bound) {
     table_alterer.reset(client_->NewTableAlterer(table_name));
     return table_alterer->AddRangePartition(fill_row(a_lower_bound).release(),
                                             fill_row(a_upper_bound).release())
@@ -1927,8 +1936,8 @@ TEST_F(AlterTableTest, TestAddRangePartitionConflictExhaustive) {
                         ->Alter();
   };
 
-  auto bounds_to_string = [] (boost::optional<int32_t> lower_bound,
-                              boost::optional<int32_t> upper_bound) -> string {
+  auto bounds_to_string = [] (optional<int32_t> lower_bound,
+                              optional<int32_t> upper_bound) -> string {
     if (!lower_bound && !upper_bound) {
       return "UNBOUNDED";
     }
@@ -1942,10 +1951,10 @@ TEST_F(AlterTableTest, TestAddRangePartitionConflictExhaustive) {
   };
 
   // Checks that b conflicts with a, when added in that order.
-  auto do_expect_range_partitions_conflict = [&] (boost::optional<int32_t> a_lower_bound,
-                                                  boost::optional<int32_t> a_upper_bound,
-                                                  boost::optional<int32_t> b_lower_bound,
-                                                  boost::optional<int32_t> b_upper_bound) {
+  auto do_expect_range_partitions_conflict = [&] (optional<int32_t> a_lower_bound,
+                                                  optional<int32_t> a_upper_bound,
+                                                  optional<int32_t> b_lower_bound,
+                                                  optional<int32_t> b_upper_bound) {
     SCOPED_TRACE(Substitute("b: $0", bounds_to_string(b_lower_bound, b_upper_bound)));
     SCOPED_TRACE(Substitute("a: $0", bounds_to_string(a_lower_bound, a_upper_bound)));
 
@@ -1998,10 +2007,10 @@ TEST_F(AlterTableTest, TestAddRangePartitionConflictExhaustive) {
   };
 
   // Checks that two range partitions conflict.
-  auto expect_range_partitions_conflict = [&] (boost::optional<int32_t> a_lower_bound,
-                                               boost::optional<int32_t> a_upper_bound,
-                                               boost::optional<int32_t> b_lower_bound,
-                                               boost::optional<int32_t> b_upper_bound) {
+  auto expect_range_partitions_conflict = [&] (optional<int32_t> a_lower_bound,
+                                               optional<int32_t> a_upper_bound,
+                                               optional<int32_t> b_lower_bound,
+                                               optional<int32_t> b_upper_bound) {
     do_expect_range_partitions_conflict(a_lower_bound, a_upper_bound,
                                         b_lower_bound, b_upper_bound);
     do_expect_range_partitions_conflict(b_lower_bound, b_upper_bound,
@@ -2034,47 +2043,47 @@ TEST_F(AlterTableTest, TestAddRangePartitionConflictExhaustive) {
 
   //  [----------)
   // [-------------->
-  expect_range_partitions_conflict(0, 100, -1, boost::none);
+  expect_range_partitions_conflict(0, 100, -1, nullopt);
 
   // [----------)
   // [-------------->
-  expect_range_partitions_conflict(0, 100, 0, boost::none);
+  expect_range_partitions_conflict(0, 100, 0, nullopt);
 
   // [----------)
   //  [------------->
-  expect_range_partitions_conflict(0, 100, 1, boost::none);
+  expect_range_partitions_conflict(0, 100, 1, nullopt);
 
   // [----------)
   //      [------------->
-  expect_range_partitions_conflict(0, 100, 50, boost::none);
+  expect_range_partitions_conflict(0, 100, 50, nullopt);
 
   // [----------)
   //           [--------->
-  expect_range_partitions_conflict(0, 100, 99, boost::none);
+  expect_range_partitions_conflict(0, 100, 99, nullopt);
 
   /// Bounded / Unbounded Below
 
   //        [----------)
   // <-------)
-  expect_range_partitions_conflict(0, 100, boost::none, 1);
+  expect_range_partitions_conflict(0, 100, nullopt, 1);
 
   //        [----------)
   // <------------)
-  expect_range_partitions_conflict(0, 100, boost::none, 50);
+  expect_range_partitions_conflict(0, 100, nullopt, 50);
 
   //        [----------)
   // <-----------------)
-  expect_range_partitions_conflict(0, 100, boost::none, 100);
+  expect_range_partitions_conflict(0, 100, nullopt, 100);
 
   //        [----------)
   // <-------------------)
-  expect_range_partitions_conflict(0, 100, boost::none, 125);
+  expect_range_partitions_conflict(0, 100, nullopt, 125);
 
   /// Bounded / Unbounded
 
   //     [----------)
   // <------------------->
-  expect_range_partitions_conflict(0, 100, boost::none, boost::none);
+  expect_range_partitions_conflict(0, 100, nullopt, nullopt);
 
   /// Bounded / Single Value
 
@@ -2094,69 +2103,69 @@ TEST_F(AlterTableTest, TestAddRangePartitionConflictExhaustive) {
 
   //    [---------->
   // [---------->
-  expect_range_partitions_conflict(0, boost::none, -10, boost::none);
+  expect_range_partitions_conflict(0, nullopt, -10, nullopt);
 
   // [---------->
   // [---------->
-  expect_range_partitions_conflict(0, boost::none, 0, boost::none);
+  expect_range_partitions_conflict(0, nullopt, 0, nullopt);
 
   /// Unbounded Above / Unbounded Below
 
   // [---------->
   // <----------)
-  expect_range_partitions_conflict(0, boost::none, boost::none, 100);
+  expect_range_partitions_conflict(0, nullopt, nullopt, 100);
 
   //        [---------->
   // <-------)
-  expect_range_partitions_conflict(0, boost::none, boost::none, 1);
+  expect_range_partitions_conflict(0, nullopt, nullopt, 1);
 
   /// Unbounded Above / Unbounded
 
   // [---------->
   // <---------->
-  expect_range_partitions_conflict(0, boost::none, boost::none, boost::none);
+  expect_range_partitions_conflict(0, nullopt, nullopt, nullopt);
 
   /// Unbounded Above / Single Value
 
   // [---------->
   // |
-  expect_range_partitions_conflict(0, boost::none, 0, 1);
+  expect_range_partitions_conflict(0, nullopt, 0, 1);
 
   // [---------->
   //   |
-  expect_range_partitions_conflict(0, boost::none, 100, 101);
+  expect_range_partitions_conflict(0, nullopt, 100, 101);
 
   /// Unbounded Below / Unbounded Below
 
   // <----------)
   // <----------)
-  expect_range_partitions_conflict(boost::none, 100, boost::none, 100);
+  expect_range_partitions_conflict(nullopt, 100, nullopt, 100);
 
   // <----------)
   // <-----)
-  expect_range_partitions_conflict(boost::none, 100, boost::none, 50);
+  expect_range_partitions_conflict(nullopt, 100, nullopt, 50);
 
   /// Unbounded Below / Unbounded
 
   // <----------)
   // <---------->
-  expect_range_partitions_conflict(boost::none, 100, boost::none, boost::none);
+  expect_range_partitions_conflict(nullopt, 100, nullopt, nullopt);
 
   /// Unbounded Below / Single Value
 
   // <----------)
   //       |
-  expect_range_partitions_conflict(boost::none, 100, 50, 51);
+  expect_range_partitions_conflict(nullopt, 100, 50, 51);
 
   // <----------)
   //           |
-  expect_range_partitions_conflict(boost::none, 100, 99, 100);
+  expect_range_partitions_conflict(nullopt, 100, 99, 100);
 
   /// Unbounded / Unbounded
 
   // <---------->
   // <---------->
-  expect_range_partitions_conflict(boost::none, boost::none, boost::none, boost::none);
+  expect_range_partitions_conflict(nullopt, nullopt, nullopt, nullopt);
 
   /// Single Value / Single Value
 
@@ -2263,7 +2272,7 @@ TEST_F(ReplicatedAlterTableTest, AlterReplicationFactor) {
 
   // 5. Set replication factor to 5, while there are only 4 tservers in the cluster.
   auto s = SetReplicationFactor(kTableName, 5);
-  ASSERT_TRUE(s.IsInvalidArgument());
+  ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
   ASSERT_STR_CONTAINS(s.ToString(), "not enough live tablet servers to alter a table with the"
                                     " requested replication factor 5; 4 tablet servers are alive");
   NO_FATALS(VerifyTabletReplicaCount(3, VerifyRowCount::kEnable));
@@ -2271,7 +2280,7 @@ TEST_F(ReplicatedAlterTableTest, AlterReplicationFactor) {
 
   // 6. Set replication factor to -1, it will fail.
   s = SetReplicationFactor(kTableName, -1);
-  ASSERT_TRUE(s.IsInvalidArgument());
+  ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
   ASSERT_STR_CONTAINS(s.ToString(), "illegal replication factor -1: minimum allowed replication"
                                     " factor is 1 (controlled by --min_num_replicas)");
   NO_FATALS(VerifyTabletReplicaCount(3, VerifyRowCount::kEnable));
@@ -2279,14 +2288,14 @@ TEST_F(ReplicatedAlterTableTest, AlterReplicationFactor) {
 
   // 7. Set replication factor to 2, it will fail.
   s = SetReplicationFactor(kTableName, 2);
-  ASSERT_TRUE(s.IsInvalidArgument());
+  ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
   ASSERT_STR_CONTAINS(s.ToString(), "illegal replication factor 2: replication factor must be odd");
   NO_FATALS(VerifyTabletReplicaCount(3, VerifyRowCount::kEnable));
   ASSERT_EQ(2, tablet_replica_->tablet()->metadata()->schema_version());
 
   // 8. Set replication factor to 9, it will fail.
   s = SetReplicationFactor(kTableName, 9);
-  ASSERT_TRUE(s.IsInvalidArgument());
+  ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
   ASSERT_STR_CONTAINS(s.ToString(), "illegal replication factor 9: maximum allowed replication "
                                     "factor is 7 (controlled by --max_num_replicas)");
   NO_FATALS(VerifyTabletReplicaCount(3, VerifyRowCount::kEnable));
@@ -2543,4 +2552,78 @@ TEST_F(AlterTableTest, TestDisableCompactAlter) {
   ASSERT_TRUE(s.IsInvalidArgument());
 }
 
+TEST_F(AlterTableTest, AddAndRemoveImmutableAttribute) {
+  InsertRows(0, 1);
+  ASSERT_OK(tablet_replica_->tablet()->Flush());
+
+  vector<string> rows;
+  ScanToStrings(&rows);
+  ASSERT_EQ(1, rows.size());
+  EXPECT_EQ("(int32 c0=0, int32 c1=0)", rows[0]);
+
+  {
+    // Add immutable attribute to a column.
+    unique_ptr<KuduTableAlterer> table_alterer(client_->NewTableAlterer(kTableName));
+    table_alterer->AlterColumn("c1")->Immutable();
+    ASSERT_OK(table_alterer->Alter());
+  }
+
+  shared_ptr<KuduTable> table;
+  ASSERT_OK(client_->OpenTable(kTableName, &table));
+  // Here we use the first column to initialize an object of KuduColumnSchema
+  // for there is no default constructor for it.
+  KuduColumnSchema col_schema = table->schema().Column(0);
+  ASSERT_TRUE(table->schema().HasColumn("c1", &col_schema));
+  ASSERT_TRUE(col_schema.is_immutable());
+
+  InsertRows(1, 1);
+  ASSERT_OK(tablet_replica_->tablet()->Flush());
+
+  ScanToStrings(&rows);
+  ASSERT_EQ(2, rows.size());
+  EXPECT_EQ("(int32 c0=0, int32 c1=0)", rows[0]);
+  EXPECT_EQ("(int32 c0=16777216, int32 c1=1)", rows[1]);
+
+  {
+    // Remove immutable attribute from a column.
+    unique_ptr<KuduTableAlterer> table_alterer(client_->NewTableAlterer(kTableName));
+    table_alterer->AlterColumn("c1")->Mutable();
+    ASSERT_OK(table_alterer->Alter());
+  }
+
+  ASSERT_OK(client_->OpenTable(kTableName, &table));
+  ASSERT_TRUE(table->schema().HasColumn("c1", &col_schema));
+  ASSERT_FALSE(col_schema.is_immutable());
+
+  InsertRows(2, 1);
+  ASSERT_OK(tablet_replica_->tablet()->Flush());
+
+  ScanToStrings(&rows);
+  ASSERT_EQ(3, rows.size());
+  EXPECT_EQ("(int32 c0=0, int32 c1=0)", rows[0]);
+  EXPECT_EQ("(int32 c0=16777216, int32 c1=1)", rows[1]);
+  EXPECT_EQ("(int32 c0=33554432, int32 c1=2)", rows[2]);
+
+  {
+    // Add a column with immutable attribute.
+    unique_ptr<KuduTableAlterer> table_alterer(client_->NewTableAlterer(kTableName));
+    table_alterer->AddColumn("c2")->Type(KuduColumnSchema::INT32)->Immutable();
+    ASSERT_OK(table_alterer->Alter());
+  }
+
+  ASSERT_OK(client_->OpenTable(kTableName, &table));
+  ASSERT_TRUE(table->schema().HasColumn("c2", &col_schema));
+  ASSERT_TRUE(col_schema.is_immutable());
+
+  InsertRows(3, 1, false);
+  ASSERT_OK(tablet_replica_->tablet()->Flush());
+
+  ScanToStrings(&rows);
+  ASSERT_EQ(4, rows.size());
+  EXPECT_EQ("(int32 c0=0, int32 c1=0, int32 c2=NULL)", rows[0]);
+  EXPECT_EQ("(int32 c0=16777216, int32 c1=1, int32 c2=NULL)", rows[1]);
+  EXPECT_EQ("(int32 c0=33554432, int32 c1=2, int32 c2=NULL)", rows[2]);
+  EXPECT_EQ("(int32 c0=50331648, int32 c1=3, int32 c2=3)", rows[3]);
+}
+
 } // namespace kudu
