@@ -26,6 +26,7 @@
 #include <set>
 #include <string>
 #include <thread>
+#include <type_traits>
 #include <unordered_map>
 #include <utility>
 #include <vector>
@@ -38,6 +39,7 @@
 #include "kudu/client/shared_ptr.h" // IWYU pragma: keep
 #include "kudu/common/common.pb.h"
 #include "kudu/common/partial_row.h"
+#include "kudu/common/partition.h"
 #include "kudu/common/schema.h"
 #include "kudu/common/wire_protocol-test-util.h"
 #include "kudu/gutil/mathlimits.h"
@@ -50,24 +52,26 @@
 #include "kudu/mini-cluster/external_mini_cluster.h"
 #include "kudu/mini-cluster/mini_cluster.h"
 #include "kudu/rpc/rpc_controller.h"
+#include "kudu/tablet/tablet.pb.h"
+#include "kudu/tools/tool_test_util.h"
+#include "kudu/tserver/tserver.pb.h"
 #include "kudu/util/atomic.h"
 #include "kudu/util/metrics.h"
 #include "kudu/util/monotime.h"
+#include "kudu/util/net/net_util.h"
+#include "kudu/util/net/sockaddr.h"
 #include "kudu/util/scoped_cleanup.h"
 #include "kudu/util/status.h"
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
 
-namespace kudu {
-class HostPort;
-}  // namespace kudu
-
 using std::multimap;
 using std::set;
 using std::string;
 using std::thread;
 using std::unique_ptr;
 using std::vector;
+using strings::Substitute;
 
 METRIC_DECLARE_entity(server);
 METRIC_DECLARE_histogram(handler_latency_kudu_tserver_TabletServerAdminService_CreateTablet);
@@ -162,7 +166,7 @@ TEST_F(CreateTableITest, TestCreateWhenMajorityOfReplicasFailCreation) {
         tablet_num_replica_map[tablet_id]++;
       }
     }
-    LOG(INFO) << strings::Substitute(
+    LOG(INFO) << Substitute(
         "Waiting for only one tablet to be left with $0 replicas. Currently have: $1 tablet(s)",
         kNumReplicas, tablet_num_replica_map.size());
     ASSERT_EQ(1, tablet_num_replica_map.size());
@@ -195,7 +199,10 @@ TEST_F(CreateTableITest, TestCreateWhenMajorityOfReplicasFailCreation) {
 TEST_F(CreateTableITest, TestSpreadReplicasEvenly) {
   const int kNumServers = 10;
   const int kNumTablets = 20;
-  NO_FATALS(StartCluster({}, {}, kNumServers));
+  vector<string> master_flags = {
+      "--enable_range_replica_placement=false",
+  };
+  NO_FATALS(StartCluster({}, master_flags, kNumServers));
 
   unique_ptr<client::KuduTableCreator> table_creator(client_->NewTableCreator());
   auto client_schema = KuduSchema::FromSchema(GetSimpleTestSchema());
@@ -395,6 +402,99 @@ TEST_F(CreateTableITest, TestSpreadReplicasEvenlyWithDimension) {
   }
 }
 
+// Tests the range aware replica placement by adding multiple tables with multiple ranges
+// and checking the replica distribution.
+TEST_F(CreateTableITest, TestSpreadReplicas) {
+  const int kNumServers = 5;
+  const int kNumReplicas = 3;
+  NO_FATALS(StartCluster({ }, { }, kNumServers));
+
+  Schema schema = Schema({ ColumnSchema("key1", INT32),
+                           ColumnSchema("key2", INT32),
+                           ColumnSchema("int_val", INT32),
+                           ColumnSchema("string_val", STRING, true) }, 2);
+  auto client_schema = KuduSchema::FromSchema(schema);
+
+  auto create_table_func = [](KuduClient* client,
+                              KuduSchema* client_schema,
+                              const string& table_name,
+                              const vector<std::pair<int32_t, int32_t>> range_bounds,
+                              const int num_buckets) {
+    unique_ptr<client::KuduTableCreator> table_creator(client->NewTableCreator());
+    table_creator->table_name(table_name)
+        .schema(client_schema)
+        .add_hash_partitions({ "key1" }, num_buckets)
+        .set_range_partition_columns({ "key2" })
+        .num_replicas(kNumReplicas);
+    for (const auto& range_bound : range_bounds) {
+      unique_ptr<KuduPartialRow> lower_bound(client_schema->NewRow());
+      RETURN_NOT_OK(lower_bound->SetInt32("key2", range_bound.first));
+      unique_ptr<KuduPartialRow> upper_bound(client_schema->NewRow());
+      RETURN_NOT_OK(upper_bound->SetInt32("key2", range_bound.second));
+      table_creator->add_range_partition(lower_bound.release(), upper_bound.release());
+    }
+    return table_creator->Create();
+  };
+
+  vector<string> tables = {"table1", "table2", "table3", "table4"};
+  vector<std::pair<int32_t, int32_t>> range_bounds =
+      { {0, 100}, {100, 200}, {200, 300}, {300, 400}};
+  const int doubleNumBuckets = 10;
+  const int numBuckets = 5;
+  for (const auto& table : tables) {
+    if (table == "table1") {
+      ASSERT_OK(create_table_func(
+          client_.get(), &client_schema, table, range_bounds, doubleNumBuckets));
+    } else {
+      ASSERT_OK(create_table_func(
+          client_.get(), &client_schema, table, range_bounds, numBuckets));
+    }
+  }
+
+  // Stats of number of replicas per range per table per tserver.
+  typedef std::unordered_map<string, std::unordered_map<string, int>> replicas_per_range_per_table;
+  std::unordered_map<int, replicas_per_range_per_table> stats;
+  for (int ts_idx = 0; ts_idx < kNumServers; ts_idx++) {
+    rpc::RpcController rpc;
+    tserver::ListTabletsRequestPB req;
+    tserver::ListTabletsResponsePB resp;
+    cluster_->tserver_proxy(ts_idx)->ListTablets(req, &resp, &rpc);
+    for (auto i = 0; i < resp.status_and_schema_size(); ++i) {
+      auto tablet_status = resp.status_and_schema(i).tablet_status();
+      if (tablet_status.has_partition()) {
+        Partition partition;
+        Partition::FromPB(tablet_status.partition(), &partition);
+        auto range_start_key = partition.begin().range_key();
+        auto table_name = tablet_status.table_name();
+        ++stats[ts_idx][table_name][range_start_key];
+      }
+    }
+  }
+
+  ASSERT_EQ(kNumServers, stats.size());
+  for (const auto& stat : stats) {
+    int tserver_replicas = 0;
+    // Verifies that four tables exist on each tserver.
+    ASSERT_EQ(tables.size(), stat.second.size());
+    for (const auto& table : stat.second) {
+      // Verifies that the four ranges exist for each table on each tserver.
+      ASSERT_EQ(range_bounds.size(), table.second.size());
+      for (const auto& ranges : table.second) {
+        // Since there are ten buckets instead of five for table "table1",
+        // we expect twice as many replicas (6 instead of 3).
+        if (table.first == "table1") {
+          ASSERT_EQ(doubleNumBuckets * kNumReplicas / kNumServers, ranges.second);;
+        } else {
+          ASSERT_EQ(numBuckets * kNumReplicas / kNumServers, ranges.second);
+        }
+        tserver_replicas += ranges.second;
+      }
+    }
+    // Verifies that 60 replicas are placed on each tserver, 300 total across 5 tservers.
+    ASSERT_EQ(60, tserver_replicas);
+  }
+}
+
 static void LookUpRandomKeysLoop(const std::shared_ptr<master::MasterServiceProxy>& master,
                                  const char* table_name,
                                  AtomicBool* quit) {
@@ -468,7 +568,7 @@ TEST_F(CreateTableITest, TestCreateTableWithDeadTServers) {
   unique_ptr<client::KuduTableCreator> table_creator(client_->NewTableCreator());
 
   // Don't bother waiting for table creation to finish; it'll never happen
-  // because all of the tservers are dead.
+  // because all the tservers are dead.
   CHECK_OK(table_creator->table_name(kTableName)
            .schema(&client_schema)
            .set_range_partition_columns({ "key" })
@@ -501,4 +601,128 @@ TEST_F(CreateTableITest, TestCreateTableWithDeadTServers) {
   }
 }
 
+// Make sure it's possible to create a table when using proxied RPC addresses,
+// and the result table locations point to the addresses set to be advertised
+// by a TCP proxy correspondingly.
+TEST_F(CreateTableITest, ProxyAdvertisedAddresses) {
+  constexpr const char* const kIpAddr = "127.0.0.1";
+  constexpr int kNumServers = 1;
+  constexpr int kNumTablets = 2;
+  const MonoDelta timeout = MonoDelta::FromSeconds(10);
+
+  uint16_t m_port = 0;
+  ASSERT_OK(GetRandomPort(kIpAddr, &m_port));
+  const HostPort m_proxied_addr(kIpAddr, m_port);
+
+  uint16_t t_port = 0;
+  ASSERT_OK(GetRandomPort(kIpAddr, &t_port));
+  const HostPort t_proxied_addr(kIpAddr, t_port);
+
+  const string m_proxy_advertised_address = "kudu.proxy.io:333";
+
+  const uint16_t t_proxy_advertised_port = 888;
+  const string t_proxy_advertised_host = "kudu.proxy.io";
+  const string t_proxy_advertised_addr = Substitute(
+      "$0:$1", t_proxy_advertised_host, t_proxy_advertised_port);
+
+  uint16_t t_bind_port = 0;
+  ASSERT_OK(GetRandomPort(kIpAddr, &t_bind_port));
+  const HostPort t_bind_addr(kIpAddr, t_bind_port);
+
+  const vector<string> master_flags = {
+    Substitute("--rpc_proxy_advertised_addresses=$0", m_proxy_advertised_address),
+    Substitute("--rpc_proxied_addresses=$0", m_proxied_addr.ToString()),
+  };
+  const vector<string> ts_flags = {
+    Substitute("--rpc_proxy_advertised_addresses=$0", t_proxy_advertised_addr),
+    Substitute("--rpc_proxied_addresses=$0", t_proxied_addr.ToString()),
+    Substitute("--rpc_bind_addresses=$0", t_bind_addr.ToString()),
+  };
+  NO_FATALS(StartCluster(ts_flags, master_flags, kNumServers));
+
+  // Build a client to send requests via the proxied RPC endpoint.
+  client::KuduClientBuilder builder;
+  builder.add_master_server_addr(m_proxied_addr.ToString());
+  client::sp::shared_ptr<client::KuduClient> c_ext;
+  ASSERT_OK(builder.Build(&c_ext));
+
+  // Create table using Kudu client sending requests as if they were proxied
+  // from outside.
+  unique_ptr<client::KuduTableCreator> table_creator(c_ext->NewTableCreator());
+  auto client_schema = KuduSchema::FromSchema(GetSimpleTestSchema());
+  ASSERT_OK(table_creator->table_name(kTableName)
+            .schema(&client_schema)
+            .set_range_partition_columns({ "key" })
+            .num_replicas(1)
+            .add_hash_partitions({ "key" }, kNumTablets)
+            .Create());
+
+  // Make sure the client receives proxy advertised addresses since the request
+  // came to the proxied RPC address.
+  const auto& master_addresses = c_ext->GetMasterAddresses();
+  ASSERT_EQ(m_proxy_advertised_address, master_addresses);
+
+  // Get information on table's locations via standard RPC endpoint.
+  {
+    master::GetTableLocationsRequestPB req;
+    req.set_intern_ts_infos_in_response(true);
+    req.mutable_table()->set_table_name(kTableName);
+
+    rpc::RpcController rpc;
+    rpc.set_timeout(timeout);
+    master::GetTableLocationsResponsePB resp;
+    auto mp = cluster_->master_proxy();
+    ASSERT_OK(mp->GetTableLocations(req, &resp, &rpc));
+
+    ASSERT_EQ(kNumTablets, resp.tablet_locations().size());
+    for (const auto& loc : resp.tablet_locations()) {
+      ASSERT_EQ(kNumServers, loc.interned_replicas_size());
+    }
+
+    ASSERT_EQ(1, resp.ts_infos_size());
+    const auto& ts_info = resp.ts_infos(0);
+    ASSERT_EQ(1, ts_info.rpc_addresses_size());
+    const auto& hp = ts_info.rpc_addresses(0);
+    ASSERT_TRUE(hp.has_host());
+    ASSERT_EQ(t_bind_addr.host(), hp.host());
+    ASSERT_TRUE(hp.has_port());
+    ASSERT_EQ(t_bind_addr.port(), hp.port());
+  }
+
+  // Get information on table's locations via endpoint for proxied RPCs.
+  {
+    Sockaddr ma;
+    ma.ParseFromNumericHostPort(m_proxied_addr);
+    auto mp = std::make_shared<master::MasterServiceProxy>(
+        cluster_->messenger(), ma, ma.host());
+
+    master::GetTableLocationsRequestPB req;
+    req.set_intern_ts_infos_in_response(true);
+    req.mutable_table()->set_table_name(kTableName);
+
+    rpc::RpcController rpc;
+    rpc.set_timeout(timeout);
+    master::GetTableLocationsResponsePB resp;
+    ASSERT_OK(mp->GetTableLocations(req, &resp, &rpc));
+
+    ASSERT_EQ(kNumTablets, resp.tablet_locations().size());
+    for (const auto& loc : resp.tablet_locations()) {
+      ASSERT_EQ(kNumServers, loc.interned_replicas_size());
+    }
+
+    ASSERT_EQ(1, resp.ts_infos_size());
+    const auto& ts_info = resp.ts_infos(0);
+    ASSERT_EQ(1, ts_info.rpc_addresses_size());
+    const auto& hp = ts_info.rpc_addresses(0);
+    ASSERT_TRUE(hp.has_host());
+    ASSERT_EQ(t_proxy_advertised_host, hp.host());
+    ASSERT_TRUE(hp.has_port());
+    ASSERT_EQ(t_proxy_advertised_port, hp.port());
+  }
+
+  // Delete the created table using the client instance communicating with
+  // the cluster through regular RPC endpoints.
+  ASSERT_OK(client_->DeleteTable(kTableName));
+}
+
 } // namespace kudu
