@@ -17,12 +17,12 @@
 
 #include <cstdint>
 #include <functional>
+#include <optional>
 #include <memory>
 #include <set>
 #include <string>
 #include <unordered_map>
 
-#include <boost/optional/optional.hpp>
 #include <gtest/gtest.h>
 
 #include "kudu/consensus/consensus-test-util.h"
@@ -44,7 +44,6 @@
 
 METRIC_DECLARE_histogram(handler_latency_kudu_master_MasterService_TSHeartbeat);
 
-using boost::none;
 using kudu::cluster::ExternalMaster;
 using kudu::cluster::ExternalTabletServer;
 using kudu::consensus::MakeOpId;
@@ -92,8 +91,11 @@ TEST_F(TombstonedVotingITest, TestTombstonedReplicaWithoutCMetaCanVote) {
   // Wait for all 3 replicas to come up and figure out where they landed.
   ASSERT_OK(inspect_->WaitForReplicaCount(3));
   master::GetTableLocationsResponsePB table_locations;
-  ASSERT_OK(itest::GetTableLocations(cluster_->master_proxy(), TestWorkload::kDefaultTableName,
-                                     kTimeout, master::VOTER_REPLICA, /*table_id=*/none,
+  ASSERT_OK(itest::GetTableLocations(cluster_->master_proxy(),
+                                     TestWorkload::kDefaultTableName,
+                                     kTimeout,
+                                     master::VOTER_REPLICA,
+                                     /*table_id=*/std::nullopt,
                                      &table_locations));
   ASSERT_EQ(1, table_locations.tablet_locations_size());
   string tablet_id = table_locations.tablet_locations(0).tablet_id();
