@@ -22,9 +22,13 @@
 #include <glog/logging.h>
 #include <gtest/gtest.h>
 
+#include "kudu/gutil/strings/substitute.h"
 #include "kudu/integration-tests/test_workload.h"
 #include "kudu/mini-cluster/external_mini_cluster.h"
 #include "kudu/mini-cluster/webui_checker.h"
+#include "kudu/util/curl_util.h"
+#include "kudu/util/faststring.h"
+#include "kudu/util/net/net_util.h"
 #include "kudu/util/monotime.h"
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
@@ -93,6 +97,17 @@ TEST_F(KuduTest, TestWebUIDoesNotCrashCluster) {
     SleepFor(MonoDelta::FromMilliseconds(10));
   }
   SleepFor(MonoDelta::FromSeconds(5));
+
+  {
+    EasyCurl curl;
+    string web_url =
+        strings::Substitute("http://$0//maintenance-manager",
+                            cluster.tablet_server(0)->bound_http_hostport().ToString());
+    faststring buf;
+    curl.FetchURL(web_url, &buf);
+    ASSERT_STR_CONTAINS(buf.ToString(), "data_retained");
+  }
+
   work.StopAndJoin();
 
   // Restart the cluster.
