@@ -33,6 +33,8 @@ static int MasterMain(int argc, char** argv) {
   SetMasterFlagDefaults();
   ParseCommandLineFlags(&argc, &argv, true);
   if (argc != 1) {
+    std::cerr << argv[0] << " accepts only flag arguments: " << argv[1] << " is not a flag argument"
+              << std::endl;
     std::cerr << "usage: " << argv[0] << std::endl;
     return 2;
   }
