@@ -24,6 +24,7 @@
 #include <limits>
 #include <map>
 #include <memory>
+#include <optional>
 #include <sstream>
 #include <string>
 #include <type_traits>
@@ -31,7 +32,6 @@
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <glog/logging.h>
 
 #include "kudu/common/common.pb.h"
@@ -49,15 +49,18 @@
 #include "kudu/gutil/strings/human_readable.h"
 #include "kudu/gutil/strings/join.h"
 #include "kudu/gutil/strings/numbers.h"
+#include "kudu/gutil/strings/strip.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/gutil/walltime.h"
 #include "kudu/master/catalog_manager.h"
 #include "kudu/master/master.h"
 #include "kudu/master/master.pb.h"
+#include "kudu/master/master_cert_authority.h"
 #include "kudu/master/sys_catalog.h"
 #include "kudu/master/table_metrics.h"
 #include "kudu/master/ts_descriptor.h"
 #include "kudu/master/ts_manager.h"
+#include "kudu/security/cert.h"
 #include "kudu/server/monitored_task.h"
 #include "kudu/server/rpc_server.h"
 #include "kudu/server/webui_util.h"
@@ -68,15 +71,15 @@
 #include "kudu/util/metrics.h"
 #include "kudu/util/monotime.h"
 #include "kudu/util/net/net_util.h"
+#include "kudu/util/openssl_util.h"
 #include "kudu/util/pb_util.h"
 #include "kudu/util/string_case.h"
 #include "kudu/util/url-coding.h"
 #include "kudu/util/web_callback_registry.h"
 
-namespace kudu {
-
-using consensus::ConsensusStatePB;
-using consensus::RaftPeerPB;
+using kudu::consensus::ConsensusStatePB;
+using kudu::consensus::RaftPeerPB;
+using kudu::security::DataFormat;
 using std::array;
 using std::map;
 using std::ostringstream;
@@ -86,6 +89,7 @@ using std::string;
 using std::vector;
 using strings::Substitute;
 
+namespace kudu {
 namespace master {
 
 namespace {
@@ -147,6 +151,12 @@ void MasterPathHandlers::HandleTabletServers(const Webserver::WebRequest& /*req*
   }
   // Process the registered tservers.
   for (const auto& desc : descs) {
+    ServerRegistrationPB reg;
+    if (auto s = desc->GetRegistration(&reg); PREDICT_FALSE(!s.ok())) {
+      LOG(WARNING) << s.ToString();
+      continue;
+    }
+
     const string& ts_uuid = desc->permanent_uuid();
     string ts_key;
     auto* ts_state_info = FindOrNull(ts_state_infos, ts_uuid);
@@ -164,8 +174,6 @@ void MasterPathHandlers::HandleTabletServers(const Webserver::WebRequest& /*req*
     }
     EasyJson ts_json = (*output)[ts_key].PushBack(EasyJson::kObject);
 
-    ServerRegistrationPB reg;
-    desc->GetRegistration(&reg);
     ts_json["uuid"] = ts_uuid;
     if (!reg.http_addresses().empty()) {
       string webserver = Substitute("$0://$1:$2",
@@ -181,7 +189,7 @@ void MasterPathHandlers::HandleTabletServers(const Webserver::WebRequest& /*req*
     ts_json["start_time"] = StartTimeToString(reg);
     reg.clear_start_time();  // Clear 'start_time' before dumping to string.
     ts_json["registration"] = pb_util::SecureShortDebugString(reg);
-    ts_json["location"] = desc->location().get_value_or("<none>");
+    ts_json["location"] = desc->location().value_or("<none>");
     version_counts[reg.software_version()][presumed_dead ? 1 : 0]++;
     has_no_live_ts &= presumed_dead;
     has_no_dead_ts &= !presumed_dead;
@@ -472,9 +480,9 @@ void MasterPathHandlers::HandleTablePage(const Webserver::WebRequest& req,
                     partition.hash_buckets().end(),
                     [] (const int32_t& bucket) { return bucket == 0; })) {
       range_partitions.emplace_back(
-          partition_schema.RangePartitionDebugString(partition.begin().range_key(),
-                                                     partition.end().range_key(),
-                                                     schema));
+          partition_schema.RangeWithCustomHashPartitionDebugString(partition.begin().range_key(),
+                                                                   partition.end().range_key(),
+                                                                   schema));
     }
 
     // Combine the tablet details and partition info for each tablet.
@@ -556,9 +564,10 @@ void MasterPathHandlers::HandleTablePage(const Webserver::WebRequest& req,
 
 void MasterPathHandlers::HandleMasters(const Webserver::WebRequest& /*req*/,
                                        Webserver::WebResponse* resp) {
+  // TODO(aserbin): update this to work with proxied RPCs
   EasyJson* output = &resp->output;
   vector<ServerEntryPB> masters;
-  Status s = master_->ListMasters(&masters);
+  Status s = master_->ListMasters(&masters, /*use_external_addr=*/false);
   if (!s.ok()) {
     string msg = s.CloneAndPrepend("Unable to list Masters").ToString();
     LOG(WARNING) << msg;
@@ -596,6 +605,39 @@ void MasterPathHandlers::HandleMasters(const Webserver::WebRequest& /*req*/,
   }
 }
 
+void MasterPathHandlers::HandleIpkiCaCert(
+    DataFormat cert_format,
+    const Webserver::WebRequest& /*req*/,
+    Webserver::PrerenderedWebResponse* resp) {
+  ostringstream& out = resp->output;
+  if (!master_->catalog_manager()->IsInitialized()) {
+    resp->status_code = HttpStatusCode::ServiceUnavailable;
+    out << "ERROR: CatalogManager is not running";
+    return;
+  }
+  const auto* ca = master_->cert_authority();
+  if (!ca || !ca->IsInitialized()) {
+    resp->status_code = HttpStatusCode::ServiceUnavailable;
+    out << "ERROR: IPKI CA isn't initialized";
+    return;
+  }
+  const auto& cert = ca->ca_cert();
+  string cert_str;
+  if (auto s = cert.ToString(&cert_str, cert_format); !s.ok()) {
+    auto err = s.CloneAndPrepend(
+        Substitute("could not convert CA cert to $0 format",
+                   security::DataFormatToString(cert_format)));
+    LOG(ERROR) << err.ToString();
+    resp->status_code = HttpStatusCode::InternalServerError;
+    out << "ERROR: " << err.ToString();
+    return;
+  }
+  if (cert_format == DataFormat::PEM) {
+    RemoveExtraWhitespace(&cert_str);
+  }
+  out << cert_str;
+}
+
 namespace {
 
 // Visitor for the catalog table which dumps tables and tablets in a JSON format. This
@@ -737,14 +779,17 @@ void MasterPathHandlers::HandleDumpEntities(const Webserver::WebRequest& /*req*/
   vector<shared_ptr<TSDescriptor> > descs;
   master_->ts_manager()->GetAllDescriptors(&descs);
   for (const auto& desc : descs) {
+    ServerRegistrationPB reg;
+    if (auto s = desc->GetRegistration(&reg); PREDICT_FALSE(!s.ok())) {
+      LOG(WARNING) << s.ToString();
+      continue;
+    }
+
     jw.StartObject();
 
     jw.String("uuid");
     jw.String(desc->permanent_uuid());
 
-    ServerRegistrationPB reg;
-    desc->GetRegistration(&reg);
-
     jw.String("rpc_addrs");
     jw.StartArray();
     for (const HostPortPB& host_port : reg.rpc_addresses()) {
@@ -781,8 +826,8 @@ void MasterPathHandlers::HandleDumpEntities(const Webserver::WebRequest& /*req*/
 }
 
 Status MasterPathHandlers::Register(Webserver* server) {
-  bool is_styled = true;
-  bool is_on_nav_bar = true;
+  constexpr const bool is_styled = true;
+  constexpr const bool is_on_nav_bar = true;
   server->RegisterPathHandler(
       "/tablet-servers", "Tablet Servers",
       [this](const Webserver::WebRequest& req, Webserver::WebResponse* resp) {
@@ -807,6 +852,23 @@ Status MasterPathHandlers::Register(Webserver* server) {
         this->HandleMasters(req, resp);
       },
       is_styled, is_on_nav_bar);
+  server->RegisterPrerenderedPathHandler(
+      "/ipki-ca-cert", "IPKI CA certificate",
+      [this](const Webserver::WebRequest& req, Webserver::PrerenderedWebResponse* resp) {
+        this->HandleIpkiCaCert(DataFormat::PEM, req, resp);
+      },
+      false /*is_styled*/, true /*is_on_nav_bar*/);
+  server->RegisterPrerenderedPathHandler(
+      "/ipki-ca-cert-pem", "IPKI CA certificate (PEM format)",
+      [this](const Webserver::WebRequest& req, Webserver::PrerenderedWebResponse* resp) {
+        this->HandleIpkiCaCert(DataFormat::PEM, req, resp);
+      },
+      false /*is_styled*/, false /*is_on_nav_bar*/);
+  server->RegisterBinaryDataPathHandler(
+      "/ipki-ca-cert-der", "IPKI CA certificate (DER format)",
+      [this](const Webserver::WebRequest& req, Webserver::PrerenderedWebResponse* resp) {
+        this->HandleIpkiCaCert(DataFormat::DER, req, resp);
+      });
   server->RegisterPrerenderedPathHandler(
       "/dump-entities", "Dump Entities",
       [this](const Webserver::WebRequest& req, Webserver::PrerenderedWebResponse* resp) {
@@ -819,8 +881,10 @@ Status MasterPathHandlers::Register(Webserver* server) {
 pair<string, string> MasterPathHandlers::TSDescToLinkPair(const TSDescriptor& desc,
                                                           const string& tablet_id) const {
   ServerRegistrationPB reg;
-  desc.GetRegistration(&reg);
-  if (reg.http_addresses().empty()) {
+  if (auto s = desc.GetRegistration(&reg); !s.ok() || reg.http_addresses().empty()) {
+    if (PREDICT_FALSE(!s.ok())) {
+      LOG(WARNING) << s.ToString();
+    }
     return std::make_pair(desc.permanent_uuid(), "");
   }
   string text = Substitute("$0:$1", reg.http_addresses(0).host(), reg.http_addresses(0).port());
@@ -857,8 +921,11 @@ string MasterPathHandlers::MasterAddrsToCsv() const {
 }
 
 Status MasterPathHandlers::GetLeaderMasterHttpAddr(string* leader_http_addr) const {
+  // TODO(aserbin): update this to work with proxied RPCs
   vector<ServerEntryPB> masters;
-  RETURN_NOT_OK_PREPEND(master_->ListMasters(&masters), "unable to list masters");
+  RETURN_NOT_OK_PREPEND(master_->ListMasters(&masters,
+                                             /*use_external_addr=*/false),
+                        "unable to list masters");
   for (const auto& master : masters) {
     if (master.has_error()) {
       continue;
