@@ -21,12 +21,12 @@
 #include <functional>
 #include <iostream>
 #include <memory>
+#include <optional>
 #include <set>
 #include <string>
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 
@@ -39,6 +39,7 @@
 #include "kudu/consensus/consensus.proxy.h"
 #include "kudu/consensus/consensus_meta_manager.h"
 #include "kudu/consensus/metadata.pb.h"
+#include "kudu/consensus/opid.pb.h"
 #include "kudu/fs/fs_manager.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/map-util.h"
@@ -52,7 +53,6 @@
 #include "kudu/master/master_options.h"
 #include "kudu/master/sys_catalog.h"
 #include "kudu/rpc/messenger.h"
-#include "kudu/rpc/response_callback.h"
 #include "kudu/rpc/rpc.h"
 #include "kudu/rpc/rpc_controller.h"
 #include "kudu/tablet/metadata.pb.h"
@@ -84,7 +84,7 @@ using kudu::pb_util::SecureShortDebugString;
 using kudu::rpc::RpcController;
 using kudu::tablet::TabletDataState;
 using kudu::tablet::TabletMetadata;
-using kudu::tserver::TabletCopyClient;
+using kudu::tserver::RemoteTabletCopyClient;
 using kudu::tserver::TSTabletManager;
 using std::set;
 using std::string;
@@ -310,12 +310,13 @@ Status ClearLocalSystemCatalogAndCopy(const HostPort& src_hp) {
   RETURN_NOT_OK(TabletMetadata::Load(&fs_manager, SysCatalogTable::kSysCatalogTabletId, &meta));
   RETURN_NOT_OK(TSTabletManager::DeleteTabletData(
       meta, cmeta_manager, TabletDataState::TABLET_DATA_DELETED,
-      /*last_logged_opid*/boost::none));
+      /*last_logged_opid*/std::nullopt));
   LOG(INFO) << "Copying system tablet from " << src_hp.ToString();
   std::shared_ptr<rpc::Messenger> messenger;
   RETURN_NOT_OK(rpc::MessengerBuilder("tablet_copy_client").Build(&messenger));
-  TabletCopyClient copy_client(SysCatalogTable::kSysCatalogTabletId, &fs_manager, cmeta_manager,
-                               messenger, nullptr /* no metrics */);
+  RemoteTabletCopyClient copy_client(SysCatalogTable::kSysCatalogTabletId,
+                                     &fs_manager, cmeta_manager,
+                                     messenger, nullptr /* no metrics */);
   RETURN_NOT_OK(copy_client.Start(src_hp, /*meta*/nullptr));
   RETURN_NOT_OK(copy_client.FetchAll(/*tablet_replica*/nullptr));
   return copy_client.Finish();
@@ -325,6 +326,7 @@ Status ClearLocalSystemCatalogAndCopy(const HostPort& src_hp) {
 
 void SetMasterFlagDefaults() {
   constexpr int32_t kDefaultRpcServiceQueueLength = 100;
+  constexpr int32_t kDefaultTabletHistoryMaxAgeSec = 300;
 
   // Reset some default values before parsing gflags.
   CHECK_NE("", SetCommandLineOptionWithMode(
@@ -345,6 +347,17 @@ void SetMasterFlagDefaults() {
       "rpc_service_queue_length",
       to_string(kDefaultRpcServiceQueueLength).c_str(),
       SET_FLAGS_DEFAULT));
+  // Master always reads the latest data snapshot from the system catalog and
+  // never uses any specific past timestamp for a read snapshot. With that,
+  // it doesn't make sense to keep a long chain of UNDO deltas in addition to the
+  // latest version in the MVCC. Keeping a short history of deltas frees CPU
+  // cycles, memory, and IO bandwidth that otherwise would be consumed by
+  // background maintenance jobs running compactions. In addition, less disk
+  // space is consumed to store the system tablet's data.
+  CHECK_NE("", SetCommandLineOptionWithMode(
+      "tablet_history_max_age_sec",
+      to_string(kDefaultTabletHistoryMaxAgeSec).c_str(),
+      SET_FLAGS_DEFAULT));
   // Setting the default value of the 'force_block_cache_capacity' flag to
   // 'false' makes the corresponding group validator enforce proper settings
   // for the memory limit and the cfile cache capacity.
