@@ -18,13 +18,14 @@
 #include "kudu/master/master_service.h"
 
 #include <memory>
+#include <optional>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 
@@ -61,6 +62,7 @@
 #include "kudu/util/scoped_cleanup.h"
 #include "kudu/util/status.h"
 
+DECLARE_bool(enable_per_range_hash_schemas);
 DECLARE_bool(hive_metastore_sasl_enabled);
 DECLARE_bool(raft_prepare_replacement_before_eviction);
 DECLARE_string(hive_metastore_uris);
@@ -101,10 +103,27 @@ TAG_FLAG(master_support_change_config, advanced);
 TAG_FLAG(master_support_change_config, runtime);
 
 DEFINE_bool(master_support_ignore_operations, true,
-            "Whether the cluster supports support ignore operations.");
+            "Whether the cluster supports ignore operations, including "
+            "INSERT_IGNORE, DELETE_IGNORE and UPDATE_IGNORE).");
 TAG_FLAG(master_support_ignore_operations, hidden);
 TAG_FLAG(master_support_ignore_operations, runtime);
 
+DEFINE_bool(master_support_upsert_ignore_operations, true,
+            "Whether the cluster supports UPSERT_IGNORE operations.");
+TAG_FLAG(master_support_upsert_ignore_operations, hidden);
+TAG_FLAG(master_support_upsert_ignore_operations, runtime);
+
+DEFINE_bool(master_support_immutable_column_attribute, true,
+            "Whether the cluster supports immutable attribute on column schema.");
+TAG_FLAG(master_support_immutable_column_attribute, hidden);
+TAG_FLAG(master_support_immutable_column_attribute, runtime);
+
+DEFINE_bool(master_support_auto_incrementing_column, true,
+            "Whether the cluster supports auto-incrementing columns. "
+            "This in turn enables the usage of non-unique primary key.");
+TAG_FLAG(master_support_auto_incrementing_column, unsafe);
+TAG_FLAG(master_support_auto_incrementing_column, experimental);
+TAG_FLAG(master_support_auto_incrementing_column, runtime);
 
 using google::protobuf::Message;
 using kudu::consensus::ReplicaManagementInfoPB;
@@ -432,11 +451,21 @@ void MasterServiceImpl::TSHeartbeat(const TSHeartbeatRequestPB* req,
   }
 
   // 4. Update tserver soft state based on the heartbeat contents.
+  // TODO(mreddy) If --enable_range_replica_placement is set to false, don't populate ranges map.
   ts_desc->UpdateHeartbeatTime();
   ts_desc->set_num_live_replicas(req->num_live_tablets());
   ts_desc->set_num_live_replicas_by_dimension(
       TabletNumByDimensionMap(req->num_live_tablets_by_dimension().begin(),
                               req->num_live_tablets_by_dimension().end()));
+  for (auto it = req->num_live_tablets_by_range_per_table().begin();
+       it != req->num_live_tablets_by_range_per_table().end(); ++it) {
+    TabletNumByRangeMap ranges;
+    for (auto range = it->second.num_live_tablets_by_range().begin();
+         range != it->second.num_live_tablets_by_range().end(); ++range) {
+      ranges[range->range_start_key()] = range->tablets();
+    }
+    ts_desc->set_num_live_replicas_by_range_per_table(it->first, ranges);
+  }
 
   // 5. Only leaders handle tablet reports.
   if (is_leader_master && req->has_tablet_report()) {
@@ -504,12 +533,17 @@ void MasterServiceImpl::GetTabletLocations(const GetTabletLocationsRequestPB* re
     SleepFor(MonoDelta::FromMilliseconds(FLAGS_master_inject_latency_on_tablet_lookups_ms));
   }
 
+  const auto use_external_addr = IsAddrOneOf(
+      rpc->local_address(), server_->rpc_proxied_addresses());
+
   CatalogManager::TSInfosDict infos_dict(resp->GetArena());
   for (const string& tablet_id : req->tablet_ids()) {
     // TODO(todd): once we have catalog data. ACL checks would also go here, probably.
     TabletLocationsPB* locs_pb = resp->add_tablet_locations();
     Status s = server_->catalog_manager()->GetTabletLocations(
-        tablet_id, req->replica_type_filter(),
+        tablet_id,
+        req->replica_type_filter(),
+        use_external_addr,
         locs_pb,
         req->intern_ts_infos_in_response() ? &infos_dict : nullptr,
         rpc->remote_user().username());
@@ -564,7 +598,30 @@ void MasterServiceImpl::DeleteTable(const DeleteTableRequestPB* req,
     return;
   }
 
-  Status s = server_->catalog_manager()->DeleteTableRpc(*req, resp, rpc);
+  Status s = server_->catalog_manager()->SoftDeleteTableRpc(*req, resp, rpc);
+  CheckRespErrorOrSetUnknown(s, resp);
+  rpc->RespondSuccess();
+}
+
+void MasterServiceImpl::RecallDeletedTable(const RecallDeletedTableRequestPB* req,
+                                           RecallDeletedTableResponsePB* resp,
+                                           rpc::RpcContext* rpc) {
+  CatalogManager::ScopedLeaderSharedLock l(server_->catalog_manager());
+  if (!l.CheckIsInitializedAndIsLeaderOrRespond(resp, rpc)) {
+    return;
+  }
+
+  // Soft-delete related functions is not supported when HMS is enabled.
+  if (hms::HmsCatalog::IsEnabled()) {
+    StatusToPB(Status::NotSupported("RecallDeletedTable is not supported when HMS is enabled."),
+               resp->mutable_error()->mutable_status());
+    resp->mutable_error()->set_code(MasterErrorPB::UNKNOWN_ERROR);
+    rpc->RespondSuccess();
+    return;
+  }
+
+  Status s = server_->catalog_manager()->RecallDeletedTableRpc(
+             *req, resp, rpc);
   CheckRespErrorOrSetUnknown(s, resp);
   rpc->RespondSuccess();
 }
@@ -639,8 +696,11 @@ void MasterServiceImpl::GetTableLocations(const GetTableLocationsRequestPB* req,
     if (PREDICT_FALSE(FLAGS_master_inject_latency_on_tablet_lookups_ms > 0)) {
       SleepFor(MonoDelta::FromMilliseconds(FLAGS_master_inject_latency_on_tablet_lookups_ms));
     }
+
+    const auto use_external_addr = IsAddrOneOf(
+        rpc->local_address(), server_->rpc_proxied_addresses());
     s = server_->catalog_manager()->GetTableLocations(
-        req, resp, rpc->remote_user().username());
+        req, resp, use_external_addr, rpc->remote_user().username());
   }
 
   CheckRespErrorOrSetUnknown(s, resp);
@@ -669,6 +729,9 @@ void MasterServiceImpl::GetTableSchema(const GetTableSchemaRequestPB* req,
 void MasterServiceImpl::ListTabletServers(const ListTabletServersRequestPB* req,
                                           ListTabletServersResponsePB* resp,
                                           rpc::RpcContext* rpc) {
+  const auto use_external_addr = IsAddrOneOf(
+      rpc->local_address(), server_->rpc_proxied_addresses());
+
   TSManager* ts_manager = server_->ts_manager();
   TServerStateMap states = req->include_states() ?
       ts_manager->GetTServerStates() : TServerStateMap();
@@ -676,10 +739,17 @@ void MasterServiceImpl::ListTabletServers(const ListTabletServersRequestPB* req,
   server_->ts_manager()->GetAllDescriptors(&descs);
   for (const std::shared_ptr<TSDescriptor>& desc : descs) {
     ListTabletServersResponsePB::Entry* entry = resp->add_servers();
+    if (auto s = desc->GetRegistration(entry->mutable_registration(),
+                                       use_external_addr); !s.ok()) {
+      LOG(WARNING) << s.ToString();
+      resp->mutable_servers()->RemoveLast();
+      continue;
+    }
     desc->GetNodeInstancePB(entry->mutable_instance_id());
-    desc->GetRegistration(entry->mutable_registration());
     entry->set_millis_since_heartbeat(desc->TimeSinceHeartbeat().ToMilliseconds());
-    if (desc->location()) entry->set_location(desc->location().get());
+    if (desc->location()) {
+      entry->set_location(*desc->location());
+    }
 
     // If we need to return states, do so.
     const auto& uuid = desc->permanent_uuid();
@@ -704,12 +774,13 @@ void MasterServiceImpl::ListTabletServers(const ListTabletServersRequestPB* req,
   rpc->RespondSuccess();
 }
 
-void MasterServiceImpl::ListMasters(const ListMastersRequestPB* req,
+void MasterServiceImpl::ListMasters(const ListMastersRequestPB* /*req*/,
                                     ListMastersResponsePB* resp,
                                     rpc::RpcContext* rpc) {
+  const auto use_external_addr = IsAddrOneOf(
+      rpc->local_address(), server_->rpc_proxied_addresses());
   vector<ServerEntryPB> masters;
-  Status s = server_->ListMasters(&masters);
-  if (!s.ok()) {
+  if (const auto s = server_->ListMasters(&masters, use_external_addr); !s.ok()) {
     StatusToPB(s, resp->mutable_error()->mutable_status());
     resp->mutable_error()->set_code(MasterErrorPB::UNKNOWN_ERROR);
 
@@ -723,9 +794,10 @@ void MasterServiceImpl::ListMasters(const ListMastersRequestPB* req,
   rpc->RespondSuccess();
 }
 
-void MasterServiceImpl::GetMasterRegistration(const GetMasterRegistrationRequestPB* req,
-                                              GetMasterRegistrationResponsePB* resp,
-                                              rpc::RpcContext* rpc) {
+void MasterServiceImpl::GetMasterRegistration(
+    const GetMasterRegistrationRequestPB* /*req*/,
+    GetMasterRegistrationResponsePB* resp,
+    rpc::RpcContext* rpc) {
   // instance_id must always be set in order for status pages to be useful.
   resp->mutable_instance_id()->CopyFrom(server_->instance_pb());
 
@@ -734,7 +806,10 @@ void MasterServiceImpl::GetMasterRegistration(const GetMasterRegistrationRequest
     return;
   }
 
-  Status s = server_->GetMasterRegistration(resp->mutable_registration());
+  const auto use_external_addr = IsAddrOneOf(
+      rpc->local_address(), server_->rpc_proxied_addresses());
+  const auto s = server_->GetMasterRegistration(
+      resp->mutable_registration(), use_external_addr);
   CheckRespErrorOrSetUnknown(s, resp);
   const auto& role_and_member = server_->catalog_manager()->GetRoleAndMemberType();
   resp->set_role(role_and_member.first);
@@ -753,17 +828,27 @@ void MasterServiceImpl::ConnectToMaster(const ConnectToMasterRequestPB* /*req*/,
 
   // Set the info about the other masters, so that the client can verify
   // it has the full set of info.
-  vector<HostPort> addresses;
-  Status s = server_->GetMasterHostPorts(&addresses);
-  if (!s.ok()) {
-    StatusToPB(s, resp->mutable_error()->mutable_status());
-    resp->mutable_error()->set_code(MasterErrorPB::UNKNOWN_ERROR);
-    rpc->RespondSuccess();
-    return;
-  }
-  resp->mutable_master_addrs()->Reserve(addresses.size());
-  for (const auto& hp : addresses) {
-    *resp->add_master_addrs() = HostPortToPB(hp);
+  {
+    // Check if the request came through the dedicated RPC endpoint meaning
+    // it's been proxied from external network.
+    if (IsAddrOneOf(rpc->local_address(), server_->rpc_proxied_addresses())) {
+      for (const auto& hp : server_->GetProxyAdvertisedHostPorts()) {
+        *resp->add_master_addrs() = HostPortToPB(hp);
+      }
+    } else {
+      vector<HostPort> addresses;
+      if (const auto s = server_->GetMasterHostPorts(&addresses);
+          PREDICT_FALSE(!s.ok())) {
+        StatusToPB(s, resp->mutable_error()->mutable_status());
+        resp->mutable_error()->set_code(MasterErrorPB::UNKNOWN_ERROR);
+        rpc->RespondSuccess();
+        return;
+      }
+      resp->mutable_master_addrs()->Reserve(addresses.size());
+      for (const auto& hp : addresses) {
+        *resp->add_master_addrs() = HostPortToPB(hp);
+      }
+    }
   }
 
   const bool is_leader = l.leader_status().ok();
@@ -881,6 +966,14 @@ bool MasterServiceImpl::SupportsFeature(uint32_t feature) const {
       return FLAGS_master_support_change_config;
     case MasterFeatures::IGNORE_OPERATIONS:
       return FLAGS_master_support_ignore_operations;
+    case MasterFeatures::RANGE_SPECIFIC_HASH_SCHEMA:
+      return FLAGS_enable_per_range_hash_schemas;
+    case MasterFeatures::UPSERT_IGNORE:
+      return FLAGS_master_support_upsert_ignore_operations;
+    case MasterFeatures::IMMUTABLE_COLUMN_ATTRIBUTE:
+      return FLAGS_master_support_immutable_column_attribute;
+    case MasterFeatures::AUTO_INCREMENTING_COLUMN:
+      return FLAGS_master_support_auto_incrementing_column;
     default:
       return false;
   }
