@@ -25,7 +25,7 @@
 #include <utility>
 #include <vector>
 
-#include <boost/functional/hash/hash.hpp>
+#include <boost/container_hash/extensions.hpp>
 #include <gflags/gflags_declare.h>
 #include <glog/logging.h>
 #include <google/protobuf/any.pb.h>
@@ -36,6 +36,7 @@
 #include "kudu/gutil/strings/join.h"
 #include "kudu/gutil/strings/split.h"
 #include "kudu/gutil/strings/substitute.h"
+#include "kudu/postgres/mini_postgres.h"
 #include "kudu/ranger/mini_ranger.h"
 #include "kudu/ranger/ranger.pb.h"
 #include "kudu/subprocess/server.h"
@@ -53,6 +54,8 @@ DECLARE_string(log_dir);
 DECLARE_string(ranger_config_path);
 DECLARE_string(ranger_log_config_dir);
 DECLARE_string(ranger_log_level);
+DECLARE_string(ranger_jar_path);
+DECLARE_string(ranger_java_extra_args);
 DECLARE_bool(ranger_logtostdout);
 DECLARE_bool(ranger_overwrite_log_config);
 
@@ -70,6 +73,7 @@ using std::vector;
 using strings::SkipEmpty;
 using strings::Split;
 using strings::Substitute;
+using kudu::postgres::MiniPostgres;
 
 namespace kudu {
 namespace ranger {
@@ -336,12 +340,31 @@ TEST_F(RangerClientTest, TestAuthorizeActionsAllAuthorized) {
   ASSERT_EQ(3, actions.size());
 }
 
+TEST_F(RangerClientTest, TestInvalidJARFails) {
+  FLAGS_ranger_config_path = test_dir_;
+  FLAGS_ranger_jar_path = "/this/is/not/a/real/location/hopefully.jar";
+  ASSERT_FALSE(ValidateRangerConfiguration());
+}
+
+TEST_F(RangerClientTest, TestMultipleInvalidJARsLeftUnchecked) {
+  FLAGS_ranger_config_path = test_dir_;
+  FLAGS_ranger_jar_path = "/this/is/not/a/real/location/hopefully.jar:/another/invalid/path.jar";
+  ASSERT_TRUE(ValidateRangerConfiguration());
+}
+
+TEST_F(RangerClientTest, TestDefaultJARPath) {
+  FLAGS_ranger_config_path = test_dir_;
+  ASSERT_TRUE(ValidateRangerConfiguration());
+}
+
+
 class RangerClientTestBase : public KuduTest {
  public:
   RangerClientTestBase()
       : test_dir_(GetTestDataDirectory()) {}
 
   void SetUp() override {
+    SKIP_IF_SLOW_NOT_ALLOWED();
     metric_entity_ = METRIC_ENTITY_server.Instantiate(&metric_registry_, "ranger_client-test");
     FLAGS_ranger_log_level = "debug";
     FLAGS_ranger_logtostdout = true;
@@ -353,7 +376,7 @@ class RangerClientTestBase : public KuduTest {
   }
 
   Status InitializeRanger() {
-    ranger_.reset(new MiniRanger("127.0.0.1"));
+    ranger_.reset(new MiniRanger("127.0.0.1", std::make_shared<MiniPostgres>("127.0.0.1")));
     RETURN_NOT_OK(ranger_->Start());
     // Create a policy so the Ranger client policy refresher can pick something
     // up. In some environments the absense of policies can cause the plugin to
@@ -380,6 +403,35 @@ class RangerClientTestBase : public KuduTest {
   std::unique_ptr<RangerClient> client_;
 };
 
+class RangerClientTestExtraJVMArgs : public RangerClientTestBase {
+ public:
+  void SetUp() override {
+    SKIP_IF_SLOW_NOT_ALLOWED();
+    metric_entity_ = METRIC_ENTITY_server.Instantiate(&metric_registry_, "ranger_client-test");
+    FLAGS_ranger_log_level = "debug";
+    FLAGS_ranger_logtostdout = true;
+    FLAGS_ranger_config_path = test_dir_;
+    FLAGS_ranger_log_config_dir = JoinPathSegments(test_dir_, "log_conf");
+    FLAGS_log_dir = JoinPathSegments(test_dir_, "logs");
+    ASSERT_OK(env_->CreateDir(FLAGS_log_dir));
+  }
+};
+
+TEST_F(RangerClientTestExtraJVMArgs, SmokeTest) {
+  FLAGS_ranger_java_extra_args = "-Xmx1G  -XX:+UseG1GC";
+  ASSERT_OK(InitializeRanger());
+
+  bool authorized;
+  ASSERT_OK(client_->AuthorizeAction("user", ActionPB::METADATA, "db", "table", /*is_owner=*/false,
+                                    /*requires_delegate_admin=*/false, &authorized));
+  ASSERT_TRUE(authorized);
+}
+
+TEST_F(RangerClientTestExtraJVMArgs, TestCrashOnInvalidArguments) {
+  FLAGS_ranger_java_extra_args = "-XX:+InvalidArgumentHopeTheyWontChooseThisNameForAFLag";
+  ASSERT_DEATH(InitializeRanger(), "The subprocess has exited with status 1");
+}
+
 namespace {
 
 // Looks in --log_dir and returns the full Kudu Ranger subprocess log filename
