@@ -32,6 +32,7 @@
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/strings/join.h"
+#include "kudu/gutil/strings/split.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/ranger/ranger.pb.h"
 #include "kudu/security/init.h"
@@ -62,10 +63,15 @@ DEFINE_string(ranger_java_path, "",
               "is used. If $JAVA_HOME is not found, Kudu will attempt to "
               "find 'java' in the Kudu user's PATH.");
 
+DEFINE_string(ranger_java_extra_args, "",
+              "Extra JVM arguments to be passed to the Ranger subprocess.");
+
 DEFINE_string(ranger_jar_path, "",
               "Path to the JAR file containing the Ranger subprocess. If "
               "not specified, the default JAR file path is expected to be "
-              "next to the master binary.");
+              "next to the master binary. It can be a colon-separated list of "
+              "JARs. If it is, the paths are not verified and passed straight to "
+              "Java.");
 
 DEFINE_string(ranger_receiver_fifo_dir, "",
               "Directory in which to create a fifo used to receive messages "
@@ -98,8 +104,13 @@ DEFINE_bool(ranger_logtostdout, false,
 TAG_FLAG(ranger_logtostdout, advanced);
 TAG_FLAG(ranger_logtostdout, evolving);
 
+DEFINE_bool(ranger_crash_master_on_subprocess_failure, true,
+            "Whether to crash the Master if the Ranger subprocess crashes.");
+TAG_FLAG(ranger_crash_master_on_subprocess_failure, advanced);
+
 DECLARE_int32(max_log_files);
-DECLARE_int32(max_log_size);
+DECLARE_uint32(max_log_size);
+DECLARE_uint32(subprocess_max_message_size_bytes);
 DECLARE_string(log_dir);
 
 METRIC_DEFINE_histogram(server, ranger_subprocess_execution_time_ms,
@@ -159,6 +170,12 @@ METRIC_DEFINE_histogram(server, ranger_server_outbound_queue_time_ms,
     "Duration of time in ms spent in the Ranger server's outbound request queue",
     kudu::MetricLevel::kInfo,
     60000LU, 1);
+METRIC_DEFINE_counter(server, ranger_server_dropped_messages,
+    "Number of messages dropped by the subprocess server",
+    kudu::MetricUnit::kMessages,
+    "Number of responses that the Ranger client had sent, but the subprocess "
+    "server failed to receive because they were oversized, corrupted, etc.",
+    kudu::MetricLevel::kWarn);
 
 DECLARE_string(keytab_file);
 DECLARE_string(principal);
@@ -222,32 +239,6 @@ string JavaPath() {
   return FLAGS_ranger_java_path;
 }
 
-bool ValidateRangerConfiguration() {
-  if (!FLAGS_ranger_config_path.empty()) {
-    // First, check the specified Java path.
-    const string java_path = JavaPath();
-    if (!Env::Default()->FileExists(java_path)) {
-      // Otherwise, since the specified path is not absolute, check if
-      // the Java binary is on the PATH.
-      string p;
-      Status s = Subprocess::Call({ "which", java_path }, "", &p);
-      if (!s.ok()) {
-        LOG(ERROR) << Substitute("--ranger_java_path has invalid java binary path: $0",
-                                 java_path);
-        return false;
-      }
-    }
-    const string ranger_jar_path = RangerJarPath();
-    if (!Env::Default()->FileExists(ranger_jar_path)) {
-      LOG(ERROR) << Substitute("--ranger_jar_path has invalid JAR file path: $0",
-                               ranger_jar_path);
-      return false;
-    }
-  }
-  return true;
-}
-GROUP_FLAG_VALIDATOR(ranger_config_flags, ValidateRangerConfiguration);
-
 bool ValidateLog4jLevel(const char* /*flagname*/, const string& value) {
   static const vector<string> kLevels = {
     "all",
@@ -345,8 +336,16 @@ Status BuildArgv(const string& fifo_path, const string& log_properties_path,
     JavaPath(),
     Substitute("-Djava.security.krb5.conf=$0", GetKrb5ConfigFile()),
     Substitute("-Dlog4j2.configurationFile=$0", log_properties_path),
-    "-cp", JavaClasspath(), kMainClass,
   };
+  if (!FLAGS_ranger_java_extra_args.empty()) {
+    vector<string> args = strings::Split(FLAGS_ranger_java_extra_args, " ", strings::SkipEmpty());
+    for (auto& arg : args) {
+      ret.emplace_back(std::move(arg));
+    }
+  }
+  ret.emplace_back("-cp");
+  ret.emplace_back(JavaClasspath());
+  ret.emplace_back(kMainClass);
   // When Kerberos is enabled in Kudu, pass both Kudu principal and keytab file
   // to the Ranger subprocess.
   if (!FLAGS_keytab_file.empty()) {
@@ -358,6 +357,8 @@ Status BuildArgv(const string& fifo_path, const string& log_properties_path,
     ret.emplace_back("-k");
     ret.emplace_back(FLAGS_keytab_file);
   }
+  ret.emplace_back("-m");
+  ret.emplace_back(std::to_string(FLAGS_subprocess_max_message_size_bytes));
   ret.emplace_back("-o");
   ret.emplace_back(fifo_path);
   *argv = std::move(ret);
@@ -366,6 +367,37 @@ Status BuildArgv(const string& fifo_path, const string& log_properties_path,
 
 } // anonymous namespace
 
+bool ValidateRangerConfiguration() {
+  if (!FLAGS_ranger_config_path.empty()) {
+    // First, check the specified Java path.
+    const string java_path = JavaPath();
+    if (!Env::Default()->FileExists(java_path)) {
+      // Otherwise, since the specified path is not absolute, check if
+      // the Java binary is on the PATH.
+      string p;
+      Status s = Subprocess::Call({ "which", java_path }, "", &p);
+      if (!s.ok()) {
+        LOG(ERROR) << Substitute("--ranger_java_path has invalid java binary path: $0",
+                                 java_path);
+        return false;
+      }
+    }
+    const string ranger_jar_path = RangerJarPath();
+
+    // If the JAR path contains a colon, we skip verifying the paths and leave
+    // it to Java.
+    if (ranger_jar_path.find(':') == string::npos &&
+        !Env::Default()->FileExists(ranger_jar_path)) {
+      LOG(ERROR) << Substitute("--ranger_jar_path has invalid JAR file path: $0",
+                               ranger_jar_path);
+      return false;
+    }
+  }
+  return true;
+}
+GROUP_FLAG_VALIDATOR(ranger_config_flags, ValidateRangerConfiguration);
+
+#define CINIT(member, x) member = METRIC_##x.Instantiate(entity)
 #define HISTINIT(member, x) member = METRIC_##x.Instantiate(entity)
 RangerSubprocessMetrics::RangerSubprocessMetrics(const scoped_refptr<MetricEntity>& entity) {
   HISTINIT(sp_inbound_queue_length, ranger_subprocess_inbound_queue_length);
@@ -377,8 +409,10 @@ RangerSubprocessMetrics::RangerSubprocessMetrics(const scoped_refptr<MetricEntit
   HISTINIT(server_inbound_queue_time_ms, ranger_server_inbound_queue_time_ms);
   HISTINIT(server_outbound_queue_size_bytes, ranger_server_outbound_queue_size_bytes);
   HISTINIT(server_outbound_queue_time_ms, ranger_server_outbound_queue_time_ms);
+  CINIT(server_dropped_messages, ranger_server_dropped_messages);
 }
 #undef HISTINIT
+#undef CINIT
 
 RangerClient::RangerClient(Env* env, const scoped_refptr<MetricEntity>& metric_entity)
     : env_(env), metric_entity_(metric_entity) {
@@ -392,8 +426,9 @@ Status RangerClient::Start() {
   const string fifo_path = SubprocessServer::FifoPath(RangerFifoBase());
   vector<string> argv;
   RETURN_NOT_OK(BuildArgv(fifo_path, log_properties_path, &argv));
-  subprocess_.reset(new RangerSubprocess(env_, fifo_path, std::move(argv), metric_entity_,
-                                         "Ranger client subprocess"));
+  subprocess_.reset(new RangerSubprocess(env_, fifo_path, argv, metric_entity_,
+                                         "Ranger client subprocess",
+                                         FLAGS_ranger_crash_master_on_subprocess_failure));
   return subprocess_->Start();
 }
 
