@@ -416,6 +416,12 @@ void Connection::CancelOutboundCall(const shared_ptr<OutboundCall> &call) {
   }
 }
 
+Status Connection::GetLocalAddress(Sockaddr* addr) const {
+  DCHECK(socket_);
+  DCHECK(addr);
+  return socket_->GetSocketAddress(addr);
+}
+
 // Inject a cancellation when 'call' is in state 'FLAGS_rpc_inject_cancellation_state'.
 void inline Connection::MaybeInjectCancellation(const shared_ptr<OutboundCall> &call) {
   if (PREDICT_FALSE(call->ShouldInjectCancellation())) {
@@ -805,7 +811,7 @@ Connection::ProcessOutboundTransfersResult Connection::ProcessOutboundTransfers(
     }
 
     last_activity_time_ = reactor_thread_->cur_time();
-    Status status = transfer->SendBuffer(*socket_);
+    Status status = transfer->SendBuffer(socket_.get());
     if (PREDICT_FALSE(!status.ok())) {
       LOG(WARNING) << ToString() << " send error: " << status.ToString();
       reactor_thread_->DestroyConnection(this, status);
