@@ -399,9 +399,9 @@ TEST_F(ExactlyOnceRpcTest, TestExactlyOnceSemanticsAfterRpcCompleted) {
 
     // The incremental usage of a new client is the size of the response itself
     // plus some fixed overhead for the client-tracking structure.
-    int expected_incremental_usage = original_resp.SpaceUsed() + 200;
+    size_t expected_incremental_usage = original_resp.SpaceUsedLong() + 200;
 
-    int mem_consumption_after = mem_tracker_->consumption();
+    size_t mem_consumption_after = mem_tracker_->consumption();
     ASSERT_GT(mem_consumption_after - mem_consumption, expected_incremental_usage);
     mem_consumption = mem_consumption_after;
   }
@@ -550,7 +550,7 @@ TEST_F(ExactlyOnceRpcTest, TestExactlyOnceSemanticsGarbageCollection) {
 
   resp.Clear();
   Status s = MakeAddCall(sequence_number, 1, &resp);
-  ASSERT_TRUE(s.IsRemoteError());
+  ASSERT_TRUE(s.IsRemoteError()) << s.ToString();
   ASSERT_STR_CONTAINS(s.ToString(), "is stale");
 
   // Sleep again, this time for 'remember_clients_ttl_ms' and run GC again.
