@@ -18,6 +18,7 @@
 #include "kudu/rpc/inbound_call.h"
 
 #include <cstdint>
+#include <limits>
 #include <memory>
 #include <ostream>
 
@@ -46,8 +47,8 @@ class FieldDescriptor;
 }
 }
 
+using google::protobuf::ArenaOptions;
 using google::protobuf::FieldDescriptor;
-using google::protobuf::Message;
 using google::protobuf::MessageLite;
 using std::string;
 using std::unique_ptr;
@@ -57,11 +58,18 @@ using strings::Substitute;
 namespace kudu {
 namespace rpc {
 
+static ArenaOptions MakeArenaOptions() {
+  ArenaOptions opts;
+  opts.start_block_size = 4096;
+  return opts;
+}
+
 InboundCall::InboundCall(Connection* conn)
   : conn_(conn),
     trace_(new Trace),
     method_info_(nullptr),
-    deadline_(MonoTime::Max()) {
+    deadline_(MonoTime::Max()),
+    arena_(MakeArenaOptions()) {
   RecordCallReceived();
 }
 
@@ -69,6 +77,8 @@ InboundCall::~InboundCall() {}
 
 Status InboundCall::ParseFrom(unique_ptr<InboundTransfer> transfer) {
   TRACE_EVENT_FLOW_BEGIN0("rpc", "InboundCall", this);
+  RETURN_NOT_OK(conn_->GetLocalAddress(&local_addr_));
+
   TRACE_EVENT0("rpc", "InboundCall::ParseFrom");
   RETURN_NOT_OK(serialization::ParseMessage(transfer->data(), &header_, &serialized_request_));
 
@@ -178,7 +188,8 @@ void InboundCall::SerializeResponseBuffer(const MessageLite& response,
     // happened.
   }
 
-  uint32_t protobuf_msg_size = response.ByteSize();
+  size_t protobuf_msg_size = response.ByteSizeLong();
+  CHECK_LE(protobuf_msg_size, std::numeric_limits<uint32_t>::max());
 
   ResponseHeader resp_hdr;
   resp_hdr.set_call_id(header_.call_id());
@@ -264,6 +275,11 @@ const Sockaddr& InboundCall::remote_address() const {
   return conn_->remote();
 }
 
+const Sockaddr& InboundCall::local_address() const {
+  DCHECK(local_addr_.is_initialized());
+  return local_addr_;
+}
+
 const scoped_refptr<Connection>& InboundCall::connection() const {
   return conn_;
 }
