@@ -25,11 +25,13 @@
 #include <functional>
 #include <memory>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <vector>
 
 #include <gflags/gflags.h>
 #include <glog/logging.h>
+#include "kudu/util/prometheus_writer.h"
 #include "kudu/util/version_info.h"
 #include "kudu/util/version_info.pb.h"
 
@@ -171,9 +173,16 @@ static void FlagsHandler(const Webserver::WebRequest& req,
   bool as_text = (req.parsed_args.find("raw") != req.parsed_args.end());
   Tags tags(as_text);
 
-  (*output) << tags.header << "Command-line Flags" << tags.end_header;
+  (*output) << tags.header << "Non-default Command-line Flags" << tags.end_header;
   (*output) << tags.pre_tag
-            << CommandlineFlagsIntoString(as_text ? EscapeMode::NONE : EscapeMode::HTML)
+            << CommandlineFlagsIntoString(as_text ? EscapeMode::NONE : EscapeMode::HTML,
+                                          Selection::NONDEFAULT)
+            << tags.end_pre_tag;
+
+  (*output) << tags.header << "All Command-line Flags" << tags.end_header;
+  (*output) << tags.pre_tag
+            << CommandlineFlagsIntoString(as_text ? EscapeMode::NONE : EscapeMode::HTML,
+                                          Selection::ALL)
             << tags.end_pre_tag;
 }
 
@@ -510,6 +519,13 @@ static void WriteMetricsAsJson(const MetricRegistry* const metrics,
   }
 }
 
+static void WriteMetricsAsPrometheus(const MetricRegistry* const metrics,
+                                     const Webserver::WebRequest& /*req*/,
+                                     Webserver::PrerenderedWebResponse* resp) {
+  PrometheusWriter writer(&resp->output);
+  WARN_NOT_OK(metrics->WriteAsPrometheus(&writer), "Couldn't write Prometheus metrics over HTTP");
+}
+
 void RegisterMetricsJsonHandler(Webserver* webserver, const MetricRegistry* const metrics) {
   auto callback = [metrics](const Webserver::WebRequest& req,
                             Webserver::PrerenderedWebResponse* resp) {
@@ -518,7 +534,7 @@ void RegisterMetricsJsonHandler(Webserver* webserver, const MetricRegistry* cons
   bool not_styled = false;
   bool not_on_nav_bar = false;
   bool is_on_nav_bar = true;
-  webserver->RegisterPrerenderedPathHandler("/metrics", "Metrics", callback,
+  webserver->RegisterPrerenderedPathHandler("/metrics", "JSON Metrics", callback,
                                             not_styled, is_on_nav_bar);
 
   // The old name -- this is preserved for compatibility with older releases of
@@ -527,4 +543,15 @@ void RegisterMetricsJsonHandler(Webserver* webserver, const MetricRegistry* cons
                                             not_styled, not_on_nav_bar);
 }
 
+void RegisterMetricsPrometheusHandler(Webserver* webserver, const MetricRegistry* const metrics) {
+  auto callback = [metrics](const Webserver::WebRequest& req,
+                            Webserver::PrerenderedWebResponse* resp) {
+    WriteMetricsAsPrometheus(metrics, req, resp);
+  };
+  constexpr bool not_styled = false;
+  constexpr bool is_on_nav_bar = true;
+  webserver->RegisterPrerenderedPathHandler("/metrics_prometheus", "Prometheus Metrics", callback,
+                                            not_styled, is_on_nav_bar);
+}
+
 } // namespace kudu
