@@ -21,12 +21,12 @@
 #include <cstdint>
 #include <functional>
 #include <mutex>
+#include <optional>
 #include <set>
 #include <sstream>
 #include <string>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 
@@ -69,6 +69,8 @@
 #include "kudu/server/tracing_path_handlers.h"
 #include "kudu/server/webserver.h"
 #include "kudu/util/atomic.h"
+#include "kudu/util/cloud/instance_detector.h"
+#include "kudu/util/cloud/instance_metadata.h"
 #include "kudu/util/env.h"
 #include "kudu/util/faststring.h"
 #include "kudu/util/file_cache.h"
@@ -76,6 +78,7 @@
 #include "kudu/util/flag_validators.h"
 #include "kudu/util/flags.h"
 #include "kudu/util/jsonwriter.h"
+#include "kudu/util/jwt-util.h"
 #include "kudu/util/logging.h"
 #include "kudu/util/mem_tracker.h"
 #include "kudu/util/metrics.h"
@@ -96,6 +99,10 @@
 #include "kudu/util/user.h"
 #include "kudu/util/version_info.h"
 
+namespace kudu {
+class JwtVerifier;
+}  // namespace kudu
+
 DEFINE_int32(num_reactor_threads, 4, "Number of libev reactor threads to start.");
 TAG_FLAG(num_reactor_threads, advanced);
 
@@ -240,6 +247,60 @@ DEFINE_uint64(server_max_open_files, 0,
               "automatically calculate this value. This is a soft limit");
 TAG_FLAG(server_max_open_files, advanced);
 
+DEFINE_bool(enable_jwt_token_auth, false,
+    "This enables JWT authentication, meaning that the server expects a valid "
+    "JWT to be sent by the client which will be verified when the connection is "
+    "being established. When true, read the JWT token out of the RPC and extract "
+    "user name from the token payload.");
+TAG_FLAG(enable_jwt_token_auth, experimental);
+
+DEFINE_string(jwks_file_path, "",
+    "File path of the pre-installed JSON Web Key Set (JWKS) for JWT verification.");
+TAG_FLAG(jwks_file_path, experimental);
+
+DEFINE_string(jwks_url, "",
+    "URL of the JSON Web Key Set (JWKS) for JWT verification.");
+TAG_FLAG(jwks_url, experimental);
+
+// Enables retrieving the JWKS URL with verifying the presented TLS certificate
+// from the server.
+DEFINE_bool(jwks_verify_server_certificate, true,
+            "Specifies if the TLS certificate of the JWKS server is verified when retrieving "
+            "the JWKS from the specified JWKS URL. A certificate is considered valid if a "
+            "trust chain can be established for it, and the certificate has a common name or "
+            "SAN that matches the server's hostname. This should only be set to false for "
+            "development / testing.");
+TAG_FLAG(jwks_verify_server_certificate, experimental);
+TAG_FLAG(jwks_verify_server_certificate, unsafe);
+
+// The targeted use-case for the wall clock jump detection is spotting sudden
+// swings of the local clock while it is still reported to be synchronized with
+// reference NTP clock.
+DEFINE_string(wall_clock_jump_detection, "auto",
+              "Whether to run a sanity check on wall clock timestamps using "
+              "the readings of the CLOCK_MONOTONIC_RAW clock as the reference. "
+              "Acceptable values for this flag are \"auto\", \"enabled\", and "
+              "\"disabled\". \"auto\" enables the sanity check in environments "
+              "known to be susceptible to such clock jumps (e.g., Azure VMs); "
+              "\"enabled\" unconditionally enables the check; \"disabled\" "
+              "unconditionally disables the check. The threshold for the "
+              "difference between deltas of consecutive timestamps read from "
+              "wall and CLOCK_MONOTONIC_RAW clocks is controlled by the "
+              "--wall_clock_jump_threshold_sec flag.");
+TAG_FLAG(wall_clock_jump_detection, experimental);
+
+// The idea behind having 900 seconds as the default threshold is to have the
+// bar set quite high, but still under 1000 seconds which is the default time
+// delta threshold for ntpd unless '-g' option is added or 'tinker panic 0'
+// or similar directive is present in ntp.conf (ntpd would exit without trying
+// to adjust time if it detects the difference between the reference time and
+// the local clock time to be greater than 1000 seconds, see 'man ntpd').
+DEFINE_uint32(wall_clock_jump_threshold_sec, 15 * 60,
+              "Maximum allowed divergence between the wall and monotonic "
+              "clocks; effective only when the clock jump protection "
+              "is enabled");
+TAG_FLAG(wall_clock_jump_threshold_sec, experimental);
+
 DECLARE_bool(use_hybrid_clock);
 DECLARE_int32(dns_resolver_max_threads_num);
 DECLARE_uint32(dns_resolver_cache_capacity_mb);
@@ -251,6 +312,8 @@ DECLARE_int64(fs_data_dirs_reserved_bytes);
 DECLARE_string(log_filename);
 DECLARE_string(keytab_file);
 DECLARE_string(principal);
+DECLARE_string(time_source);
+DECLARE_string(trusted_certificate_file);
 
 METRIC_DECLARE_gauge_size(merged_entities_count_of_server);
 METRIC_DEFINE_gauge_int64(server, uptime,
@@ -271,6 +334,17 @@ METRIC_DEFINE_gauge_int64(server, data_dirs_space_available_bytes,
                           "-1 if reading any of the disks fails",
                           kudu::MetricLevel::kInfo);
 
+#ifdef TCMALLOC_ENABLED
+METRIC_DEFINE_gauge_int64(server, memory_usage,
+                          "Current Memory Usage",
+                          kudu::MetricUnit::kBytes,
+                          "Current memory usage of the server process",
+                          kudu::MetricLevel::kInfo);
+#endif // #ifdef TCMALLOC_ENABLED
+
+using kudu::cloud::CloudType;
+using kudu::cloud::InstanceDetector;
+using kudu::cloud::InstanceMetadata;
 using kudu::security::RpcAuthentication;
 using kudu::security::RpcEncryption;
 using std::ostringstream;
@@ -294,6 +368,58 @@ namespace server {
 
 namespace {
 
+enum class TriState {
+  AUTO,
+  ENABLED,
+  DISABLED,
+};
+
+// This is a helper function to parse a flag that has three possible values:
+// "auto", "enabled", "disabled".  That directly maps into the TriState enum.
+Status ParseTriStateFlag(const string& name,
+                         const string& value,
+                         TriState* result = nullptr) {
+  if (iequals(value, "auto")) {
+    if (result) {
+      *result = TriState::AUTO;
+    }
+    return Status::OK();
+  }
+  if (iequals(value, "enabled")) {
+    if (result) {
+      *result = TriState::ENABLED;
+    }
+    return Status::OK();
+  }
+  if (iequals(value, "disabled")) {
+    if (result) {
+      *result = TriState::DISABLED;
+    }
+    return Status::OK();
+  }
+  return Status::InvalidArgument(
+      Substitute("$0: invalid value for flag --$1", value, name));
+}
+
+bool ValidateWallClockJumpDetection(const char* name, const string& value) {
+  const auto s = ParseTriStateFlag(name, value);
+  if (s.ok()) {
+    return true;
+  }
+  LOG(ERROR) << s.ToString();
+  return false;
+}
+DEFINE_validator(wall_clock_jump_detection, &ValidateWallClockJumpDetection);
+
+bool ValidateWallClockJumpThreshold(const char* name, uint32_t value) {
+  if (value == 0) {
+    LOG(ERROR) << Substitute("--$0 must be greater than 0", name);
+    return false;
+  }
+  return true;
+}
+DEFINE_validator(wall_clock_jump_threshold_sec, &ValidateWallClockJumpThreshold);
+
 bool ValidateTlsProtocol(const char* /*flagname*/, const string& value) {
   return IsValidTlsProtocolStr(value);
 }
@@ -336,6 +462,27 @@ bool ValidateKeytabPermissions() {
 }
 GROUP_FLAG_VALIDATOR(keytab_permissions, &ValidateKeytabPermissions);
 
+bool ValidateJWKSNotEmpty() {
+  if (FLAGS_enable_jwt_token_auth && (FLAGS_jwks_file_path.empty() && FLAGS_jwks_url.empty())) {
+    LOG(ERROR) << "'jwt_token_auth' is enabled, but 'jwks_filepath' and 'jwks_url' are both empty";
+    return false;
+  }
+
+  return true;
+}
+
+GROUP_FLAG_VALIDATOR(jwk_not_empty_validator, &ValidateJWKSNotEmpty);
+
+bool ValidateEitherJWKSFilePathOrUrlSet() {
+  if (!FLAGS_jwks_url.empty() && !FLAGS_jwks_file_path.empty()) {
+    LOG(ERROR) << "only set either 'jwks_url' or 'jwks_file_path' but not both";
+    return false;
+  }
+  return true;
+}
+
+GROUP_FLAG_VALIDATOR(jwks_file_or_url_set_validator, &ValidateEitherJWKSFilePathOrUrlSet);
+
 } // namespace
 
 static bool ValidateRpcAuthentication(const char* flag_name, const string& flag_value) {
@@ -499,12 +646,12 @@ ServerBase::ServerBase(string name, const ServerBaseOptions& options,
       result_tracker_(new rpc::ResultTracker(shared_ptr<MemTracker>(
           MemTracker::CreateTracker(-1, "result-tracker", mem_tracker_)))),
       is_first_run_(false),
+      stop_background_threads_latch_(1),
       dns_resolver_(new DnsResolver(
           FLAGS_dns_resolver_max_threads_num,
           FLAGS_dns_resolver_cache_capacity_mb * 1024 * 1024,
           MonoDelta::FromSeconds(FLAGS_dns_resolver_cache_ttl_sec))),
-      options_(options),
-      stop_background_threads_latch_(1) {
+      options_(options) {
   metric_entity_->NeverRetire(
       METRIC_merged_entities_count_of_server.InstantiateHidden(metric_entity_, 1));
 
@@ -517,13 +664,6 @@ ServerBase::ServerBase(string name, const ServerBaseOptions& options,
   fs_opts.file_cache = file_cache_.get();
   fs_manager_.reset(new FsManager(options.env, std::move(fs_opts)));
 
-  if (FLAGS_use_hybrid_clock) {
-    clock_.reset(new clock::HybridClock(metric_entity_));
-  } else {
-    clock_.reset(new clock::LogicalClock(Timestamp::kInitialTimestamp,
-                                         metric_entity_));
-  }
-
   if (FLAGS_webserver_enabled) {
     web_server_.reset(new Webserver(options.webserver_opts));
   }
@@ -567,6 +707,25 @@ void ServerBase::GenerateInstanceID() {
 }
 
 Status ServerBase::Init() {
+  if (!FLAGS_use_hybrid_clock) {
+    clock_.reset(new clock::LogicalClock(Timestamp::kInitialTimestamp,
+                                         metric_entity_));
+  } else {
+    uint64_t threshold_usec = 0;
+    unique_ptr<InstanceMetadata> im;
+    RETURN_NOT_OK(WallClockJumpDetectionNeeded(&threshold_usec, &im));
+    if (threshold_usec > 0) {
+      LOG(INFO) << "enabling wall clock jump detection";
+    }
+    clock_.reset(new clock::HybridClock(
+        metric_entity_, threshold_usec, std::move(im)));
+  }
+
+  // Initialize the clock immediately. This checks that the clock is synchronized
+  // so we're less likely to get into a partially initialized state on disk during startup
+  // if we're having clock problems.
+  RETURN_NOT_OK_PREPEND(clock_->Init(), "Cannot initialize clock");
+
   Timer* init = startup_path_handler_->init_progress();
   Timer* read_filesystem = startup_path_handler_->read_filesystem_progress();
   init->Start();
@@ -576,10 +735,6 @@ Status ServerBase::Init() {
 
   InitSpinLockContentionProfiling();
 
-  // Initialize the clock immediately. This checks that the clock is synchronized
-  // so we're less likely to get into a partially initialized state on disk during startup
-  // if we're having clock problems.
-  RETURN_NOT_OK_PREPEND(clock_->Init(), "Cannot initialize clock");
   RETURN_NOT_OK(security::InitKerberosForServer(FLAGS_principal, FLAGS_keytab_file));
   RETURN_NOT_OK(file_cache_->Init());
 
@@ -611,7 +766,14 @@ Status ServerBase::Init() {
   if (s.IsNotFound()) {
     LOG(INFO) << "This appears to be a new deployment of Kudu; creating new FS layout";
     is_first_run_ = true;
-    s = fs_manager_->CreateInitialFileSystemLayout();
+    if (options_.server_key.empty()) {
+      s = fs_manager_->CreateInitialFileSystemLayout();
+    } else {
+      s = fs_manager_->CreateInitialFileSystemLayout(std::nullopt,
+                                                     options_.server_key,
+                                                     options_.server_key_iv,
+                                                     options_.server_key_version);
+    }
     if (s.IsAlreadyPresent()) {
       return s.CloneAndPrepend("FS layout already exists; not overwriting existing layout");
     }
@@ -629,6 +791,21 @@ Status ServerBase::Init() {
 
   // Create the Messenger.
   rpc::MessengerBuilder builder(name_);
+  shared_ptr<JwtVerifier> jwt_verifier = nullptr;
+  if (FLAGS_enable_jwt_token_auth) {
+    if (!FLAGS_jwks_url.empty()) {
+      jwt_verifier =
+          std::make_shared<KeyBasedJwtVerifier>(FLAGS_jwks_url,
+                                                FLAGS_jwks_verify_server_certificate,
+                                                FLAGS_trusted_certificate_file);
+    } else if (!FLAGS_jwks_file_path.empty()) {
+      jwt_verifier =
+          std::make_shared<KeyBasedJwtVerifier>(FLAGS_jwks_file_path);
+    } else {
+      LOG(WARNING) << Substitute("JWT authentication enabled, but neither 'jwks_url' or "
+          "'jwks_file_path' are set!");
+    }
+  }
   builder.set_num_reactors(FLAGS_num_reactor_threads)
          .set_min_negotiation_threads(FLAGS_min_negotiation_threads)
          .set_max_negotiation_threads(FLAGS_max_negotiation_threads)
@@ -644,6 +821,7 @@ Status ServerBase::Init() {
          .set_epki_cert_key_files(FLAGS_rpc_certificate_file, FLAGS_rpc_private_key_file)
          .set_epki_certificate_authority_file(FLAGS_rpc_ca_certificate_file)
          .set_epki_private_password_key_cmd(FLAGS_rpc_private_key_password_cmd)
+         .set_jwt_verifier(std::move(jwt_verifier))
          .set_keytab_file(FLAGS_keytab_file)
          .enable_inbound_tls();
 
@@ -682,9 +860,58 @@ Status ServerBase::Init() {
   return rpc_server_->Bind();
 }
 
+Status ServerBase::WallClockJumpDetectionNeeded(
+    uint64_t* threshold_usec, unique_ptr<InstanceMetadata>* im) {
+  DCHECK(threshold_usec);
+  DCHECK(im);
+
+  TriState st = TriState::AUTO;
+  RETURN_NOT_OK(ParseTriStateFlag(
+      "wall_clock_jump_detection", FLAGS_wall_clock_jump_detection, &st));
+  switch (st) {
+    case TriState::DISABLED:
+      *threshold_usec = 0;
+      return Status::OK();
+    case TriState::ENABLED:
+      *threshold_usec = FLAGS_wall_clock_jump_threshold_sec * 1000UL * 1000UL;
+      return Status::OK();
+    default:
+      break;
+  }
+  DCHECK(st == TriState::AUTO);
+
+  InstanceDetector detector;
+  unique_ptr<InstanceMetadata> metadata;
+  if (const auto s = detector.Detect(&metadata); !s.ok()) {
+    LOG(INFO) << Substitute("$0: unable to detect cloud type of this node, "
+                            "probably running in non-cloud environment", s.ToString());
+    *threshold_usec = 0;
+    return Status::OK();
+  }
+  LOG(INFO) << Substitute("running on $0 node",
+                          cloud::TypeToString(metadata->type()));
+
+  // Enable an extra check to detect clock jumps on Azure VMs: those seem
+  // to be prone to clock jumps that aren't reflected in NTP clock
+  // synchronization status reported by ntp_adjtime()/ntp_gettime().
+  // Perhaps, that's due to the VMICTimeSync service interfering with the
+  // kernel NTP discipline after a 'memory preserving' maintenance: when a
+  // VM is 'unfrozen' after the update, the VMICTimeSync service updates the
+  // VM's clock to compensate for the pause (see [1]).
+  // [1] https://learn.microsoft.com/en-us/azure/virtual-machines/linux/time-sync
+  if (metadata->type() == CloudType::AZURE) {
+    *threshold_usec = FLAGS_wall_clock_jump_threshold_sec * 1000UL * 1000UL;
+  } else {
+    *threshold_usec = 0;
+  }
+  *im = std::move(metadata);
+
+  return Status::OK();
+}
+
 Status ServerBase::InitAcls() {
   string service_user;
-  boost::optional<string> keytab_user = security::GetLoggedInUsernameFromKeytab();
+  std::optional<string> keytab_user = security::GetLoggedInUsernameFromKeytab();
   if (keytab_user) {
     // If we're logged in from a keytab, then everyone should be, and we expect them
     // to use the same mapped username.
@@ -932,6 +1159,14 @@ Status ServerBase::Start() {
       metric_entity_,
       [this]() {return (MonoTime::Now() - this->start_time()).ToMicroseconds();})->
           AutoDetachToLastValue(&metric_detacher_);
+
+#ifdef TCMALLOC_ENABLED
+  METRIC_memory_usage.InstantiateFunctionGauge(
+      metric_entity_,
+      []() {return process_memory::CurrentConsumption();})->
+          AutoDetachToLastValue(&metric_detacher_);
+#endif // #ifdef TCMALLOC_ENABLED
+
   METRIC_data_dirs_space_available_bytes.InstantiateFunctionGauge(
       metric_entity_,
       [this]() {
@@ -962,6 +1197,7 @@ Status ServerBase::Start() {
     AddPostInitializedDefaultPathHandlers(web_server_.get());
     AddRpczPathHandlers(messenger_, web_server_.get());
     RegisterMetricsJsonHandler(web_server_.get(), metric_registry_.get());
+    RegisterMetricsPrometheusHandler(web_server_.get(), metric_registry_.get());
     TracingPathHandlers::RegisterHandlers(web_server_.get());
     web_server_->set_footer_html(FooterHtml());
     web_server_->SetStartupComplete(true);
