@@ -21,13 +21,14 @@
 #include <cstddef>
 #include <cstdint>
 #include <memory>
+#include <optional>
 #include <ostream>
 #include <string>
 #include <tuple>
+#include <type_traits>
 #include <unordered_set>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 #include <glog/stl_logging.h>
@@ -54,18 +55,16 @@
 #include "kudu/tablet/delta_key.h"
 #include "kudu/tablet/delta_store.h"
 #include "kudu/tablet/delta_tracker.h"
-#include "kudu/tablet/deltamemstore.h"
 #include "kudu/tablet/diskrowset-test-base.h"
 #include "kudu/tablet/mvcc.h"
 #include "kudu/tablet/rowset.h"
-#include "kudu/tablet/rowset_metadata.h"
 #include "kudu/tablet/tablet-test-util.h"
-#include "kudu/tablet/tablet.h"
 #include "kudu/tablet/tablet.pb.h"
 #include "kudu/tablet/tablet_mem_trackers.h"
 #include "kudu/util/bloom_filter.h"
 #include "kudu/util/faststring.h"
 #include "kudu/util/memory/arena.h"
+#include "kudu/util/monotime.h"
 #include "kudu/util/random.h"
 #include "kudu/util/slice.h"
 #include "kudu/util/status.h"
@@ -73,11 +72,19 @@
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
 
+namespace kudu {
+namespace tablet {
+class RowSetMetadata;
+}  // namespace tablet
+}  // namespace kudu
+
 DEFINE_double(update_fraction, 0.1f, "fraction of rows to update");
 DECLARE_bool(cfile_lazy_open);
 DECLARE_bool(crash_on_eio);
 DECLARE_int32(cfile_default_block_size);
 DECLARE_double(env_inject_eio);
+DECLARE_int32(tablet_history_max_age_sec);
+DECLARE_bool(rowset_metadata_store_keys);
 DECLARE_double(tablet_delta_store_major_compact_min_ratio);
 DECLARE_int32(tablet_delta_store_minor_compact_max);
 
@@ -212,7 +219,7 @@ TEST_F(TestRowSet, TestRowSetUpdate) {
   OperationResultPB result;
   ProbeStats stats;
   Status s = rs->MutateRow(timestamp, probe, enc.as_changelist(), op_id_, nullptr, &stats, &result);
-  ASSERT_TRUE(s.IsNotFound());
+  ASSERT_TRUE(s.IsNotFound()) << s.ToString();
   ASSERT_EQ(0, result.mutated_stores_size());
 
   // Now read back the value column, and verify that the updates
@@ -247,7 +254,7 @@ TEST_F(TestRowSet, TestErrorDuringUpdate) {
   ProbeStats stats;
   Status s = rs->MutateRow(timestamp, probe, enc.as_changelist(), op_id_, nullptr, &stats, &result);
   LOG(INFO) << s.ToString();
-  ASSERT_TRUE(s.IsIOError());
+  ASSERT_TRUE(s.IsIOError()) << s.ToString();
 }
 
 TEST_F(TestRowSet, TestRandomRead) {
@@ -539,18 +546,18 @@ TEST_F(TestRowSet, TestMakeDeltaIteratorMergerUnlocked) {
   ASSERT_OK(OpenTestRowSet(&rs));
   UpdateExistingRows(rs.get(), FLAGS_update_fraction, nullptr);
   ASSERT_OK(rs->FlushDeltas(nullptr));
-  DeltaTracker *dt = rs->delta_tracker();
-  int num_stores = dt->redo_delta_stores_.size();
+  const DeltaTracker& dt = rs->delta_tracker();
+  size_t num_stores = dt.redo_delta_stores_.size();
+  ASSERT_GT(num_stores, 0);
   vector<shared_ptr<DeltaStore> > compacted_stores;
   vector<BlockId> compacted_blocks;
   unique_ptr<DeltaIterator> merge_iter;
-  ASSERT_OK(dt->MakeDeltaIteratorMergerUnlocked(nullptr, 0, num_stores - 1, &schema_,
-                                                &compacted_stores,
-                                                &compacted_blocks, &merge_iter));
+  ASSERT_OK(dt.MakeDeltaIteratorMergerUnlocked(nullptr, 0, num_stores - 1, &schema_,
+                                               &compacted_stores,
+                                               &compacted_blocks, &merge_iter));
   vector<string> results;
   ASSERT_OK(DebugDumpDeltaIterator(REDO, merge_iter.get(), schema_,
-                                          ITERATE_OVER_ALL_ROWS,
-                                          &results));
+                                   ITERATE_OVER_ALL_ROWS, &results));
   for (const string &str : results) {
     VLOG(1) << str;
   }
@@ -607,10 +614,10 @@ TEST_F(TestRowSet, TestCompactStores) {
       RowSet::MAJOR_DELTA_COMPACTION)));
 
   // Compact the deltafiles
-  DeltaTracker *dt = rs->delta_tracker();
+  auto* dt = rs->mutable_delta_tracker();
   int num_stores = dt->redo_delta_stores_.size();
   VLOG(1) << "Number of stores before compaction: " << num_stores;
-  ASSERT_EQ(num_stores, 3);
+  ASSERT_EQ(3, num_stores);
   ASSERT_OK(dt->CompactStores(nullptr, 0, num_stores - 1));
   num_stores = dt->redo_delta_stores_.size();
   VLOG(1) << "Number of stores after compaction: " << num_stores;
@@ -637,6 +644,57 @@ TEST_F(TestRowSet, TestCompactStores) {
   ASSERT_TRUE(is_sorted(results.begin(), results.end()));
 }
 
+TEST_F(TestRowSet, TestGCScheduleRedoFilesWithFullOfDeleteOP) {
+  // Make major delta compaction runnable even with tiny amount of data accumulated
+  // across rowset's deltas.
+  FLAGS_tablet_delta_store_major_compact_min_ratio = 0.0001;
+  // Write the min/max keys to the rowset metadata. In this way, we can simplify the
+  // test scenario by focusing on the REDO delta store files.
+  FLAGS_rowset_metadata_store_keys = true;
+
+  WriteTestRowSet();
+  {
+    shared_ptr<DiskRowSet> rs;
+    ASSERT_OK(OpenTestRowSet(&rs));
+
+    // Generate base data.
+    ASSERT_OK(rs->FlushDeltas(nullptr));
+  }
+
+  // Reopen the rowset.
+  {
+    shared_ptr<DiskRowSet> rs;
+    ASSERT_OK(OpenTestRowSet(&rs));
+    const DeltaTracker& dt = rs->delta_tracker();
+    ASSERT_EQ(0, dt.CountUndoDeltaStores());
+    ASSERT_EQ(0, dt.CountRedoDeltaStores());
+
+    int loop_cnt = 10;
+    int loop_per_idx = FLAGS_roundtrip_num_rows / loop_cnt;
+    // Generate delta files with DELETE operations.
+    for (int i = 0; i < loop_cnt; i++) {
+      NO_FATALS(DeleteExistingRows(rs.get(), loop_per_idx * i, loop_per_idx * (i + 1), nullptr));
+      ASSERT_OK(rs->FlushDeltas(nullptr));
+      ASSERT_EQ(0, dt.CountUndoDeltaStores());
+      ASSERT_EQ(i + 1, dt.CountRedoDeltaStores());
+    }
+
+    ASSERT_EQ(0, rs->DeltaStoresCompactionPerfImprovementScore(RowSet::MAJOR_DELTA_COMPACTION));
+
+    // Modify the threshold so we can trigger a new round of gc scheduling.
+    FLAGS_tablet_history_max_age_sec = 1;
+    SleepFor(MonoDelta::FromSeconds(2));
+
+    // Major delta compaction of the DRS should obtain a non-zero score.
+    NO_FATALS(BetweenZeroAndOne(rs->DeltaStoresCompactionPerfImprovementScore(
+              RowSet::MAJOR_DELTA_COMPACTION)));
+
+    ASSERT_OK(rs->MajorCompactDeltaStores(nullptr, HistoryGcOpts::Disabled()));
+    ASSERT_EQ(0, dt.CountUndoDeltaStores());
+    ASSERT_EQ(1, dt.CountRedoDeltaStores());
+  }
+}
+
 TEST_F(TestRowSet, TestGCAncientStores) {
   // Disable lazy open so that major delta compactions don't require manual REDO initialization.
   FLAGS_cfile_lazy_open = false;
@@ -646,7 +704,7 @@ TEST_F(TestRowSet, TestGCAncientStores) {
   WriteTestRowSet();
   shared_ptr<DiskRowSet> rs;
   ASSERT_OK(OpenTestRowSet(&rs));
-  DeltaTracker *dt = rs->delta_tracker();
+  DeltaTracker* dt = rs->mutable_delta_tracker();
   ASSERT_EQ(0, dt->CountUndoDeltaStores());
   ASSERT_EQ(0, dt->CountRedoDeltaStores());
 
@@ -710,6 +768,70 @@ TEST_F(TestRowSet, TestDiskSizeEstimation) {
   ASSERT_GT(drss.redo_deltas_size, 0);
 }
 
+TEST_F(TestRowSet, TestGCDRSWithoutLiveRowCount) {
+  // Write a rowset.
+  WriteTestRowSet();
+  shared_ptr<DiskRowSet> rs;
+
+  ASSERT_OK(OpenTestRowSet(&rs));
+  uint64_t rs_live_rows;
+  // The rowset's live row count should be equal when there are no live row count
+  // stats available if no DMS exists.
+  ASSERT_OK(rs->CountLiveRows(&rs_live_rows));
+  ASSERT_EQ(rs_live_rows, FLAGS_roundtrip_num_rows);
+  uint64_t rs_live_rows_without_lrc;
+  CHECK_OK(rs->CountLiveRowsWithoutLiveRowCountStats(&rs_live_rows_without_lrc));
+  ASSERT_EQ(rs_live_rows_without_lrc, rs_live_rows);
+
+  // Write a delta file.
+  UpdateExistingRows(rs.get(), static_cast<float>(FLAGS_update_fraction), nullptr);
+  ASSERT_OK(rs->FlushDeltas(nullptr));
+
+  // The live row count is 0 if the DRS has been fully deleted if no DMS exists.
+  NO_FATALS(DeleteExistingRows(rs.get(), 0, FLAGS_roundtrip_num_rows, nullptr));
+  ASSERT_OK(rs->FlushDeltas(nullptr));
+  CHECK_OK(rs->CountLiveRows(&rs_live_rows));
+  ASSERT_EQ(rs_live_rows, 0);
+  CHECK_OK(rs->CountLiveRowsWithoutLiveRowCountStats(&rs_live_rows_without_lrc));
+  ASSERT_EQ(rs_live_rows_without_lrc, 0);
+}
+
+TEST_F(TestRowSet, TestCountLiveRowsWithoutLiveRowCountStats) {
+  // Write a rowset.
+  WriteTestRowSet();
+  shared_ptr<DiskRowSet> rs;
+
+  ASSERT_OK(OpenTestRowSet(&rs));
+  uint64_t rs_live_rows;
+  // The rowset's live row count from different way should be equal if if no
+  // DMS exists.
+  CHECK_OK(rs->CountLiveRows(&rs_live_rows));
+  ASSERT_EQ(rs_live_rows, FLAGS_roundtrip_num_rows);
+  uint64_t rs_live_rows_without_lrc;
+  CHECK_OK(rs->CountLiveRowsWithoutLiveRowCountStats(&rs_live_rows_without_lrc));
+  ASSERT_EQ(rs_live_rows_without_lrc, rs_live_rows);
+
+  // Write a delta file.
+  UpdateExistingRows(rs.get(), static_cast<float>(FLAGS_update_fraction), nullptr);
+  ASSERT_OK(rs->FlushDeltas(nullptr));
+
+  // The delta file is OK to deal with without a DMS.
+  CHECK_OK(rs->CountLiveRows(&rs_live_rows));
+  ASSERT_EQ(rs_live_rows, FLAGS_roundtrip_num_rows);
+  CHECK_OK(rs->CountLiveRowsWithoutLiveRowCountStats(&rs_live_rows_without_lrc));
+  ASSERT_EQ(rs_live_rows_without_lrc, rs_live_rows);
+
+  // Get a new DMS.
+  UpdateExistingRows(rs.get(), static_cast<float>(FLAGS_update_fraction), nullptr);
+
+  // The 'CountLiveRowsWithoutLiveRowCountStats' may get a smaller value than
+  // 'CountLiveRows' if a DMS exists.
+  CHECK_OK(rs->CountLiveRows(&rs_live_rows));
+  CHECK_OK(rs->CountLiveRowsWithoutLiveRowCountStats(&rs_live_rows_without_lrc));
+  ASSERT_LE(rs_live_rows_without_lrc, rs_live_rows);
+  ASSERT_GE(rs_live_rows_without_lrc, FLAGS_roundtrip_num_rows);
+}
+
 class DiffScanRowSetTest : public KuduRowSetTest,
                            public ::testing::WithParamInterface<tuple<bool, bool>> {
  public:
@@ -787,6 +909,7 @@ TEST_P(DiffScanRowSetTest, TestFuzz) {
     if (add_vc_is_deleted) {
       bool read_default = false;
       col_schemas.emplace_back("is_deleted", IS_DELETED, /*is_nullable=*/ false,
+                               /*is_immutable=*/ false, /*is_auto_incrementing=*/ false,
                                &read_default);
       col_ids.emplace_back(schema_.max_col_id() + 1);
     }
@@ -837,14 +960,14 @@ TEST_P(DiffScanRowSetTest, TestFuzz) {
   // 'col_idx'. If 'val' is unset, deletes the row.
   auto mutate_row = [&](rowid_t row_idx,
                         size_t col_idx,
-                        boost::optional<uint32_t> val) {
+                        std::optional<uint32_t> val) {
     // Build the mutation.
     faststring buf;
     RowChangeListEncoder enc(&buf);
     if (val) {
       enc.AddColumnUpdate(schema_.column(col_idx),
                           schema_.column_id(col_idx),
-                          val.get_ptr());
+                          &(*val));
     } else {
       enc.SetToDelete();
     }
@@ -893,7 +1016,7 @@ TEST_P(DiffScanRowSetTest, TestFuzz) {
   NO_FATALS(maybe_flush_compact());
   NO_FATALS(mutate_row(3, 1, 400)); // ts 4
   NO_FATALS(maybe_flush_compact());
-  NO_FATALS(mutate_row(3, 1, boost::none)); // ts 5
+  NO_FATALS(mutate_row(3, 1, std::nullopt)); // ts 5
   NO_FATALS(maybe_flush_compact());
 
   // Run the diff scan test on every permutation of time bounds.
