@@ -19,10 +19,10 @@
 
 #include <algorithm>
 #include <map>
+#include <optional>
 #include <ostream>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 #include <glog/stl_logging.h>
@@ -39,6 +39,7 @@
 #include "kudu/common/types.h"
 #include "kudu/fs/block_manager.h"
 #include "kudu/fs/fs_manager.h"
+#include "kudu/fs/io_context.h"
 #include "kudu/gutil/port.h"
 #include "kudu/tablet/cfile_set.h"
 #include "kudu/tablet/compaction.h"
@@ -50,6 +51,7 @@
 #include "kudu/tablet/multi_column_writer.h"
 #include "kudu/tablet/mutation.h"
 #include "kudu/tablet/mvcc.h"
+#include "kudu/tablet/rowset_metadata.h"
 #include "kudu/util/compression/compression.pb.h"
 #include "kudu/util/debug/trace_event.h"
 #include "kudu/util/flag_tags.h"
@@ -79,6 +81,19 @@ DEFINE_bool(rowset_metadata_store_keys, false,
             "metadata. If false, keys will be read from the data blocks.");
 TAG_FLAG(rowset_metadata_store_keys, experimental);
 
+using kudu::cfile::BloomFileWriter;
+using kudu::fs::BlockManager;
+using kudu::fs::BlockCreationTransaction;
+using kudu::fs::CreateBlockOptions;
+using kudu::fs::IOContext;
+using kudu::fs::WritableBlock;
+using kudu::log::LogAnchorRegistry;
+using std::optional;
+using std::shared_ptr;
+using std::string;
+using std::unique_ptr;
+using std::vector;
+
 namespace kudu {
 
 class Mutex;
@@ -89,18 +104,6 @@ class OpId;
 
 namespace tablet {
 
-using cfile::BloomFileWriter;
-using fs::BlockManager;
-using fs::BlockCreationTransaction;
-using fs::CreateBlockOptions;
-using fs::IOContext;
-using fs::WritableBlock;
-using log::LogAnchorRegistry;
-using std::shared_ptr;
-using std::string;
-using std::unique_ptr;
-using std::vector;
-
 const char *DiskRowSet::kMinKeyMetaEntryName = "min_key";
 const char *DiskRowSet::kMaxKeyMetaEntryName = "max_key";
 
@@ -557,7 +560,7 @@ Status DiskRowSet::MinorCompactDeltaStores(const IOContext* io_context) {
 Status DiskRowSet::MajorCompactDeltaStores(const IOContext* io_context,
                                            HistoryGcOpts history_gc_opts) {
   vector<ColumnId> col_ids;
-  delta_tracker_->GetColumnIdsWithUpdates(&col_ids);
+  delta_tracker_->GetColumnIdsToCompact(&col_ids);
 
   if (col_ids.empty()) {
     VLOG_WITH_PREFIX(2) << "There are no column ids with updates";
@@ -572,8 +575,8 @@ Status DiskRowSet::MajorCompactDeltaStoresWithColumnIds(const vector<ColumnId>&
                                                         HistoryGcOpts history_gc_opts) {
   VLOG_WITH_PREFIX(1) << "Major compacting REDO delta stores (cols: " << col_ids << ")";
   TRACE_EVENT0("tablet", "DiskRowSet::MajorCompactDeltaStoresWithColumnIds");
-  std::lock_guard<Mutex> l(*delta_tracker()->compact_flush_lock());
-  RETURN_NOT_OK(delta_tracker()->CheckWritableUnlocked());
+  std::lock_guard<Mutex> l(*mutable_delta_tracker()->compact_flush_lock());
+  RETURN_NOT_OK(mutable_delta_tracker()->CheckWritableUnlocked());
 
   // TODO(todd): do we need to lock schema or anything here?
   unique_ptr<MajorDeltaCompaction> compaction;
@@ -630,10 +633,10 @@ Status DiskRowSet::NewMajorDeltaCompaction(const vector<ColumnId>& col_ids,
   DCHECK(open_);
   shared_lock<rw_spinlock> l(component_lock_);
 
-  const Schema* schema = &rowset_metadata_->tablet_schema();
+  const SchemaPtr schema_ptr = rowset_metadata_->tablet_schema();
 
   RowIteratorOptions opts;
-  opts.projection = schema;
+  opts.projection = schema_ptr.get();
   opts.io_context = io_context;
   vector<shared_ptr<DeltaStore>> included_stores;
   unique_ptr<DeltaIterator> delta_iter;
@@ -641,7 +644,7 @@ Status DiskRowSet::NewMajorDeltaCompaction(const vector<ColumnId>& col_ids,
       opts, REDO, &included_stores, &delta_iter));
 
   out->reset(new MajorDeltaCompaction(rowset_metadata_->fs_manager(),
-                                      *schema,
+                                      *schema_ptr,
                                       base_data_.get(),
                                       std::move(delta_iter),
                                       std::move(included_stores),
@@ -686,9 +689,9 @@ Status DiskRowSet::MutateRow(Timestamp timestamp,
 #endif
   shared_lock<rw_spinlock> l(component_lock_);
 
-  boost::optional<rowid_t> row_idx;
+  optional<rowid_t> row_idx;
   RETURN_NOT_OK(base_data_->FindRow(probe, io_context, &row_idx, stats));
-  if (PREDICT_FALSE(row_idx == boost::none)) {
+  if (PREDICT_FALSE(!row_idx)) {
     return Status::NotFound("row not found");
   }
 #ifndef NDEBUG
@@ -703,9 +706,7 @@ Status DiskRowSet::MutateRow(Timestamp timestamp,
     return Status::NotFound("row not found");
   }
 
-  RETURN_NOT_OK(delta_tracker_->Update(timestamp, *row_idx, update, op_id, result));
-
-  return Status::OK();
+  return delta_tracker_->Update(timestamp, *row_idx, update, op_id, result);
 }
 
 Status DiskRowSet::CheckRowPresent(const RowSetKeyProbe &probe,
@@ -749,6 +750,17 @@ Status DiskRowSet::CountRows(const IOContext* io_context, rowid_t *count) const
   return Status::OK();
 }
 
+Status DiskRowSet::CountLiveRowsWithoutLiveRowCountStats(uint64_t* count) const {
+  IOContext io_context({ rowset_metadata_->tablet_metadata()->tablet_id() });
+  rowid_t rows_count;
+  // TODO(kedeng) : get operation count in DMS.
+  RETURN_NOT_OK(CountRows(&io_context, &rows_count));
+  int64_t deleted_count = delta_tracker_->CountDeletedRowsInRedos();
+  DCHECK_GE(rows_count, deleted_count);
+  *count = rows_count - deleted_count;
+  return Status::OK();
+}
+
 Status DiskRowSet::CountLiveRows(uint64_t* count) const {
   DCHECK_GE(rowset_metadata_->live_row_count(), delta_tracker_->CountDeletedRows());
   *count = rowset_metadata_->live_row_count() - delta_tracker_->CountDeletedRows();
@@ -846,10 +858,9 @@ double DiskRowSet::DeltaStoresCompactionPerfImprovementScore(DeltaCompactionType
   }
 
   if (type == RowSet::MAJOR_DELTA_COMPACTION) {
-    vector<ColumnId> col_ids_with_updates;
-    delta_tracker_->GetColumnIdsWithUpdates(&col_ids_with_updates);
-    // If we have files but no updates, we don't want to major compact.
-    if (!col_ids_with_updates.empty()) {
+    vector<ColumnId> col_ids_for_compact;
+    delta_tracker_->GetColumnIdsToCompact(&col_ids_for_compact);
+    if (!col_ids_for_compact.empty()) {
       DiskRowSetSpace drss;
       GetDiskRowSetSpaceUsage(&drss);
       double ratio = static_cast<double>(drss.redo_deltas_size) / drss.base_data_size;
@@ -867,15 +878,23 @@ double DiskRowSet::DeltaStoresCompactionPerfImprovementScore(DeltaCompactionType
   return std::min(1.0, perf_improv);
 }
 
-Status DiskRowSet::EstimateBytesInPotentiallyAncientUndoDeltas(Timestamp ancient_history_mark,
-                                                               int64_t* bytes) {
-  return delta_tracker_->EstimateBytesInPotentiallyAncientUndoDeltas(ancient_history_mark, bytes);
+Status DiskRowSet::EstimateBytesInPotentiallyAncientUndoDeltas(
+    Timestamp ancient_history_mark,
+    EstimateType estimate_type,
+    int64_t* bytes) {
+  return delta_tracker_->EstimateBytesInPotentiallyAncientUndoDeltas(
+      ancient_history_mark, estimate_type, bytes);
 }
 
 Status DiskRowSet::IsDeletedAndFullyAncient(Timestamp ancient_history_mark,
                                             bool* deleted_and_ancient) {
   uint64_t live_row_count = 0;
-  RETURN_NOT_OK(CountLiveRows(&live_row_count));
+  if (rowset_metadata_->tablet_metadata()->supports_live_row_count()) {
+    RETURN_NOT_OK(CountLiveRows(&live_row_count));
+  } else {
+    RETURN_NOT_OK(CountLiveRowsWithoutLiveRowCountStats(&live_row_count));
+  }
+
   if (live_row_count > 0) {
     *deleted_and_ancient = false;
     return Status::OK();
@@ -906,14 +925,14 @@ Status DiskRowSet::DeleteAncientUndoDeltas(Timestamp ancient_history_mark,
                                                  blocks_deleted, bytes_deleted);
 }
 
-Status DiskRowSet::DebugDump(vector<string> *lines) {
+Status DiskRowSet::DebugDumpImpl(int64_t* rows_left, vector<string>* lines) {
   // Using CompactionInput to dump our data is an easy way of seeing all the
   // rows and deltas.
   unique_ptr<CompactionInput> input;
-  RETURN_NOT_OK(NewCompactionInput(&rowset_metadata_->tablet_schema(),
+  RETURN_NOT_OK(NewCompactionInput(rowset_metadata_->tablet_schema().get(),
                                    MvccSnapshot::CreateSnapshotIncludingAllOps(),
                                    nullptr, &input));
-  return DebugDumpCompactionInput(input.get(), lines);
+  return DebugDumpCompactionInput(input.get(), rows_left, lines);
 }
 
 } // namespace tablet
