@@ -26,6 +26,7 @@
 #include <memory>
 #include <mutex>
 #include <string>
+#include <type_traits>
 #include <vector>
 
 #include <glog/logging.h>
@@ -40,7 +41,6 @@
 #include "kudu/tablet/delta_key.h"
 #include "kudu/tablet/delta_tracker.h"
 #include "kudu/tablet/rowset.h"
-#include "kudu/tablet/rowset_metadata.h"
 #include "kudu/tablet/tablet_mem_trackers.h"
 #include "kudu/tablet/tablet_metadata.h"
 #include "kudu/util/bloom_filter.h"
@@ -57,6 +57,10 @@ class RowChangeList;
 class RowwiseIterator;
 class Timestamp;
 
+namespace tablet {
+class RowSetMetadata;
+}  // namespace tablet
+
 namespace cfile {
 class BloomFileWriter;
 class CFileWriter;
@@ -75,6 +79,13 @@ namespace log {
 class LogAnchorRegistry;
 }
 
+namespace tools {
+Status DumpRowSetInternal(const fs::IOContext& ctx,
+                          const std::shared_ptr<tablet::RowSetMetadata>& rs_meta,
+                          int indent,
+                          int64_t* rows_left);
+}
+
 namespace tablet {
 
 class CFileSet;
@@ -369,14 +380,18 @@ class DiskRowSet :
 
   // Gets the number of rows in this rowset, checking 'num_rows_' first. If not
   // yet set, consults the base data and stores the result in 'num_rows_'.
-  Status CountRows(const fs::IOContext* io_context, rowid_t *count) const final override;
+  Status CountRows(const fs::IOContext* io_context, rowid_t *count) const final;
 
   // Count the number of live rows in this DRS.
-  virtual Status CountLiveRows(uint64_t* count) const override;
+  Status CountLiveRows(uint64_t* count) const override;
+  // Because possible operations in DMS and will be ignored, mainly because there is no API
+  // available in the old version(earlier than 1.10) of kudu to obtain this data, this API
+  // can only obtain an approximate live row count.
+  Status CountLiveRowsWithoutLiveRowCountStats(uint64_t* count) const;
 
   // See RowSet::GetBounds(...)
-  virtual Status GetBounds(std::string* min_encoded_key,
-                           std::string* max_encoded_key) const override;
+  Status GetBounds(std::string* min_encoded_key,
+                   std::string* max_encoded_key) const override;
 
   void GetDiskRowSetSpaceUsage(DiskRowSetSpace* drss) const;
 
@@ -400,8 +415,10 @@ class DiskRowSet :
 
   double DeltaStoresCompactionPerfImprovementScore(DeltaCompactionType type) const override;
 
-  Status EstimateBytesInPotentiallyAncientUndoDeltas(Timestamp ancient_history_mark,
-                                                     int64_t* bytes) override;
+  Status EstimateBytesInPotentiallyAncientUndoDeltas(
+      Timestamp ancient_history_mark,
+      EstimateType estimate_type,
+      int64_t* bytes) override;
 
   Status IsDeletedAndFullyAncient(Timestamp ancient_history_mark,
                                   bool* deleted_and_ancient) override;
@@ -430,8 +447,14 @@ class DiskRowSet :
     has_been_compacted_.store(true);
   }
 
-  DeltaTracker *delta_tracker() {
-    return DCHECK_NOTNULL(delta_tracker_.get());
+  DeltaTracker* mutable_delta_tracker() {
+    DCHECK(delta_tracker_);
+    return delta_tracker_.get();
+  }
+
+  const DeltaTracker& delta_tracker() const {
+    DCHECK(delta_tracker_);
+    return *delta_tracker_;
   }
 
   std::shared_ptr<RowSetMetadata> metadata() override {
@@ -449,8 +472,6 @@ class DiskRowSet :
         ToString());
   }
 
-  virtual Status DebugDump(std::vector<std::string> *lines) override;
-
  protected:
   DiskRowSet(std::shared_ptr<RowSetMetadata> rowset_metadata,
              log::LogAnchorRegistry* log_anchor_registry,
@@ -464,6 +485,11 @@ class DiskRowSet :
 
   friend class CompactionInput;
   friend class Tablet;
+  friend Status kudu::tools::DumpRowSetInternal(
+      const fs::IOContext& ctx,
+      const std::shared_ptr<tablet::RowSetMetadata>& rs_meta,
+      int indent,
+      int64_t* rows_left);
 
   Status Open(const fs::IOContext* io_context);
 
@@ -478,6 +504,8 @@ class DiskRowSet :
                                               const fs::IOContext* io_context,
                                               HistoryGcOpts history_gc_opts);
 
+  Status DebugDumpImpl(int64_t* rows_left, std::vector<std::string>* lines) override;
+
   std::shared_ptr<RowSetMetadata> rowset_metadata_;
 
   bool open_;
