@@ -21,6 +21,7 @@
 #include <memory>
 #include <mutex>
 #include <ostream>
+#include <type_traits>
 #include <utility>
 
 #include <glog/logging.h>
@@ -39,12 +40,14 @@
 #include "kudu/rpc/result_tracker.h"
 #include "kudu/rpc/rpc_header.pb.h"
 #include "kudu/tablet/mvcc.h"
-#include "kudu/tablet/tablet.h"
-#include "kudu/tablet/tablet_replica.h"
 #include "kudu/tablet/op_order_verifier.h"
 #include "kudu/tablet/ops/op_tracker.h"
+#include "kudu/tablet/tablet.h"
+#include "kudu/tablet/tablet_metrics.h"
+#include "kudu/tablet/tablet_replica.h"
 #include "kudu/util/debug/trace_event.h"
 #include "kudu/util/logging.h"
+#include "kudu/util/metrics.h"
 #include "kudu/util/pb_util.h"
 #include "kudu/util/status_callback.h"
 #include "kudu/util/threadpool.h"
@@ -106,7 +109,8 @@ OpDriver::OpDriver(OpTracker *op_tracker,
                    Log* log,
                    ThreadPoolToken* prepare_pool_token,
                    ThreadPool* apply_pool,
-                   OpOrderVerifier* order_verifier)
+                   OpOrderVerifier* order_verifier,
+                   MonoTime deadline)
     : op_tracker_(op_tracker),
       consensus_(consensus),
       log_(log),
@@ -114,6 +118,7 @@ OpDriver::OpDriver(OpTracker *op_tracker,
       apply_pool_(apply_pool),
       order_verifier_(order_verifier),
       trace_(new Trace()),
+      deadline_(deadline),
       start_time_(MonoTime::Now()),
       replication_state_(NOT_REPLICATING),
       prepare_state_(NOT_PREPARED) {
@@ -232,9 +237,18 @@ void OpDriver::ExecuteAsync() {
 
 void OpDriver::PrepareTask() {
   TRACE_EVENT_FLOW_END0("op", "PrepareTask", this);
-  Status prepare_status = Prepare();
+  if (PREDICT_FALSE(deadline_ <= MonoTime::Now())) {
+    static const Status kTimedOut = Status::TimedOut(
+        "operation timed out while waiting in prepare queue");
+    if (auto* m = op_->state()->tablet_replica()->tablet()->metrics();
+        PREDICT_TRUE(m)) {
+      m->ops_timed_out_in_prepare_queue->Increment();
+    }
+    return HandleFailure(kTimedOut);
+  }
+  const auto prepare_status = Prepare();
   if (PREDICT_FALSE(!prepare_status.ok())) {
-    HandleFailure(prepare_status);
+    return HandleFailure(prepare_status);
   }
 }
 
@@ -314,7 +328,7 @@ Status OpDriver::Prepare() {
                         mutable_state()->consensus_round()->replicate_msg()));
       RETURN_NOT_OK(op_->Start());
       VLOG_WITH_PREFIX(4) << "Triggering consensus replication.";
-      TRACE("REPLICATION: Starting.");
+      TRACE("REPLICATION: starting");
       // Trigger consensus replication.
       {
         std::lock_guard<simple_spinlock> lock(lock_);
@@ -427,7 +441,7 @@ void OpDriver::ReplicationFinished(const Status& status) {
   }
 
   TRACE_COUNTER_INCREMENT("replication_time_us", replication_duration.ToMicroseconds());
-  TRACE("REPLICATION: Finished.");
+  TRACE("REPLICATION: finished");
 
   // If we have prepared and replicated, we're ready
   // to move ahead and apply this operation.
@@ -533,7 +547,7 @@ void OpDriver::ApplyTask() {
     // until now.earliest > prepare_latest. Only after this are the locks
     // released.
     if (mutable_state()->external_consistency_mode() == COMMIT_WAIT) {
-      TRACE("APPLY: Commit Wait.");
+      TRACE("APPLY: CommitWait");
       // If we can't commit wait and have already applied we might have consistency
       // issues if we still reply to the client that the operation was a success.
       // On the other hand we don't have rollbacks as of yet thus we can't undo the
