@@ -22,11 +22,12 @@
 #include <cstdint>
 #include <ctime>
 #include <new>
+#include <optional>
 #include <ostream>
+#include <type_traits>
 #include <vector>
 
 #include <boost/container/small_vector.hpp>
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 #include <google/protobuf/arena.h>
@@ -71,8 +72,15 @@ DEFINE_int32(tablet_inject_latency_on_apply_write_op_ms, 0,
 TAG_FLAG(tablet_inject_latency_on_apply_write_op_ms, unsafe);
 TAG_FLAG(tablet_inject_latency_on_apply_write_op_ms, runtime);
 
+DEFINE_int32(tablet_inject_latency_on_prepare_write_op_ms, 0,
+             "How much latency to inject when a write op is prepared. "
+             "For testing only!");
+TAG_FLAG(tablet_inject_latency_on_prepare_write_op_ms, unsafe);
+TAG_FLAG(tablet_inject_latency_on_prepare_write_op_ms, runtime);
+
 DECLARE_bool(enable_txn_partition_lock);
 
+using std::optional;
 using std::string;
 using std::unique_ptr;
 using std::vector;
@@ -81,11 +89,12 @@ using strings::Substitute;
 namespace kudu {
 namespace tablet {
 
-using pb_util::SecureShortDebugString;
 using consensus::CommitMsg;
 using consensus::DriverType;
 using consensus::ReplicateMsg;
 using consensus::WRITE_OP;
+using pb_util::SecureShortDebugString;
+using tserver::ResourceMetricsPB;
 using tserver::TabletServerErrorPB;
 using tserver::WriteRequestPB;
 using tserver::WriteResponsePB;
@@ -111,6 +120,7 @@ void AddWritePrivilegesForRowOperations(const RowOperationsPB::Type& op_type,
       InsertIfNotPresent(privileges, WritePrivilegeType::INSERT);
       break;
     case RowOperationsPB::UPSERT:
+    case RowOperationsPB::UPSERT_IGNORE:
       InsertIfNotPresent(privileges, WritePrivilegeType::INSERT);
       InsertIfNotPresent(privileges, WritePrivilegeType::UPDATE);
       break;
@@ -151,7 +161,8 @@ WriteOp::WriteOp(unique_ptr<WriteOpState> state, DriverType type)
 void WriteOp::NewReplicateMsg(unique_ptr<ReplicateMsg>* replicate_msg) {
   replicate_msg->reset(new ReplicateMsg);
   (*replicate_msg)->set_op_type(consensus::OperationType::WRITE_OP);
-  (*replicate_msg)->mutable_write_request()->CopyFrom(*state()->request());
+  auto* write_req = (*replicate_msg)->mutable_write_request();
+  write_req->CopyFrom(*state()->request());
   if (state()->are_results_tracked()) {
     (*replicate_msg)->mutable_request_id()->CopyFrom(state()->request_id());
   }
@@ -159,7 +170,15 @@ void WriteOp::NewReplicateMsg(unique_ptr<ReplicateMsg>* replicate_msg) {
 
 Status WriteOp::Prepare() {
   TRACE_EVENT0("op", "WriteOp::Prepare");
-  TRACE("PREPARE: Starting.");
+
+  if (PREDICT_FALSE(FLAGS_tablet_inject_latency_on_prepare_write_op_ms) > 0) {
+    TRACE("injecting $0ms of latency due to --tablet_inject_latency_on_prepare_write_op_ms",
+          FLAGS_tablet_inject_latency_on_prepare_write_op_ms);
+    SleepFor(MonoDelta::FromMilliseconds(FLAGS_tablet_inject_latency_on_prepare_write_op_ms));
+  }
+
+  Tablet* tablet = state()->tablet_replica()->tablet();
+  TRACE(Substitute("PREPARE: starting on tablet $0", tablet->tablet_id()));
   // Decode everything first so that we give up if something major is wrong.
   Schema client_schema;
   RETURN_NOT_OK_PREPEND(SchemaFromPB(state_->request()->schema(), &client_schema),
@@ -172,8 +191,6 @@ Status WriteOp::Prepare() {
     return s;
   }
 
-  Tablet* tablet = state()->tablet_replica()->tablet();
-
   // Before taking any other locks, acquire the transaction state lock and
   // ensure it is open.
   Status s;
@@ -184,8 +201,22 @@ Status WriteOp::Prepare() {
       return s;
     }
   }
-
-  s = tablet->DecodeWriteOperations(&client_schema, state());
+  // In case of leader replica, set the auto-incrementing column value in the raft consensus
+  // replicate message and in case of follower replica update the auto increment counter to
+  // the value present in the raft consensus replicate message.
+  bool is_leader = type() == consensus::LEADER;
+  if (state_->consensus_round() &&
+      state_->tablet_replica()->tablet()->schema()->has_auto_incrementing()) {
+    if (is_leader) {
+      auto* write_req = state_->consensus_round()->replicate_msg()->mutable_write_request();
+      write_req->mutable_auto_incrementing_column()->set_auto_incrementing_counter(
+          tablet->GetAutoIncrementingCounter());
+    } else {
+      tablet->SetAutoIncrementingCounter(state_->consensus_round()->
+          replicate_msg()->write_request().auto_incrementing_column().auto_incrementing_counter());
+    }
+  }
+  s = tablet->DecodeWriteOperations(&client_schema, state(), is_leader);
   if (!s.ok()) {
     // TODO(unknown): is MISMATCHED_SCHEMA always right here? probably not.
     state()->completion_callback()->set_error(s, TabletServerErrorPB::MISMATCHED_SCHEMA);
@@ -224,7 +255,7 @@ Status WriteOp::Prepare() {
   }
   RETURN_NOT_OK(tablet->AcquireRowLocks(state()));
 
-  TRACE("PREPARE: Finished.");
+  TRACE("PREPARE: finished");
   return Status::OK();
 }
 
@@ -243,19 +274,20 @@ Status WriteOp::Start() {
   return Status::OK();
 }
 
-void WriteOp::UpdatePerRowErrors() {
-  // Add per-row errors to the result, update metrics.
-  for (int i = 0; i < state()->row_ops().size(); ++i) {
-    const RowOp* op = state()->row_ops()[i];
+void WriteOp::UpdatePerRowMetricsAndErrors() {
+  // Update metrics or add per-row errors to the result.
+  size_t idx = 0;
+  for (const auto* op : state()->row_ops()) {
     if (op->result->has_failed_status()) {
       // Replicas disregard the per row errors, for now
       // TODO(unknown): check the per-row errors against the leader's, at least in debug mode
       WriteResponsePB::PerRowErrorPB* error = state()->response()->add_per_row_errors();
-      error->set_row_index(i);
+      error->set_row_index(idx);
       error->mutable_error()->CopyFrom(op->result->failed_status());
+    } else {
+      state()->UpdateMetricsForOp(*op);
     }
-
-    state()->UpdateMetricsForOp(*op);
+    ++idx;
   }
 }
 
@@ -263,7 +295,7 @@ void WriteOp::UpdatePerRowErrors() {
 // it seems pointless to return a Status!
 Status WriteOp::Apply(CommitMsg** commit_msg) {
   TRACE_EVENT0("op", "WriteOp::Apply");
-  TRACE("APPLY: Starting.");
+  TRACE("APPLY: starting");
 
   if (PREDICT_FALSE(ANNOTATE_UNPROTECTED_READ(
       FLAGS_tablet_inject_latency_on_apply_write_op_ms) > 0)) {
@@ -274,9 +306,9 @@ Status WriteOp::Apply(CommitMsg** commit_msg) {
 
   Tablet* tablet = state()->tablet_replica()->tablet();
   RETURN_NOT_OK(tablet->ApplyRowOperations(state()));
-  TRACE("APPLY: Finished.");
+  TRACE("APPLY: finished");
 
-  UpdatePerRowErrors();
+  UpdatePerRowMetricsAndErrors();
 
   // Create the Commit message
   *commit_msg = google::protobuf::Arena::CreateMessage<CommitMsg>(state_->pb_arena());
@@ -289,32 +321,39 @@ Status WriteOp::Apply(CommitMsg** commit_msg) {
 void WriteOp::Finish(OpResult result) {
   TRACE_EVENT0("op", "WriteOp::Finish");
 
+  if (result == Op::APPLIED) {
+    // Populate response metrics.
+    state()->FillResponseMetrics(type());
+  }
+
   state()->FinishApplyingOrAbort(result);
 
   if (PREDICT_FALSE(result == Op::ABORTED)) {
-    TRACE("FINISH: Op aborted.");
+    TRACE("FINISH: Op aborted");
     return;
   }
 
   DCHECK_EQ(result, Op::APPLIED);
 
-  TRACE("FINISH: Updating metrics.");
+  TRACE("FINISH: Updating metrics");
 
-  TabletMetrics* metrics = state_->tablet_replica()->tablet()->metrics();
-  if (metrics) {
+  if (auto* metrics = state_->tablet_replica()->tablet()->metrics();
+      PREDICT_TRUE(metrics != nullptr)) {
     // TODO(unknown): should we change this so it's actually incremented by the
     // Tablet code itself instead of this wrapper code?
-    metrics->rows_inserted->IncrementBy(state_->metrics().successful_inserts);
-    metrics->insert_ignore_errors->IncrementBy(state_->metrics().insert_ignore_errors);
-    metrics->rows_upserted->IncrementBy(state_->metrics().successful_upserts);
-    metrics->rows_updated->IncrementBy(state_->metrics().successful_updates);
-    metrics->update_ignore_errors->IncrementBy(state_->metrics().update_ignore_errors);
-    metrics->rows_deleted->IncrementBy(state_->metrics().successful_deletes);
-    metrics->delete_ignore_errors->IncrementBy(state_->metrics().delete_ignore_errors);
+    const auto& op_m = state_->metrics();
+    metrics->rows_inserted->IncrementBy(op_m.successful_inserts);
+    metrics->insert_ignore_errors->IncrementBy(op_m.insert_ignore_errors);
+    metrics->rows_upserted->IncrementBy(op_m.successful_upserts);
+    metrics->upsert_ignore_errors->IncrementBy(op_m.upsert_ignore_errors);
+    metrics->rows_updated->IncrementBy(op_m.successful_updates);
+    metrics->update_ignore_errors->IncrementBy(op_m.update_ignore_errors);
+    metrics->rows_deleted->IncrementBy(op_m.successful_deletes);
+    metrics->delete_ignore_errors->IncrementBy(op_m.delete_ignore_errors);
 
     if (type() == consensus::LEADER) {
       if (state()->external_consistency_mode() == COMMIT_WAIT) {
-        metrics->commit_wait_duration->Increment(state_->metrics().commit_wait_duration_usec);
+        metrics->commit_wait_duration->Increment(op_m.commit_wait_duration_usec);
       }
       uint64_t op_duration_usec =
           (MonoTime::Now() - start_time_).ToMicroseconds();
@@ -346,7 +385,7 @@ WriteOpState::WriteOpState(TabletReplica* tablet_replica,
                            const tserver::WriteRequestPB *request,
                            const rpc::RequestIdPB* request_id,
                            tserver::WriteResponsePB *response,
-                           boost::optional<WriteAuthorizationContext> authz_ctx)
+                           optional<WriteAuthorizationContext> authz_ctx)
   : OpState(tablet_replica),
     request_(DCHECK_NOTNULL(request)),
     response_(response),
@@ -477,9 +516,7 @@ void WriteOpState::ReleaseTxResultPB(TxResultPB* result) const {
 }
 
 void WriteOpState::UpdateMetricsForOp(const RowOp& op) {
-  if (op.result->has_failed_status()) {
-    return;
-  }
+  DCHECK(!op.result->has_failed_status());
   switch (op.decoded_op.type) {
     case RowOperationsPB::INSERT:
       DCHECK(!op.error_ignored);
@@ -496,6 +533,17 @@ void WriteOpState::UpdateMetricsForOp(const RowOp& op) {
       DCHECK(!op.error_ignored);
       op_metrics_.successful_upserts++;
       break;
+    case RowOperationsPB::UPSERT_IGNORE:
+      if (op.error_ignored) {
+        op_metrics_.upsert_ignore_errors++;
+      }
+      // This op may be completed even if it's error_ignored. It make sense
+      // when attempting to update immutable cells, the rest of cells may be updated
+      // except the immutable cells.
+      if (!op.failed) {
+        op_metrics_.successful_upserts++;
+      }
+      break;
     case RowOperationsPB::UPDATE:
       DCHECK(!op.error_ignored);
       op_metrics_.successful_updates++;
@@ -503,7 +551,11 @@ void WriteOpState::UpdateMetricsForOp(const RowOp& op) {
     case RowOperationsPB::UPDATE_IGNORE:
       if (op.error_ignored) {
         op_metrics_.update_ignore_errors++;
-      } else {
+      }
+      // This op may be completed even if it's error_ignored. It make sense
+      // when attempting to update immutable cells, the rest of cells may be updated
+      // except the immutable cells.
+      if (!op.failed) {
         op_metrics_.successful_updates++;
       }
       break;
@@ -655,5 +707,21 @@ string WriteOpState::ToString() const {
                     row_ops_str);
 }
 
+void WriteOpState::FillResponseMetrics(consensus::DriverType type) {
+  const auto& op_m = op_metrics_;
+  tserver::ResourceMetricsPB* resp_metrics = response_->mutable_resource_metrics();
+  resp_metrics->set_successful_inserts(op_m.successful_inserts);
+  resp_metrics->set_insert_ignore_errors(op_m.insert_ignore_errors);
+  resp_metrics->set_successful_upserts(op_m.successful_upserts);
+  resp_metrics->set_upsert_ignore_errors(op_m.upsert_ignore_errors);
+  resp_metrics->set_successful_updates(op_m.successful_updates);
+  resp_metrics->set_update_ignore_errors(op_m.update_ignore_errors);
+  resp_metrics->set_successful_deletes(op_m.successful_deletes);
+  resp_metrics->set_delete_ignore_errors(op_m.delete_ignore_errors);
+  if (type == consensus::LEADER && external_consistency_mode() == COMMIT_WAIT) {
+    resp_metrics->set_commit_wait_duration_usec(op_m.commit_wait_duration_usec);
+  }
+}
+
 }  // namespace tablet
 }  // namespace kudu
