@@ -21,13 +21,14 @@
 #include <map>
 #include <memory>
 #include <mutex>
+#include <optional>
 #include <string>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
 #include <boost/container/flat_map.hpp>
 #include <boost/container/vector.hpp>
-#include <boost/optional/optional.hpp>
 #include <glog/logging.h>
 
 #include "kudu/common/schema.h"
@@ -99,7 +100,7 @@ class RowSetMetadata {
 
   int64_t id() const { return id_; }
 
-  const Schema& tablet_schema() const {
+  const SchemaPtr tablet_schema() const {
     return tablet_metadata_->schema();
   }
 
@@ -119,14 +120,14 @@ class RowSetMetadata {
 
   // Atomically commit the new redo delta block to RowSetMetadata.
   // This atomic operation includes updates to last_durable_redo_dms_id_ and live_row_count_.
-  Status CommitRedoDeltaDataBlock(int64_t dms_id,
-                                  int64_t num_deleted_rows,
-                                  const BlockId& block_id);
+  void CommitRedoDeltaDataBlock(int64_t dms_id,
+                                int64_t num_deleted_rows,
+                                const BlockId& block_id);
 
-  Status CommitUndoDeltaDataBlock(const BlockId& block_id);
+  void CommitUndoDeltaDataBlock(const BlockId& block_id);
 
   bool has_encoded_keys_unlocked() const {
-    return min_encoded_key_ != boost::none && max_encoded_key_ != boost::none;
+    return min_encoded_key_ && max_encoded_key_;
   }
 
   bool has_encoded_keys() const {
@@ -146,13 +147,13 @@ class RowSetMetadata {
 
   std::string min_encoded_key() const {
     std::lock_guard<LockType> l(lock_);
-    DCHECK(min_encoded_key_ != boost::none);
+    DCHECK(min_encoded_key_);
     return *min_encoded_key_;
   }
 
   std::string max_encoded_key() const {
     std::lock_guard<LockType> l(lock_);
-    DCHECK(max_encoded_key_ != boost::none);
+    DCHECK(max_encoded_key_);
     return *max_encoded_key_;
   }
 
@@ -226,7 +227,9 @@ class RowSetMetadata {
 
   void ToProtobuf(RowSetDataPB *pb);
 
-  BlockIdContainer GetAllBlocks();
+  BlockIdContainer GetAllBlocks() const;
+
+  BlockId GetMaxLiveBlockId() const;
 
   // Increase the row count.
   // Note:
@@ -270,8 +273,8 @@ class RowSetMetadata {
   mutable LockType lock_;
 
   // The min and max keys of the rowset.
-  boost::optional<std::string> min_encoded_key_;
-  boost::optional<std::string> max_encoded_key_;
+  std::optional<std::string> min_encoded_key_;
+  std::optional<std::string> max_encoded_key_;
 
   BlockId bloom_block_;
   BlockId adhoc_index_block_;
