@@ -22,14 +22,15 @@
 #include <iterator>
 #include <map>
 #include <memory>
+#include <optional>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <unordered_set>
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 #include <google/protobuf/arena.h>
@@ -75,7 +76,7 @@
 #include "kudu/tablet/ops/write_op.h"
 #include "kudu/tablet/row_op.h"
 #include "kudu/tablet/rowset.h"
-#include "kudu/tablet/rowset_metadata.h"
+#include "kudu/tablet/rowset_metadata.h" // IWYU pragma: keep
 #include "kudu/tablet/tablet.h"
 #include "kudu/tablet/tablet.pb.h"
 #include "kudu/tablet/tablet_metadata.h"
@@ -1535,11 +1536,11 @@ Status TabletBootstrap::PlayAlterSchemaRequest(const IOContext* /*io_context*/,
   AlterSchemaRequestPB* alter_schema = replicate_msg->mutable_alter_schema_request();
 
   // Decode schema
-  Schema schema;
-  RETURN_NOT_OK(SchemaFromPB(alter_schema->schema(), &schema));
+  SchemaPtr schema_ptr = std::make_shared<Schema>();
+  RETURN_NOT_OK(SchemaFromPB(alter_schema->schema(), schema_ptr.get()));
 
   AlterSchemaOpState op_state(nullptr, alter_schema, nullptr);
-  RETURN_NOT_OK(tablet_->CreatePreparedAlterSchema(&op_state, &schema));
+  RETURN_NOT_OK(tablet_->CreatePreparedAlterSchema(&op_state, schema_ptr));
 
   // Apply the alter schema to the tablet
   RETURN_NOT_OK_PREPEND(tablet_->AlterSchema(&op_state), "Failed to AlterSchema:");
@@ -1547,7 +1548,7 @@ Status TabletBootstrap::PlayAlterSchemaRequest(const IOContext* /*io_context*/,
   if (!op_state.error()) {
     // If the alter completed successfully, update the log segment header. Note
     // that our new log isn't hooked up to the tablet yet.
-    log_->SetSchemaForNextLogSegment(std::move(schema), op_state.schema_version());
+    log_->SetSchemaForNextLogSegment(*schema_ptr, op_state.schema_version());
   }
 
   return AppendCommitMsg(commit_msg);
@@ -1681,7 +1682,8 @@ Status TabletBootstrap::ApplyOperations(const IOContext* io_context,
     switch (op->decoded_op.type) {
       case RowOperationsPB::INSERT:
       case RowOperationsPB::INSERT_IGNORE:
-      case RowOperationsPB::UPSERT: {
+      case RowOperationsPB::UPSERT:
+      case RowOperationsPB::UPSERT_IGNORE: {
         // TODO(unknown): should we have a separate counter for upserts?
         stats_.inserts_seen++;
         if (op->has_result()) {
