@@ -20,11 +20,12 @@
 #include <cstdint>
 #include <functional>
 #include <memory>
+#include <optional>
 #include <ostream>
 #include <string>
+#include <type_traits>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags_declare.h>
 #include <glog/logging.h>
 #include <gtest/gtest.h>
@@ -32,7 +33,6 @@
 #include "kudu/clock/clock.h"
 #include "kudu/clock/hybrid_clock.h"
 #include "kudu/clock/mock_ntp.h"
-#include "kudu/clock/time_service.h"
 #include "kudu/common/common.pb.h"
 #include "kudu/common/partial_row.h"
 #include "kudu/common/schema.h"
@@ -47,7 +47,6 @@
 #include "kudu/tablet/diskrowset.h"
 #include "kudu/tablet/local_tablet_writer.h"
 #include "kudu/tablet/rowset.h"
-#include "kudu/tablet/rowset_metadata.h"
 #include "kudu/tablet/tablet-harness.h"
 #include "kudu/tablet/tablet-test-base.h"
 #include "kudu/tablet/tablet-test-util.h"
@@ -62,8 +61,10 @@
 DECLARE_bool(enable_maintenance_manager);
 DECLARE_int32(tablet_history_max_age_sec);
 DECLARE_string(time_source);
+DECLARE_bool(enable_gc_deleted_rowsets_without_live_row_count);
 
 using kudu::clock::HybridClock;
+using std::nullopt;
 using std::string;
 using std::vector;
 using strings::Substitute;
@@ -112,6 +113,20 @@ class TabletHistoryGcTest : public TabletTestBase<IntKeyTestSetup<INT64>> {
     ASSERT_GT(metrics->deleted_rowset_gc_duration->TotalCount(), orig_count);
   }
 
+  // Attempt to run the deleted rowset GC, it is our expectation that GC will make no sense
+  // and nothing will change.
+  void TryRunningDeletedRowsetGCWithoutEffect() {
+    const auto& metrics = tablet()->metrics();
+    const int orig_bytes = metrics->deleted_rowset_gc_bytes_deleted->value();
+    const int orig_count = metrics->deleted_rowset_gc_duration->TotalCount();
+    int64_t bytes = 0;
+    ASSERT_OK(tablet()->GetBytesInAncientDeletedRowsets(&bytes));
+    ASSERT_EQ(0, bytes);
+    ASSERT_OK(tablet()->DeleteAncientDeletedRowsets());
+    ASSERT_EQ(orig_bytes, metrics->deleted_rowset_gc_bytes_deleted->value());
+    ASSERT_EQ(orig_count, metrics->deleted_rowset_gc_duration->TotalCount());
+  }
+
   // Helper functions that mutate rows in batches of keys:
   //   [0, rows_per_rowset)
   //   [rows_per_rowset, 2*rows_per_rowset)
@@ -206,7 +221,7 @@ TEST_F(TabletHistoryGcTest, TestNoGenerateUndoOnMajorDeltaCompaction) {
     }
     // We must flush the DMS before major compaction can operate on these REDOs.
     for (int i = 0; i < kNumRowsets; i++) {
-      tablet()->FlushBiggestDMS();
+      ASSERT_OK(tablet()->FlushBiggestDMSForTests());
     }
     post_update_ts[val - 1] = clock()->Now();
   }
@@ -259,7 +274,7 @@ TEST_F(TabletHistoryGcTest, TestMajorDeltaCompactionOnSubsetOfColumns) {
     ASSERT_OK_FAST(writer.Update(row));
   }
   for (int i = 0; i < kNumRowsets; i++) {
-    tablet()->FlushBiggestDMS();
+    ASSERT_OK(tablet()->FlushBiggestDMSForTests());
   }
 
   NO_FATALS(AddTimeToHybridClock(MonoDelta::FromSeconds(200)));
@@ -305,11 +320,11 @@ TEST_F(TabletHistoryGcTest, TestNoGenerateUndoOnMRSFlush) {
   NO_FATALS(AddTimeToHybridClock(MonoDelta::FromSeconds(200)));
 
   NO_FATALS(VerifyTestRowsWithTimestampAndVerifier(kStartRow, 0,
-                                                   time_before_insert, boost::none));
+                                                   time_before_insert, nullopt));
   NO_FATALS(VerifyTestRowsWithTimestampAndVerifier(kStartRow, TotalNumRows(),
                                                    time_after_insert, kRowsEqual0));
   NO_FATALS(VerifyTestRowsWithTimestampAndVerifier(kStartRow, 0,
-                                                   time_after_delete, boost::none));
+                                                   time_after_delete, nullopt));
 
   // Now flush the MRS. No trace should remain after this.
   ASSERT_OK(tablet()->Flush());
@@ -345,7 +360,8 @@ TEST_F(TabletHistoryGcTest, TestUndoGCOnMergeCompaction) {
   NO_FATALS(VerifyTestRowsWithVerifier(kStartRow, TotalNumRows(), kRowsEqual0));
 
   // The earliest thing we can see is an empty tablet.
-  NO_FATALS(VerifyTestRowsWithTimestampAndVerifier(kStartRow, 0, time_before_insert, boost::none));
+  NO_FATALS(VerifyTestRowsWithTimestampAndVerifier(
+      kStartRow, 0, time_before_insert, nullopt));
 
   // Move the clock so the insert is prior to the AHM, then compact.
   NO_FATALS(AddTimeToHybridClock(MonoDelta::FromSeconds(2)));
@@ -393,7 +409,8 @@ TEST_F(TabletHistoryGcTest, TestRowRemovalGCOnMergeCompaction) {
   // will be GCed.
   ASSERT_OK(tablet()->Compact(Tablet::FORCE_COMPACT_ALL));
   NO_FATALS(VerifyDebugDumpRowsMatch(""));
-  NO_FATALS(VerifyTestRowsWithTimestampAndVerifier(kStartRow, 0, prev_time, boost::none));
+  NO_FATALS(VerifyTestRowsWithTimestampAndVerifier(
+      kStartRow, 0, prev_time, nullopt));
   ASSERT_EQ(0, tablet()->OnDiskDataSize());
 }
 
@@ -673,7 +690,25 @@ TEST_F(TabletHistoryGcNoMaintMgrTest, TestUndoDeltaBlockGc) {
   ASSERT_EQ(1, tablet()->metrics()->undo_delta_block_gc_delete_duration->TotalCount());
 }
 
-TEST_F(TabletHistoryGcNoMaintMgrTest, TestGCDeletedRowsetsWithRedoFiles) {
+class TabletDeletedRowsetGcTest : public TabletHistoryGcNoMaintMgrTest,
+                                  public ::testing::WithParamInterface<bool> {
+public:
+  void SetSupportLiveRowCount() {
+    NO_FATALS(tablet()->metadata()->
+        set_supports_live_row_count_for_tests(/* supports_live_row_count */ GetParam()));
+  }
+
+  void SetUp() override {
+    TabletHistoryGcNoMaintMgrTest::SetUp();
+    NO_FATALS(SetSupportLiveRowCount());
+    FLAGS_enable_gc_deleted_rowsets_without_live_row_count = GetParam();
+  }
+};
+
+INSTANTIATE_TEST_SUITE_P(, TabletDeletedRowsetGcTest,
+                         ::testing::Values(true, false));
+
+TEST_P(TabletDeletedRowsetGcTest, TestGCDeletedRowsetsWithRedoFiles) {
   FLAGS_tablet_history_max_age_sec = 1000;
   NO_FATALS(InsertOriginalRows(kNumRowsets, rows_per_rowset_));
   ASSERT_EQ(kNumRowsets, tablet()->CountUndoDeltasForTests());
@@ -704,15 +739,23 @@ TEST_F(TabletHistoryGcNoMaintMgrTest, TestGCDeletedRowsetsWithRedoFiles) {
   ASSERT_EQ(0, tablet()->CountRedoDeltasForTests());
 }
 
-TEST_F(TabletHistoryGcNoMaintMgrTest, TestGCDeletedRowsetsWithDMS) {
+TEST_P(TabletDeletedRowsetGcTest, TestGCDeletedRowsetsWithDMS) {
   FLAGS_tablet_history_max_age_sec = 1000;
   NO_FATALS(InsertOriginalRows(kNumRowsets, rows_per_rowset_));
   NO_FATALS(DeleteOriginalRows(kNumRowsets, rows_per_rowset_, /*flush_dms*/false));
   NO_FATALS(AddTimeToHybridClock(MonoDelta::FromSeconds(FLAGS_tablet_history_max_age_sec + 1)));
-  NO_FATALS(TryRunningDeletedRowsetGC());
+  if (GetParam()) {
+    NO_FATALS(TryRunningDeletedRowsetGC());
+  } else {
+    // We can't get the exact count of live row without live_row_count feature.
+    // If we disable the live_row_count, the imprecise counting will not trigger the GC.
+    // However, this is expected and will not cause any errors, so we can directly ignore this
+    // unit testcase at this time.
+    NO_FATALS(TryRunningDeletedRowsetGCWithoutEffect());
+  }
 }
 
-TEST_F(TabletHistoryGcNoMaintMgrTest, TestGCDeletedRowsetsAfterMinorCompaction) {
+TEST_P(TabletDeletedRowsetGcTest, TestGCDeletedRowsetsAfterMinorCompaction) {
   FLAGS_tablet_history_max_age_sec = 1000;
   NO_FATALS(InsertOriginalRows(kNumRowsets, rows_per_rowset_));
   // Flush twice as frequently when deleting so we end up with multiple delta
@@ -727,7 +770,7 @@ TEST_F(TabletHistoryGcNoMaintMgrTest, TestGCDeletedRowsetsAfterMinorCompaction)
   NO_FATALS(TryRunningDeletedRowsetGC());
 }
 
-TEST_F(TabletHistoryGcNoMaintMgrTest, TestGCDeletedRowsetsAfterMajorCompaction) {
+TEST_P(TabletDeletedRowsetGcTest, TestGCDeletedRowsetsAfterMajorCompaction) {
   FLAGS_tablet_history_max_age_sec = 1000;
   NO_FATALS(InsertOriginalRows(kNumRowsets, rows_per_rowset_));
   // Major delta compaction is a no-op if we only have deletes, so trick the
