@@ -19,16 +19,19 @@
 #include <atomic>
 #include <cstdint>
 #include <memory>
+#include <mutex>
+#include <optional>
 #include <string>
 #include <unordered_map>
 #include <unordered_set>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <glog/logging.h>
 
 #include "kudu/common/common.pb.h"
 #include "kudu/common/partition.h"
+#include "kudu/common/schema.h"
+#include "kudu/consensus/opid.pb.h"
 #include "kudu/fs/block_id.h"
 #include "kudu/gutil/atomicops.h"
 #include "kudu/gutil/macros.h"
@@ -43,13 +46,8 @@ namespace kudu {
 
 class BlockIdPB;
 class FsManager;
-class Schema;
 class Timestamp;
 
-namespace consensus {
-class OpId;
-} // namespace consensus
-
 namespace log {
 class MinLogIndexAnchorer;
 } // namespace log
@@ -58,6 +56,7 @@ namespace tablet {
 
 class RowSetMetadata;
 class TxnMetadata;
+
 enum TxnState : int8_t;
 
 typedef std::vector<std::shared_ptr<RowSetMetadata>> RowSetMetadataVector;
@@ -93,11 +92,11 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
                           const PartitionSchema& partition_schema,
                           const Partition& partition,
                           const TabletDataState& initial_tablet_data_state,
-                          boost::optional<consensus::OpId> tombstone_last_logged_opid,
+                          std::optional<consensus::OpId> tombstone_last_logged_opid,
                           bool supports_live_row_count,
-                          boost::optional<TableExtraConfigPB> extra_config,
-                          boost::optional<std::string> dimension_label,
-                          boost::optional<TableTypePB> table_type,
+                          std::optional<TableExtraConfigPB> extra_config,
+                          std::optional<std::string> dimension_label,
+                          std::optional<TableTypePB> table_type,
                           scoped_refptr<TabletMetadata>* metadata);
 
   // Load existing metadata from disk.
@@ -118,16 +117,18 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
                              const PartitionSchema& partition_schema,
                              const Partition& partition,
                              const TabletDataState& initial_tablet_data_state,
-                             boost::optional<consensus::OpId> tombstone_last_logged_opid,
-                             boost::optional<TableExtraConfigPB> extra_config,
-                             boost::optional<std::string> dimension_label,
-                             boost::optional<TableTypePB> table_type,
+                             std::optional<consensus::OpId> tombstone_last_logged_opid,
+                             std::optional<TableExtraConfigPB> extra_config,
+                             std::optional<std::string> dimension_label,
+                             std::optional<TableTypePB> table_type,
                              scoped_refptr<TabletMetadata>* metadata);
 
   static std::vector<BlockIdPB> CollectBlockIdPBs(
       const TabletSuperBlockPB& superblock);
 
-  BlockIdContainer CollectBlockIds();
+  BlockIdContainer CollectBlockIds() const;
+
+  BlockId GetMaxLiveBlockId() const;
 
   const std::string& tablet_id() const {
     DCHECK_NE(state_, kNotLoadedYet);
@@ -144,25 +145,24 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
     return table_id_;
   }
 
-  const boost::optional<TableTypePB>& table_type() const;
+  const std::optional<TableTypePB>& table_type() const;
 
   std::string table_name() const;
 
   uint32_t schema_version() const;
 
-  void SetSchema(const Schema& schema, uint32_t version);
+  void SetSchema(const SchemaPtr& schema, uint32_t version);
 
   void SetTableName(const std::string& table_name);
 
   void SetExtraConfig(TableExtraConfigPB extra_config);
 
-  // Return a reference to the current schema.
+  // Return a scoped_refptr to the current schema.
   // This pointer will be valid until the TabletMetadata is destructed,
   // even if the schema is changed.
-  const Schema& schema() const {
-    const Schema* s = reinterpret_cast<const Schema*>(
-        base::subtle::Acquire_Load(reinterpret_cast<const AtomicWord*>(&schema_ptr_)));
-    return *s;
+  const SchemaPtr schema() const {
+    std::lock_guard<LockType> l(data_lock_);
+    return schema_;
   }
 
   // Returns the partition schema of the tablet's table.
@@ -226,7 +226,7 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
   // Note: this will always update the in-memory state, but upon failure,
   // may not update the on-disk state.
   Status DeleteTabletData(TabletDataState delete_type,
-                          const boost::optional<consensus::OpId>& last_logged_opid);
+                          const std::optional<consensus::OpId>& last_logged_opid);
 
   // Return true if this metadata references no blocks (either live or orphaned) and is
   // already marked as tombstoned. If this is the case, then calling DeleteTabletData
@@ -300,13 +300,13 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
   void SetPreFlushCallback(StatusClosure callback);
 
   // Return the last-logged opid of a tombstoned tablet, if known.
-  boost::optional<consensus::OpId> tombstone_last_logged_opid() const;
+  std::optional<consensus::OpId> tombstone_last_logged_opid() const;
 
   // Returns the table's extra configuration properties.
-  boost::optional<TableExtraConfigPB> extra_config() const;
+  std::optional<TableExtraConfigPB> extra_config() const;
 
   // Returns the table's dimension label.
-  boost::optional<std::string> dimension_label() const;
+  std::optional<std::string> dimension_label() const;
 
   // Loads the currently-flushed superblock from disk into the given protobuf.
   Status ReadSuperBlockFromDisk(TabletSuperBlockPB* superblock) const;
@@ -366,16 +366,16 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
                  const Schema& schema, PartitionSchema partition_schema,
                  Partition partition,
                  const TabletDataState& tablet_data_state,
-                 boost::optional<consensus::OpId> tombstone_last_logged_opid,
+                 std::optional<consensus::OpId> tombstone_last_logged_opid,
                  bool supports_live_row_count,
-                 boost::optional<TableExtraConfigPB> extra_config,
-                 boost::optional<std::string> dimension_label,
-                 boost::optional<TableTypePB> table_type);
+                 std::optional<TableExtraConfigPB> extra_config,
+                 std::optional<std::string> dimension_label,
+                 std::optional<TableTypePB> table_type);
 
   // Constructor for loading an existing tablet.
   TabletMetadata(FsManager* fs_manager, std::string tablet_id);
 
-  void SetSchemaUnlocked(std::unique_ptr<Schema> schema, uint32_t version);
+  void SwapSchemaUnlocked(SchemaPtr schema, uint32_t version, SchemaPtr* old_schema);
 
   Status LoadFromDisk();
 
@@ -443,21 +443,11 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
   std::unordered_map<int64_t, scoped_refptr<TxnMetadata>> txn_metadata_by_txn_id_;;
 
   // The current schema version.
-  std::unique_ptr<Schema> schema_;
-  // The raw pointer to the schema for an atomic swap.
-  Schema* schema_ptr_;
-
+  SchemaPtr schema_;
   uint32_t schema_version_;
   std::string table_name_;
   PartitionSchema partition_schema_;
 
-  // Previous values of 'schema_'.
-  // These are currently kept alive forever, under the assumption that
-  // a given tablet won't have thousands of "alter table" calls.
-  // They are kept alive so that callers of schema() don't need to
-  // worry about reference counting or locking.
-  std::vector<std::unique_ptr<Schema>> old_schemas_;
-
   // Protected by 'data_lock_'.
   BlockIdSet orphaned_blocks_;
 
@@ -467,16 +457,16 @@ class TabletMetadata : public RefCountedThreadSafe<TabletMetadata> {
   // Record of the last opid logged by the tablet before it was last
   // tombstoned. Has no meaning for non-tombstoned tablets.
   // Protected by 'data_lock_'.
-  boost::optional<consensus::OpId> tombstone_last_logged_opid_;
+  std::optional<consensus::OpId> tombstone_last_logged_opid_;
 
   // Table extra config.
-  boost::optional<TableExtraConfigPB> extra_config_;
+  std::optional<TableExtraConfigPB> extra_config_;
 
   // Tablet's dimension label.
-  boost::optional<std::string> dimension_label_;
+  std::optional<std::string> dimension_label_;
 
   // The table type of the table this tablet belongs to.
-  boost::optional<TableTypePB> table_type_;
+  std::optional<TableTypePB> table_type_;
 
   // If this counter is > 0 then Flush() will not write any data to
   // disk.
