@@ -18,14 +18,14 @@
 #include "kudu/tablet/tablet_mm_ops.h"
 
 #include <mutex>
+#include <optional>
 #include <ostream>
+#include <type_traits>
 #include <utility>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 
-#include "kudu/common/common.pb.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/tablet/rowset.h"
@@ -89,6 +89,12 @@ DEFINE_bool(enable_workload_score_for_perf_improvement_ops, false,
 TAG_FLAG(enable_workload_score_for_perf_improvement_ops, experimental);
 TAG_FLAG(enable_workload_score_for_perf_improvement_ops, runtime);
 
+DEFINE_int32(update_stats_log_throttling_interval_sec, 600,
+    "Log throttling interval for messages on disabled compaction emitted by UpdateStats "
+    "method.");
+TAG_FLAG(update_stats_log_throttling_interval_sec, runtime);
+TAG_FLAG(update_stats_log_throttling_interval_sec, experimental);
+
 using std::string;
 using strings::Substitute;
 
@@ -122,15 +128,16 @@ bool TabletOpBase::DisableCompaction() const {
 ////////////////////////////////////////////////////////////
 
 CompactRowSetsOp::CompactRowSetsOp(Tablet* tablet)
-  : TabletOpBase(Substitute("CompactRowSetsOp($0)", tablet->tablet_id()),
-                 MaintenanceOp::HIGH_IO_USAGE, tablet),
-    last_num_mrs_flushed_(0),
-    last_num_rs_compacted_(0) {
+    : TabletOpBase(Substitute("CompactRowSetsOp($0)", tablet->tablet_id()),
+                   MaintenanceOp::HIGH_IO_USAGE, tablet),
+      last_num_mrs_flushed_(0),
+      last_num_rs_compacted_(0),
+      last_num_undo_deltas_gced_(0) {
 }
 
 void CompactRowSetsOp::UpdateStats(MaintenanceOpStats* stats) {
   if (PREDICT_FALSE(!FLAGS_enable_rowset_compaction || DisableCompaction())) {
-    KLOG_EVERY_N_SECS(WARNING, 300)
+    KLOG_EVERY_N_SECS(WARNING, FLAGS_update_stats_log_throttling_interval_sec)
         << Substitute("Rowset compaction is disabled (check --enable_rowset_compaction "
            "and disable_compaction in extra_config for tablet:$0)", tablet_->tablet_id());
     stats->set_runnable(false);
@@ -142,21 +149,26 @@ void CompactRowSetsOp::UpdateStats(MaintenanceOpStats* stats) {
   double workload_score = FLAGS_enable_workload_score_for_perf_improvement_ops ?
                           tablet_->CollectAndUpdateWorkloadStats(MaintenanceOp::COMPACT_OP) : 0;
 
-  // Any operation that changes the on-disk row layout invalidates the
-  // cached stats.
+  // Any operation that changes the on-disk row layout invalidates the cached
+  // stats. Also, UNDO delta GC do the same since the runnable state of the
+  // CompactRowSetsOp depends on the fraction of ancient deltas in the rowsets.
   TabletMetrics* metrics = tablet_->metrics();
   if (metrics) {
     uint64_t new_num_mrs_flushed = metrics->flush_mrs_duration->TotalCount();
     uint64_t new_num_rs_compacted = metrics->compact_rs_duration->TotalCount();
+    uint64_t new_num_undo_deltas_gced =
+        metrics->undo_delta_block_gc_perform_duration->TotalCount();
     if (prev_stats_.valid() &&
         new_num_mrs_flushed == last_num_mrs_flushed_ &&
-        new_num_rs_compacted == last_num_rs_compacted_) {
+        new_num_rs_compacted == last_num_rs_compacted_ &&
+        new_num_undo_deltas_gced == last_num_undo_deltas_gced_) {
       prev_stats_.set_workload_score(workload_score);
       *stats = prev_stats_;
       return;
     }
     last_num_mrs_flushed_ = new_num_mrs_flushed;
     last_num_rs_compacted_ = new_num_rs_compacted;
+    last_num_undo_deltas_gced_ = new_num_undo_deltas_gced;
   }
 
   tablet_->UpdateCompactionStats(&prev_stats_);
@@ -206,7 +218,7 @@ MinorDeltaCompactionOp::MinorDeltaCompactionOp(Tablet* tablet)
 
 void MinorDeltaCompactionOp::UpdateStats(MaintenanceOpStats* stats) {
   if (PREDICT_FALSE(!FLAGS_enable_minor_delta_compaction || DisableCompaction())) {
-    KLOG_EVERY_N_SECS(WARNING, 300)
+    KLOG_EVERY_N_SECS(WARNING, FLAGS_update_stats_log_throttling_interval_sec)
         << Substitute("Minor delta compaction is disabled (check --enable_minor_delta_compaction "
            "and disable_compaction in extra_config for tablet:$0)", tablet_->tablet_id());
     stats->set_runnable(false);
@@ -290,7 +302,7 @@ MajorDeltaCompactionOp::MajorDeltaCompactionOp(Tablet* tablet)
 
 void MajorDeltaCompactionOp::UpdateStats(MaintenanceOpStats* stats) {
   if (PREDICT_FALSE(!FLAGS_enable_major_delta_compaction || DisableCompaction())) {
-    KLOG_EVERY_N_SECS(WARNING, 300)
+    KLOG_EVERY_N_SECS(WARNING, FLAGS_update_stats_log_throttling_interval_sec)
         << Substitute("Major delta compaction is disabled (check --enable_major_delta_compaction "
            "and disable_compaction in extra_config for tablet:$0)", tablet_->tablet_id());
     stats->set_runnable(false);
@@ -373,7 +385,7 @@ UndoDeltaBlockGCOp::UndoDeltaBlockGCOp(Tablet* tablet)
 
 void UndoDeltaBlockGCOp::UpdateStats(MaintenanceOpStats* stats) {
   if (PREDICT_FALSE(!FLAGS_enable_undo_delta_block_gc)) {
-    KLOG_EVERY_N_SECS(WARNING, 300)
+    KLOG_EVERY_N_SECS(WARNING, FLAGS_update_stats_log_throttling_interval_sec)
         << "Undo delta block GC is disabled (check --enable_undo_delta_block_gc)";
     stats->set_runnable(false);
     return;
