@@ -20,11 +20,13 @@
 #include <cstdint>
 #include <functional>
 #include <memory>
+#include <optional>
 #include <ostream>
 #include <string>
+#include <thread>
+#include <type_traits>
 #include <utility>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 #include <gtest/gtest.h>
@@ -32,13 +34,13 @@
 #include "kudu/common/common.pb.h"
 #include "kudu/common/partial_row.h"
 #include "kudu/common/row_operations.h"
+#include "kudu/common/row_operations.pb.h"
 #include "kudu/common/schema.h"
 #include "kudu/common/wire_protocol.h"
-#include "kudu/common/wire_protocol.pb.h"
 #include "kudu/consensus/consensus.pb.h"
 #include "kudu/consensus/log.h"
 #include "kudu/consensus/log_anchor_registry.h"
-#include "kudu/consensus/log_reader.h"
+#include "kudu/consensus/log_reader.h"  // IWYU pragma: keep
 #include "kudu/consensus/log_util.h"
 #include "kudu/consensus/opid.pb.h"
 #include "kudu/consensus/opid_util.h"
@@ -46,6 +48,7 @@
 #include "kudu/fs/fs_manager.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/ref_counted.h"
+#include "kudu/tablet/lock_manager.h"
 #include "kudu/tablet/ops/alter_schema_op.h"
 #include "kudu/tablet/ops/op.h"
 #include "kudu/tablet/ops/op_driver.h"
@@ -57,12 +60,14 @@
 #include "kudu/tablet/tablet_replica_mm_ops.h"
 #include "kudu/tserver/tserver.pb.h"
 #include "kudu/tserver/tserver_admin.pb.h"
+#include "kudu/util/array_view.h"
 #include "kudu/util/countdown_latch.h"
 #include "kudu/util/maintenance_manager.h"
 #include "kudu/util/metrics.h"
 #include "kudu/util/monotime.h"
 #include "kudu/util/pb_util.h"
 #include "kudu/util/random.h"
+#include "kudu/util/slice.h"
 #include "kudu/util/status.h"
 #include "kudu/util/test_macros.h"
 #include "kudu/util/test_util.h"
@@ -89,6 +94,7 @@ using kudu::tserver::WriteRequestPB;
 using kudu::tserver::WriteResponsePB;
 using std::shared_ptr;
 using std::string;
+using std::thread;
 using std::unique_ptr;
 
 namespace kudu {
@@ -213,8 +219,8 @@ class TabletReplicaTest : public TabletReplicaTestBase {
   // Assert that the Log GC() anchor is earlier than the latest OpId in the Log.
   void AssertLogAnchorEarlierThanLogLatest() {
     log::RetentionIndexes retention = tablet_replica_->GetRetentionIndexes();
-    boost::optional<OpId> last_log_opid = tablet_replica_->consensus()->GetLastOpId(RECEIVED_OPID);
-    ASSERT_NE(boost::none, last_log_opid);
+    std::optional<OpId> last_log_opid = tablet_replica_->consensus()->GetLastOpId(RECEIVED_OPID);
+    ASSERT_TRUE(last_log_opid);
     ASSERT_LT(retention.for_durability, last_log_opid->index())
       << "Expected valid log anchor, got earliest opid: " << retention.for_durability
       << " (expected any value earlier than last log id: " << SecureShortDebugString(*last_log_opid)
@@ -323,8 +329,8 @@ TEST_F(TabletReplicaTest, TestDMSAnchorPreventsLogGC) {
   log->reader()->GetSegmentsSnapshot(&segments);
   ASSERT_EQ(2, segments.size());
 
-  boost::optional<OpId> id = consensus->GetLastOpId(consensus::RECEIVED_OPID);
-  ASSERT_NE(boost::none, id);
+  std::optional<OpId> id = consensus->GetLastOpId(consensus::RECEIVED_OPID);
+  ASSERT_TRUE(id);
   LOG(INFO) << "Before: " << *id;
 
   // We currently have no anchors and the last operation in the log is 0.3
@@ -357,7 +363,7 @@ TEST_F(TabletReplicaTest, TestDMSAnchorPreventsLogGC) {
   ASSERT_EQ(5, segments.size());
 
   // Flush DMS to release the anchor.
-  tablet_replica_->tablet()->FlushBiggestDMS();
+  ASSERT_OK(tablet_replica_->tablet()->FlushBiggestDMSForTests());
 
   // Verify no anchors after Flush().
   ASSERT_EVENTUALLY([&]{ AssertNoLogAnchors(); });
@@ -425,7 +431,8 @@ TEST_F(TabletReplicaTest, TestActiveOpPreventsLogGC) {
 
     scoped_refptr<OpDriver> driver;
     ASSERT_OK(tablet_replica_->NewLeaderOpDriver(std::move(op),
-                                                 &driver));
+                                                 &driver,
+                                                 MonoTime::Max()));
     driver->ExecuteAsync();
     apply_started.Wait();
     ASSERT_TRUE(driver->GetOpId().IsInitialized())
@@ -452,7 +459,7 @@ TEST_F(TabletReplicaTest, TestActiveOpPreventsLogGC) {
   log->reader()->GetSegmentsSnapshot(&segments);
   ASSERT_EQ(5, segments.size());
   ASSERT_EQ(1, tablet_replica_->log_anchor_registry()->GetAnchorCountForTests());
-  tablet_replica_->tablet()->FlushBiggestDMS();
+  ASSERT_OK(tablet_replica_->tablet()->FlushBiggestDMSForTests());
 
   ASSERT_EVENTUALLY([&]{
       AssertNoLogAnchors();
@@ -475,7 +482,7 @@ TEST_F(TabletReplicaTest, TestActiveOpPreventsLogGC) {
   rpc_latch.Wait();
   tablet_replica_->op_tracker_.WaitForAllToFinish();
   ASSERT_EQ(0, tablet_replica_->op_tracker_.GetNumPendingForTests());
-  tablet_replica_->tablet()->FlushBiggestDMS();
+  ASSERT_OK(tablet_replica_->tablet()->FlushBiggestDMSForTests());
   ASSERT_EVENTUALLY([&]{ AssertNoLogAnchors(); });
 
   // All should be deleted except the two last segments.
@@ -540,11 +547,11 @@ TEST_F(TabletReplicaTest, TestRollLogSegmentSchemaOnAlter) {
   ConsensusBootstrapInfo info;
   ASSERT_OK(StartReplicaAndWaitUntilLeader(info));
   SchemaPB orig_schema_pb;
-  ASSERT_OK(SchemaToPB(SchemaBuilder(tablet()->metadata()->schema()).Build(), &orig_schema_pb));
+  ASSERT_OK(SchemaToPB(SchemaBuilder(*tablet()->metadata()->schema()).Build(), &orig_schema_pb));
   const int orig_schema_version = tablet()->metadata()->schema_version();
 
   // Add a new column.
-  SchemaBuilder builder(tablet()->metadata()->schema());
+  SchemaBuilder builder(*tablet()->metadata()->schema());
   ASSERT_OK(builder.AddColumn("new_col", INT32));
   Schema new_client_schema = builder.BuildWithoutIds();
   SchemaPB new_schema;
@@ -581,11 +588,11 @@ TEST_F(TabletReplicaTest, Kudu2690Test) {
   ConsensusBootstrapInfo info;
   ASSERT_OK(StartReplicaAndWaitUntilLeader(info));
   SchemaPB orig_schema_pb;
-  ASSERT_OK(SchemaToPB(SchemaBuilder(tablet()->metadata()->schema()).Build(), &orig_schema_pb));
+  ASSERT_OK(SchemaToPB(SchemaBuilder(*tablet()->metadata()->schema()).Build(), &orig_schema_pb));
   const int orig_schema_version = tablet()->metadata()->schema_version();
 
   // First things first, add a new column.
-  SchemaBuilder builder(tablet()->metadata()->schema());
+  SchemaBuilder builder(*tablet()->metadata()->schema());
   ASSERT_OK(builder.AddColumn("new_col", INT32));
   Schema new_client_schema = builder.BuildWithoutIds();
   SchemaPB new_schema;
@@ -676,7 +683,7 @@ TEST_F(TabletReplicaTest, TestRestartAfterGCDeletedRowsets) {
   ASSERT_EQ(0, live_row_count->value());
 
   // Now do that again but with deltafiles.
-  ASSERT_OK(tablet->FlushBiggestDMS());
+  ASSERT_OK(tablet->FlushBiggestDMSForTests());
   ASSERT_OK(RestartReplica());
   tablet = tablet_replica_->tablet();
   ASSERT_EQ(1, tablet->num_rowsets());
@@ -689,5 +696,58 @@ TEST_F(TabletReplicaTest, TestRestartAfterGCDeletedRowsets) {
   ASSERT_EQ(1, tablet->num_rowsets());
 }
 
+// This is a trivial test scenario to check how row locking works in case of
+// concurrent attempts to lock the same row with relatively long waiting times.
+// The thread attempting to acquire the row lock for long times should be able
+// to acquire the lock eventually and log about its attempts to acquire the log.
+// The logging part isn't covered by any special assertions, though.
+// An alternative place to add this scenario could be lock_manager-test.cc, but
+// for proper logging a real WriteOpState backed by a tablet is necessary.
+TEST_F(TabletReplicaTest, RowLocksLongWaitAndLogging) {
+  SKIP_IF_SLOW_NOT_ALLOWED();
+
+  constexpr const char* const kKey = "key";
+  constexpr int32_t kValue = 0;
+
+  const Schema schema(GetTestSchema());
+
+  Slice key[]{kKey};
+  unique_ptr<WriteRequestPB> req(new WriteRequestPB);
+  req->set_tablet_id(tablet()->tablet_id());
+  CHECK_OK(SchemaToPB(schema, req->mutable_schema()));
+  KuduPartialRow row(&schema);
+  CHECK_OK(row.SetInt32(kKey, kValue));
+  {
+    RowOperationsPBEncoder enc(req->mutable_row_operations());
+    enc.Add(RowOperationsPB::DELETE, row);
+  }
+  unique_ptr<WriteResponsePB> resp(new WriteResponsePB);
+  LockManager lock_manager;
+
+  thread t0([&]{
+    unique_ptr<WriteOpState> op_state(new WriteOpState(
+        tablet_replica_.get(), req.get(), nullptr, resp.get()));
+    ScopedRowLock row_lock(
+        &lock_manager, op_state.get(), key, LockManager::LOCK_EXCLUSIVE);
+    CHECK(row_lock.acquired());
+    // Pause for a while when the other thread tries to acquire the lock,
+    // so the other thread logs about its attempts to acquire the row lock.
+    SleepFor(MonoDelta::FromMilliseconds(3000));
+  });
+
+  thread t1([&]{
+    // Let the other thread acquire the lock first.
+    SleepFor(MonoDelta::FromMilliseconds(500));
+    unique_ptr<WriteOpState> op_state(new WriteOpState(
+        tablet_replica_.get(), req.get(), nullptr, resp.get()));
+    ScopedRowLock row_lock(
+        &lock_manager, op_state.get(), key, LockManager::LOCK_EXCLUSIVE);
+    CHECK(row_lock.acquired());
+  });
+
+  t0.join();
+  t1.join();
+}
+
 } // namespace tablet
 } // namespace kudu
