@@ -22,13 +22,13 @@
 #include <functional>
 #include <memory>
 #include <mutex>
+#include <optional>
 #include <ostream>
 #include <string>
 #include <type_traits>
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 
@@ -49,7 +49,6 @@
 #include "kudu/gutil/basictypes.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/port.h"
-#include "kudu/gutil/stl_util.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/rpc/result_tracker.h"
 #include "kudu/rpc/rpc_header.pb.h"
@@ -58,6 +57,7 @@
 #include "kudu/tablet/ops/op_driver.h"
 #include "kudu/tablet/ops/participant_op.h"
 #include "kudu/tablet/ops/write_op.h"
+#include "kudu/tablet/tablet.h"
 #include "kudu/tablet/tablet.pb.h"
 #include "kudu/tablet/tablet_replica_mm_ops.h"
 #include "kudu/tablet/txn_coordinator.h"
@@ -186,6 +186,13 @@ TabletReplica::TabletReplica(
       last_status_("Tablet initializing...") {
 }
 
+TabletReplica::TabletReplica()
+    : apply_pool_(nullptr),
+      reload_txn_status_tablet_pool_(nullptr),
+      state_(SHUTDOWN),
+      last_status_("Fake replica created") {
+}
+
 TabletReplica::~TabletReplica() {
   // We are required to call Shutdown() before destroying a TabletReplica.
   CHECK(state_ == SHUTDOWN || state_ == FAILED)
@@ -584,13 +591,14 @@ Status TabletReplica::UnregisterTxnOpDispatcher(int64_t txn_id,
   return unregister_status;
 }
 
-Status TabletReplica::SubmitWrite(unique_ptr<WriteOpState> op_state) {
+Status TabletReplica::SubmitWrite(unique_ptr<WriteOpState> op_state,
+                                  MonoTime deadline) {
   RETURN_NOT_OK(CheckRunning());
 
   op_state->SetResultTracker(result_tracker_);
   unique_ptr<WriteOp> op(new WriteOp(std::move(op_state), consensus::LEADER));
   scoped_refptr<OpDriver> driver;
-  RETURN_NOT_OK(NewLeaderOpDriver(std::move(op), &driver));
+  RETURN_NOT_OK(NewLeaderOpDriver(std::move(op), &driver, deadline));
   driver->ExecuteAsync();
   return Status::OK();
 }
@@ -601,18 +609,19 @@ Status TabletReplica::SubmitTxnParticipantOp(std::unique_ptr<ParticipantOpState>
   op_state->SetResultTracker(result_tracker_);
   unique_ptr<ParticipantOp> op(new ParticipantOp(std::move(op_state), consensus::LEADER));
   scoped_refptr<OpDriver> driver;
-  RETURN_NOT_OK(NewLeaderOpDriver(std::move(op), &driver));
+  RETURN_NOT_OK(NewLeaderOpDriver(std::move(op), &driver, MonoTime::Max()));
   driver->ExecuteAsync();
   return Status::OK();
 }
 
-Status TabletReplica::SubmitAlterSchema(unique_ptr<AlterSchemaOpState> state) {
+Status TabletReplica::SubmitAlterSchema(unique_ptr<AlterSchemaOpState> state,
+                                        MonoTime deadline) {
   RETURN_NOT_OK(CheckRunning());
 
   unique_ptr<AlterSchemaOp> op(
       new AlterSchemaOp(std::move(state), consensus::LEADER));
   scoped_refptr<OpDriver> driver;
-  RETURN_NOT_OK(NewLeaderOpDriver(std::move(op), &driver));
+  RETURN_NOT_OK(NewLeaderOpDriver(std::move(op), &driver, deadline));
   driver->ExecuteAsync();
   return Status::OK();
 }
@@ -890,14 +899,16 @@ void TabletReplica::FinishConsensusOnlyRound(ConsensusRound* round) {
 }
 
 Status TabletReplica::NewLeaderOpDriver(unique_ptr<Op> op,
-                                        scoped_refptr<OpDriver>* driver) {
+                                        scoped_refptr<OpDriver>* driver,
+                                        MonoTime deadline) {
   scoped_refptr<OpDriver> op_driver = new OpDriver(
     &op_tracker_,
     consensus_.get(),
     log_.get(),
     prepare_pool_token_.get(),
     apply_pool_,
-    &op_order_verifier_);
+    &op_order_verifier_,
+    deadline);
   RETURN_NOT_OK(op_driver->Init(std::move(op), consensus::LEADER));
   *driver = std::move(op_driver);
 
@@ -930,19 +941,16 @@ void TabletReplica::RegisterMaintenanceOps(MaintenanceManager* maint_mgr) {
     return;
   }
 
-  vector<MaintenanceOp*> maintenance_ops;
+  vector<unique_ptr<MaintenanceOp>> maintenance_ops;
 
-  unique_ptr<MaintenanceOp> mrs_flush_op(new FlushMRSOp(this));
-  maint_mgr->RegisterOp(mrs_flush_op.get());
-  maintenance_ops.push_back(mrs_flush_op.release());
+  maintenance_ops.emplace_back(new FlushMRSOp(this));
+  maint_mgr->RegisterOp(maintenance_ops.back().get());
 
-  unique_ptr<MaintenanceOp> dms_flush_op(new FlushDeltaMemStoresOp(this));
-  maint_mgr->RegisterOp(dms_flush_op.get());
-  maintenance_ops.push_back(dms_flush_op.release());
+  maintenance_ops.emplace_back(new FlushDeltaMemStoresOp(this));
+  maint_mgr->RegisterOp(maintenance_ops.back().get());
 
-  unique_ptr<MaintenanceOp> log_gc(new LogGCOp(this));
-  maint_mgr->RegisterOp(log_gc.get());
-  maintenance_ops.push_back(log_gc.release());
+  maintenance_ops.emplace_back(new LogGCOp(this));
+  maint_mgr->RegisterOp(maintenance_ops.back().get());
 
   std::shared_ptr<Tablet> tablet;
   {
@@ -956,22 +964,22 @@ void TabletReplica::RegisterMaintenanceOps(MaintenanceManager* maint_mgr) {
 
 void TabletReplica::CancelMaintenanceOpsForTests() {
   std::lock_guard<simple_spinlock> l(lock_);
-  for (MaintenanceOp* op : maintenance_ops_) {
+  for (auto& op : maintenance_ops_) {
     op->CancelAndDisable();
   }
 }
 
 void TabletReplica::UnregisterMaintenanceOps() {
   DCHECK(state_change_lock_.is_locked());
-  vector<MaintenanceOp*> maintenance_ops;
+  decltype(maintenance_ops_) ops;
   {
     std::lock_guard<simple_spinlock> l(lock_);
-    maintenance_ops = std::move(maintenance_ops_);
+    ops = std::move(maintenance_ops_);
+    maintenance_ops_.clear();
   }
-  for (MaintenanceOp* op : maintenance_ops) {
+  for (auto& op : ops) {
     op->Unregister();
   }
-  STLDeleteElements(&maintenance_ops);
 }
 
 size_t TabletReplica::OnDiskSize() const {
@@ -1157,12 +1165,14 @@ void TabletReplica::MakeUnavailable(const Status& error) {
   {
     std::lock_guard<simple_spinlock> lock(lock_);
     tablet = tablet_;
-    for (MaintenanceOp* op : maintenance_ops_) {
+    for (auto& op : maintenance_ops_) {
       op->CancelAndDisable();
     }
   }
   // Stop the Tablet from doing further I/O.
-  if (tablet) tablet->Stop();
+  if (tablet) {
+    tablet->Stop();
+  }
 
   // Set the error; when the replica is shut down, it will end up FAILED.
   SetError(error);
