@@ -24,6 +24,7 @@
 #include <memory>
 #include <mutex>
 #include <string>
+#include <type_traits>
 #include <unordered_map>
 #include <vector>
 
@@ -40,11 +41,12 @@
 #include "kudu/tablet/ops/op.h"
 #include "kudu/tablet/ops/op_tracker.h"
 #include "kudu/tablet/ops/write_op.h"
-#include "kudu/tablet/tablet.h"
+#include "kudu/tablet/tablet.h" // IWYU pragma: keep
 #include "kudu/tablet/tablet_metadata.h"
 #include "kudu/tserver/tserver.pb.h"
 #include "kudu/util/locks.h"
 #include "kudu/util/metrics.h"
+#include "kudu/util/monotime.h"
 #include "kudu/util/status.h"
 
 namespace kudu {
@@ -52,7 +54,6 @@ class AlterTableTest;
 class DnsResolver;
 class MaintenanceManager;
 class MaintenanceOp;
-class MonoDelta;
 class ThreadPool;
 class ThreadPoolToken;
 class TxnOpDispatcherITest;
@@ -82,6 +83,10 @@ class Messenger;
 class ResultTracker;
 } // namespace rpc
 
+namespace tools {
+class TabletCopier;
+} // namespace tools
+
 namespace tablet {
 class AlterSchemaOpState;
 class OpDriver;
@@ -175,8 +180,10 @@ class TabletReplica : public RefCountedThreadSafe<TabletReplica>,
 
   // Submits a write to a tablet and executes it asynchronously.
   // The caller is expected to build and pass a WriteOpState that points to the
-  // RPC's WriteRequest, and WriteResponse.
-  Status SubmitWrite(std::unique_ptr<WriteOpState> op_state);
+  // RPC's WriteRequest, and WriteResponse. The 'deadline' parameter corresponds
+  // to the timeout of the corresponding RPC, if present/specified.
+  Status SubmitWrite(std::unique_ptr<WriteOpState> op_state,
+                     MonoTime deadline = MonoTime::Max());
 
   // Submits an op to update transaction participant state, executing it
   // asynchonously.
@@ -185,7 +192,8 @@ class TabletReplica : public RefCountedThreadSafe<TabletReplica>,
   // Called by the tablet service to start an alter schema op.
   //
   // The op contains all the information required to execute the
-  // AlterSchema operation and send the response back.
+  // AlterSchema operation and send the response back. The 'deadline' parameter
+  // corresponds to the timeout of the corresponding RPC, if present/specified.
   //
   // If the returned Status is OK, the response to the client will be sent
   // asynchronously. Otherwise the tablet service will have to send the response directly.
@@ -193,7 +201,8 @@ class TabletReplica : public RefCountedThreadSafe<TabletReplica>,
   // The AlterSchema operation is taking the tablet component lock in exclusive mode
   // meaning that no other operation on the tablet can be executed while the
   // AlterSchema is in progress.
-  Status SubmitAlterSchema(std::unique_ptr<AlterSchemaOpState> op_state);
+  Status SubmitAlterSchema(std::unique_ptr<AlterSchemaOpState> op_state,
+                           MonoTime deadline = MonoTime::Max());
 
   void GetTabletStatusPB(TabletStatusPB* status_pb_out) const;
 
@@ -303,8 +312,12 @@ class TabletReplica : public RefCountedThreadSafe<TabletReplica>,
   // Convenience method to return the permanent_uuid of this peer.
   std::string permanent_uuid() const { return tablet_->metadata()->fs_manager()->uuid(); }
 
+  // The 'deadline' parameter is used to short-cuircut processing operations
+  // that have timed out while in the prepare queue. For most call sites,
+  // the deadline is naturally defined by the corresponding RPC.
   Status NewLeaderOpDriver(std::unique_ptr<Op> op,
-                           scoped_refptr<OpDriver>* driver);
+                           scoped_refptr<OpDriver>* driver,
+                           MonoTime deadline);
 
   Status NewReplicaOpDriver(std::unique_ptr<Op> op,
                             scoped_refptr<OpDriver>* driver);
@@ -390,6 +403,7 @@ class TabletReplica : public RefCountedThreadSafe<TabletReplica>,
   void BeginTxnParticipantOp(int64_t txn_id, RegisteredTxnCallback began_txn_cb);
 
  private:
+  friend class kudu::tools::TabletCopier;
   friend class kudu::AlterTableTest;
   friend class RefCountedThreadSafe<TabletReplica>;
   friend class TabletReplicaTest;
@@ -400,6 +414,9 @@ class TabletReplica : public RefCountedThreadSafe<TabletReplica>,
   FRIEND_TEST(TabletReplicaTest, TestMRSAnchorPreventsLogGC);
   FRIEND_TEST(kudu::TxnOpDispatcherITest, LifecycleBasic);
 
+  // Only for CLI tools and tests.
+  TabletReplica();
+
   // A class to properly dispatch transactional write operations arriving
   // with TabletServerService::Write() RPC for the specified tablet replica.
   // Before submitting the operations via TabletReplica::SubmitWrite(), it's
@@ -594,9 +611,9 @@ class TabletReplica : public RefCountedThreadSafe<TabletReplica>,
 
   clock::Clock* clock_;
 
-  // List of maintenance operations for the tablet that need information that only the peer
-  // can provide.
-  std::vector<MaintenanceOp*> maintenance_ops_;
+  // Maintenance operations for the tablet that need information that only
+  // the peer can provide.
+  std::vector<std::unique_ptr<MaintenanceOp>> maintenance_ops_;
 
   // The result tracker for writes.
   scoped_refptr<rpc::ResultTracker> result_tracker_;
