@@ -22,6 +22,7 @@
 #include <functional>
 #include <iostream>
 #include <map>
+#include <optional>
 #include <set>
 #include <string>
 #include <unordered_map>
@@ -29,7 +30,6 @@
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 
@@ -591,7 +591,7 @@ Status KsckChecksummer::ChecksumData(const KsckChecksumOptions& opts,
 
   // Clear the contents of 'checksum_results' because we always overwrite it
   // with whatever results are obtained (and with nothing if there's no results).
-  checksum_results->snapshot_timestamp = boost::none;
+  checksum_results->snapshot_timestamp.reset();
   checksum_results->tables.clear();
 
   TabletInfoMap tablet_infos;
