@@ -21,14 +21,16 @@
 #include <cstddef>
 #include <cstdint>
 #include <cstring>
+#include <functional>
 #include <iomanip>
 #include <iostream>
 #include <iterator>
 #include <map>
 #include <memory>
+#include <optional>
 #include <set>
+#include <type_traits>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 #include <rapidjson/document.h>
@@ -48,6 +50,7 @@
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/stl_util.h"
+#include "kudu/gutil/strings/numbers.h"
 #include "kudu/gutil/strings/split.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/util/bitmap.h"
@@ -75,7 +78,10 @@ using kudu::client::KuduValue;
 using kudu::client::KuduWriteOperation;
 using kudu::iequals;
 using std::endl;
+using std::function;
 using std::map;
+using std::nullopt;
+using std::optional;
 using std::ostream;
 using std::ostringstream;
 using std::right;
@@ -88,8 +94,17 @@ using strings::Substitute;
 
 DEFINE_bool(create_table, true,
             "Whether to create the destination table if it doesn't exist.");
+DEFINE_int32(create_table_replication_factor, -1,
+             "The replication factor of the destination table if the table will be created. "
+             "By default, the replication factor of source table will be used.");
+DEFINE_string(create_table_hash_bucket_nums, "",
+              "The number of hash buckets in each hash dimension seperated by comma");
 DEFINE_bool(fill_cache, true,
             "Whether to fill block cache when scanning.");
+DEFINE_bool(fault_tolerant, false,
+            "Whether to make scans resumable at another tablet server if current server fails. "
+            "Fault-tolerant scans typically have lower throughput than non fault-tolerant scans, "
+            "but the results are returned in primary key order for a single tablet.");
 DEFINE_string(predicates, "",
               "Query predicates on columns. Unlike traditional SQL syntax, "
               "the scan tool's simple query predicates are represented in a "
@@ -114,9 +129,12 @@ DEFINE_bool(report_scanner_stats, false,
 DEFINE_bool(show_values, false,
             "Whether to show values of scanned rows.");
 DEFINE_string(write_type, "insert",
-              "How data should be copied to the destination table. Valid values are 'insert', "
-              "'upsert' or the empty string. If the empty string, data will not be copied "
-              "(useful when create_table is 'true').");
+              "Write operation type to use when populating the destination "
+              "table with the rows from the source table. Choose from "
+              "'insert', 'insert_ignore', 'upsert', 'upsert_ignore', or an "
+              "empty string. Empty string means the data isn't going to be "
+              "copied, which is useful with --create_table=true when just "
+              "creating the destination table without copying the data.");
 DEFINE_string(replica_selection, "CLOSEST",
               "Replica selection for scan operations. Acceptable values are: "
               "CLOSEST, LEADER (maps into KuduClient::CLOSEST_REPLICA and "
@@ -150,18 +168,32 @@ bool IsFlagValueAcceptable(const char* flag_name,
   return false;
 }
 
+constexpr const char* const kWriteTypeInsert = "insert";
+constexpr const char* const kWriteTypeInsertIgnore = "insert_ignore";
+constexpr const char* const kWriteTypeUpsert = "upsert";
+constexpr const char* const kWriteTypeUpsertIgnore = "upsert_ignore";
+
 bool ValidateWriteType(const char* flag_name,
                        const string& flag_value) {
-  static const vector<string> kWriteTypes = { "insert", "upsert", "" };
+  static const vector<string> kWriteTypes = {
+    "",
+    kWriteTypeInsert,
+    kWriteTypeInsertIgnore,
+    kWriteTypeUpsert,
+    kWriteTypeUpsertIgnore,
+  };
   return IsFlagValueAcceptable(flag_name, flag_value, kWriteTypes);
 }
 
 constexpr const char* const kReplicaSelectionClosest = "closest";
+constexpr const char* const kReplicaSelectionFirst = "first";
 constexpr const char* const kReplicaSelectionLeader = "leader";
+
 bool ValidateReplicaSelection(const char* flag_name,
                               const string& flag_value) {
   static const vector<string> kReplicaSelections = {
     kReplicaSelectionClosest,
+    kReplicaSelectionFirst,
     kReplicaSelectionLeader,
   };
   return IsFlagValueAcceptable(flag_name, flag_value, kReplicaSelections);
@@ -283,9 +315,9 @@ KuduPredicate* NewInListPredicate(const client::sp::shared_ptr<KuduTable>& table
 Status AddPredicate(const client::sp::shared_ptr<KuduTable>& table,
                     const string& predicate_type,
                     const string& column_name,
-                    const boost::optional<const rapidjson::Value*>& value,
+                    const optional<const rapidjson::Value*>& value,
                     const JsonReader& reader,
-                    KuduScanTokenBuilder& builder) {
+                    KuduScanTokenBuilder* builder) {
   if (predicate_type.empty() || column_name.empty()) {
     return Status::OK();
   }
@@ -302,7 +334,7 @@ Status AddPredicate(const client::sp::shared_ptr<KuduTable>& table,
     case PredicateType::Equality:
     case PredicateType::Range:
       CHECK(value);
-      predicate = NewComparisonPredicate(table, type, predicate_type, column_name, value.get());
+      predicate = NewComparisonPredicate(table, type, predicate_type, column_name, *value);
       break;
     case PredicateType::IsNotNull:
     case PredicateType::IsNull:
@@ -311,20 +343,20 @@ Status AddPredicate(const client::sp::shared_ptr<KuduTable>& table,
       break;
     case PredicateType::InList: {
       CHECK(value);
-      predicate = NewInListPredicate(table, type, column_name, reader, value.get());
+      predicate = NewInListPredicate(table, type, column_name, reader, *value);
       break;
     }
     default:
       return Status::NotSupported(Substitute("not support predicate_type $0", predicate_type));
   }
   CHECK(predicate);
-  RETURN_NOT_OK(builder.AddConjunctPredicate(predicate));
+  RETURN_NOT_OK(builder->AddConjunctPredicate(predicate));
 
   return Status::OK();
 }
 
 Status AddPredicates(const client::sp::shared_ptr<KuduTable>& table,
-                     KuduScanTokenBuilder& builder) {
+                     KuduScanTokenBuilder* builder) {
   if (FLAGS_predicates.empty()) {
     return Status::OK();
   }
@@ -355,8 +387,8 @@ Status AddPredicates(const client::sp::shared_ptr<KuduTable>& table,
       RETURN_NOT_OK(AddPredicate(table,
           elements[0]->GetString(),
           elements[1]->GetString(),
-          elements.size() == 2 ?
-            boost::none : boost::optional<const rapidjson::Value*>(elements[2]),
+          elements.size() == 2 ? nullopt
+                               : optional<const rapidjson::Value*>(elements[2]),
           reader,
           builder));
     } else {
@@ -415,17 +447,60 @@ Status CreateDstTableIfNeeded(const client::sp::shared_ptr<KuduTable>& src_table
   };
 
   // Table schema and replica number.
+  int num_replicas = FLAGS_create_table_replication_factor == -1 ?
+      src_table->num_replicas() : FLAGS_create_table_replication_factor;
   unique_ptr<KuduTableCreator> table_creator(dst_client->NewTableCreator());
   table_creator->table_name(dst_table_name)
       .schema(&dst_table_schema)
-      .num_replicas(src_table->num_replicas());
+      .num_replicas(num_replicas);
 
   // Add hash partition schema.
+  vector<int> hash_bucket_nums;
+  if (!partition_schema.hash_schema().empty()) {
+    vector<string> hash_bucket_nums_str = Split(FLAGS_create_table_hash_bucket_nums,
+                                                ",", strings::SkipEmpty());
+    // FLAGS_create_table_hash_bucket_nums is not defined, set it to -1 defaultly.
+    if (hash_bucket_nums_str.empty()) {
+      for (int i = 0; i < partition_schema.hash_schema().size(); i++) {
+        hash_bucket_nums.push_back(-1);
+      }
+    } else {
+      // If the --create_table_hash_bucket_nums flag is set, the number
+      // of comma-separated elements must be equal to the number of hash schema dimensions.
+      if (partition_schema.hash_schema().size() != hash_bucket_nums_str.size()) {
+        return Status::InvalidArgument("The count of hash bucket numbers must be equal to the "
+                                       "number of hash schema dimensions.");
+      }
+      for (int i = 0; i < hash_bucket_nums_str.size(); i++) {
+        int bucket_num = 0;
+        bool is_number = safe_strto32(hash_bucket_nums_str[i], &bucket_num);
+        if (!is_number) {
+          return Status::InvalidArgument(Substitute("'$0': cannot parse the number "
+                                                    "of hash buckets.",
+                                                    hash_bucket_nums_str[i]));
+        }
+        if (bucket_num < 2) {
+          return Status::InvalidArgument("The number of hash buckets must not be less than 2.");
+        }
+        hash_bucket_nums.push_back(bucket_num);
+      }
+    }
+  }
+
+  if (partition_schema.hash_schema().empty() &&
+      !FLAGS_create_table_hash_bucket_nums.empty()) {
+    return Status::InvalidArgument("There are no hash partitions defined in this table.");
+  }
+
+  int i = 0;
   for (const auto& hash_dimension : partition_schema.hash_schema()) {
+    int num_buckets = hash_bucket_nums[i] != -1 ? hash_bucket_nums[i] :
+                                                  hash_dimension.num_buckets;
     auto hash_columns = convert_column_ids_to_names(hash_dimension.column_ids);
     table_creator->add_hash_partitions(hash_columns,
-                                       hash_dimension.num_buckets,
+                                       num_buckets,
                                        hash_dimension.seed);
+    i++;
   }
 
   // Add range partition schema.
@@ -461,6 +536,12 @@ Status CreateDstTableIfNeeded(const client::sp::shared_ptr<KuduTable>& src_table
     table_creator->add_range_partition(lower.release(), upper.release());
   }
 
+  if (partition_schema.hash_schema().empty() &&
+      partition_schema.range_schema().column_ids.empty()) {
+    // This src table is unpartitioned, just create a table range partitioned on no columns.
+    table_creator->set_range_partition_columns({});
+  }
+
   // Create table.
   RETURN_NOT_OK(table_creator->Create());
   LOG(INFO) << "Table " << dst_table_name << " created successfully";
@@ -468,7 +549,7 @@ Status CreateDstTableIfNeeded(const client::sp::shared_ptr<KuduTable>& src_table
   return Status::OK();
 }
 
-void CheckPendingErrors(const client::sp::shared_ptr<KuduSession>& session) {
+void CheckPendingErrors(KuduSession* session) {
   vector<KuduError*> errors;
   ElementDeleter d(&errors);
   session->GetPendingErrors(&errors, nullptr);
@@ -480,8 +561,8 @@ void CheckPendingErrors(const client::sp::shared_ptr<KuduSession>& session) {
 TableScanner::TableScanner(
     client::sp::shared_ptr<client::KuduClient> client,
     std::string table_name,
-    boost::optional<client::sp::shared_ptr<client::KuduClient>> dst_client,
-    boost::optional<std::string> dst_table_name)
+    optional<client::sp::shared_ptr<client::KuduClient>> dst_client,
+    optional<std::string> dst_table_name)
     : total_count_(0),
       client_(std::move(client)),
       table_name_(std::move(table_name)),
@@ -492,9 +573,9 @@ TableScanner::TableScanner(
   CHECK_OK(SetReplicaSelection(FLAGS_replica_selection));
 }
 
-Status TableScanner::ScanData(const std::vector<kudu::client::KuduScanToken*>& tokens,
-                              const std::function<void(const KuduScanBatch& batch)>& cb) {
-  for (auto token : tokens) {
+Status TableScanner::ScanData(const vector<KuduScanToken*>& tokens,
+                              const function<Status(const KuduScanBatch& batch)>& cb) {
+  for (const auto* token : tokens) {
     Stopwatch sw(Stopwatch::THIS_THREAD);
     sw.start();
 
@@ -512,7 +593,7 @@ Status TableScanner::ScanData(const std::vector<kudu::client::KuduScanToken*>& t
       count += batch.NumRows();
       total_count_ += batch.NumRows();
       ++next_batch_calls;
-      cb(batch);
+      RETURN_NOT_OK(cb(batch));
     }
     sw.stop();
 
@@ -533,6 +614,7 @@ Status TableScanner::ScanData(const std::vector<kudu::client::KuduScanToken*>& t
 }
 
 void TableScanner::ScanTask(const vector<KuduScanToken*>& tokens, Status* thread_status) {
+  DCHECK(thread_status);
   *thread_status = ScanData(tokens, [&](const KuduScanBatch& batch) {
     if (out_ && FLAGS_show_values) {
       MutexLock l(output_lock_);
@@ -541,29 +623,65 @@ void TableScanner::ScanTask(const vector<KuduScanToken*>& tokens, Status* thread
       }
       out_->flush();
     }
+    return Status::OK();
   });
 }
 
 void TableScanner::CopyTask(const vector<KuduScanToken*>& tokens, Status* thread_status) {
+#define TASK_RET_NOT_OK(s) do {   \
+    const Status& _s = (s);       \
+    if (PREDICT_FALSE(!_s.ok())) {\
+      *thread_status = _s;        \
+      return;                     \
+    }                             \
+  } while (0)
+
+  DCHECK(thread_status);
+  KuduWriteOperation::Type op_type;
+  const auto& op_type_str = FLAGS_write_type;
+  if (op_type_str == kWriteTypeInsert) {
+    op_type = KuduWriteOperation::INSERT;
+  } else if (op_type_str == kWriteTypeInsertIgnore) {
+    op_type = KuduWriteOperation::INSERT_IGNORE;
+  } else if (op_type_str == kWriteTypeUpsert) {
+    op_type = KuduWriteOperation::UPSERT;
+  } else if (op_type_str == kWriteTypeUpsertIgnore) {
+    op_type = KuduWriteOperation::UPSERT_IGNORE;
+  } else {
+    *thread_status = Status::InvalidArgument(Substitute(
+        "invalid write operation type: $0", op_type_str));
+    return;
+  }
+
   client::sp::shared_ptr<KuduTable> dst_table;
-  CHECK_OK(dst_client_.get()->OpenTable(*dst_table_name_, &dst_table));
-  const KuduSchema& dst_table_schema = dst_table->schema();
+  TASK_RET_NOT_OK((*dst_client_)->OpenTable(*dst_table_name_, &dst_table));
 
   // One session per thread.
-  client::sp::shared_ptr<KuduSession> session(dst_client_.get()->NewSession());
-  CHECK_OK(session->SetFlushMode(KuduSession::AUTO_FLUSH_BACKGROUND));
-  CHECK_OK(session->SetErrorBufferSpace(1024));
+  client::sp::shared_ptr<KuduSession> session((*dst_client_)->NewSession());
+  TASK_RET_NOT_OK(session->SetFlushMode(KuduSession::AUTO_FLUSH_BACKGROUND));
+  TASK_RET_NOT_OK(session->SetErrorBufferSpace(1024 * 1024));
   session->SetTimeoutMillis(FLAGS_timeout_ms);
 
-  *thread_status = ScanData(tokens, [&](const KuduScanBatch& batch) {
+  // The callback's lambda of ScanData() keeps references to the session and
+  // the destination table objects, making sure they are alive when the callback
+  // is invoked.
+  *thread_status = ScanData(tokens, [table = std::move(dst_table),
+                                     session = std::move(session),
+                                     op_type] (const KuduScanBatch& batch) {
+    auto* s_ptr = session.get();
+    auto* t_ptr = table.get();
     for (const auto& row : batch) {
-      CHECK_OK(AddRow(dst_table, dst_table_schema, row, session));
+      RETURN_NOT_OK(AddRow(s_ptr, t_ptr, row, op_type));
     }
-    CheckPendingErrors(session);
-    // Flush here to make sure all write operations have been sent,
-    // and all strings reference to batch are still valid.
-    CHECK_OK(session->Flush());
+    // Flush the session to make sure all write operations have been sent
+    // to the server. If any error happens, CheckPendingErrors() will report
+    // on them.
+    auto s = s_ptr->Flush();
+    CheckPendingErrors(s_ptr);
+    return s;
   });
+
+#undef TASK_RET_NOT_OK
 }
 
 void TableScanner::SetOutput(ostream* out) {
@@ -575,7 +693,7 @@ void TableScanner::SetReadMode(KuduScanner::ReadMode mode) {
 }
 
 Status TableScanner::SetReplicaSelection(const string& selection_str) {
-  KuduClient::ReplicaSelection selection;
+  KuduClient::ReplicaSelection selection = KuduClient::ReplicaSelection::CLOSEST_REPLICA;
   RETURN_NOT_OK(ParseReplicaSelection(selection_str, &selection));
   replica_selection_ = selection;
   return Status::OK();
@@ -585,12 +703,12 @@ void TableScanner::SetScanBatchSize(int32_t scan_batch_size) {
   scan_batch_size_ = scan_batch_size;
 }
 
-Status TableScanner::StartWork(WorkType type) {
+Status TableScanner::StartWork(WorkType work_type) {
   client::sp::shared_ptr<KuduTable> src_table;
   RETURN_NOT_OK(client_->OpenTable(table_name_, &src_table));
 
   // Create destination table if needed.
-  if (type == WorkType::kCopy) {
+  if (work_type == WorkType::kCopy) {
     RETURN_NOT_OK(CreateDstTableIfNeeded(src_table, *dst_client_, *dst_table_name_));
     if (FLAGS_write_type.empty()) {
       // Create table only.
@@ -601,7 +719,7 @@ Status TableScanner::StartWork(WorkType type) {
   KuduScanTokenBuilder builder(src_table.get());
   RETURN_NOT_OK(builder.SetCacheBlocks(FLAGS_fill_cache));
   if (mode_) {
-    RETURN_NOT_OK(builder.SetReadMode(mode_.get()));
+    RETURN_NOT_OK(builder.SetReadMode(*mode_));
   }
   if (scan_batch_size_ >= 0) {
     // Batch size of 0 is valid and has special semantics: the server sends
@@ -611,9 +729,17 @@ Status TableScanner::StartWork(WorkType type) {
   }
   RETURN_NOT_OK(builder.SetSelection(replica_selection_));
   RETURN_NOT_OK(builder.SetTimeoutMillis(FLAGS_timeout_ms));
+  if (FLAGS_fault_tolerant) {
+    // TODO(yingchun): push down this judgement to ScanConfiguration::SetFaultTolerant
+    if (mode_ && *mode_ != KuduScanner::READ_AT_SNAPSHOT) {
+      return Status::InvalidArgument(Substitute("--fault_tolerant conflicts with "
+          "the non-READ_AT_SNAPSHOT read mode"));
+    }
+    RETURN_NOT_OK(builder.SetFaultTolerant());
+  }
 
   // Set projection if needed.
-  if (type == WorkType::kScan) {
+  if (work_type == WorkType::kScan) {
     const auto project_all = FLAGS_columns == "*" || FLAGS_columns.empty();
     if (!project_all || FLAGS_row_count_only) {
       vector<string> projected_column_names;
@@ -624,42 +750,64 @@ Status TableScanner::StartWork(WorkType type) {
     }
   }
 
+  if (work_type == WorkType::kCopy) {
+    // If we are copying a table we do not want to scan the auto-incrementing column as it would be
+    // populated on the server side. This would avoid scanning an entire column of the table.
+    if (src_table->schema().GetAutoIncrementingColumnIndex() != -1) {
+      vector<string> projected_column_names;
+      for (int i = 0; i < src_table->schema().num_columns(); i++) {
+        if (src_table->schema().Column(i).name() == KuduSchema::GetAutoIncrementingColumnName()) {
+          continue;
+        }
+        projected_column_names.emplace_back(src_table->schema().Column(i).name());
+      }
+      RETURN_NOT_OK(builder.SetProjectedColumnNames(projected_column_names));
+    }
+    // Ensure both the source and destination table schemas are identical at this point.
+    client::sp::shared_ptr<KuduTable> dst_table;
+    RETURN_NOT_OK(dst_client_->get()->OpenTable(*dst_table_name_, &dst_table));
+    if (dst_table->schema() != src_table->schema()) {
+      Status::InvalidArgument("source and destination tables should have the same schema");
+    }
+  }
+
   // Set predicates.
-  RETURN_NOT_OK(AddPredicates(src_table, builder));
+  RETURN_NOT_OK(AddPredicates(src_table, &builder));
 
   vector<KuduScanToken*> tokens;
   ElementDeleter deleter(&tokens);
   RETURN_NOT_OK(builder.Build(&tokens));
 
+  const int num_threads = FLAGS_num_threads;
+
   // Set tablet filter.
   const set<string>& tablet_id_filters = Split(FLAGS_tablets, ",", strings::SkipWhitespace());
   map<int, vector<KuduScanToken*>> thread_tokens;
   int i = 0;
   for (auto* token : tokens) {
     if (tablet_id_filters.empty() || ContainsKey(tablet_id_filters, token->tablet().id())) {
-      thread_tokens[i++ % FLAGS_num_threads].emplace_back(token);
+      thread_tokens[i++ % num_threads].emplace_back(token);
     }
   }
 
-  // Initialize statuses for each thread.
-  vector<Status> thread_statuses(FLAGS_num_threads);
-
   RETURN_NOT_OK(ThreadPoolBuilder("table_scan_pool")
-                  .set_max_threads(FLAGS_num_threads)
+                  .set_max_threads(num_threads)
                   .set_idle_timeout(MonoDelta::FromMilliseconds(1))
                   .Build(&thread_pool_));
 
-  Status end_status = Status::OK();
+  // Initialize statuses for each thread.
+  vector<Status> thread_statuses(num_threads);
+
   Stopwatch sw(Stopwatch::THIS_THREAD);
   sw.start();
-  for (i = 0; i < FLAGS_num_threads; ++i) {
+  for (i = 0; i < num_threads; ++i) {
     auto* t_tokens = &thread_tokens[i];
     auto* t_status = &thread_statuses[i];
-    if (type == WorkType::kScan) {
+    if (work_type == WorkType::kScan) {
       RETURN_NOT_OK(thread_pool_->Submit([this, t_tokens, t_status]()
                                          { this->ScanTask(*t_tokens, t_status); }));
     } else {
-      CHECK(type == WorkType::kCopy);
+      DCHECK(work_type == WorkType::kCopy);
       RETURN_NOT_OK(thread_pool_->Submit([this, t_tokens, t_status]()
                                          { this->CopyTask(*t_tokens, t_status); }));
     }
@@ -675,18 +823,20 @@ Status TableScanner::StartWork(WorkType type) {
         << " cost " << sw.elapsed().wall_seconds() << " seconds" << endl;
   }
 
-  for (i = 0; i < FLAGS_num_threads; ++i) {
-    if (!thread_statuses[i].ok()) {
+  const auto& operation = work_type == WorkType::kScan ? "Scanning" : "Copying";
+  Status result_status;
+  for (const auto& s : thread_statuses) {
+    if (!s.ok()) {
       if (out_) {
-        *out_ << "Scanning failed " << thread_statuses[i].ToString() << endl;
+        *out_ << operation << " failed: " << s.ToString() << endl;
       }
-      if (end_status.ok()) {
-        end_status = thread_statuses[i];
+      if (result_status.ok()) {
+        result_status = s;
       }
     }
   }
 
-  return end_status;
+  return result_status;
 }
 
 Status TableScanner::StartScan() {
@@ -700,23 +850,53 @@ Status TableScanner::StartCopy() {
   return StartWork(WorkType::kCopy);
 }
 
-Status TableScanner::AddRow(const client::sp::shared_ptr<KuduTable>& table,
-                            const KuduSchema& table_schema,
+Status TableScanner::AddRow(KuduSession* session,
+                            KuduTable* table,
                             const KuduScanBatch::RowPtr& src_row,
-                            const client::sp::shared_ptr<KuduSession>& session) {
+                            KuduWriteOperation::Type write_op_type) {
   unique_ptr<KuduWriteOperation> write_op;
-  if (FLAGS_write_type == "insert") {
-    write_op.reset(table->NewInsert());
-  } else if (FLAGS_write_type == "upsert") {
-    write_op.reset(table->NewUpsert());
-  } else {
-    LOG(FATAL) << Substitute("invalid write_type: $0", FLAGS_write_type);
+  switch (write_op_type) {
+    case KuduWriteOperation::INSERT:
+      write_op.reset(table->NewInsert());
+      break;
+    case KuduWriteOperation::INSERT_IGNORE:
+      write_op.reset(table->NewInsertIgnore());
+      break;
+    case KuduWriteOperation::UPSERT:
+      write_op.reset(table->NewUpsert());
+      break;
+    case KuduWriteOperation::UPSERT_IGNORE:
+      write_op.reset(table->NewUpsertIgnore());
+      break;
+    default:
+      return Status::InvalidArgument(
+          Substitute("unexpected op type: $0", write_op_type));
+      break;  // unreachable
   }
 
-  KuduPartialRow* dst_row = write_op->mutable_row();
-  size_t row_size = ContiguousRowHelper::row_size(*src_row.schema_);
-  memcpy(dst_row->row_data_, src_row.row_data_, row_size);
-  BitmapChangeBits(dst_row->isset_bitmap_, 0, table_schema.num_columns(), true);
+  // If the destination table has auto-incrementing column, we do not set it
+  // as we skip scanning the auto-incrementing column while scanning the source table.
+  auto* dst_row = write_op->mutable_row();
+  const int auto_incrementing_col_idx = table->schema().GetAutoIncrementingColumnIndex();
+  if (auto_incrementing_col_idx == Schema::kColumnNotFound) {
+    memcpy(dst_row->row_data_, src_row.row_data_,
+           ContiguousRowHelper::row_size(*src_row.schema_));
+    BitmapChangeBits(dst_row->isset_bitmap_, 0, table->schema().num_columns(), true);
+  } else {
+    int src_iterator = 0;
+    for (int dst_iterator = 0; dst_iterator < table->schema().num_columns(); dst_iterator++) {
+      if (auto_incrementing_col_idx != dst_iterator) {
+        if (src_row.IsNull(src_iterator)) {
+          RETURN_NOT_OK(dst_row->SetNull(dst_iterator));
+        } else {
+          RETURN_NOT_OK(dst_row->Set(dst_iterator, src_row.row_data_ +
+              src_row->schema_->column_offset(src_iterator)));
+        }
+        BitmapChange(dst_row->isset_bitmap_, dst_iterator, true);
+        src_iterator++;
+      }
+    }
+  }
 
   return session->Apply(write_op.release());
 }
@@ -729,6 +909,8 @@ Status TableScanner::ParseReplicaSelection(
     *selection = KuduClient::ReplicaSelection::CLOSEST_REPLICA;
   } else if (iequals(kReplicaSelectionLeader, selection_str)) {
     *selection = KuduClient::ReplicaSelection::LEADER_ONLY;
+  } else if (iequals(kReplicaSelectionFirst, selection_str)) {
+    *selection = KuduClient::ReplicaSelection::FIRST_REPLICA;
   } else {
     return Status::InvalidArgument("invalid replica selection", selection_str);
   }
