@@ -53,8 +53,13 @@
 #include "kudu/consensus/log.pb.h"
 #include "kudu/consensus/log_util.h"
 #include "kudu/consensus/opid.pb.h"
+#include "kudu/fs/fs.pb.h"
+#include "kudu/fs/default_key_provider.h"
+#include "kudu/fs/key_provider.h"
+#include "kudu/fs/ranger_kms_key_provider.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/ref_counted.h"
+#include "kudu/gutil/strings/escaping.h"
 #include "kudu/gutil/strings/join.h"
 #include "kudu/gutil/strings/numbers.h"
 #include "kudu/gutil/strings/split.h"
@@ -96,6 +101,9 @@ DEFINE_bool(force, false, "If true, allows the set_flag command to set a flag "
             "which is not explicitly marked as runtime-settable. Such flag "
             "changes may be simply ignored on the server, or may cause the "
             "server to crash.");
+DEFINE_bool(run_consistency_check, true, "If true, Kudu server checks all flags for consistency "
+            "upon setting a flag. In this mode, the server rolls the flag back to its previous "
+            "value and sends corresponding error response if an inconsistency is detected.");
 DEFINE_bool(print_meta, true, "Include metadata in output");
 DEFINE_string(print_entries, "decoded",
               "How to print entries:\n"
@@ -133,8 +141,7 @@ DEFINE_string(memtracker_output, "table",
               "the memtracker hierarchy.");
 
 DEFINE_int32(num_threads, 2,
-             "Number of threads to run. Each thread runs its own "
-             "KuduSession.");
+             "Number of threads to run.");
 static bool ValidateNumThreads(const char* flag_name, int32_t flag_value) {
   if (flag_value <= 0) {
     LOG(ERROR) << strings::Substitute("'$0' flag should have a positive value",
@@ -161,6 +168,13 @@ DEFINE_bool(row_count_only, false,
 
 DECLARE_bool(show_values);
 
+DEFINE_string(instance_file, "",
+              "Path to the instance file containing the encrypted encryption key.");
+
+DECLARE_string(encryption_key_provider);
+DECLARE_string(ranger_kms_url);
+DECLARE_string(encryption_cluster_key_name);
+
 bool ValidateTimeoutSettings() {
   if (FLAGS_timeout_ms < FLAGS_negotiation_timeout_ms) {
     LOG(ERROR) << strings::Substitute(
@@ -211,6 +225,9 @@ using kudu::rpc::MessengerBuilder;
 using kudu::rpc::RequestIdPB;
 using kudu::rpc::ResponseCallback;
 using kudu::rpc::RpcController;
+using kudu::security::KeyProvider;
+using kudu::security::DefaultKeyProvider;
+using kudu::security::RangerKMSKeyProvider;
 using kudu::server::GenericServiceProxy;
 using kudu::server::GetFlagsRequestPB;
 using kudu::server::GetFlagsResponsePB;
@@ -236,7 +253,9 @@ using std::setw;
 using std::shared_ptr;
 using std::string;
 using std::unique_ptr;
+using std::unordered_set;
 using std::vector;
+using strings::a2b_hex;
 using strings::Split;
 using strings::Substitute;
 
@@ -244,6 +263,7 @@ namespace kudu {
 namespace tools {
 
 const char* const kMasterAddressesArg = "master_addresses";
+const char* const kCurrentLeaderUUIDArg = "current_leader_uuid";
 const char* const kMasterAddressesArgDesc = "Either comma-separated list of Kudu "
     "master addresses where each address is of form 'hostname:port', or a cluster name if it has "
     "been configured in ${KUDU_CONFIG}/kudurc";
@@ -257,6 +277,9 @@ const char* const kTabletIdArgDesc = "Tablet Identifier";
 const char* const kTabletIdsCsvArg = "tablet_ids";
 const char* const kTabletIdsCsvArgDesc =
     "Comma-separated list of Tablet Identifiers";
+const char* const kRowsetIdsCsvArg = "rowset_ids";
+const char* const kRowsetIdsCsvArgDesc =
+    "Comma-separated list of Rowset Identifiers";
 
 const char* const kMasterAddressArg = "master_address";
 const char* const kMasterAddressDesc = "Address of a Kudu Master of form "
@@ -325,8 +348,15 @@ Status PrintDecodedWriteRequestPB(const string& indent,
   Arena arena(32 * 1024);
   RowOperationsPBDecoder dec(&write.row_operations(), &request_schema, &tablet_schema, &arena);
   vector<DecodedRowOperation> ops;
-  RETURN_NOT_OK(dec.DecodeOperations<DecoderMode::WRITE_OPS>(&ops));
-
+  if (write.has_auto_incrementing_column()) {
+    // Define auto_incrementing_counter and use it as in-out parameter during decoding of the ops
+    // in DecodeOperations().
+    int64_t auto_incrementing_counter =
+        write.auto_incrementing_column().auto_incrementing_counter();
+    RETURN_NOT_OK(dec.DecodeOperations<DecoderMode::WRITE_OPS>(&ops, &auto_incrementing_counter));
+  } else {
+    RETURN_NOT_OK(dec.DecodeOperations<DecoderMode::WRITE_OPS>(&ops));
+  }
   cout << indent << "Tablet: " << write.tablet_id() << endl;
   cout << indent << "RequestId: "
       << (request_id ? SecureShortDebugString(*request_id) : "None") << endl;
@@ -335,6 +365,10 @@ Status PrintDecodedWriteRequestPB(const string& indent,
   if (write.has_propagated_timestamp()) {
     cout << indent << "Propagated TS: " << write.propagated_timestamp() << endl;
   }
+  if (write.has_auto_incrementing_column()) {
+    cout << indent << "Auto Incrementing Counter: "
+        << write.auto_incrementing_column().auto_incrementing_counter() << endl;
+  }
 
   int i = 0;
   for (const DecodedRowOperation& op : ops) {
@@ -415,17 +449,39 @@ Status GetServerFlags(const string& address,
   rpc.set_timeout(MonoDelta::FromMilliseconds(FLAGS_timeout_ms));
 
   req.set_all_flags(all_flags);
-  for (StringPiece tag : strings::Split(flag_tags, ",", strings::SkipEmpty())) {
+  const vector<string> filter_tags = strings::Split(flag_tags, ",", strings::SkipEmpty());
+  for (StringPiece tag : filter_tags) {
     req.add_tags(tag.as_string());
   }
-  for (StringPiece flag: strings::Split(flags_to_get, ",", strings::SkipEmpty())) {
+  const unordered_set<string> filter_flags =
+      strings::Split(flags_to_get, ",", strings::SkipEmpty());
+  for (StringPiece flag: filter_flags) {
     req.add_flags(flag.as_string());
   }
 
   RETURN_NOT_OK(proxy->GetFlags(req, &resp, &rpc));
 
   flags->clear();
-  std::move(resp.flags().begin(), resp.flags().end(), std::back_inserter(*flags));
+  // Filter flags according to -flags_to_get and -flags_tags. Currently, the filter
+  // logic has been implemented on the server side, but some old versions do not support
+  // the filter logic on the server side, which makes new version kudu tool can not be
+  // used in old version of Kudu server. Therefore, it is better to implement filter
+  // logic on the kudu tool side again.
+  for (const auto& flag : resp.flags()) {
+    if (!filter_flags.empty() && !ContainsKey(filter_flags, flag.name())) {
+      continue;
+    }
+    bool matches = filter_tags.empty();
+    unordered_set<string> contained_tags(flag.tags().begin(), flag.tags().end());
+    for (const auto& tag : filter_tags) {
+      if (ContainsKey(contained_tags, tag)) {
+        matches = true;
+        break;
+      }
+    }
+    if (!matches) continue;
+    flags->push_back(flag);
+  }
   return Status::OK();
 }
 
@@ -523,6 +579,10 @@ Status GetServerStatus(const string& address, uint16_t default_port,
 Status GetReplicas(TabletServerServiceProxy* proxy,
                    vector<ListTabletsResponsePB::StatusAndSchemaPB>* replicas) {
   ListTabletsRequestPB req;
+  // Even with FLAGS_include_schema=false, don't set need_schema_info=false
+  // in the request. The reason is that the schema is still needed to decode
+  // the partition of each replica, and the partition information is pretty
+  // much always nice to have.
   req.set_need_schema_info(true);
   ListTabletsResponsePB resp;
   RpcController rpc;
@@ -613,6 +673,7 @@ Status SetServerFlag(const string& address, uint16_t default_port,
   req.set_flag(flag);
   req.set_value(value);
   req.set_force(FLAGS_force);
+  req.set_run_consistency_check(FLAGS_run_consistency_check);
 
   RETURN_NOT_OK(proxy->SetFlag(req, &resp, &rpc));
   switch (resp.result()) {
@@ -894,6 +955,39 @@ Status GetKuduToolAbsolutePathSafe(string* path) {
   return Status::OK();
 }
 
+Status SetServerKey() {
+  if (FLAGS_instance_file.empty()) {
+    return Status::OK();
+  }
+
+  InstanceMetadataPB instance;
+  RETURN_NOT_OK_PREPEND(pb_util::ReadPBContainerFromPath(Env::Default(), FLAGS_instance_file,
+                                                         &instance, pb_util::NOT_SENSITIVE),
+                        "Could not open instance file");
+
+  if (string key = instance.server_key();
+      !key.empty()) {
+    unique_ptr<security::KeyProvider> key_provider;
+    if (FLAGS_encryption_key_provider == "ranger-kms"
+        || FLAGS_encryption_key_provider == "ranger_kms") {
+      key_provider.reset(new RangerKMSKeyProvider(FLAGS_ranger_kms_url,
+                                                  FLAGS_encryption_cluster_key_name));
+    } else {
+      key_provider.reset(new DefaultKeyProvider());
+    }
+
+    string server_key;
+    RETURN_NOT_OK(key_provider->DecryptServerKey(instance.server_key(),
+                                                instance.server_key_iv(),
+                                                instance.server_key_version(),
+                                                &server_key));
+    Env::Default()->SetEncryptionKey(reinterpret_cast<const uint8_t*>(a2b_hex(server_key).c_str()),
+                                     key.length() * 4);
+  }
+
+  return Status::OK();
+}
+
 namespace {
 
 // Pretty print a table using the psql format. For example:
