@@ -73,6 +73,8 @@ extern const char* const kTabletIdArg;
 extern const char* const kTabletIdArgDesc;
 extern const char* const kTabletIdsCsvArg;
 extern const char* const kTabletIdsCsvArgDesc;
+extern const char* const kRowsetIdsCsvArg;
+extern const char* const kRowsetIdsCsvArgDesc;
 
 extern const char* const kMasterAddressArg;
 extern const char* const kMasterAddressDesc;
@@ -80,6 +82,8 @@ extern const char* const kMasterAddressDesc;
 extern const char* const kTServerAddressArg;
 extern const char* const kTServerAddressDesc;
 
+extern const char* const kCurrentLeaderUUIDArg;
+
 // Builder for actions involving RPC communications, either with a whole Kudu
 // cluster or a particular Kudu RPC server.
 class RpcActionBuilder : public ActionBuilder {
@@ -241,6 +245,10 @@ Status MasterAddressesToSet(
 // as reported in ConnectToMasterResponsePB::master_addrs.
 Status VerifyMasterAddressList(const std::vector<std::string>& master_addresses);
 
+// Parses the instance file set by the 'instance_file' flag, and sets the
+// server key on the default Env.
+Status SetServerKey();
+
 // A table of data to present to the user.
 //
 // Supports formatting based on the --format flag.
