@@ -36,6 +36,7 @@
 #include <glog/logging.h>
 #include <gtest/gtest.h>
 
+#include "kudu/common/timestamp.h"
 #include "kudu/consensus/raft_consensus.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/ref_counted.h"
@@ -68,6 +69,8 @@ using std::unique_ptr;
 using std::unordered_set;
 using std::vector;
 
+DECLARE_bool(enable_txn_system_client_init);
+DECLARE_bool(txn_schedule_background_tasks);
 DECLARE_uint32(txn_keepalive_interval_ms);
 DECLARE_uint32(txn_staleness_tracker_interval_ms);
 METRIC_DECLARE_entity(tablet);
@@ -91,8 +94,10 @@ class TxnStatusManagerTest : public TabletReplicaTestBase {
   void SetUp() override {
     // Using shorter intervals for transaction staleness tracking to speed up
     // test scenarios verifying related functionality.
+    FLAGS_enable_txn_system_client_init = true;
     FLAGS_txn_keepalive_interval_ms = 200;
     FLAGS_txn_staleness_tracker_interval_ms = 50;
+    FLAGS_txn_schedule_background_tasks = false;
 
     NO_FATALS(TabletReplicaTestBase::SetUp());
     ConsensusBootstrapInfo info;
@@ -101,7 +106,7 @@ class TxnStatusManagerTest : public TabletReplicaTestBase {
   }
 
   Status ResetTxnStatusManager() {
-    txn_manager_.reset(new TxnStatusManager(tablet_replica_.get()));
+    txn_manager_.reset(new TxnStatusManager(tablet_replica_.get(), nullptr, nullptr));
     return txn_manager_->LoadFromTablet();
   }
 
@@ -119,60 +124,62 @@ TEST_F(TxnStatusManagerTest, TestStartTransactions) {
     { 3, { kParticipant1, kParticipant2 } },
   };
 
-  ASSERT_TRUE(txn_manager_->GetParticipantsByTxnIdForTests().empty());
+  {
+    TxnStatusManager::ScopedLeaderSharedLock lock(txn_manager_.get());
+    ASSERT_TRUE(txn_manager_->GetParticipantsByTxnIdForTests().empty());
 
-  TabletServerErrorPB ts_error;
-  for (const auto& txn_id_and_prts : expected_prts_by_txn_id) {
-    const auto& txn_id = txn_id_and_prts.first;
-    int64_t highest_seen_txn_id = -1;
-    ASSERT_OK(txn_manager_->BeginTransaction(
-        txn_id, kOwner, &highest_seen_txn_id, &ts_error));
-    ASSERT_GE(highest_seen_txn_id, 0);
-    ASSERT_EQ(highest_seen_txn_id, txn_id);
-    for (const auto& prt : txn_id_and_prts.second) {
-      ASSERT_OK(txn_manager_->RegisterParticipant(txn_id, prt, kOwner, &ts_error));
+    TabletServerErrorPB ts_error;
+    for (const auto& txn_id_and_prts : expected_prts_by_txn_id) {
+      const auto& txn_id = txn_id_and_prts.first;
+      int64_t highest_seen_txn_id = -1;
+      ASSERT_OK(txn_manager_->BeginTransaction(
+          txn_id, kOwner, &highest_seen_txn_id, &ts_error));
+      ASSERT_GE(highest_seen_txn_id, 0);
+      ASSERT_EQ(highest_seen_txn_id, txn_id);
+      for (const auto& prt : txn_id_and_prts.second) {
+        ASSERT_OK(txn_manager_->RegisterParticipant(txn_id, prt, kOwner, &ts_error));
+      }
+    }
+    // Registering a participant that's already open is harmless, presuming the
+    // participant is still open.
+    ASSERT_OK(txn_manager_->RegisterParticipant(3, kParticipant1, kOwner, &ts_error));
+
+    // Starting a transaction that's already been started should result in an
+    // error, even if it's not currently in flight.
+    {
+      int64_t highest_seen_txn_id = -1;
+      auto s = txn_manager_->BeginTransaction(
+          1, kOwner, &highest_seen_txn_id, &ts_error);
+      ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
+      ASSERT_EQ(3, highest_seen_txn_id);
+    }
+    {
+      int64_t highest_seen_txn_id = -1;
+      auto s = txn_manager_->BeginTransaction(
+          2, kOwner, &highest_seen_txn_id, &ts_error);
+      ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
+      ASSERT_EQ(3, highest_seen_txn_id);
+    }
+
+    // Registering participants to transactions that don't exist should also
+    // result in errors.
+    auto s = txn_manager_->RegisterParticipant(2, kParticipant1, kOwner, &ts_error);
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
+
+    // The underlying participants map should only reflect the successful
+    // operations.
+    ASSERT_EQ(expected_prts_by_txn_id,
+              txn_manager_->GetParticipantsByTxnIdForTests());
+    ASSERT_EQ(3, txn_manager_->highest_txn_id());
+    {
+      // Reload the TxnStatusManager from disk and verify the state.
+      TxnStatusManager txn_manager_reloaded(tablet_replica_.get(), nullptr, nullptr);
+      ASSERT_OK(txn_manager_reloaded.LoadFromTablet());
+      ASSERT_EQ(expected_prts_by_txn_id,
+                txn_manager_reloaded.GetParticipantsByTxnIdForTests());
+      ASSERT_EQ(3, txn_manager_reloaded.highest_txn_id());
     }
   }
-  // Registering a participant that's already open is harmless, presuming the
-  // participant is still open.
-  ASSERT_OK(txn_manager_->RegisterParticipant(3, kParticipant1, kOwner, &ts_error));
-
-  // Starting a transaction that's already been started should result in an
-  // error, even if it's not currently in flight.
-  {
-    int64_t highest_seen_txn_id = -1;
-    auto s = txn_manager_->BeginTransaction(
-        1, kOwner, &highest_seen_txn_id, &ts_error);
-    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
-    ASSERT_EQ(3, highest_seen_txn_id);
-  }
-  {
-    int64_t highest_seen_txn_id = -1;
-    auto s = txn_manager_->BeginTransaction(
-        2, kOwner, &highest_seen_txn_id, &ts_error);
-    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
-    ASSERT_EQ(3, highest_seen_txn_id);
-  }
-
-  // Registering participants to transactions that don't exist should also
-  // result in errors.
-  auto s = txn_manager_->RegisterParticipant(2, kParticipant1, kOwner, &ts_error);
-  ASSERT_TRUE(s.IsNotFound()) << s.ToString();
-
-  // The underlying participants map should only reflect the successful
-  // operations.
-  ASSERT_EQ(expected_prts_by_txn_id,
-            txn_manager_->GetParticipantsByTxnIdForTests());
-  ASSERT_EQ(3, txn_manager_->highest_txn_id());
-  {
-    // Reload the TxnStatusManager from disk and verify the state.
-    TxnStatusManager txn_manager_reloaded(tablet_replica_.get());
-    ASSERT_OK(txn_manager_reloaded.LoadFromTablet());
-    ASSERT_EQ(expected_prts_by_txn_id,
-              txn_manager_reloaded.GetParticipantsByTxnIdForTests());
-    ASSERT_EQ(3, txn_manager_reloaded.highest_txn_id());
-  }
-
   // Now rebuild the underlying replica and rebuild the TxnStatusManager.
   ASSERT_OK(RestartReplica());
   NO_FATALS(ResetTxnStatusManager());
@@ -184,7 +191,8 @@ TEST_F(TxnStatusManagerTest, TestStartTransactions) {
   // if the transaction status tablet's data is not loaded yet.
   ASSERT_OK(RestartReplica());
   {
-    TxnStatusManager tsm(tablet_replica_.get());
+    TxnStatusManager tsm(tablet_replica_.get(), nullptr, nullptr);
+    TxnStatusManager::ScopedLeaderSharedLock lock(&tsm);
     // Check for the special value of the highest_txn_id when the data from
     // the transaction status tablet isn't loaded yet.
     ASSERT_EQ(-2, tsm.highest_txn_id());
@@ -203,7 +211,7 @@ TEST_F(TxnStatusManagerTest, TestStartTransactions) {
       ASSERT_TRUE(s.IsServiceUnavailable());
       ASSERT_STR_CONTAINS(s.ToString(), kErrMsg);
 
-      s = tsm.FinalizeCommitTransaction(txn_id, &ts_error);
+      s = tsm.FinalizeCommitTransaction(txn_id, Timestamp::kInitialTimestamp, &ts_error);
       ASSERT_TRUE(s.IsServiceUnavailable());
       ASSERT_STR_CONTAINS(s.ToString(), kErrMsg);
 
@@ -261,6 +269,7 @@ TEST_F(TxnStatusManagerTest, TestStartTransactionsConcurrently) {
         // Synchronize the threads so we're inserting to a single range at a
         // time.
         barriers[b]->Wait();
+        TxnStatusManager::ScopedLeaderSharedLock leader_shared_lock(txn_manager_.get());
         auto txn_id = txns_to_insert[b][i];
         TabletServerErrorPB ts_error;
         int64_t highest_seen_txn_id = -1;
@@ -308,6 +317,7 @@ TEST_F(TxnStatusManagerTest, TestRegisterParticipantsConcurrently) {
   CountDownLatch begun_txn(1);
   threads.reserve(1 + kParticipantsInParallel);
   threads.emplace_back([&] {
+    TxnStatusManager::ScopedLeaderSharedLock leader_shared_lock(txn_manager_.get());
     TabletServerErrorPB ts_error;
     CHECK_OK(txn_manager_->BeginTransaction(kTxnId, kOwner, nullptr, &ts_error));
     begun_txn.CountDown();
@@ -322,6 +332,7 @@ TEST_F(TxnStatusManagerTest, TestRegisterParticipantsConcurrently) {
         // at least some of the participant registrations succeed.
         begun_txn.Wait();
       }
+      TxnStatusManager::ScopedLeaderSharedLock leader_shared_lock(txn_manager_.get());
       string prt = ParticipantId(i % kUniqueParticipantIds);
       TabletServerErrorPB ts_error;
       Status s = txn_manager_->RegisterParticipant(kTxnId, prt, kOwner, &ts_error);
@@ -357,13 +368,14 @@ TEST_F(TxnStatusManagerTest, TestUpdateStateConcurrently) {
   const int kNumTransactions = 10;
   const int kNumUpdatesInParallel = 20;
   for (int i = 0; i < kNumTransactions; i++) {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
     TabletServerErrorPB ts_error;
     ASSERT_OK(txn_manager_->BeginTransaction(i, kOwner, nullptr, &ts_error));
   }
   typedef std::pair<int64_t, TxnStatePB> IdAndUpdate;
   vector<IdAndUpdate> all_updates;
   for (int i = 0; i < kNumTransactions; i++) {
-    all_updates.emplace_back(std::make_pair(i, TxnStatePB::ABORTED));
+    all_updates.emplace_back(std::make_pair(i, TxnStatePB::ABORT_IN_PROGRESS));
     all_updates.emplace_back(std::make_pair(i, TxnStatePB::COMMIT_IN_PROGRESS));
     all_updates.emplace_back(std::make_pair(i, TxnStatePB::COMMITTED));
   }
@@ -376,17 +388,20 @@ TEST_F(TxnStatusManagerTest, TestUpdateStateConcurrently) {
   // Start a bunch of threads that update transaction states.
   for (int i = 0; i < kNumUpdatesInParallel; i++) {
     threads.emplace_back([&, i] {
+      TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
       const auto& txn_id = updates[i].first;
       TabletServerErrorPB ts_error;
       switch (updates[i].second) {
-        case TxnStatePB::ABORTED:
+        case TxnStatePB::ABORT_IN_PROGRESS:
           statuses[i] = txn_manager_->AbortTransaction(txn_id, kOwner, &ts_error);
           break;
         case TxnStatePB::COMMIT_IN_PROGRESS:
           statuses[i] = txn_manager_->BeginCommitTransaction(txn_id, kOwner, &ts_error);
           break;
         case TxnStatePB::COMMITTED:
-          statuses[i] = txn_manager_->FinalizeCommitTransaction(txn_id, &ts_error);
+          statuses[i] = txn_manager_->FinalizeCommitTransaction(txn_id,
+                                                                Timestamp::kInitialTimestamp,
+                                                                &ts_error);
           break;
         default:
           FAIL() << "bad update";
@@ -407,7 +422,7 @@ TEST_F(TxnStatusManagerTest, TestUpdateStateConcurrently) {
       continue;
     }
     switch (updates[i].second) {
-      case TxnStatePB::ABORTED:
+      case TxnStatePB::ABORT_IN_PROGRESS:
         EmplaceIfNotPresent(&txns_with_abort, txn_id);
         break;
       case TxnStatePB::COMMIT_IN_PROGRESS:
@@ -438,65 +453,87 @@ TEST_F(TxnStatusManagerTest, TestUpdateStateConcurrently) {
 // TxnStatusManager::GetTransactionStatus() method.
 TEST_F(TxnStatusManagerTest, GetTransactionStatus) {
   {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
     TxnStatusEntryPB txn_status;
     TabletServerErrorPB ts_error;
     auto s = txn_manager_->GetTransactionStatus(
         1, kOwner, &txn_status, &ts_error);
-    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
     ASSERT_STR_CONTAINS(s.ToString(), "transaction ID 1 not found");
     ASSERT_FALSE(txn_status.has_state());
     ASSERT_FALSE(txn_status.has_user());
   }
+  constexpr const int64_t kNoTxnId = 0;
+  constexpr const int64_t kCommittedTxnId = 1;
+  constexpr const int64_t kAbortInProgressTxnId = 2;
+  constexpr const int64_t kCommitInProgressTxnId = 3;
+  constexpr const int64_t kOpenTxnId = 4;
   {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
     TabletServerErrorPB ts_error;
-    ASSERT_OK(txn_manager_->BeginTransaction(1, kOwner, nullptr, &ts_error));
+    ASSERT_OK(txn_manager_->BeginTransaction(kCommittedTxnId, kOwner, nullptr, &ts_error));
 
     TxnStatusEntryPB txn_status;
     ASSERT_OK(txn_manager_->GetTransactionStatus(
-        1, kOwner, &txn_status, &ts_error));
+        kCommittedTxnId, kOwner, &txn_status, &ts_error));
     ASSERT_TRUE(txn_status.has_state());
     ASSERT_EQ(TxnStatePB::OPEN, txn_status.state());
     ASSERT_TRUE(txn_status.has_user());
     ASSERT_EQ(kOwner, txn_status.user());
 
-    ASSERT_OK(txn_manager_->BeginCommitTransaction(1, kOwner, &ts_error));
+    ASSERT_OK(txn_manager_->BeginCommitTransaction(kCommittedTxnId, kOwner, &ts_error));
     ASSERT_OK(txn_manager_->GetTransactionStatus(
-        1, kOwner, &txn_status, &ts_error));
+        kCommittedTxnId, kOwner, &txn_status, &ts_error));
     ASSERT_TRUE(txn_status.has_state());
     ASSERT_EQ(TxnStatePB::COMMIT_IN_PROGRESS, txn_status.state());
     ASSERT_TRUE(txn_status.has_user());
     ASSERT_EQ(kOwner, txn_status.user());
 
-    ASSERT_OK(txn_manager_->FinalizeCommitTransaction(1, &ts_error));
+    ASSERT_OK(txn_manager_->FinalizeCommitTransaction(kCommittedTxnId,
+                                                      Timestamp::kInitialTimestamp, &ts_error));
     ASSERT_OK(txn_manager_->GetTransactionStatus(
-        1, kOwner, &txn_status, &ts_error));
+        kCommittedTxnId, kOwner, &txn_status, &ts_error));
     ASSERT_TRUE(txn_status.has_state());
+    ASSERT_TRUE(txn_status.has_commit_timestamp());
+    ASSERT_EQ(TxnStatePB::FINALIZE_IN_PROGRESS, txn_status.state());
+    ASSERT_TRUE(txn_status.has_user());
+    ASSERT_EQ(kOwner, txn_status.user());
+
+    ASSERT_OK(txn_manager_->CompleteCommitTransaction(kCommittedTxnId));
+    ASSERT_OK(txn_manager_->GetTransactionStatus(
+        kCommittedTxnId, kOwner, &txn_status, &ts_error));
+    ASSERT_TRUE(txn_status.has_state());
+    ASSERT_TRUE(txn_status.has_commit_timestamp());
     ASSERT_EQ(TxnStatePB::COMMITTED, txn_status.state());
     ASSERT_TRUE(txn_status.has_user());
     ASSERT_EQ(kOwner, txn_status.user());
   }
 
   {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
     TabletServerErrorPB ts_error;
-    ASSERT_OK(txn_manager_->BeginTransaction(2, kOwner, nullptr, &ts_error));
-    ASSERT_OK(txn_manager_->AbortTransaction(2, kOwner, &ts_error));
+    ASSERT_OK(txn_manager_->BeginTransaction(kAbortInProgressTxnId, kOwner, nullptr, &ts_error));
+    ASSERT_OK(txn_manager_->AbortTransaction(kAbortInProgressTxnId, kOwner, &ts_error));
 
     TxnStatusEntryPB txn_status;
     ASSERT_OK(txn_manager_->GetTransactionStatus(
-        2, kOwner, &txn_status, &ts_error));
+        kAbortInProgressTxnId, kOwner, &txn_status, &ts_error));
     ASSERT_TRUE(txn_status.has_state());
-    ASSERT_EQ(TxnStatePB::ABORTED, txn_status.state());
+    ASSERT_EQ(TxnStatePB::ABORT_IN_PROGRESS, txn_status.state());
     ASSERT_TRUE(txn_status.has_user());
     ASSERT_EQ(kOwner, txn_status.user());
   }
 
-  // Start another transaction and start its commit phase.
-  TabletServerErrorPB ts_error;
-  ASSERT_OK(txn_manager_->BeginTransaction(3, kOwner, nullptr, &ts_error));
-  ASSERT_OK(txn_manager_->BeginCommitTransaction(3, kOwner, &ts_error));
+  {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
+    // Start another transaction and start its commit phase.
+    TabletServerErrorPB ts_error;
+    ASSERT_OK(txn_manager_->BeginTransaction(kCommitInProgressTxnId, kOwner, nullptr, &ts_error));
+    ASSERT_OK(txn_manager_->BeginCommitTransaction(kCommitInProgressTxnId, kOwner, &ts_error));
 
-  // Start just another transaction.
-  ASSERT_OK(txn_manager_->BeginTransaction(4, kOwner, nullptr, &ts_error));
+    // Start just another transaction.
+    ASSERT_OK(txn_manager_->BeginTransaction(kOpenTxnId, kOwner, nullptr, &ts_error));
+  }
 
   // Make the TxnStatusManager start from scratch.
   ASSERT_OK(RestartReplica());
@@ -506,31 +543,32 @@ TEST_F(TxnStatusManagerTest, GetTransactionStatus) {
   // TxnStatusManager even after restarting the underlying replica and
   // rebuilding the TxnStatusManager from scratch.
   {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
     TxnStatusEntryPB txn_status;
     TabletServerErrorPB ts_error;
     ASSERT_OK(txn_manager_->GetTransactionStatus(
-        1, kOwner, &txn_status, &ts_error));
+        kCommittedTxnId, kOwner, &txn_status, &ts_error));
     ASSERT_TRUE(txn_status.has_state());
     ASSERT_EQ(TxnStatePB::COMMITTED, txn_status.state());
     ASSERT_TRUE(txn_status.has_user());
     ASSERT_EQ(kOwner, txn_status.user());
 
     ASSERT_OK(txn_manager_->GetTransactionStatus(
-        2, kOwner, &txn_status, &ts_error));
+        kAbortInProgressTxnId, kOwner, &txn_status, &ts_error));
     ASSERT_TRUE(txn_status.has_state());
-    ASSERT_EQ(TxnStatePB::ABORTED, txn_status.state());
+    ASSERT_EQ(TxnStatePB::ABORT_IN_PROGRESS, txn_status.state());
     ASSERT_TRUE(txn_status.has_user());
     ASSERT_EQ(kOwner, txn_status.user());
 
     ASSERT_OK(txn_manager_->GetTransactionStatus(
-        3, kOwner, &txn_status, &ts_error));
+        kCommitInProgressTxnId, kOwner, &txn_status, &ts_error));
     ASSERT_TRUE(txn_status.has_state());
     ASSERT_EQ(TxnStatePB::COMMIT_IN_PROGRESS, txn_status.state());
     ASSERT_TRUE(txn_status.has_user());
     ASSERT_EQ(kOwner, txn_status.user());
 
     ASSERT_OK(txn_manager_->GetTransactionStatus(
-        4, kOwner, &txn_status, &ts_error));
+        kOpenTxnId, kOwner, &txn_status, &ts_error));
     ASSERT_TRUE(txn_status.has_state());
     ASSERT_EQ(TxnStatePB::OPEN, txn_status.state());
     ASSERT_TRUE(txn_status.has_user());
@@ -539,29 +577,32 @@ TEST_F(TxnStatusManagerTest, GetTransactionStatus) {
 
   // Supplying wrong user.
   {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
     TxnStatusEntryPB txn_status;
     TabletServerErrorPB ts_error;
     auto s = txn_manager_->GetTransactionStatus(
-        1, "stranger", &txn_status, &ts_error);
+        kCommittedTxnId, "stranger", &txn_status, &ts_error);
     ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
   }
 
   // Supplying not-yet-used transaction ID.
   {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
     TxnStatusEntryPB txn_status;
     TabletServerErrorPB ts_error;
     auto s = txn_manager_->GetTransactionStatus(
-        0, kOwner, &txn_status, &ts_error);
-    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+        kNoTxnId, kOwner, &txn_status, &ts_error);
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
   }
 
   // Supplying wrong user and not-yet-used transaction ID.
   {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
     TxnStatusEntryPB txn_status;
     TabletServerErrorPB ts_error;
     auto s = txn_manager_->GetTransactionStatus(
-        0, "stranger", &txn_status, &ts_error);
-    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+        kNoTxnId, "stranger", &txn_status, &ts_error);
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
   }
 }
 
@@ -571,24 +612,29 @@ TEST_F(TxnStatusManagerTest, GetTransactionStatus) {
 TEST_F(TxnStatusManagerTest, KeepTransactionAlive) {
   // Supplying not-yet-registered transaction ID.
   {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
     TabletServerErrorPB ts_error;
     auto s = txn_manager_->KeepTransactionAlive(1, kOwner, &ts_error);
-    ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+    ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
     ASSERT_STR_CONTAINS(s.ToString(), "transaction ID 1 not found");
   }
 
-  // OPEN --> COMMIT_IN_PROGRESS --> COMMITTED
+  const auto not_authorized_keep_alive = [&] (const int64_t txn_id) {
+    TabletServerErrorPB ts_error;
+    auto s = txn_manager_->KeepTransactionAlive(txn_id, "stranger", &ts_error);
+    ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
+    ASSERT_STR_CONTAINS(s.ToString(),
+                        Substitute("transaction ID $0 not owned by stranger", txn_id));
+  };
+
+  // OPEN --> COMMIT_IN_PROGRESS --> FINALIZE_IN_PROGRESS
   {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
     TabletServerErrorPB ts_error;
     ASSERT_OK(txn_manager_->BeginTransaction(1, kOwner, nullptr, &ts_error));
     ASSERT_OK(txn_manager_->KeepTransactionAlive(1, kOwner, &ts_error));
     // Supplying wrong user for transaction in OPEN state.
-    {
-      auto s = txn_manager_->KeepTransactionAlive(1, "stranger", &ts_error);
-      ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
-      ASSERT_STR_CONTAINS(s.ToString(),
-                          "transaction ID 1 not owned by stranger");
-    }
+    NO_FATALS(not_authorized_keep_alive(1));
 
     ASSERT_OK(txn_manager_->BeginCommitTransaction(1, kOwner, &ts_error));
     auto s = txn_manager_->KeepTransactionAlive(1, kOwner, &ts_error);
@@ -596,29 +642,28 @@ TEST_F(TxnStatusManagerTest, KeepTransactionAlive) {
     ASSERT_STR_CONTAINS(s.ToString(),
                         "transaction ID 1 is in commit phase");
     // Supplying wrong user for transaction in COMMIT_IN_PROGRESS state.
-    {
-      auto s = txn_manager_->KeepTransactionAlive(1, "stranger", &ts_error);
-      ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
-      ASSERT_STR_CONTAINS(s.ToString(),
-                          "transaction ID 1 not owned by stranger");
-    }
+    NO_FATALS(not_authorized_keep_alive(1));
 
-    ASSERT_OK(txn_manager_->FinalizeCommitTransaction(1, &ts_error));
+    ASSERT_OK(txn_manager_->FinalizeCommitTransaction(1, Timestamp::kInitialTimestamp, &ts_error));
     s = txn_manager_->KeepTransactionAlive(1, kOwner, &ts_error);
     ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
     ASSERT_STR_CONTAINS(s.ToString(),
-                        "transaction ID 1 is already in terminal state");
+                        "transaction ID 1 is in commit phase");
+    // Supplying wrong user for transaction in FINALIZE_IN_PROGRESS state.
+    NO_FATALS(not_authorized_keep_alive(1));
+
+    ASSERT_OK(txn_manager_->CompleteCommitTransaction(1));
+    s = txn_manager_->KeepTransactionAlive(1, kOwner, &ts_error);
+    ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
+    ASSERT_STR_CONTAINS(s.ToString(),
+                        "transaction ID 1 is not available for further commits");
     // Supplying wrong user for transaction in COMMITTED state.
-    {
-      auto s = txn_manager_->KeepTransactionAlive(1, "stranger", &ts_error);
-      ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
-      ASSERT_STR_CONTAINS(s.ToString(),
-                          "transaction ID 1 not owned by stranger");
-    }
+    NO_FATALS(not_authorized_keep_alive(1));
   }
 
-  // OPEN --> COMMIT_IN_PROGRESS --> ABORTED
+  // OPEN --> COMMIT_IN_PROGRESS --> ABORT_IN_PROGRESS
   {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
     TabletServerErrorPB ts_error;
     ASSERT_OK(txn_manager_->BeginTransaction(2, kOwner, nullptr, &ts_error));
     ASSERT_OK(txn_manager_->KeepTransactionAlive(2, kOwner, &ts_error));
@@ -633,17 +678,14 @@ TEST_F(TxnStatusManagerTest, KeepTransactionAlive) {
     s = txn_manager_->KeepTransactionAlive(2, kOwner, &ts_error);
     ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
     ASSERT_STR_CONTAINS(s.ToString(),
-                        "transaction ID 2 is already in terminal state");
+                        "transaction ID 2 is not available for further commits");
     // Supplying wrong user for transaction in ABORTED state.
-    {
-      auto s = txn_manager_->KeepTransactionAlive(2, "stranger", &ts_error);
-      ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
-      ASSERT_STR_CONTAINS(s.ToString(), "transaction ID 2 not owned by stranger");
-    }
+    NO_FATALS(not_authorized_keep_alive(2));
   }
 
-  // OPEN --> ABORTED
+  // OPEN --> ABORT_IN_PROGRESS
   {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
     TabletServerErrorPB ts_error;
     ASSERT_OK(txn_manager_->BeginTransaction(3, kOwner, nullptr, &ts_error));
     ASSERT_OK(txn_manager_->KeepTransactionAlive(3, kOwner, &ts_error));
@@ -652,11 +694,12 @@ TEST_F(TxnStatusManagerTest, KeepTransactionAlive) {
     auto s = txn_manager_->KeepTransactionAlive(3, kOwner, &ts_error);
     ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
     ASSERT_STR_CONTAINS(s.ToString(),
-                        "transaction ID 3 is already in terminal state");
+                        "transaction ID 3 is not available for further commits");
   }
 
   // Open a new transaction just before restarting the TxnStatusManager.
   {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
     TabletServerErrorPB ts_error;
     ASSERT_OK(txn_manager_->BeginTransaction(4, kOwner, nullptr, &ts_error));
   }
@@ -670,47 +713,37 @@ TEST_F(TxnStatusManagerTest, KeepTransactionAlive) {
   // rebuilding the TxnStatusManager from scratch, so KeepTransactionAlive()
   // should behave the same as if no restart has happened.
   {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
     TabletServerErrorPB ts_error;
     auto s = txn_manager_->KeepTransactionAlive(1, kOwner, &ts_error);
     ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
     ASSERT_STR_CONTAINS(s.ToString(),
-                        "transaction ID 1 is already in terminal state");
-    // Supplying wrong user for transaction in COMMITTED state.
-    {
-      auto s = txn_manager_->KeepTransactionAlive(1, "stranger", &ts_error);
-      ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
-      ASSERT_STR_CONTAINS(s.ToString(),
-                          "transaction ID 1 not owned by stranger");
-    }
+                        "transaction ID 1 is not available for further commits");
+    // Supplying wrong user for transaction in FINALIZE_IN_PROGRESS state.
+    NO_FATALS(not_authorized_keep_alive(1));
   }
   {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
     TabletServerErrorPB ts_error;
     auto s = txn_manager_->KeepTransactionAlive(2, kOwner, &ts_error);
     ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
     ASSERT_STR_CONTAINS(s.ToString(),
-                        "transaction ID 2 is already in terminal state");
-    // Supplying wrong user for transaction in ABORTED state.
-    {
-      auto s = txn_manager_->KeepTransactionAlive(2, "stranger", &ts_error);
-      ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
-      ASSERT_STR_CONTAINS(s.ToString(), "transaction ID 2 not owned by stranger");
-    }
+                        "transaction ID 2 is not available for further commits");
+    // Supplying wrong user for transaction in ABORT_IN_PROGRESS state.
+    NO_FATALS(not_authorized_keep_alive(2));
   }
   {
+    TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
     TabletServerErrorPB ts_error;
     ASSERT_OK(txn_manager_->KeepTransactionAlive(4, kOwner, &ts_error));
     // Supplying wrong user for transaction in OPEN state.
-    {
-      auto s = txn_manager_->KeepTransactionAlive(4, "stranger", &ts_error);
-      ASSERT_TRUE(s.IsNotAuthorized()) << s.ToString();
-      ASSERT_STR_CONTAINS(s.ToString(),
-                          "transaction ID 4 not owned by stranger");
-    }
+    NO_FATALS(not_authorized_keep_alive(4));
   }
 }
 
 // Test that performing actions as the wrong user will return errors.
 TEST_F(TxnStatusManagerTest, TestWrongUser) {
+  TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
   const string kWrongUser = "stranger";
   int64_t highest_seen_txn_id = -1;
   TabletServerErrorPB ts_error;
@@ -744,6 +777,7 @@ TEST_F(TxnStatusManagerTest, TestWrongUser) {
 // Test that we can only update a transaction's state when it's in an
 // appropriate state.
 TEST_F(TxnStatusManagerTest, TestUpdateTransactionState) {
+  TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
   const int64_t kTxnId1 = 1;
   TabletServerErrorPB ts_error;
   ASSERT_OK(txn_manager_->BeginTransaction(kTxnId1, kOwner, nullptr, &ts_error));
@@ -757,39 +791,54 @@ TEST_F(TxnStatusManagerTest, TestUpdateTransactionState) {
   // We can't begin or finalize a commit if we've aborted.
   Status s = txn_manager_->BeginCommitTransaction(kTxnId1, kOwner, &ts_error);
   ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
-  s = txn_manager_->FinalizeCommitTransaction(kTxnId1, &ts_error);
+  s = txn_manager_->FinalizeCommitTransaction(kTxnId1, Timestamp::kInitialTimestamp, &ts_error);
+  ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
+  s = txn_manager_->CompleteCommitTransaction(kTxnId1);
   ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
 
   // We can't finalize a commit that hasn't begun committing.
   const int64_t kTxnId2 = 2;
   ASSERT_OK(txn_manager_->BeginTransaction(kTxnId2, kOwner, nullptr, &ts_error));
-  s = txn_manager_->FinalizeCommitTransaction(kTxnId2, &ts_error);
+  s = txn_manager_->FinalizeCommitTransaction(kTxnId2, Timestamp::kInitialTimestamp, &ts_error);
+  ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
+  s = txn_manager_->CompleteCommitTransaction(kTxnId2);
   ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
 
   // We can't abort a transaction that has finished committing.
   ASSERT_OK(txn_manager_->BeginCommitTransaction(kTxnId2, kOwner, &ts_error));
-  ASSERT_OK(txn_manager_->FinalizeCommitTransaction(kTxnId2, &ts_error));
+  ASSERT_OK(txn_manager_->FinalizeCommitTransaction(
+      kTxnId2, Timestamp::kInitialTimestamp, &ts_error));
   s = txn_manager_->AbortTransaction(kTxnId2, kOwner, &ts_error);
   ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
 
-  // Redundant finalize calls are also benign.
-  ASSERT_OK(txn_manager_->FinalizeCommitTransaction(kTxnId2, &ts_error));
+  // Redundant begin commit or finalize calls are also benign.
+  ASSERT_OK(txn_manager_->BeginCommitTransaction(kTxnId2, kOwner, &ts_error));
+  ASSERT_OK(txn_manager_->FinalizeCommitTransaction(
+      kTxnId2, Timestamp::kInitialTimestamp, &ts_error));
 
-  // Calls to begin committing should return an error if we've already
-  // finalized the commit.
-  s = txn_manager_->BeginCommitTransaction(kTxnId2, kOwner, &ts_error);
+  // We can't abort a transaction that has been completely committed.
+  ASSERT_OK(txn_manager_->CompleteCommitTransaction(kTxnId2));
+  s = txn_manager_->AbortTransaction(kTxnId2, kOwner, &ts_error);
   ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
+
+  // Redundant begin commit, finalize calls, and complete calls are also
+  // benign.
+  ASSERT_OK(txn_manager_->BeginCommitTransaction(kTxnId2, kOwner, &ts_error));
+  ASSERT_OK(txn_manager_->FinalizeCommitTransaction(
+      kTxnId2, Timestamp::kInitialTimestamp, &ts_error));
+  ASSERT_OK(txn_manager_->CompleteCommitTransaction(kTxnId2));
 }
 
 // Test that we can only add participants to a transaction when it's in an
 // appropriate state.
 TEST_F(TxnStatusManagerTest, TestRegisterParticipantsWithStates) {
+  TxnStatusManager::ScopedLeaderSharedLock l(txn_manager_.get());
   TabletServerErrorPB ts_error;
   const int64_t kTxnId1 = 1;
 
   // We can't register a participant to a transaction that hasn't started.
   Status s = txn_manager_->RegisterParticipant(kTxnId1, ParticipantId(1), kOwner, &ts_error);
-  ASSERT_TRUE(s.IsNotFound()) << s.ToString();
+  ASSERT_TRUE(s.IsInvalidArgument()) << s.ToString();
 
   ASSERT_OK(txn_manager_->BeginTransaction(kTxnId1, kOwner, nullptr, &ts_error));
   ASSERT_OK(txn_manager_->RegisterParticipant(kTxnId1, ParticipantId(1), kOwner, &ts_error));
@@ -802,8 +851,14 @@ TEST_F(TxnStatusManagerTest, TestRegisterParticipantsWithStates) {
   s = txn_manager_->RegisterParticipant(kTxnId1, ParticipantId(2), kOwner, &ts_error);
   ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
 
-  // We can't register participants when we've finished committnig.
-  ASSERT_OK(txn_manager_->FinalizeCommitTransaction(kTxnId1, &ts_error));
+  // We can't register participants when we've finalized the commit.
+  ASSERT_OK(txn_manager_->FinalizeCommitTransaction(
+      kTxnId1, Timestamp::kInitialTimestamp, &ts_error));
+  s = txn_manager_->RegisterParticipant(kTxnId1, ParticipantId(2), kOwner, &ts_error);
+  ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
+
+  // We can't register participants when we've completely committed.
+  ASSERT_OK(txn_manager_->CompleteCommitTransaction(kTxnId1));
   s = txn_manager_->RegisterParticipant(kTxnId1, ParticipantId(2), kOwner, &ts_error);
   ASSERT_TRUE(s.IsIllegalState()) << s.ToString();
 
