@@ -23,13 +23,16 @@
 #include <functional>
 #include <memory>
 #include <mutex>
+#include <optional>
 #include <ostream>
 #include <string>
+#include <tuple>
+#include <type_traits>
 #include <unordered_map>
+#include <unordered_set>
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 #include <google/protobuf/stubs/common.h>
@@ -42,6 +45,7 @@
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/port.h"
 #include "kudu/gutil/ref_counted.h"
+#include "kudu/gutil/strings/join.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/master/master.pb.h"
 #include "kudu/master/master.proxy.h"
@@ -106,6 +110,7 @@ TAG_FLAG(heartbeat_inject_required_feature_flag, unsafe);
 DECLARE_bool(raft_prepare_replacement_before_eviction);
 
 using kudu::consensus::ReplicaManagementInfoPB;
+using kudu::master::MasterFeatures_Name;
 using kudu::master::MasterErrorPB;
 using kudu::master::MasterFeatures;
 using kudu::master::MasterServiceProxy;
@@ -336,12 +341,32 @@ void Heartbeater::Thread::SetupCommonField(master::TSToMasterCommonPB* common) {
 Status Heartbeater::Thread::SetupRegistration(ServerRegistrationPB* reg) {
   reg->Clear();
 
-  vector<HostPort> hps;
-  RETURN_NOT_OK(server_->rpc_server()->GetAdvertisedHostPorts(&hps));
-  for (const auto& hp : hps) {
-    auto* pb = reg->add_rpc_addresses();
-    pb->set_host(hp.host());
-    pb->set_port(hp.port());
+  // Populate the RPC addresses for communicating within the internal network.
+  {
+    vector<HostPort> hps;
+    RETURN_NOT_OK(server_->rpc_server()->GetAdvertisedHostPorts(&hps));
+    for (const auto& hp : hps) {
+      auto* pb = reg->add_rpc_addresses();
+      pb->set_host(hp.host());
+      pb->set_port(hp.port());
+    }
+  }
+
+  // Populate the RPC addresses for communicating with clients from
+  // external networks.
+  {
+    const auto* srv = server_->rpc_server();
+    const auto& hps = srv->GetProxyAdvertisedHostPorts();
+    for (const auto& hp : hps) {
+      auto* pb = reg->add_rpc_proxy_addresses();
+      pb->set_host(hp.host());
+      pb->set_port(hp.port());
+    }
+    // The RPC proxy-advertised addresses and the proxied RPC addresses should
+    // be either both empty or both non-empty. However, the number addresses
+    // a server uses to process proxied RPCs might differ from the number of
+    // advertised RPC addresses at proxy.
+    DCHECK_EQ(hps.empty(), srv->GetRpcProxiedAddresses().empty());
   }
   // Now fetch any UNIX domain sockets.
   vector<Sockaddr> addrs;
@@ -450,9 +475,7 @@ Status Heartbeater::Thread::DoHeartbeat(MasterErrorPB* error,
 
   // Check with the TS cert manager if it has a cert that needs signing.
   // If so, send the CSR in the heartbeat for the master to sign.
-  boost::optional<security::CertSignRequest> csr =
-      server_->mutable_tls_context()->GetCsrIfNecessary();
-  if (csr != boost::none) {
+  if (auto csr = server_->mutable_tls_context()->GetCsrIfNecessary(); csr) {
     RETURN_NOT_OK(csr->ToString(req.mutable_csr_der(), security::DataFormat::DER));
     VLOG(1) << "Sending a CSR to the master in the next heartbeat";
   }
@@ -484,6 +507,19 @@ Status Heartbeater::Thread::DoHeartbeat(MasterErrorPB* error,
   auto num_live_tablets_by_dimension = server_->tablet_manager()->GetNumLiveTabletsByDimension();
   req.mutable_num_live_tablets_by_dimension()->insert(num_live_tablets_by_dimension.begin(),
                                                       num_live_tablets_by_dimension.end());
+  auto num_live_tablets_by_range_and_table =
+      server_->tablet_manager()->GetNumLiveTabletsByRangePerTable();
+  for (const auto& [table, ranges] : num_live_tablets_by_range_and_table) {
+    master::TabletsByRangePerTablePB table_pb;
+    table_pb.mutable_num_live_tablets_by_range()->Reserve(ranges.size());
+    for (const auto& range : ranges) {
+      master::TabletsByRangePB* range_pb = table_pb.add_num_live_tablets_by_range();
+      range_pb->set_range_start_key(range.first);
+      range_pb->set_tablets(range.second);
+    }
+    auto pair = google::protobuf::MapPair(table, table_pb);
+    req.mutable_num_live_tablets_by_range_per_table()->insert(pair);
+  }
 
   VLOG(2) << "Sending heartbeat:\n" << SecureDebugString(req);
   master::TSHeartbeatResponsePB resp;
@@ -615,7 +651,13 @@ void Heartbeater::Thread::RunThread() {
         msg = Substitute("master detected incompatibility: $0", err_msg);
       }
       if (s.IsRemoteError() && error_status.unsupported_feature_flags_size() > 0) {
-        msg = Substitute("master does not support required feature flags: $0", err_msg);
+        const auto required_features_str = JoinMapped(
+            error_status.unsupported_feature_flags(),
+            [](uint32_t feature) {
+              return MasterFeatures_Name(feature);
+            }, ",");
+        msg = Substitute("master does not support required feature flag(s) $0: $1",
+                         required_features_str, err_msg);
       }
       if (!msg.empty()) {
         if (FLAGS_heartbeat_incompatible_replica_management_is_fatal) {
