@@ -21,17 +21,19 @@
 #include <functional>
 #include <memory>
 #include <mutex>
+#include <optional>
 #include <ostream>
 #include <set>
 #include <string>
+#include <type_traits>
 #include <utility>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <gflags/gflags.h>
 #include <glog/logging.h>
 
 #include "kudu/common/common.pb.h"
+#include "kudu/common/partition.h"
 #include "kudu/common/wire_protocol.h"
 #include "kudu/common/wire_protocol.pb.h"
 #include "kudu/consensus/consensus.pb.h"
@@ -53,7 +55,6 @@
 #include "kudu/rpc/result_tracker.h"
 #include "kudu/server/rpc_server.h"
 #include "kudu/tablet/metadata.pb.h"
-#include "kudu/tablet/tablet.h"
 #include "kudu/tablet/tablet_bootstrap.h"
 #include "kudu/tablet/tablet_metadata.h"
 #include "kudu/tablet/tablet_replica.h"
@@ -75,6 +76,12 @@
 #include "kudu/util/timer.h"
 #include "kudu/util/trace.h"
 
+namespace kudu {
+namespace tablet {
+class Tablet;
+}  // namespace tablet
+}  // namespace kudu
+
 DEFINE_int32(num_tablets_to_copy_simultaneously, 10,
              "Number of threads available to copy tablets from remote servers.");
 TAG_FLAG(num_tablets_to_copy_simultaneously, advanced);
@@ -288,8 +295,9 @@ using kudu::tablet::TabletMetadata;
 using kudu::tablet::TabletReplica;
 using kudu::transactions::TxnStatusManager;
 using kudu::transactions::TxnStatusManagerFactory;
-using kudu::tserver::TabletCopyClient;
 using std::make_shared;
+using std::nullopt;
+using std::optional;
 using std::set;
 using std::shared_ptr;
 using std::string;
@@ -410,6 +418,7 @@ protected:
 };
 
 TSTabletManager::~TSTabletManager() {
+  Shutdown();
 }
 
 Status TSTabletManager::Init(Timer* start_tablets,
@@ -542,7 +551,7 @@ Status TSTabletManager::Init(Timer* start_tablets,
     }
     open_tablet_pool_->Wait();
     if (seen_error) {
-      LOG_AND_RETURN(WARNING, first_error);
+      LOG_AND_RETURN(ERROR, first_error);
     }
 
     LOG(INFO) << Substitute("Loaded tablet metadata ($0 total tablets, $1 live tablets)",
@@ -613,9 +622,9 @@ Status TSTabletManager::CreateNewTablet(const string& table_id,
                                         const Schema& schema,
                                         const PartitionSchema& partition_schema,
                                         RaftConfigPB config,
-                                        boost::optional<TableExtraConfigPB> extra_config,
-                                        boost::optional<string> dimension_label,
-                                        boost::optional<TableTypePB> table_type,
+                                        optional<TableExtraConfigPB> extra_config,
+                                        optional<string> dimension_label,
+                                        optional<TableTypePB> table_type,
                                         scoped_refptr<TabletReplica>* replica) {
   CHECK_EQ(state(), MANAGER_RUNNING);
   CHECK(IsRaftConfigMember(server_->instance_pb().permanent_uuid(), config));
@@ -652,7 +661,7 @@ Status TSTabletManager::CreateNewTablet(const string& table_id,
                               partition_schema,
                               partition,
                               TABLET_DATA_READY,
-                              boost::none,
+                              nullopt,
                               /*supports_live_row_count=*/ true,
                               std::move(extra_config),
                               std::move(dimension_label),
@@ -734,7 +743,7 @@ void TSTabletManager::StartTabletCopy(
   runnable->DisableCallback();
 
   // Check if the tablet is already in transition (i.e. being copied).
-  boost::optional<string> transition;
+  optional<string> transition;
   {
     // Lock must be dropped before executing callbacks.
     shared_lock<RWMutex> lock(lock_);
@@ -822,7 +831,7 @@ void TSTabletManager::RunTabletCopy(
         LOG(FATAL) << LogPrefix(tablet_id) << "Tablet Copy: "
                    << "Found tablet in TABLET_DATA_COPYING state during StartTabletCopy()";
       case TABLET_DATA_TOMBSTONED: {
-        boost::optional<OpId> last_logged_opid = meta->tombstone_last_logged_opid();
+        optional<OpId> last_logged_opid = meta->tombstone_last_logged_opid();
         if (last_logged_opid) {
           CALLBACK_RETURN_NOT_OK_WITH_ERROR(CheckLeaderTermNotLower(tablet_id, leader_term,
                                                                     last_logged_opid->term()),
@@ -839,7 +848,7 @@ void TSTabletManager::RunTabletCopy(
           CALLBACK_AND_RETURN(
               Status::IllegalState("consensus unavailable: tablet not running", tablet_id));
         }
-        boost::optional<OpId> opt_last_logged_opid = consensus->GetLastOpId(RECEIVED_OPID);
+        optional<OpId> opt_last_logged_opid = consensus->GetLastOpId(RECEIVED_OPID);
         if (!opt_last_logged_opid) {
           CALLBACK_AND_RETURN(
               Status::IllegalState("cannot determine last-logged opid: tablet not running",
@@ -924,8 +933,8 @@ void TSTabletManager::RunTabletCopy(
   //   to the leader.
   //
   // TODO(aserbin): make this robust and more optimal than it is now.
-  TabletCopyClient tc_client(tablet_id, fs_manager_, cmeta_manager_,
-                             server_->messenger(), &tablet_copy_metrics_);
+  RemoteTabletCopyClient tc_client(tablet_id, fs_manager_, cmeta_manager_,
+                                   server_->messenger(), &tablet_copy_metrics_);
 
   // Download and persist the remote superblock in TABLET_DATA_COPYING state.
   if (replacing_tablet) {
@@ -1052,7 +1061,7 @@ public:
   DeleteTabletRunnable(TSTabletManager* ts_tablet_manager,
                        string tablet_id,
                        tablet::TabletDataState delete_type,
-                       const boost::optional<int64_t>& cas_config_index, // NOLINT
+                       const optional<int64_t>& cas_config_index, // NOLINT
                        std::function<void(const Status&, TabletServerErrorPB::Code)> cb)
       : TabletManagerRunnable(ts_tablet_manager, std::move(cb)),
         tablet_id_(std::move(tablet_id)),
@@ -1070,7 +1079,7 @@ public:
 private:
   const string tablet_id_;
   const tablet::TabletDataState delete_type_;
-  const boost::optional<int64_t> cas_config_index_;
+  const optional<int64_t> cas_config_index_;
 
   DISALLOW_COPY_AND_ASSIGN(DeleteTabletRunnable);
 };
@@ -1078,7 +1087,7 @@ private:
 void TSTabletManager::DeleteTabletAsync(
     const string& tablet_id,
     tablet::TabletDataState delete_type,
-    const boost::optional<int64_t>& cas_config_index,
+    const optional<int64_t>& cas_config_index,
     const std::function<void(const Status&, TabletServerErrorPB::Code)>& cb) {
   auto runnable = make_shared<DeleteTabletRunnable>(this, tablet_id, delete_type,
                                                     cas_config_index, cb);
@@ -1096,7 +1105,7 @@ void TSTabletManager::DeleteTabletAsync(
 Status TSTabletManager::DeleteTablet(
     const string& tablet_id,
     TabletDataState delete_type,
-    const boost::optional<int64_t>& cas_config_index,
+    const optional<int64_t>& cas_config_index,
     TabletServerErrorPB::Code* error_code) {
 
   if (delete_type != TABLET_DATA_DELETED && delete_type != TABLET_DATA_TOMBSTONED) {
@@ -1154,7 +1163,7 @@ Status TSTabletManager::DeleteTablet(
 
   replica->Stop();
 
-  boost::optional<OpId> opt_last_logged_opid;
+  optional<OpId> opt_last_logged_opid;
   if (consensus) {
     opt_last_logged_opid = consensus->GetLastOpId(RECEIVED_OPID);
     DCHECK(!opt_last_logged_opid || opt_last_logged_opid->IsInitialized());
@@ -1468,27 +1477,37 @@ void TSTabletManager::Shutdown() {
 
   // Stop copying tablets.
   // TODO(mpercy): Cancel all outstanding tablet copy tasks (KUDU-1795).
-  tablet_copy_pool_->Shutdown();
+  if (tablet_copy_pool_ != nullptr) {
+    tablet_copy_pool_->Shutdown();
+  }
 
   // Shut down the bootstrap pool, so no new tablets are registered after this point.
-  open_tablet_pool_->Shutdown();
+  if (open_tablet_pool_ != nullptr) {
+    open_tablet_pool_->Shutdown();
+  }
 
   // Shut down the delete pool, so no new tablets are deleted after this point.
-  delete_tablet_pool_->Shutdown();
+  if (delete_tablet_pool_ != nullptr) {
+    delete_tablet_pool_->Shutdown();
+  }
 
   // Shut down the transaction participant registration pool.
-  txn_participant_registration_pool_->Shutdown();
+  if (txn_participant_registration_pool_ != nullptr) {
+    txn_participant_registration_pool_->Shutdown();
+  }
 
-  // Signal the only task running on the txn_status_manager_pool_ to wrap up.
-  shutdown_latch_.CountDown();
-  // Shut down the pool running the dedicated TxnStatusManager-related task.
-  txn_status_manager_pool_->Shutdown();
+  if (txn_status_manager_pool_ != nullptr) {
+    // Signal the only task running on the txn_status_manager_pool_ to wrap up.
+    shutdown_latch_.CountDown();
+    // Shut down the pool running the dedicated TxnStatusManager-related task.
+    txn_status_manager_pool_->Shutdown();
+  }
 
   // Take a snapshot of the replicas list -- that way we don't have to hold
   // on to the lock while shutting them down, which might cause a lock
   // inversion. (see KUDU-308 for example).
   vector<scoped_refptr<TabletReplica>> replicas_to_shutdown;
-  GetTabletReplicas(&replicas_to_shutdown);
+  GetTabletReplicasImpl(&replicas_to_shutdown);
 
   for (const scoped_refptr<TabletReplica>& replica : replicas_to_shutdown) {
     replica->Shutdown();
@@ -1498,11 +1517,15 @@ void TSTabletManager::Shutdown() {
   // The shutdown takes place after the replicas are fully shutdown, to ensure
   // on-going reloading metadata tasks of the transaction status managers are
   // properly executed to unblock the shutdown process of replicas.
-  reload_txn_status_tablet_pool_->Shutdown();
+  if (reload_txn_status_tablet_pool_ != nullptr) {
+    reload_txn_status_tablet_pool_->Shutdown();
+  }
 
   // Now that our TxnStatusManagers have shut down, clean up the threadpool
   // used for commit tasks.
-  txn_commit_pool_->Shutdown();
+  if (txn_commit_pool_ != nullptr) {
+    txn_commit_pool_->Shutdown();
+  }
 
   {
     std::lock_guard<RWMutex> l(lock_);
@@ -1561,11 +1584,16 @@ const NodeInstancePB& TSTabletManager::NodeInstance() const {
   return server_->instance_pb();
 }
 
-void TSTabletManager::GetTabletReplicas(vector<scoped_refptr<TabletReplica> >* replicas) const {
+void TSTabletManager::GetTabletReplicasImpl(
+    vector<scoped_refptr<TabletReplica>>* replicas) const {
   shared_lock<RWMutex> l(lock_);
   AppendValuesFromMap(tablet_map_, replicas);
 }
 
+void TSTabletManager::GetTabletReplicas(vector<scoped_refptr<TabletReplica>>* replicas) const {
+  GetTabletReplicasImpl(replicas);
+}
+
 void TSTabletManager::MarkTabletDirty(const string& tablet_id, const string& reason) {
   VLOG(2) << Substitute("$0 Marking dirty. Reason: $1. Will report this "
       "tablet to the Master in the next heartbeat",
@@ -1598,7 +1626,7 @@ TabletNumByDimensionMap TSTabletManager::GetNumLiveTabletsByDimension() const {
     tablet::TabletStatePB state = entry.second->state();
     if (state == tablet::BOOTSTRAPPING ||
         state == tablet::RUNNING) {
-      boost::optional<string> dimension_label = entry.second->tablet_metadata()->dimension_label();
+      optional<string> dimension_label = entry.second->tablet_metadata()->dimension_label();
       if (dimension_label) {
         result[*dimension_label]++;
       }
@@ -1607,6 +1635,21 @@ TabletNumByDimensionMap TSTabletManager::GetNumLiveTabletsByDimension() const {
   return result;
 }
 
+TabletNumByRangePerTableMap TSTabletManager::GetNumLiveTabletsByRangePerTable() const {
+  TabletNumByRangePerTableMap result;
+  shared_lock<RWMutex> l(lock_);
+  for (const auto& entry : tablet_map_) {
+    tablet::TabletStatePB state = entry.second->state();
+    if (state == tablet::BOOTSTRAPPING ||
+        state == tablet::RUNNING) {
+      const string& table_id = entry.second->tablet_metadata()->table_id();
+      const string& range_key = entry.second->tablet_metadata()->partition().begin().range_key();
+      result[table_id][range_key]++;
+    }
+  }
+  return result;
+}
+
 void TSTabletManager::InitLocalRaftPeerPB() {
   DCHECK_EQ(state(), MANAGER_INITIALIZING);
   local_peer_pb_.set_permanent_uuid(fs_manager_->uuid());
@@ -1798,7 +1841,7 @@ Status TSTabletManager::HandleNonReadyTabletOnStartup(const scoped_refptr<Tablet
 
   if (!skip_deletion) {
     // Passing no OpId will retain the last_logged_opid that was previously in the metadata.
-    RETURN_NOT_OK(DeleteTabletData(meta, cmeta_manager_, data_state, boost::none));
+    RETURN_NOT_OK(DeleteTabletData(meta, cmeta_manager_, data_state, nullopt));
   }
 
   // Register TOMBSTONED tablets so that they get reported to the Master, which
@@ -1817,7 +1860,7 @@ Status TSTabletManager::DeleteTabletData(
     const scoped_refptr<TabletMetadata>& meta,
     const scoped_refptr<consensus::ConsensusMetadataManager>& cmeta_manager,
     TabletDataState delete_type,
-    boost::optional<OpId> last_logged_opid) {
+    optional<OpId> last_logged_opid) {
   const string& tablet_id = meta->tablet_id();
   LOG(INFO) << LogPrefix(tablet_id, meta->fs_manager())
             << "Deleting tablet data with delete state "
