@@ -22,6 +22,7 @@
 #include <functional>
 #include <map>
 #include <memory>
+#include <optional>
 #include <string>
 #include <unordered_map>
 #include <unordered_set>
@@ -46,11 +47,6 @@
 #include "kudu/util/rw_mutex.h"
 #include "kudu/util/status.h"
 
-namespace boost {
-template <class T>
-class optional;
-}
-
 namespace kudu {
 
 class FsManager;
@@ -86,9 +82,15 @@ class TabletServer;
 // Map of tablet id -> transition reason string.
 typedef std::unordered_map<std::string, std::string> TransitionInProgressMap;
 
-// Map of dimension -> tablets number.
+// Map of dimension -> number of tablets.
 typedef std::unordered_map<std::string, int32_t> TabletNumByDimensionMap;
 
+// Map of range start key -> number of tablets in that range.
+typedef std::unordered_map<std::string, int32_t> TabletNumByRangeMap;
+
+// Map of table id -> number of tablets by each range in that table.
+typedef std::unordered_map<std::string, TabletNumByRangeMap> TabletNumByRangePerTableMap;
+
 class TransitionInProgressDeleter;
 
 // Keeps track of the tablets hosted on the tablet server side.
@@ -118,7 +120,7 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
   // the first tablet whose bootstrap failed.
   Status WaitForAllBootstrapsToFinish();
 
-  // Shut down all of the tablets, gracefully flushing before shutdown.
+  // Shut down all the tablets, gracefully flushing before shutdown.
   void Shutdown();
 
   // Create a new tablet and register it with the tablet manager. The new tablet
@@ -135,9 +137,9 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
                          const Schema& schema,
                          const PartitionSchema& partition_schema,
                          consensus::RaftConfigPB config,
-                         boost::optional<TableExtraConfigPB> extra_config,
-                         boost::optional<std::string> dimension_label,
-                         boost::optional<TableTypePB> table_type,
+                         std::optional<TableExtraConfigPB> extra_config,
+                         std::optional<std::string> dimension_label,
+                         std::optional<TableTypePB> table_type,
                          scoped_refptr<tablet::TabletReplica>* replica);
 
   // Delete the specified tablet asynchronously with callback 'cb'.
@@ -151,14 +153,14 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
   //   CAS_FAILED.
   void DeleteTabletAsync(const std::string& tablet_id,
                          tablet::TabletDataState delete_type,
-                         const boost::optional<int64_t>& cas_config_index,
+                         const std::optional<int64_t>& cas_config_index,
                          const std::function<void(const Status&, TabletServerErrorPB::Code)>& cb);
 
   // Delete the specified tablet synchronously.
   // See DeleteTabletAsync() for more information.
   Status DeleteTablet(const std::string& tablet_id,
                       tablet::TabletDataState delete_type,
-                      const boost::optional<int64_t>& cas_config_index,
+                      const std::optional<int64_t>& cas_config_index,
                       TabletServerErrorPB::Code* error_code = nullptr);
 
   // Lookup the given tablet replica by its ID.
@@ -196,9 +198,9 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
   void PopulateIncrementalTabletReport(master::TabletReportPB* report,
                                        const std::vector<std::string>& tablet_ids) const;
 
-  // Get all of the tablets currently hosted on this server.
-  virtual void GetTabletReplicas(
-      std::vector<scoped_refptr<tablet::TabletReplica> >* replicas) const override;
+  // Get all the tablets currently hosted on this server.
+  void GetTabletReplicas(
+      std::vector<scoped_refptr<tablet::TabletReplica>>* replicas) const override;
 
   // Marks tablet with 'tablet_id' as dirty so that it'll be included in the
   // next round of master heartbeats.
@@ -216,6 +218,10 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
   // Get the number of tablets in RUNNING or BOOTSTRAPPING state in each dimension.
   TabletNumByDimensionMap GetNumLiveTabletsByDimension() const;
 
+  // Get the number of tablets in RUNNING or BOOTSTRAPPING state in each range for each table.
+  // TODO(mreddy) Include tablets in INITIALIZED state, but need to check if it's tombstoned.
+  TabletNumByRangePerTableMap GetNumLiveTabletsByRangePerTable() const;
+
   Status RunAllLogGC();
 
   // Delete the tablet using the specified delete_type as the final metadata
@@ -223,13 +229,13 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
   //
   // If set, 'last_logged_opid' will be persisted in the
   // 'tombstone_last_logged_opid' field in the tablet metadata. Otherwise, if
-  // 'last_logged_opid' is equal to boost::none, the tablet metadata will
+  // 'last_logged_opid' is equal to std::nullopt, the tablet metadata will
   // retain its previous value of 'tombstone_last_logged_opid', if any.
   static Status DeleteTabletData(
       const scoped_refptr<tablet::TabletMetadata>& meta,
       const scoped_refptr<consensus::ConsensusMetadataManager>& cmeta_manager,
       tablet::TabletDataState delete_type,
-      boost::optional<consensus::OpId> last_logged_opid);
+      std::optional<consensus::OpId> last_logged_opid);
 
   // Synchronously makes the specified tablet unavailable for further I/O and
   // schedules its asynchronous shutdown.
@@ -292,6 +298,10 @@ class TSTabletManager : public tserver::TabletReplicaLookupIf {
   // Returns Status::OK() iff state_ == MANAGER_RUNNING.
   Status CheckRunningUnlocked(TabletServerErrorPB::Code* error_code) const;
 
+  // Get all of the tablets currently hosted on this server.
+  void GetTabletReplicasImpl(
+      std::vector<scoped_refptr<tablet::TabletReplica>>* replicas) const;
+
   // Registers the start of a tablet state transition by inserting the tablet
   // id and reason string into the transition_in_progress_ map.
   // 'reason' is a string included in the Status return when there is
