@@ -43,7 +43,11 @@
 #include <glog/raw_logging.h>
 #ifdef __linux__
 #define UNW_LOCAL_ONLY
+#ifdef __aarch64__
+#include <libunwind-aarch64.h>
+#else
 #include <libunwind.h>
+#endif //__aarch64__
 #endif
 
 #include "kudu/gutil/basictypes.h"
@@ -104,7 +108,7 @@ extern int GetStackTrace(void** result, int max_depth, int skip_count);
 // symbol name to "out".  The symbol name is demangled if possible
 // (supports symbols generated by GCC 3.x or newer).  Otherwise,
 // returns false.
-bool Symbolize(void *pc, char *out, int out_size);
+bool Symbolize(void *pc, char *out, size_t out_size);
 
 namespace glog_internal_namespace_ {
 extern void DumpStackTraceToString(string *s);
