@@ -7,6 +7,8 @@
 #include <fnmatch.h>
 #include <fts.h>
 #include <glob.h>
+#include <openssl/rand.h>
+#include <openssl/ssl.h>
 #include <pthread.h>
 #include <sys/resource.h>
 #include <sys/socket.h>
@@ -20,16 +22,15 @@
 #include <algorithm>
 #include <cerrno>
 #include <climits>
-#include <cstddef>
 #include <cstdint>
 #include <cstdio>
 #include <cstdlib>
 #include <cstring>
 #include <ctime>
 #include <functional>
-#include <map>
 #include <memory>
 #include <numeric>
+#include <optional>
 #include <ostream>
 #include <string>
 #include <type_traits>
@@ -40,6 +41,7 @@
 
 #include "kudu/gutil/atomicops.h"
 #include "kudu/gutil/basictypes.h"
+#include "kudu/gutil/integral_types.h"
 #include "kudu/gutil/macros.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/once.h"
@@ -49,6 +51,7 @@
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/gutil/strings/util.h"
 #include "kudu/util/array_view.h"
+#include "kudu/util/coding-inl.h"
 #include "kudu/util/debug/trace_event.h"
 #include "kudu/util/env.h"
 #include "kudu/util/errno.h"
@@ -58,6 +61,7 @@
 #include "kudu/util/logging.h"
 #include "kudu/util/malloc.h"
 #include "kudu/util/monotime.h"
+#include "kudu/util/openssl_util.h"
 #include "kudu/util/path_util.h"
 #include "kudu/util/scoped_cleanup.h"
 #include "kudu/util/slice.h"
@@ -197,6 +201,14 @@ DEFINE_string(env_inject_lock_failure_globs, "",
               "will fail.");
 TAG_FLAG(env_inject_lock_failure_globs, hidden);
 
+DEFINE_bool(encrypt_data_at_rest, false,
+            "Whether sensitive files should be encrypted on the file system.");
+DEFINE_int32(encryption_key_length, 128, "Encryption key length.");
+TAG_FLAG(encryption_key_length, advanced);
+
+DEFINE_validator(encryption_key_length,
+                 [](const char* /*n*/, int32 v) { return v == 128 || v == 192 || v == 256; });
+
 static __thread uint64_t thread_local_id;
 static Atomic64 cur_thread_local_id_;
 
@@ -204,14 +216,78 @@ namespace kudu {
 
 const char* const Env::kInjectedFailureStatusMsg = "INJECTED FAILURE";
 
+const uint8_t kEncryptionBlockSize = 16;
+
+const uint8_t kEncryptionHeaderSize = 64;
+
+const char* const kEncryptionHeaderMagic = "kuduenc";
+
+using evp_ctx_unique_ptr = std::unique_ptr<EVP_CIPHER_CTX, decltype(&EVP_CIPHER_CTX_free)>;
+
 namespace {
 
+
 struct FreeDeleter {
   inline void operator()(void* ptr) const {
     free(ptr);
   }
 };
 
+enum class EncryptionAlgorithm {
+  AES128CTR = 0x00,
+  AES192CTR = 0x01,
+  AES256CTR = 0x02,
+  // ECB mode below only used to encrypt keys.
+  AES128ECB = 0xFD,
+  AES192ECB = 0XFE,
+  AES256ECB = 0xFF,
+};
+
+// The encryption header is stored on disk as follows:
+//
+// *----------------------------------------------*
+// | "kuduenc" magic string (7 bytes)             |
+// *----------------------------------------------*
+// | Algorithm and key length (1 byte)            |
+// *----------------------------------------------*
+// | Encrypted File Key (right-padded) (32 bytes) |
+// *----------------------------------------------*
+// | Reserved for future use (24 bytes)           |
+// *----------------------------------------------*
+//
+// The algorithm and key length mapping is:
+//
+// *------*-------------*
+// | 0x00 | AES-128-CTR |
+// *------*-------------*
+// | 0x01 | AES-192-CTR |
+// *------*-------------*
+// | 0x02 | AES-256-CTR |
+// *------*-------------*
+struct EncryptionHeader {
+  EncryptionAlgorithm algorithm;
+  uint8_t key[32];
+};
+
+const EVP_CIPHER* GetEVPCipher(EncryptionAlgorithm algorithm) {
+  switch (algorithm) {
+    case EncryptionAlgorithm::AES128CTR:
+      return EVP_aes_128_ctr();
+    case EncryptionAlgorithm::AES192CTR:
+      return EVP_aes_192_ctr();
+    case EncryptionAlgorithm::AES256CTR:
+      return EVP_aes_256_ctr();
+    case EncryptionAlgorithm::AES128ECB:
+      return EVP_aes_128_ecb();
+    case EncryptionAlgorithm::AES192ECB:
+      return EVP_aes_192_ecb();
+    case EncryptionAlgorithm::AES256ECB:
+      return EVP_aes_256_ecb();
+    default:
+      return nullptr;
+  }
+}
+
 #if defined(__APPLE__)
 // Simulates Linux's fallocate file preallocation API on OS X.
 int fallocate(int fd, int mode, off_t offset, off_t len) {
@@ -364,6 +440,102 @@ Status DoOpen(const string& filename, int flags, const string& reason, int* fd)
   return Status::OK();
 }
 
+// Encrypts the data in 'cleartext' and writes it to 'ciphertext'. It requires
+// 'offset' to be set in the file as it's used to set the initialization vector.
+Status DoEncryptV(const EncryptionHeader* eh,
+                  uint64_t offset,
+                  ArrayView<const Slice> cleartext,
+                  ArrayView<Slice> ciphertext) {
+  DCHECK_EQ(cleartext.size(), ciphertext.size());
+  // Set the initialization vector based on the offset.
+  uint8_t iv[16];
+  InlineBigEndianEncodeFixed64(&iv[0], 0);
+  InlineBigEndianEncodeFixed64(&iv[8], offset / kEncryptionBlockSize);
+
+  evp_ctx_unique_ptr ctx(EVP_CIPHER_CTX_new(), EVP_CIPHER_CTX_free);
+
+  OPENSSL_RET_NOT_OK(EVP_EncryptInit_ex(ctx.get(), GetEVPCipher(eh->algorithm),
+                                        nullptr, eh->key, iv),
+                     "Failed to initialize encryption");
+  OPENSSL_RET_NOT_OK(EVP_CIPHER_CTX_set_padding(ctx.get(), 0),
+                     "failed to disable padding");
+  const size_t offset_mod = offset % kEncryptionBlockSize;
+  if (offset_mod) {
+    unsigned char scratch_clear[kEncryptionBlockSize];
+    unsigned char scratch_cipher[kEncryptionBlockSize];
+    int out_length;
+    OPENSSL_RET_NOT_OK(EVP_EncryptUpdate(ctx.get(), scratch_cipher, &out_length,
+                                         scratch_clear, offset_mod),
+                       "Failed to encrypt scratch data");
+    DCHECK_LE(out_length, kEncryptionBlockSize);
+  }
+  for (auto i = 0; i < cleartext.size(); ++i) {
+    int out_length;
+    OPENSSL_RET_NOT_OK(EVP_EncryptUpdate(ctx.get(),
+                                         ciphertext[i].mutable_data(),
+                                         &out_length,
+                                         cleartext[i].data(),
+                                         cleartext[i].size()),
+                       "Failed to encrypt data");
+    DCHECK_EQ(out_length, cleartext[i].size());
+    DCHECK_LE(out_length, ciphertext[i].size());
+  }
+  // EVP_EncryptFinal_ex() would be needed after the last chunk of data
+  // encrypted with EVP_EncryptUpdate() when padding is enabled. However,
+  // the logic above assumes no padding, and it is turned off explicitly
+  // for the cipher context above.
+  //
+  // TODO(aserbin): maybe, call EVP_EncryptFinal_ex() in DEBUG mode to make sure
+  //                no data remains in a partial block?
+  return Status::OK();
+}
+
+// Decrypts 'data'. Uses 'offset' in the file to set the initialization vector.
+Status DoDecryptV(const EncryptionHeader* eh, uint64_t offset, ArrayView<Slice> data) {
+  // Set the initialization vector based on the offset.
+  uint8_t iv[16];
+  InlineBigEndianEncodeFixed64(&iv[0], 0);
+  InlineBigEndianEncodeFixed64(&iv[8], offset / kEncryptionBlockSize);
+
+  evp_ctx_unique_ptr ctx(EVP_CIPHER_CTX_new(), EVP_CIPHER_CTX_free);
+  OPENSSL_RET_NOT_OK(EVP_DecryptInit_ex(ctx.get(), GetEVPCipher(eh->algorithm),
+                                        nullptr, eh->key, iv),
+                     "Failed to initialize decryption");
+  OPENSSL_RET_NOT_OK(EVP_CIPHER_CTX_set_padding(ctx.get(), 0),
+                     "failed to disable padding");
+  const size_t offset_mod = offset % kEncryptionBlockSize;
+  if (offset_mod) {
+    unsigned char scratch_clear[kEncryptionBlockSize];
+    unsigned char scratch_cipher[kEncryptionBlockSize];
+    int out_length;
+    OPENSSL_RET_NOT_OK(EVP_DecryptUpdate(ctx.get(),
+                                         scratch_clear,
+                                         &out_length,
+                                         scratch_cipher,
+                                         offset_mod),
+                       "Failed to decrypt scratch data");
+  }
+
+  for (auto i = 0; i < data.size(); ++i) {
+    const Slice& ciphertext_slice = data[i];
+    int in_length = ciphertext_slice.size();
+    if (!in_length || IsAllZeros(ciphertext_slice)) continue;
+    int out_length;
+    OPENSSL_RET_NOT_OK(EVP_DecryptUpdate(ctx.get(),
+                                         data[i].mutable_data(),
+                                         &out_length,
+                                         ciphertext_slice.data(),
+                                         in_length),
+                       "Failed to decrypt data");
+  }
+  // We don't call EVP_DecryptFinal_ex() after EVP_DecryptUpdate() for the
+  // same reason we don't call EVP_EncryptFinal_ex() in DoEncryptV().
+  //
+  // TODO(aserbin): maybe, call EVP_DecryptFinal_ex() in DEBUG mode to make sure
+  //                no data remains in a partial block?
+  return Status::OK();
+}
+
 Status DoOpen(const string& filename, Env::OpenMode mode, int* fd) {
   MAYBE_RETURN_EIO(filename, IOError(Env::kInjectedFailureStatusMsg, EIO));
   ThreadRestrictions::AssertIOAllowed();
@@ -392,8 +564,12 @@ Status DoOpen(const string& filename, Env::OpenMode mode, int* fd) {
   return Status::OK();
 }
 
-Status DoReadV(int fd, const string& filename, uint64_t offset,
-               ArrayView<Slice> results) {
+Status DoReadV(
+    int fd,
+    const string& filename,
+    uint64_t offset,
+    ArrayView<Slice> results,
+    const EncryptionHeader* eh) {
   MAYBE_RETURN_EIO(filename, IOError(Env::kInjectedFailureStatusMsg, EIO));
   ThreadRestrictions::AssertIOAllowed();
 
@@ -405,7 +581,7 @@ Status DoReadV(int fd, const string& filename, uint64_t offset,
   for (size_t i = 0; i < iov_size; i++) {
     Slice& result = results[i];
     bytes_req += result.size();
-    iov[i] = { result.mutable_data(), result.size() };
+    iov[i] = {result.mutable_data(), result.size()};
   }
 
   uint64_t cur_offset = offset;
@@ -432,11 +608,17 @@ Status DoReadV(int fd, const string& filename, uint64_t offset,
     }
     if (PREDICT_FALSE(r == 0)) {
       // EOF.
+      if (eh) {
+        RETURN_NOT_OK(DoDecryptV(eh, offset, results));
+      }
       return Status::EndOfFile(
           Substitute("EOF trying to read $0 bytes at offset $1", bytes_req, offset));
     }
     if (PREDICT_TRUE(r == rem)) {
       // All requested bytes were read. This is almost always the case.
+      if (eh) {
+        RETURN_NOT_OK(DoDecryptV(eh, offset, results));
+      }
       return Status::OK();
     }
     DCHECK_LE(r, rem);
@@ -458,11 +640,19 @@ Status DoReadV(int fd, const string& filename, uint64_t offset,
     cur_offset += r;
     rem -= r;
   }
+  if (eh) {
+    RETURN_NOT_OK(DoDecryptV(eh, offset, results));
+  }
   DCHECK_EQ(0, rem);
   return Status::OK();
 }
 
-Status DoWriteV(int fd, const string& filename, uint64_t offset, ArrayView<const Slice> data) {
+Status DoWriteV(
+    int fd,
+    const string& filename,
+    uint64_t offset,
+    ArrayView<const Slice> data,
+    const EncryptionHeader* eh) {
   MAYBE_RETURN_EIO(filename, IOError(Env::kInjectedFailureStatusMsg, EIO));
   ThreadRestrictions::AssertIOAllowed();
 
@@ -471,10 +661,32 @@ Status DoWriteV(int fd, const string& filename, uint64_t offset, ArrayView<const
   size_t bytes_req = 0;
   size_t iov_size = data.size();
   struct iovec iov[iov_size];
-  for (size_t i = 0; i < iov_size; i++) {
-    const Slice& result = data[i];
-    bytes_req += result.size();
-    iov[i] = { const_cast<uint8_t*>(result.data()), result.size() };
+  std::vector<Slice> encrypted_data(iov_size);
+  SCOPED_CLEANUP({
+    if (eh) {
+      delete[] encrypted_data[0].mutable_data();
+    }
+  });
+  if (eh) {
+    for (size_t i = 0; i < iov_size; i++) {
+      bytes_req += data[i].size();
+    }
+    unique_ptr<unsigned char[]> encrypted_buf(new unsigned char[bytes_req]);
+    size_t buffer_offset = 0;
+    for (size_t i = 0; i < iov_size; i++) {
+      size_t size = data[i].size();
+      encrypted_data[i] = Slice(&encrypted_buf[buffer_offset], size);
+      buffer_offset += size;
+      iov[i] = {const_cast<uint8_t*>(encrypted_data[i].data()), size};
+    }
+    RETURN_NOT_OK(DoEncryptV(eh, offset, data, encrypted_data));
+    encrypted_buf.release();
+  } else {
+    for (size_t i = 0; i < iov_size; i++) {
+      const Slice& result = data[i];
+      bytes_req += result.size();
+      iov[i] = {const_cast<uint8_t*>(result.data()), result.size()};
+    }
   }
 
   uint64_t cur_offset = offset;
@@ -528,6 +740,69 @@ Status DoWriteV(int fd, const string& filename, uint64_t offset, ArrayView<const
   return Status::OK();
 }
 
+Status GenerateHeader(EncryptionHeader* eh) {
+  switch (FLAGS_encryption_key_length) {
+    case 128:
+      eh->algorithm = EncryptionAlgorithm::AES128CTR;
+      break;
+    case 192:
+      eh->algorithm = EncryptionAlgorithm::AES192CTR;
+      break;
+    case 256:
+      eh->algorithm = EncryptionAlgorithm::AES256CTR;
+      break;
+    default:
+      return Status::InvalidArgument(
+          "Supported key lengths for AES encryption are 128, 192, and 256.");
+  }
+  OPENSSL_RET_NOT_OK(RAND_bytes(eh->key, FLAGS_encryption_key_length / 8),
+                     "Failed to generate random key");
+
+  return Status::OK();
+}
+
+Status WriteEncryptionHeader(int fd, const string& filename, const EncryptionHeader& server_key,
+                             const EncryptionHeader& eh) {
+  vector<Slice> headerv = { kEncryptionHeaderMagic };
+  uint32_t key_size;
+  uint8_t algorithm[1];
+  switch (eh.algorithm) {
+    case EncryptionAlgorithm::AES128CTR:
+      algorithm[0] = 0;
+      key_size = 16;
+      break;
+    case EncryptionAlgorithm::AES192CTR:
+      algorithm[0] = 1;
+      // As the keys are encrypted in ECB mode which requires padding, we need
+      // 32 bytes instead of 24 to encrypt and write a 192-bit key.
+      key_size = 32;
+      break;
+    case EncryptionAlgorithm::AES256CTR:
+      algorithm[0] = 2;
+      key_size = 32;
+      break;
+    default:
+      return Status::InvalidArgument(Substitute("Unknown encryption algorithm: $0", algorithm));
+  }
+  headerv.emplace_back(Slice(algorithm, 1));
+  Slice file_key(eh.key, key_size);
+
+  uint8_t encrypted_file_key[32];
+  Slice efk(encrypted_file_key, key_size);
+  vector<Slice> clear = {file_key};
+  vector<Slice> cipher = {efk};
+  RETURN_NOT_OK(DoEncryptV(&server_key, 0, clear, cipher));
+
+  // Add the encrypted file key and trailing zeros to the header.
+  headerv.emplace_back(efk);
+  static const uint8_t padding[40] = {0};
+  // 7 bytes of magic + 1 byte of algorithm and key length.
+  constexpr int kMagicAndAlgorithmSize = 8;
+  Slice padding_slice(padding, kEncryptionHeaderSize - kMagicAndAlgorithmSize - key_size);
+  headerv.emplace_back(padding_slice);
+  return DoWriteV(fd, filename, 0, headerv, nullptr);
+}
+
 Status DoIsOnXfsFilesystem(const string& path, bool* result) {
 #ifdef __APPLE__
   *result = false;
@@ -545,6 +820,40 @@ Status DoIsOnXfsFilesystem(const string& path, bool* result) {
   return Status::OK();
 }
 
+Status ReadEncryptionHeader(int fd, const string& filename, const EncryptionHeader& server_key,
+                            EncryptionHeader* eh) {
+  char magic[7];
+  uint8_t algorithm[1];
+  char file_key[32];
+  vector<Slice> headerv({ Slice(magic, 7), Slice(algorithm, 1), Slice(file_key, 32) });
+  RETURN_NOT_OK(DoReadV(fd, filename, 0, headerv, nullptr));
+  if (strncmp(magic, kEncryptionHeaderMagic, 7) != 0) {
+    return Status::Corruption(Substitute("Invalid encryption header: $0", magic));
+  }
+  uint16_t key_size;
+  eh->algorithm = EncryptionAlgorithm(algorithm[0]);
+  switch (eh->algorithm) {
+    case EncryptionAlgorithm::AES128CTR:
+      key_size = 16;
+      break;
+    case EncryptionAlgorithm::AES192CTR:
+      key_size = 24;
+      break;
+    case EncryptionAlgorithm::AES256CTR:
+      key_size = 32;
+      break;
+    default:
+      return Status::Corruption(Substitute("Unknown encryption algorithm: $0", algorithm));
+  }
+  // Round up to the nearest multiple of 16 bytes when reading and decrypting
+  // the file. The actual key size can be used when storing the key in memory.
+  // See WriteEncryptionHeader for more info.
+  vector<Slice> v = {Slice(file_key, (key_size + 15) & -16)};
+  RETURN_NOT_OK(DoDecryptV(&server_key, 0, v));
+  memcpy(&eh->key, file_key, key_size);
+  return Status::OK();
+}
+
 const char* ResourceLimitTypeToString(Env::ResourceLimitType t) {
   switch (t) {
     case Env::ResourceLimitType::OPEN_FILES_PER_PROCESS:
@@ -582,6 +891,10 @@ class PosixFifo : public Fifo {
  public:
   explicit PosixFifo(string fname) : filename_(std::move(fname)) {}
 
+  size_t GetEncryptionHeaderSize() const override {
+    return 0;
+  }
+
   const string& filename() const override {
     return filename_;
   }
@@ -625,10 +938,18 @@ class PosixSequentialFile: public SequentialFile {
  private:
   const string filename_;
   FILE* const file_;
+  const bool encrypted_;
+  size_t offset_;
+  const EncryptionHeader encryption_header_;
 
  public:
-  PosixSequentialFile(string fname, FILE* f)
-      : filename_(std::move(fname)), file_(f) {}
+  PosixSequentialFile(string fname, bool encrypted, FILE* f, EncryptionHeader eh)
+    : filename_(std::move(fname)),
+      file_(f),
+      encrypted_(encrypted),
+      offset_(encrypted ? kEncryptionHeaderSize : 0),
+      encryption_header_(eh) {}
+
   ~PosixSequentialFile() {
     int err;
     RETRY_ON_EINTR(err, fclose(file_));
@@ -653,6 +974,10 @@ class PosixSequentialFile: public SequentialFile {
         return IOError(filename_, errno);
       }
     }
+    if (encrypted_) {
+      RETURN_NOT_OK(DoDecryptV(&encryption_header_, offset_, ArrayView<Slice>(result, 1)));
+    }
+    offset_ += r;
     return Status::OK();
   }
 
@@ -663,10 +988,15 @@ class PosixSequentialFile: public SequentialFile {
     if (fseek(file_, n, SEEK_CUR)) {
       return IOError(filename_, errno);
     }
+    offset_ += n;
     return Status::OK();
   }
 
   virtual const string& filename() const OVERRIDE { return filename_; }
+
+  size_t GetEncryptionHeaderSize() const override {
+    return encrypted_ ? kEncryptionHeaderSize : 0;
+  }
 };
 
 // pread() based random-access
@@ -674,20 +1004,29 @@ class PosixRandomAccessFile: public RandomAccessFile {
  private:
   const string filename_;
   const int fd_;
+  const bool encrypted_;
+  const EncryptionHeader encryption_header_;
 
  public:
-  PosixRandomAccessFile(string fname, int fd)
-      : filename_(std::move(fname)), fd_(fd) {}
+  PosixRandomAccessFile(string fname, int fd, bool encrypted, EncryptionHeader eh)
+      : filename_(std::move(fname)),
+        fd_(fd),
+        encrypted_(encrypted),
+        encryption_header_(eh) {}
   ~PosixRandomAccessFile() {
     DoClose(fd_);
   }
 
   virtual Status Read(uint64_t offset, Slice result) const OVERRIDE {
-    return DoReadV(fd_, filename_, offset, ArrayView<Slice>(&result, 1));
+    DCHECK_GE(offset, GetEncryptionHeaderSize());
+    return DoReadV(fd_, filename_, offset, ArrayView<Slice>(&result, 1),
+                   encrypted_ ? &encryption_header_ : nullptr);
   }
 
   virtual Status ReadV(uint64_t offset, ArrayView<Slice> results) const OVERRIDE {
-    return DoReadV(fd_, filename_, offset, results);
+    DCHECK_GE(offset, GetEncryptionHeaderSize());
+    return DoReadV(fd_, filename_, offset, results,
+                   encrypted_ ? &encryption_header_ : nullptr);
   }
 
   virtual Status Size(uint64_t *size) const OVERRIDE {
@@ -704,6 +1043,10 @@ class PosixRandomAccessFile: public RandomAccessFile {
 
   virtual const string& filename() const OVERRIDE { return filename_; }
 
+  size_t GetEncryptionHeaderSize() const override {
+    return encrypted_ ? kEncryptionHeaderSize : 0;
+  }
+
   virtual size_t memory_footprint() const OVERRIDE {
     return kudu_malloc_usable_size(this) + filename_.capacity();
   }
@@ -715,15 +1058,17 @@ class PosixRandomAccessFile: public RandomAccessFile {
 // order to further improve Sync() performance.
 class PosixWritableFile : public WritableFile {
  public:
-  PosixWritableFile(string fname, int fd, uint64_t file_size,
-                    bool sync_on_close)
+  PosixWritableFile(string fname, int fd, uint64_t file_size, bool sync_on_close,
+                     bool encrypted, EncryptionHeader eh)
       : filename_(std::move(fname)),
         fd_(fd),
         sync_on_close_(sync_on_close),
         filesize_(file_size),
         pre_allocated_size_(0),
         pending_sync_(false),
-        closed_(false) {}
+        closed_(false),
+        encrypted_(encrypted),
+        encryption_header_(eh) {}
 
   ~PosixWritableFile() {
     WARN_NOT_OK(Close(), "Failed to close " + filename_);
@@ -735,7 +1080,8 @@ class PosixWritableFile : public WritableFile {
 
   virtual Status AppendV(ArrayView<const Slice> data) OVERRIDE {
     ThreadRestrictions::AssertIOAllowed();
-    RETURN_NOT_OK(DoWriteV(fd_, filename_, filesize_, data));
+    RETURN_NOT_OK(DoWriteV(fd_, filename_, filesize_, data,
+                           encrypted_ ? &encryption_header_ : nullptr));
     // Calculate the amount of data written
     size_t bytes_written = accumulate(data.begin(), data.end(), static_cast<size_t>(0),
                                       [&](int sum, const Slice& curr) {
@@ -848,6 +1194,10 @@ class PosixWritableFile : public WritableFile {
 
   virtual const string& filename() const OVERRIDE { return filename_; }
 
+  size_t GetEncryptionHeaderSize() const override {
+    return encrypted_ ? kEncryptionHeaderSize : 0;
+  }
+
  private:
   const string filename_;
   const int fd_;
@@ -857,27 +1207,36 @@ class PosixWritableFile : public WritableFile {
   uint64_t pre_allocated_size_;
   bool pending_sync_;
   bool closed_;
+  const bool encrypted_;
+  const EncryptionHeader encryption_header_;
 };
 
 class PosixRWFile : public RWFile {
  public:
-  PosixRWFile(string fname, int fd, bool sync_on_close)
+  PosixRWFile(string fname, int fd, bool sync_on_close, bool encrypted,
+              EncryptionHeader eh)
       : filename_(std::move(fname)),
         fd_(fd),
         sync_on_close_(sync_on_close),
         is_on_xfs_(false),
-        closed_(false) {}
+        closed_(false),
+        encrypted_(encrypted),
+        encryption_header_(eh) {}
 
   ~PosixRWFile() {
     WARN_NOT_OK(Close(), "Failed to close " + filename_);
   }
 
   virtual Status Read(uint64_t offset, Slice result) const OVERRIDE {
-    return DoReadV(fd_, filename_, offset, ArrayView<Slice>(&result, 1));
+    DCHECK_GE(offset, GetEncryptionHeaderSize());
+    return DoReadV(fd_, filename_, offset, ArrayView<Slice>(&result, 1),
+                   encrypted_ ? &encryption_header_ : nullptr);
   }
 
   virtual Status ReadV(uint64_t offset, ArrayView<Slice> results) const OVERRIDE {
-    return DoReadV(fd_, filename_, offset, results);
+    DCHECK_GE(offset, GetEncryptionHeaderSize());
+    return DoReadV(fd_, filename_, offset, results,
+                   encrypted_ ? &encryption_header_ : nullptr);
   }
 
   virtual Status Write(uint64_t offset, const Slice& data) OVERRIDE {
@@ -885,7 +1244,9 @@ class PosixRWFile : public RWFile {
   }
 
   virtual Status WriteV(uint64_t offset, ArrayView<const Slice> data) OVERRIDE {
-    return DoWriteV(fd_, filename_, offset, data);
+    DCHECK_GE(offset, GetEncryptionHeaderSize());
+    return DoWriteV(fd_, filename_, offset, data,
+                    encrypted_ ? &encryption_header_ : nullptr);
   }
 
   virtual Status PreAllocate(uint64_t offset,
@@ -955,7 +1316,8 @@ class PosixRWFile : public RWFile {
     } else {
       int ret;
       RETRY_ON_EINTR(ret, fallocate(
-          fd_, FALLOC_FL_PUNCH_HOLE | FALLOC_FL_KEEP_SIZE, offset, length));
+          fd_, FALLOC_FL_PUNCH_HOLE | FALLOC_FL_KEEP_SIZE,
+          offset, length));
       if (ret != 0) {
         return IOError(filename_, errno);
       }
@@ -1093,10 +1455,18 @@ class PosixRWFile : public RWFile {
 #endif
   }
 
+  bool IsEncrypted() const override {
+    return encrypted_;
+  }
+
   virtual const string& filename() const OVERRIDE {
     return filename_;
   }
 
+  size_t GetEncryptionHeaderSize() const override {
+    return encrypted_ ? kEncryptionHeaderSize : 0;
+  }
+
  private:
   static void InitIsOnXFS(void* arg) {
     PosixRWFile* rwf = reinterpret_cast<PosixRWFile*>(arg);
@@ -1118,6 +1488,8 @@ class PosixRWFile : public RWFile {
   GoogleOnceDynamic once_;
   bool is_on_xfs_;
   bool closed_;
+  const bool encrypted_;
+  const EncryptionHeader encryption_header_;
 };
 
 int LockOrUnlock(int fd, bool lock) {
@@ -1139,14 +1511,24 @@ class PosixFileLock : public FileLock {
   int fd_;
 };
 
+static Env* default_env;
+
 class PosixEnv : public Env {
  public:
   ~PosixEnv() {
-    fprintf(stderr, "Destroying Env::Default()\n");
-    exit(1);
+    if (this == default_env) {
+      fprintf(stderr, "Destroying Env::Default()\n");
+      exit(1);
+    }
   }
 
   virtual Status NewSequentialFile(const string& fname,
+                                   unique_ptr<SequentialFile>* result) override {
+    return NewSequentialFile(SequentialFileOptions(), fname, result);
+  }
+
+  virtual Status NewSequentialFile(const SequentialFileOptions& opts,
+                                   const string& fname,
                                    unique_ptr<SequentialFile>* result) OVERRIDE {
     TRACE_EVENT1("io", "PosixEnv::NewSequentialFile", "path", fname);
     MAYBE_RETURN_EIO(fname, IOError(Env::kInjectedFailureStatusMsg, EIO));
@@ -1156,7 +1538,18 @@ class PosixEnv : public Env {
     if (f == nullptr) {
       return IOError(fname, errno);
     }
-    result->reset(new PosixSequentialFile(fname, f));
+    bool encrypted = opts.is_sensitive && IsEncryptionEnabled();
+    EncryptionHeader header;
+    if (encrypted) {
+      DCHECK(server_key_);
+      int fd;
+      RETURN_NOT_OK(DoOpen(fname, OpenMode::MUST_EXIST, &fd));
+      RETURN_NOT_OK(ReadEncryptionHeader(fd, fname, *server_key_, &header));
+      if (fseek(f, kEncryptionHeaderSize, SEEK_CUR)) {
+        return IOError(fname, errno);
+      }
+    }
+    result->reset(new PosixSequentialFile(fname, encrypted, f, header));
     return Status::OK();
   }
 
@@ -1176,8 +1569,14 @@ class PosixEnv : public Env {
     if (fd < 0) {
       return IOError(fname, errno);
     }
-
-    result->reset(new PosixRandomAccessFile(fname, fd));
+    EncryptionHeader header;
+    bool encrypted = opts.is_sensitive && IsEncryptionEnabled();
+    if (encrypted) {
+      DCHECK(server_key_);
+      RETURN_NOT_OK(ReadEncryptionHeader(fd, fname, *server_key_, &header));
+    }
+    result->reset(new PosixRandomAccessFile(fname, fd,
+                  encrypted, header));
     return Status::OK();
   }
 
@@ -1218,8 +1617,27 @@ class PosixEnv : public Env {
                            unique_ptr<RWFile>* result) OVERRIDE {
     TRACE_EVENT1("io", "PosixEnv::NewRWFile", "path", fname);
     int fd;
+    bool encrypt = opts.is_sensitive && IsEncryptionEnabled();
+    uint64_t size = 0;
+    if (opts.mode == MUST_EXIST) {
+      RETURN_NOT_OK(GetFileSize(fname, &size));
+    } else if (encrypt) {
+      GetFileSize(fname, &size);
+    }
+
     RETURN_NOT_OK(DoOpen(fname, opts.mode, &fd));
-    result->reset(new PosixRWFile(fname, fd, opts.sync_on_close));
+    EncryptionHeader eh;
+    if (encrypt) {
+      DCHECK(server_key_);
+      if (size >= kEncryptionHeaderSize) {
+        RETURN_NOT_OK(ReadEncryptionHeader(fd, fname, *server_key_, &eh));
+      } else {
+        RETURN_NOT_OK(GenerateHeader(&eh));
+        RETURN_NOT_OK(WriteEncryptionHeader(fd, fname, *server_key_, eh));
+      }
+    }
+    result->reset(new PosixRWFile(fname, fd, opts.sync_on_close,
+                                  encrypt, eh));
     return Status::OK();
   }
 
@@ -1228,7 +1646,15 @@ class PosixEnv : public Env {
     TRACE_EVENT1("io", "PosixEnv::NewTempRWFile", "template", name_template);
     int fd = 0;
     RETURN_NOT_OK(MkTmpFile(name_template, &fd, created_filename));
-    res->reset(new PosixRWFile(*created_filename, fd, opts.sync_on_close));
+    bool encrypt = opts.is_sensitive && IsEncryptionEnabled();
+    EncryptionHeader eh;
+    if (encrypt) {
+      DCHECK(server_key_);
+      RETURN_NOT_OK(GenerateHeader(&eh));
+      RETURN_NOT_OK(WriteEncryptionHeader(fd, *created_filename, *server_key_, eh));
+    }
+    res->reset(new PosixRWFile(*created_filename, fd, opts.sync_on_close,
+                               encrypt, eh));
     return Status::OK();
   }
 
@@ -1348,7 +1774,7 @@ class PosixEnv : public Env {
         });
   }
 
-  virtual Status GetFileSize(const string& fname, uint64_t* size) OVERRIDE {
+  virtual Status GetFileSize(const string& fname, uint64_t* size) override {
     TRACE_EVENT1("io", "PosixEnv::GetFileSize", "path", fname);
     MAYBE_RETURN_EIO(fname, IOError(Env::kInjectedFailureStatusMsg, EIO));
     ThreadRestrictions::AssertIOAllowed();
@@ -1443,6 +1869,7 @@ class PosixEnv : public Env {
     RETURN_NOT_OK(StatVfs(path, &buf));
     space_info->capacity_bytes = buf.f_frsize * buf.f_blocks;
     space_info->free_bytes = buf.f_frsize * buf.f_bavail;
+    space_info->filesystem_id = buf.f_fsid;
     return Status::OK();
   }
 
@@ -1843,6 +2270,27 @@ class PosixEnv : public Env {
     return result;
   }
 
+  bool IsEncryptionEnabled() const override { return FLAGS_encrypt_data_at_rest; }
+
+  void SetEncryptionKey(const uint8_t* server_key, size_t key_size) override {
+    EncryptionHeader eh;
+    switch (key_size) {
+      case 128:
+        eh.algorithm = EncryptionAlgorithm::AES128ECB;
+        break;
+      case 192:
+        eh.algorithm = EncryptionAlgorithm::AES192ECB;
+        break;
+      case 256:
+        eh.algorithm = EncryptionAlgorithm::AES256ECB;
+        break;
+      default:
+        LOG(FATAL) << "Illegal key size: " << key_size;
+    }
+    memcpy(eh.key, server_key, key_size / 8);
+    server_key_ = eh;
+  }
+
  private:
   // unique_ptr Deleter implementation for fts_close
   struct FtsCloser {
@@ -1884,10 +2332,25 @@ class PosixEnv : public Env {
                                     const WritableFileOptions& opts,
                                     unique_ptr<WritableFile>* result) {
     uint64_t file_size = 0;
+    Status s = GetFileSize(fname, &file_size);
     if (opts.mode == MUST_EXIST) {
-      RETURN_NOT_OK(GetFileSize(fname, &file_size));
+      RETURN_NOT_OK(s);
     }
-    result->reset(new PosixWritableFile(fname, fd, file_size, opts.sync_on_close));
+    bool encrypt = opts.is_sensitive && IsEncryptionEnabled();
+    EncryptionHeader eh;
+    if (encrypt) {
+      DCHECK(server_key_);
+      if (file_size < kEncryptionHeaderSize) {
+        RETURN_NOT_OK(GenerateHeader(&eh));
+        RETURN_NOT_OK(WriteEncryptionHeader(fd, fname, *server_key_, eh));
+        file_size = kEncryptionHeaderSize;
+      } else {
+        RETURN_NOT_OK(ReadEncryptionHeader(fd, fname, *server_key_, &eh));
+      }
+    }
+    result->reset(new PosixWritableFile(fname, fd, file_size, opts.sync_on_close,
+                                        encrypt, eh));
+
     return Status::OK();
   }
 
@@ -1909,6 +2372,10 @@ class PosixEnv : public Env {
     }
   }
 
+  size_t GetEncryptionHeaderSize() const override {
+    return IsEncryptionEnabled() ? kEncryptionHeaderSize : 0;
+  }
+
   Status GetFileSizeOnDiskRecursivelyCb(uint64_t* bytes_used,
                                         FileType type,
                                         const string& dirname,
@@ -1929,12 +2396,13 @@ class PosixEnv : public Env {
     }
     return Status::OK();
   }
+
+  std::optional<EncryptionHeader> server_key_;
 };
 
 }  // namespace
 
 static pthread_once_t once = PTHREAD_ONCE_INIT;
-static Env* default_env;
 static void InitDefaultEnv() { default_env = new PosixEnv; }
 
 Env* Env::Default() {
@@ -1942,6 +2410,10 @@ Env* Env::Default() {
   return default_env;
 }
 
+unique_ptr<Env> Env::NewEnv() {
+  return unique_ptr<Env>(new PosixEnv());
+}
+
 std::ostream& operator<<(std::ostream& o, Env::ResourceLimitType t) {
   return o << ResourceLimitTypeToString(t);
 }
