@@ -44,6 +44,7 @@
 
 DECLARE_bool(cache_force_single_shard);
 DECLARE_int32(file_cache_expiry_period_ms);
+DECLARE_bool(encrypt_data_at_rest);
 
 using std::shared_ptr;
 using std::string;
@@ -51,6 +52,9 @@ using std::unique_ptr;
 using std::vector;
 using strings::Substitute;
 
+namespace {
+} // namespace
+
 namespace kudu {
 
 template <class FileType>
@@ -94,8 +98,10 @@ class FileCacheTest : public KuduTest {
 
   Status WriteTestFile(const string& name, const string& data) {
     unique_ptr<RWFile> f;
-    RETURN_NOT_OK(env_->NewRWFile(name, &f));
-    RETURN_NOT_OK(f->Write(0, data));
+    RWFileOptions opts;
+    opts.is_sensitive = true;
+    RETURN_NOT_OK(env_->NewRWFile(opts, name, &f));
+    RETURN_NOT_OK(f->Write(f->GetEncryptionHeaderSize(), data));
     return Status::OK();
   }
 
@@ -146,7 +152,7 @@ TYPED_TEST(FileCacheTest, TestBasicOperations) {
     for (int i = 0; i < 3; i++) {
       uint64_t size;
       ASSERT_OK(f1->Size(&size));
-      ASSERT_EQ(kData1.size(), size);
+      ASSERT_EQ(kData1.size(), size - Env::Default()->GetEncryptionHeaderSize());
       NO_FATALS(this->AssertFdsAndDescriptors(1, 1));
     }
 
@@ -253,10 +259,12 @@ TYPED_TEST(FileCacheTest, TestInvalidation) {
   ASSERT_OK(this->WriteTestFile(kFile2, kData2));
   ASSERT_OK(this->env_->RenameFile(kFile2, kFile1));
 
+  const uint8_t kHeaderSize = Env::Default()->GetEncryptionHeaderSize();
+
   // We should still be able to access the file, since it has a cached fd.
   uint64_t size;
   ASSERT_OK(f->Size(&size));
-  ASSERT_EQ(kData1.size(), size);
+  ASSERT_EQ(kData1.size(), size - kHeaderSize);
 
   // If we invalidate it from the cache and try again, it should crash because
   // the existing descriptor was invalidated.
@@ -268,7 +276,7 @@ TYPED_TEST(FileCacheTest, TestInvalidation) {
   shared_ptr<TypeParam> f2;
   ASSERT_OK(this->cache_->template OpenFile<Env::MUST_EXIST>(kFile1, &f2));
   ASSERT_OK(f2->Size(&size));
-  ASSERT_EQ(kData2.size(), size);
+  ASSERT_EQ(kData2.size(), size - kHeaderSize);
 }
 
 
@@ -302,8 +310,8 @@ TYPED_TEST(FileCacheTest, TestHeavyReads) {
     const auto& f = opened_files[idx];
     uint64_t size;
     ASSERT_OK(f->Size(&size));
-    Slice s(buf.get(), size);
-    ASSERT_OK(f->Read(0, s));
+    Slice s(buf.get(), size - f->GetEncryptionHeaderSize());
+    ASSERT_OK(f->Read(f->GetEncryptionHeaderSize(), s));
     ASSERT_EQ(data, s);
     ASSERT_LE(this->CountOpenFds(),
               this->initial_open_fds_ + kCacheCapacity);
@@ -338,10 +346,15 @@ TYPED_TEST(FileCacheTest, TestNoRecursiveDeadlock) {
   }
 }
 
-class RandomAccessFileCacheTest : public FileCacheTest<RandomAccessFile> {
+class RandomAccessFileCacheTest :
+  public FileCacheTest<RandomAccessFile>,
+  public ::testing::WithParamInterface<bool> {
 };
 
-TEST_F(RandomAccessFileCacheTest, TestMemoryFootprintDoesNotCrash) {
+INSTANTIATE_TEST_SUITE_P(, RandomAccessFileCacheTest, ::testing::Values(false, true));
+
+TEST_P(RandomAccessFileCacheTest, TestMemoryFootprintDoesNotCrash) {
+  SetEncryptionFlags(GetParam());
   const string kFile = this->GetTestPath("foo");
   ASSERT_OK(this->WriteTestFile(kFile, "test data"));
 
@@ -353,10 +366,15 @@ TEST_F(RandomAccessFileCacheTest, TestMemoryFootprintDoesNotCrash) {
   LOG(INFO) << f->memory_footprint();
 }
 
-class RWFileCacheTest : public FileCacheTest<RWFile> {
+class RWFileCacheTest :
+  public FileCacheTest<RWFile>,
+  public ::testing::WithParamInterface<bool> {
 };
 
-TEST_F(RWFileCacheTest, TestOpenMustCreate) {
+INSTANTIATE_TEST_SUITE_P(, RWFileCacheTest, ::testing::Values(false, true));
+
+TEST_P(RWFileCacheTest, TestOpenMustCreate) {
+  SetEncryptionFlags(GetParam());
   const string kFile1 = this->GetTestPath("foo");
   const string kFile2 = this->GetTestPath("bar");
 
@@ -385,7 +403,8 @@ TEST_F(RWFileCacheTest, TestOpenMustCreate) {
   }
 }
 
-TEST_F(RWFileCacheTest, TestOpenCreateOrOpen) {
+TEST_P(RWFileCacheTest, TestOpenCreateOrOpen) {
+  SetEncryptionFlags(GetParam());
   const string kFile1 = this->GetTestPath("foo");
   const string kFile2 = this->GetTestPath("bar");
 
@@ -407,10 +426,15 @@ TEST_F(RWFileCacheTest, TestOpenCreateOrOpen) {
   ASSERT_TRUE(rwf1->Sync().IsNotFound());
 }
 
-class MixedFileCacheTest : public KuduTest {
+class MixedFileCacheTest :
+  public KuduTest,
+  public ::testing::WithParamInterface<bool> {
 };
 
-TEST_F(MixedFileCacheTest, TestBothFileTypes) {
+INSTANTIATE_TEST_SUITE_P(, MixedFileCacheTest, ::testing::Values(false, true));
+
+TEST_P(MixedFileCacheTest, TestBothFileTypes) {
+  SetEncryptionFlags(GetParam());
   const string kFile1 = GetTestPath("foo");
   const string kData1 = "test data 1";
   const string kFile2 = GetTestPath("foo2");
@@ -419,10 +443,13 @@ TEST_F(MixedFileCacheTest, TestBothFileTypes) {
   // Create the two test files.
   {
     unique_ptr<RWFile> f;
-    ASSERT_OK(env_->NewRWFile(kFile1, &f));
-    ASSERT_OK(f->Write(0, kData1));
-    ASSERT_OK(env_->NewRWFile(kFile2, &f));
-    ASSERT_OK(f->Write(0, kData2));
+    const uint8_t kHeaderSize = Env::Default()->GetEncryptionHeaderSize();
+    RWFileOptions opts;
+    opts.is_sensitive = true;
+    ASSERT_OK(env_->NewRWFile(opts, kFile1, &f));
+    ASSERT_OK(f->Write(kHeaderSize, kData1));
+    ASSERT_OK(env_->NewRWFile(opts, kFile2, &f));
+    ASSERT_OK(f->Write(kHeaderSize, kData2));
   }
 
   FileCache cache("test", env_, 1, /*entity=*/ nullptr);
@@ -438,12 +465,12 @@ TEST_F(MixedFileCacheTest, TestBothFileTypes) {
   uint64_t size;
   ASSERT_OK(rwf->Size(&size));
   uint8_t buf[16];
-  Slice s1(buf, size);
-  ASSERT_OK(rwf->Read(0, s1));
+  Slice s1(buf, size - rwf->GetEncryptionHeaderSize());
+  ASSERT_OK(rwf->Read(rwf->GetEncryptionHeaderSize(), s1));
   ASSERT_EQ(kData1, s1);
   ASSERT_OK(raf->Size(&size));
-  Slice s2(buf, size);
-  ASSERT_OK(raf->Read(0, s2));
+  Slice s2(buf, size - raf->GetEncryptionHeaderSize());
+  ASSERT_OK(raf->Read(raf->GetEncryptionHeaderSize(), s2));
   ASSERT_EQ(kData2, s2);
 
   // It's okay to reopen the test file using the same file type, but not with a
