@@ -30,12 +30,12 @@
 #include <limits>
 #include <memory>
 #include <mutex>
+#include <optional>
 #include <ostream>
 #include <string>
 #include <unordered_set>
 #include <vector>
 
-#include <boost/optional/optional.hpp>
 #include <glog/logging.h>
 #include <google/protobuf/descriptor.h>
 #include <google/protobuf/descriptor.pb.h>
@@ -62,7 +62,6 @@
 #include "kudu/util/debug/sanitizer_scopes.h"
 #include "kudu/util/debug/trace_event.h"
 #include "kudu/util/env.h"
-#include "kudu/util/env_util.h"
 #include "kudu/util/faststring.h"
 #include "kudu/util/jsonwriter.h"
 #include "kudu/util/logging.h"
@@ -93,6 +92,7 @@ using kudu::pb_util::internal::WritableFileOutputStream;
 using std::deque;
 using std::endl;
 using std::initializer_list;
+using std::optional;
 using std::ostream;
 using std::shared_ptr;
 using std::string;
@@ -243,8 +243,8 @@ Status ParseAndCompareChecksum(const uint8_t* checksum_buf,
 // If 'cached_file_size' already has a value, this is a no-op.
 template<typename ReadableFileType>
 Status CacheFileSize(ReadableFileType* reader,
-                     boost::optional<uint64_t>* cached_file_size) {
-  if (*cached_file_size) {
+                     optional<uint64_t>* cached_file_size) {
+  if (cached_file_size->has_value()) {
     return Status::OK();
   }
 
@@ -286,15 +286,15 @@ Status RestOfFileIsAllZeros(ReadableFileType* reader,
 // updated with the new offset on success. On failure, 'offset' is not modified.
 template<typename ReadableFileType>
 Status ReadPBStartingAt(ReadableFileType* reader, int version,
-                        boost::optional<uint64_t>* cached_file_size,
+                        optional<uint64_t>* cached_file_size,
                         uint64_t* offset, Message* msg) {
   uint64_t tmp_offset = *offset;
   VLOG(1) << "Reading PB with version " << version << " starting at offset " << *offset;
 
   RETURN_NOT_OK(CacheFileSize(reader, cached_file_size));
-  uint64_t file_size = cached_file_size->get();
+  const uint64_t file_size = *(*cached_file_size);
 
-  if (tmp_offset == *cached_file_size) {
+  if (tmp_offset == file_size) {
     return Status::EndOfFile("Reached end of file");
   }
 
@@ -379,9 +379,9 @@ Status ReadPBStartingAt(ReadableFileType* reader, int version,
 // Status::Incomplete() for V1 format files.
 template<typename ReadableFileType>
 Status ReadFullPB(ReadableFileType* reader, int version,
-                  boost::optional<uint64_t>* cached_file_size,
+                  optional<uint64_t>* cached_file_size,
                   uint64_t* offset, Message* msg) {
-  bool had_cached_size = *cached_file_size != boost::none;
+  const bool had_cached_size = cached_file_size->has_value();
   Status s = ReadPBStartingAt(reader, version, cached_file_size, offset, msg);
   if (PREDICT_FALSE(s.IsIncomplete() && version == 1)) {
     return Status::Corruption("Unrecoverable incomplete record", s.ToString());
@@ -389,7 +389,7 @@ Status ReadFullPB(ReadableFileType* reader, int version,
   // If we hit EOF, but we were using a cached view of the file size, then it might be
   // that the file has been extended. Invalidate the cache and try again.
   if (had_cached_size && (s.IsIncomplete() || s.IsEndOfFile())) {
-    *cached_file_size = boost::none;
+    cached_file_size->reset();
     return ReadFullPB(reader, version, cached_file_size, offset, msg);
   }
   return s;
@@ -397,10 +397,10 @@ Status ReadFullPB(ReadableFileType* reader, int version,
 
 // Read and parse the protobuf container file-level header documented in pb_util.h.
 template<typename ReadableFileType>
-Status ParsePBFileHeader(ReadableFileType* reader, boost::optional<uint64_t>* cached_file_size,
+Status ParsePBFileHeader(ReadableFileType* reader, optional<uint64_t>* cached_file_size,
                          uint64_t* offset, int* version) {
   RETURN_NOT_OK(CacheFileSize(reader, cached_file_size));
-  uint64_t file_size = cached_file_size->get();
+  const uint64_t file_size = *(*cached_file_size);
 
   // We initially read enough data for a V2+ file header. This optimizes for
   // V2+ and is valid on a V1 file because we don't consider these files valid
@@ -453,7 +453,7 @@ Status ParsePBFileHeader(ReadableFileType* reader, boost::optional<uint64_t>* ca
 // Read and parse the supplemental header from the container file.
 template<typename ReadableFileType>
 Status ReadSupplementalHeader(ReadableFileType* reader, int version,
-                              boost::optional<uint64_t>* cached_file_size,
+                              optional<uint64_t>* cached_file_size,
                               uint64_t* offset,
                               ContainerSupHeaderPB* sup_header) {
   RETURN_NOT_OK_PREPEND(ReadFullPB(reader, version, cached_file_size, offset, sup_header),
@@ -516,7 +516,9 @@ Status WritePBToPath(Env* env, const std::string& path,
   string tmp_path;
 
   unique_ptr<WritableFile> file;
-  RETURN_NOT_OK(env->NewTempWritableFile(WritableFileOptions(), tmp_template, &tmp_path, &file));
+  WritableFileOptions opts;
+  opts.is_sensitive = false;
+  RETURN_NOT_OK(env->NewTempWritableFile(opts, tmp_template, &tmp_path, &file));
   auto tmp_deleter = MakeScopedCleanup([&]() {
     WARN_NOT_OK(env->DeleteFile(tmp_path), "Could not delete file " + tmp_path);
   });
@@ -540,8 +542,10 @@ Status WritePBToPath(Env* env, const std::string& path,
 }
 
 Status ReadPBFromPath(Env* env, const std::string& path, MessageLite* msg) {
-  shared_ptr<SequentialFile> rfile;
-  RETURN_NOT_OK(env_util::OpenFileForSequential(env, path, &rfile));
+  unique_ptr<SequentialFile> rfile;
+  SequentialFileOptions opts;
+  opts.is_sensitive = false;
+  RETURN_NOT_OK(env->NewSequentialFile(opts, path, &rfile));
   RETURN_NOT_OK(ParseFromSequentialFile(msg, rfile.get()));
   return Status::OK();
 }
@@ -665,10 +669,9 @@ string SecureShortDebugString(const Message& msg) {
 
 WritablePBContainerFile::WritablePBContainerFile(shared_ptr<RWFile> writer)
   : state_(FileState::NOT_INITIALIZED),
-    offset_(0),
+    offset_(writer->GetEncryptionHeaderSize()),
     version_(kPBContainerDefaultVersion),
-    writer_(std::move(writer)) {
-}
+    writer_(std::move(writer)) {}
 
 WritablePBContainerFile::~WritablePBContainerFile() {
   WARN_NOT_OK(Close(), "Could not Close() when destroying file");
@@ -727,12 +730,12 @@ Status WritablePBContainerFile::CreateNew(const Message& msg) {
 
 Status WritablePBContainerFile::OpenExisting() {
   DCHECK_EQ(FileState::NOT_INITIALIZED, state_);
-  boost::optional<uint64_t> size;
+  optional<uint64_t> size;
   RETURN_NOT_OK(ParsePBFileHeader(writer_.get(), &size, &offset_, &version_));
   ContainerSupHeaderPB sup_header;
   RETURN_NOT_OK(ReadSupplementalHeader(writer_.get(), version_, &size,
                                        &offset_, &sup_header));
-  offset_ = size.get(); // Reset the write offset to the end of the file.
+  offset_ = *size; // Reset the write offset to the end of the file.
   state_ = FileState::OPEN;
   return Status::OK();
 }
@@ -772,6 +775,11 @@ Status WritablePBContainerFile::Sync() {
   return Status::OK();
 }
 
+uint64_t WritablePBContainerFile::Offset() const {
+  std::lock_guard<Mutex> l(offset_lock_);
+  return offset_;
+}
+
 Status WritablePBContainerFile::Close() {
   if (state_ != FileState::CLOSED) {
     state_ = FileState::CLOSED;
@@ -881,7 +889,7 @@ void WritablePBContainerFile::PopulateDescriptorSet(
 ReadablePBContainerFile::ReadablePBContainerFile(shared_ptr<RandomAccessFile> reader)
   : state_(FileState::NOT_INITIALIZED),
     version_(kPBContainerInvalidVersion),
-    offset_(0),
+    offset_(reader->GetEncryptionHeaderSize()),
     reader_(std::move(reader)) {
 }
 
@@ -994,9 +1002,13 @@ Status ReadablePBContainerFile::Dump(ostream* os, ReadablePBContainerFile::Forma
         *os << SecureDebugString(*msg) << endl;
         break;
       case Format::JSON:
+      case Format::JSON_PRETTY:
         buf.clear();
-        const auto& google_status = google::protobuf::util::MessageToJsonString(
-            *msg, &buf, google::protobuf::util::JsonPrintOptions());
+        auto opt = google::protobuf::util::JsonPrintOptions();
+        if (format == Format::JSON_PRETTY) {
+            opt.add_whitespace = true;
+        }
+        const auto& google_status = google::protobuf::util::MessageToJsonString(*msg, &buf, opt);
         if (!google_status.ok()) {
           return Status::RuntimeError("could not convert PB to JSON", google_status.ToString());
         }
@@ -1033,9 +1045,12 @@ uint64_t ReadablePBContainerFile::offset() const {
   return offset_;
 }
 
-Status ReadPBContainerFromPath(Env* env, const std::string& path, Message* msg) {
+Status ReadPBContainerFromPath(Env* env, const std::string& path,
+                               Message* msg, SensitivityMode sensitivity_mode) {
   unique_ptr<RandomAccessFile> file;
-  RETURN_NOT_OK(env->NewRandomAccessFile(path, &file));
+  RandomAccessFileOptions opts;
+  opts.is_sensitive = sensitivity_mode == SENSITIVE;
+  RETURN_NOT_OK(env->NewRandomAccessFile(opts, path, &file));
 
   ReadablePBContainerFile pb_file(std::move(file));
   RETURN_NOT_OK(pb_file.Open());
@@ -1046,7 +1061,8 @@ Status ReadPBContainerFromPath(Env* env, const std::string& path, Message* msg)
 Status WritePBContainerToPath(Env* env, const std::string& path,
                               const Message& msg,
                               CreateMode create,
-                              SyncMode sync) {
+                              SyncMode sync,
+                              SensitivityMode sensitivity_mode) {
   TRACE_EVENT2("io", "WritePBContainerToPath",
                "path", path,
                "msg_type", msg.GetTypeName());
@@ -1059,7 +1075,9 @@ Status WritePBContainerToPath(Env* env, const std::string& path,
   string tmp_path;
 
   unique_ptr<RWFile> file;
-  RETURN_NOT_OK(env->NewTempRWFile(RWFileOptions(), tmp_template, &tmp_path, &file));
+  RWFileOptions opts;
+  opts.is_sensitive = sensitivity_mode == SENSITIVE;
+  RETURN_NOT_OK(env->NewTempRWFile(opts, tmp_template, &tmp_path, &file));
   auto tmp_deleter = MakeScopedCleanup([&]() {
     WARN_NOT_OK(env->DeleteFile(tmp_path), "Could not delete file " + tmp_path);
   });
