@@ -27,7 +27,7 @@
 #include <glog/logging.h>
 
 #include "kudu/gutil/map-util.h"
-#include "kudu/util/int128_util.h"
+#include "kudu/util/int128.h"
 #include "kudu/util/random.h"
 
 namespace kudu {
