@@ -30,6 +30,7 @@
 #include <ostream>
 #include <string>
 #include <thread>
+#include <type_traits>
 #include <vector>
 
 #include <boost/smart_ptr/shared_ptr.hpp>
@@ -90,6 +91,15 @@ class ThreadPoolTest : public KuduTest {
         .Build(&pool_);
   }
 
+  Status RebuildPoolWithScheduler(int min_threads, int max_threads, int period_ms = 100) {
+    return ThreadPoolBuilder(kDefaultPoolName)
+        .set_min_threads(min_threads)
+        .set_max_threads(max_threads)
+        .set_enable_scheduler()
+        .set_schedule_period_ms(period_ms)
+        .Build(&pool_);
+  }
+
  protected:
   unique_ptr<ThreadPool> pool_;
 };
@@ -97,6 +107,8 @@ class ThreadPoolTest : public KuduTest {
 TEST_F(ThreadPoolTest, TestNoTaskOpenClose) {
   ASSERT_OK(RebuildPoolWithMinMax(4, 4));
   pool_->Shutdown();
+  ASSERT_OK(RebuildPoolWithScheduler(4, 4));
+  pool_->Shutdown();
 }
 
 static void SimpleTaskMethod(int n, Atomic32* counter) {
@@ -122,6 +134,8 @@ class SimpleTask {
 };
 
 TEST_F(ThreadPoolTest, TestSimpleTasks) {
+  constexpr int kDelayMs = 500;
+
   ASSERT_OK(RebuildPoolWithMinMax(4, 4));
 
   Atomic32 counter(0);
@@ -134,6 +148,19 @@ TEST_F(ThreadPoolTest, TestSimpleTasks) {
   ASSERT_OK(pool_->Submit([&counter]() { SimpleTaskMethod(123, &counter); }));
   pool_->Wait();
   ASSERT_EQ(10 + 15 + 20 + 15 + 123, base::subtle::NoBarrier_Load(&counter));
+
+  ASSERT_OK(RebuildPoolWithScheduler(4, 4));
+  unique_ptr<ThreadPoolToken> token = pool_->NewToken(ThreadPool::ExecutionMode::SERIAL);
+  ASSERT_OK(token->Schedule([&counter]() {
+    SimpleTaskMethod(13, &counter);
+  }, kDelayMs));
+
+  // make sure the tasks scheduled and executed in SchedulerThread.
+  SleepFor(MonoDelta::FromMilliseconds(2 * kDelayMs));
+  // make sure all tasks executed in thread pool.
+  pool_->Wait();
+  // check the results.
+  ASSERT_EQ(10 + 15 + 20 + 15 + 123 + 13, base::subtle::NoBarrier_Load(&counter));
 }
 
 static void IssueTraceStatement() {
@@ -154,12 +181,106 @@ TEST_F(ThreadPoolTest, TestTracePropagation) {
   ASSERT_STR_CONTAINS(t->DumpToString(), "hello from task");
 }
 
+static void ContinuousIssueTraceStatement(ThreadPoolToken* token) {
+  Random r(SeedRandom());
+  // delay_ms is a random time: 1ms or 2ms or 3ms.
+  int delay_ms = 1 + static_cast<int>(r.Uniform(3));
+  Status status = token->Schedule(std::bind(&ContinuousIssueTraceStatement, token), delay_ms);
+  // At the case, 'pool_->Shutdown();' would shutdown its SchedulerThread firstly, after that
+  // 'token->Scheduler(...)' would return a IllegalState.
+  LOG_IF(WARNING, !status.ok()) << "ContinuousIssueTraceStatement: " << status.ToString();
+  ASSERT_TRUE(status.ok() || status.IsIllegalState());
+}
+
+TEST_F(ThreadPoolTest, TestExtremeScheduler) {
+  // The case is for the scenario:
+  //   1. There are many tasks in SchedulerThread need to be scheduled.
+  //   2. Every periodic schedule of SchedulerThread, many tasks can be scheduled.
+  //   3. At 1, 2 conditions, SchedulerThread's shutdown happened
+  //      and program should work well(no threads data race and core dump).
+  ASSERT_OK(RebuildPoolWithScheduler(1, 1, 1));
+  unique_ptr<ThreadPoolToken> token = pool_->NewToken(ThreadPool::ExecutionMode::SERIAL);
+  for (int i = 0; i < 2048; i++) {
+    ContinuousIssueTraceStatement(token.get());
+  }
+  SleepFor(MonoDelta::FromMilliseconds(5));
+  pool_->Shutdown();
+}
+
 TEST_F(ThreadPoolTest, TestSubmitAfterShutdown) {
   ASSERT_OK(RebuildPoolWithMinMax(1, 1));
   pool_->Shutdown();
   Status s = pool_->Submit(&IssueTraceStatement);
   ASSERT_EQ("Service unavailable: The pool has been shut down.",
             s.ToString());
+
+  ASSERT_OK(RebuildPoolWithScheduler(1, 1));
+  unique_ptr<ThreadPoolToken> token = pool_->NewToken(ThreadPool::ExecutionMode::SERIAL);
+  Atomic32 counter(0);
+  constexpr const int delay_ms = 100;
+  ASSERT_OK(token->Schedule([&counter]() { SimpleTaskMethod(10, &counter); }, delay_ms));
+  SleepFor(MonoDelta::FromMilliseconds(2 * delay_ms));
+  ASSERT_EQ(10, base::subtle::NoBarrier_Load(&counter));
+  token->Shutdown();
+  ASSERT_OK(token->Schedule([&counter]() { SimpleTaskMethod(3, &counter); }, delay_ms));
+  SleepFor(MonoDelta::FromMilliseconds(2 * delay_ms));
+  // The result does not change.
+  ASSERT_EQ(10, base::subtle::NoBarrier_Load(&counter));
+  pool_->Shutdown();
+  // Submit a delayed task, but it will not really executed.
+  ASSERT_TRUE(
+      token->Schedule([&counter]() { SimpleTaskMethod(5, &counter); }, delay_ms).IsIllegalState());
+  // The result does not change.
+  ASSERT_EQ(10, base::subtle::NoBarrier_Load(&counter));
+}
+
+TEST_F(ThreadPoolTest, TokenShutdownBeforeSchedulerExecute) {
+  ASSERT_OK(RebuildPoolWithScheduler(1, 1));
+  unique_ptr<ThreadPoolToken> token = pool_->NewToken(ThreadPool::ExecutionMode::SERIAL);
+  Atomic32 counter(0);
+  constexpr const int delay_ms = 200;
+  ASSERT_OK(token->Schedule([&counter]() { SimpleTaskMethod(10, &counter); }, delay_ms));
+  token->Shutdown();
+  ASSERT_EQ(0, base::subtle::NoBarrier_Load(&counter));
+  SleepFor(MonoDelta::FromMilliseconds(2 * delay_ms));
+  ASSERT_OK(token->Schedule([&counter]() { SimpleTaskMethod(3, &counter); }, delay_ms));
+  SleepFor(MonoDelta::FromMilliseconds(2 * delay_ms));
+  ASSERT_EQ(0, base::subtle::NoBarrier_Load(&counter));
+  pool_->Shutdown();
+  ASSERT_TRUE(
+      token->Schedule([&counter]() { SimpleTaskMethod(3, &counter); }, delay_ms).IsIllegalState());
+  SleepFor(MonoDelta::FromMilliseconds(2 * delay_ms));
+  ASSERT_EQ(0, base::subtle::NoBarrier_Load(&counter));
+}
+
+TEST_F(ThreadPoolTest, SchedulerShutdownBeforeTokenShutdown) {
+  ASSERT_OK(RebuildPoolWithScheduler(1, 1));
+  unique_ptr<ThreadPoolToken> token = pool_->NewToken(ThreadPool::ExecutionMode::SERIAL);
+  ASSERT_OK(token->Schedule(&IssueTraceStatement, 200));
+  SleepFor(MonoDelta::FromMilliseconds(500));
+  pool_->Shutdown();
+  Status status = token->Schedule(&IssueTraceStatement, 200);
+  ASSERT_TRUE(status.IsIllegalState() || status.IsServiceUnavailable());
+  token->Shutdown();
+}
+
+TEST_F(ThreadPoolTest, SchedulerWithNullToken) {
+  ASSERT_OK(RebuildPoolWithScheduler(1, 1));
+  ASSERT_OK(pool_->Schedule(nullptr, &IssueTraceStatement, MonoTime::Now()));
+
+  Atomic32 counter(0);
+  SimpleTask task(2, &counter);
+
+  ASSERT_OK(pool_->Submit([&counter]() { SimpleTaskMethod(1, &counter); }));
+  ASSERT_OK(pool_->Submit([&task]() { task.Run(); }));
+  // token is null is ok, but the task will ignore.
+  ASSERT_OK(pool_->Schedule(
+      nullptr, [&counter]() { SimpleTaskMethod(3, &counter); }, MonoTime::Now()));
+  // make sure the delayed task execute, the task would be ignored.
+  SleepFor(MonoDelta::FromMilliseconds(200));
+  // make sure all tasks executed in thread pool.
+  pool_->Wait();
+  ASSERT_EQ(1 + 2, base::subtle::NoBarrier_Load(&counter));
 }
 
 TEST_F(ThreadPoolTest, TestThreadPoolWithNoMinimum) {
@@ -198,6 +319,52 @@ TEST_F(ThreadPoolTest, TestThreadPoolWithNoMinimum) {
   ASSERT_EQ(0, pool_->num_threads());
 }
 
+TEST_F(ThreadPoolTest, TestThreadPoolWithSchedulerAndNoMinimum) {
+  constexpr int kIdleTimeoutMs = 1;
+  constexpr int kDelayMs = 1000;
+  ASSERT_OK(RebuildPoolWithBuilder(
+      ThreadPoolBuilder(kDefaultPoolName)
+      .set_min_threads(0)
+      .set_max_threads(4)
+      .set_enable_scheduler()
+      .set_idle_timeout(MonoDelta::FromMilliseconds(kIdleTimeoutMs))));
+  // There are no threads to start with.
+  ASSERT_EQ(0, pool_->active_threads_);
+  ASSERT_EQ(0, pool_->num_threads());
+
+  CountDownLatch latch(1);
+  SCOPED_CLEANUP({
+      latch.CountDown();
+  });
+  ASSERT_OK(pool_->Submit([&latch]() { latch.Wait(); }));
+  ASSERT_OK(pool_->Submit([&latch]() { latch.Wait(); }));
+  ASSERT_OK(pool_->Submit([&latch]() { latch.Wait(); }));
+  ASSERT_OK(pool_->Submit([&latch]() { latch.Wait(); }));
+  ASSERT_EQ(4, pool_->num_threads());
+  unique_ptr<ThreadPoolToken> token = pool_->NewToken(ThreadPool::ExecutionMode::SERIAL);
+  ASSERT_OK(token->Submit([&latch]() { latch.Wait(); }));
+  ASSERT_EQ(4, pool_->num_threads());
+  ASSERT_OK(token->Schedule([&latch]() { latch.Wait(); }, kDelayMs));
+  ASSERT_OK(token->Schedule([&latch]() { latch.Wait(); }, static_cast<int>(kDelayMs * 1.2)));
+  ASSERT_EQ(4, pool_->num_threads());
+  latch.CountDown();
+  pool_->Wait();
+
+  latch.Reset(1);
+  ASSERT_EQ(0, pool_->num_active_threads());
+  SleepFor(MonoDelta::FromMilliseconds(static_cast<int>(kDelayMs * 1.5)));
+  ASSERT_GT(pool_->num_active_threads(), 0);
+  ASSERT_OK(token->Schedule([&latch]() { latch.Wait(); }, kDelayMs));
+  latch.CountDown();
+  pool_->Wait();
+  ASSERT_EQ(0, pool_->num_active_threads());
+  SleepFor(MonoDelta::FromMilliseconds(10 * kIdleTimeoutMs));
+  ASSERT_EQ(0, pool_->num_threads());
+  ASSERT_EQ(0, pool_->num_active_threads());
+  pool_->Shutdown();
+  ASSERT_EQ(nullptr, pool_->scheduler_);
+}
+
 TEST_F(ThreadPoolTest, TestThreadPoolWithNoMaxThreads) {
   // By default a threadpool's max_threads is set to the number of CPUs, so
   // this test submits more tasks than that to ensure that the number of CPUs
@@ -713,8 +880,8 @@ class ThreadPoolPerformanceTest :
     public ThreadPoolTest,
     public testing::WithParamInterface<bool> {
 };
-INSTANTIATE_TEST_CASE_P(LoadMeterPresence, ThreadPoolPerformanceTest,
-                        ::testing::Values(false, true));
+INSTANTIATE_TEST_SUITE_P(LoadMeterPresence, ThreadPoolPerformanceTest,
+                         ::testing::Values(false, true));
 
 // A scenario to assess ThreadPool's performance in the absence/presence
 // of the QueueLoadMeter. The scenario uses a mix of serial and concurrent
@@ -846,9 +1013,9 @@ TEST_F(ThreadPoolTest, TestSlowDestructor) {
 class ThreadPoolTestTokenTypes : public ThreadPoolTest,
                                  public testing::WithParamInterface<ThreadPool::ExecutionMode> {};
 
-INSTANTIATE_TEST_CASE_P(Tokens, ThreadPoolTestTokenTypes,
-                        ::testing::Values(ThreadPool::ExecutionMode::SERIAL,
-                                          ThreadPool::ExecutionMode::CONCURRENT));
+INSTANTIATE_TEST_SUITE_P(Tokens, ThreadPoolTestTokenTypes,
+                         ::testing::Values(ThreadPool::ExecutionMode::SERIAL,
+                                           ThreadPool::ExecutionMode::CONCURRENT));
 
 
 TEST_P(ThreadPoolTestTokenTypes, TestTokenSubmitAndWait) {
@@ -973,8 +1140,9 @@ TEST_P(ThreadPoolTestTokenTypes, TestTokenWaitForAll) {
   const int kNumSubmissions = 20;
   Random r(SeedRandom());
   vector<unique_ptr<ThreadPoolToken>> tokens;
+  tokens.resize(kNumTokens);
   for (int i = 0; i < kNumTokens; i++) {
-    tokens.emplace_back(pool_->NewToken(GetParam()));
+    tokens[i] = pool_->NewToken(GetParam());
   }
 
   atomic<int32_t> v(0);
@@ -1095,7 +1263,7 @@ TEST_P(ThreadPoolTestTokenTypes, TestTokenSubmissionsAdhereToMaxQueueSize) {
   ASSERT_OK(t->Submit([&latch]() { latch.Wait(); }));
   ASSERT_OK(t->Submit([&latch]() { latch.Wait(); }));
   Status s = t->Submit([&latch]() { latch.Wait(); });
-  ASSERT_TRUE(s.IsServiceUnavailable());
+  ASSERT_TRUE(s.IsServiceUnavailable()) << s.ToString();
 }
 
 TEST_F(ThreadPoolTest, TestTokenConcurrency) {
