@@ -17,23 +17,23 @@
 
 #include "kudu/util/threadpool.h"
 
-#include <algorithm>
-#include <cstdint>
 #include <deque>
 #include <functional>
 #include <limits>
 #include <memory>
 #include <ostream>
 #include <string>
+#include <utility>
+#include <vector>
 
 #include <glog/logging.h>
 
-#include "kudu/gutil/macros.h"
 #include "kudu/gutil/map-util.h"
 #include "kudu/gutil/strings/substitute.h"
 #include "kudu/gutil/sysinfo.h"
 #include "kudu/gutil/walltime.h"
 #include "kudu/util/metrics.h"
+#include "kudu/util/monotime.h"
 #include "kudu/util/scoped_cleanup.h"
 #include "kudu/util/thread.h"
 #include "kudu/util/trace.h"
@@ -42,9 +42,9 @@
 namespace kudu {
 
 using std::deque;
-using std::shared_ptr;
 using std::string;
 using std::unique_ptr;
+using std::vector;
 using strings::Substitute;
 
 ////////////////////////////////////////////////////////
@@ -56,7 +56,9 @@ ThreadPoolBuilder::ThreadPoolBuilder(string name)
       min_threads_(0),
       max_threads_(base::NumCPUs()),
       max_queue_size_(std::numeric_limits<int>::max()),
-      idle_timeout_(MonoDelta::FromMilliseconds(500)) {}
+      idle_timeout_(MonoDelta::FromMilliseconds(500)),
+      enable_scheduler_(false),
+      schedule_period_ms_(100) {}
 
 ThreadPoolBuilder& ThreadPoolBuilder::set_trace_metric_prefix(const string& prefix) {
   trace_metric_prefix_ = prefix;
@@ -90,6 +92,16 @@ ThreadPoolBuilder& ThreadPoolBuilder::set_metrics(ThreadPoolMetrics metrics) {
   return *this;
 }
 
+ThreadPoolBuilder& ThreadPoolBuilder::set_enable_scheduler() {
+  enable_scheduler_ = true;
+  return *this;
+}
+
+ThreadPoolBuilder& ThreadPoolBuilder::set_schedule_period_ms(uint32_t schedule_period_ms) {
+  schedule_period_ms_ = schedule_period_ms;
+  return *this;
+}
+
 ThreadPoolBuilder& ThreadPoolBuilder::set_queue_overload_threshold(
     const MonoDelta& threshold) {
   queue_overload_threshold_ = threshold;
@@ -101,6 +113,66 @@ Status ThreadPoolBuilder::Build(unique_ptr<ThreadPool>* pool) const {
   return (*pool)->Init();
 }
 
+SchedulerThread::SchedulerThread(string thread_pool_name, uint32_t schedule_period_ms)
+    : thread_pool_name_(std::move(thread_pool_name)),
+      schedule_period_ms_(schedule_period_ms),
+      shutdown_(1) {}
+
+SchedulerThread::~SchedulerThread() {
+  if (thread_) {
+    Shutdown();
+  }
+}
+
+Status SchedulerThread::Start() {
+  return Thread::Create(
+      thread_pool_name_, "scheduler", [this]() { this->RunLoop(); }, &thread_);
+}
+
+Status SchedulerThread::Shutdown() {
+  if (thread_) {
+    shutdown_.CountDown();
+    thread_->Join();
+  }
+  return Status::OK();
+}
+
+void SchedulerThread::RunLoop() {
+  while (!shutdown_.WaitFor(MonoDelta::FromMilliseconds(schedule_period_ms_))) {
+    MonoTime now = MonoTime::Now();
+    vector<SchedulerTask> pending_tasks;
+    {
+      MutexLock auto_lock(mutex_);
+      auto upper_it = future_tasks_.upper_bound(now);
+      for (auto it = future_tasks_.begin(); it != upper_it; it++) {
+        pending_tasks.emplace_back(std::move(it->second));
+      }
+      future_tasks_.erase(future_tasks_.begin(), upper_it);
+    }
+
+    for (const auto& task : pending_tasks) {
+      ThreadPoolToken* token = task.thread_pool_token();
+      while (token != nullptr) {
+        Status s = token->Submit(task.func());
+        if (s.ok()) {
+          break;
+        }
+        DCHECK(s.IsServiceUnavailable())
+            << Substitute("threadpool token Submit status: $0", s.ToString());
+
+        if (!token->MaySubmitNewTasks()) {
+          // threadpool token is Shutdown, skip the task.
+          break;
+        }
+        // If developers use ThreadPoolToken::Schedule(...) too frequent, blocking queue's
+        // capacity will be full, then retry submit the task again.
+        VLOG(1) << Substitute("threadpool token Submit status: $0, retry the task", s.ToString());
+        SleepFor(MonoDelta::FromMilliseconds(1));
+      }
+    }
+  }
+}
+
 ////////////////////////////////////////////////////////
 // ThreadPoolToken
 ////////////////////////////////////////////////////////
@@ -164,7 +236,7 @@ void ThreadPoolToken::Shutdown() {
         break;
       }
       Transition(State::QUIESCING);
-      FALLTHROUGH_INTENDED;
+      [[fallthrough]];
     case State::QUIESCING:
       // The token is already quiescing. Just wait for a worker thread to
       // switch it to QUIESCED.
@@ -185,6 +257,14 @@ void ThreadPoolToken::Shutdown() {
   }
 }
 
+// Submit a task, running after delay_ms delay some time
+Status ThreadPoolToken::Schedule(std::function<void()> f, int64_t delay_ms) {
+  CHECK(mode() == ThreadPool::ExecutionMode::SERIAL);
+  MonoTime execute_time = MonoTime::Now();
+  execute_time.AddDelta(MonoDelta::FromMilliseconds(delay_ms));
+  return pool_->Schedule(this, std::move(f), execute_time);
+}
+
 void ThreadPoolToken::Wait() {
   MutexLock unique_lock(pool_->lock_);
   pool_->CheckNotPoolThreadUnlocked();
@@ -285,7 +365,10 @@ ThreadPool::ThreadPool(const ThreadPoolBuilder& builder)
       active_threads_(0),
       total_queued_tasks_(0),
       tokenless_(NewToken(ExecutionMode::CONCURRENT)),
-      metrics_(builder.metrics_) {
+      metrics_(builder.metrics_),
+      scheduler_(nullptr),
+      schedule_period_ms_(builder.schedule_period_ms_),
+      enable_scheduler_(builder.enable_scheduler_) {
   string prefix = !builder.trace_metric_prefix_.empty() ?
       builder.trace_metric_prefix_ : builder.name_;
 
@@ -323,13 +406,24 @@ Status ThreadPool::Init() {
       return status;
     }
   }
+  if (enable_scheduler_) {
+    scheduler_ = new SchedulerThread(name_, schedule_period_ms_);
+    RETURN_NOT_OK(scheduler_->Start());
+  }
   return Status::OK();
 }
 
 void ThreadPool::Shutdown() {
+  {
+    MutexLock l(scheduler_lock_);
+    if (scheduler_) {
+      delete scheduler_;
+      scheduler_ = nullptr;
+    }
+  }
+
   MutexLock unique_lock(lock_);
   CheckNotPoolThreadUnlocked();
-
   // Note: this is the same error seen at submission if the pool is at
   // capacity, so clients can't tell them apart. This isn't really a practical
   // concern though because shutting down a pool typically requires clients to
@@ -430,6 +524,17 @@ Status ThreadPool::Submit(std::function<void()> f) {
   return DoSubmit(std::move(f), tokenless_.get());
 }
 
+Status ThreadPool::Schedule(ThreadPoolToken* token,
+                            std::function<void()> f,
+                            MonoTime execute_time) {
+  MutexLock l(scheduler_lock_);
+  if (!scheduler_) {
+    return Status::IllegalState("scheduler thread has been shutdown");
+  }
+  scheduler_->Schedule(token, std::move(f), execute_time);
+  return Status::OK();
+}
+
 Status ThreadPool::DoSubmit(std::function<void()> f, ThreadPoolToken* token) {
   DCHECK(token);
   const MonoTime submit_time = MonoTime::Now();
@@ -516,14 +621,22 @@ Status ThreadPool::DoSubmit(std::function<void()> f, ThreadPoolToken* token) {
     idle_threads_.pop_front();
   }
   NotifyLoadMeterUnlocked();
+
+  // Update the token's queue length histogram under a lock. Submitting a task
+  // via a pool token and destroying the token might be run concurrently by
+  // differen threads. If updating the metric after releasing the 'lock_', the
+  // destructor for the token might run after 'lock_' is released since the
+  // other thread might already be executing the code of the ThreadPoolToken's
+  // destructor, just waiting to acquire 'lock_' in ThreadPoolToken::Shutdown().
+  if (token->metrics_.queue_length_histogram) {
+    token->metrics_.queue_length_histogram->Increment(length_at_submit);
+  }
+
   guard.Unlock();
 
   if (metrics_.queue_length_histogram) {
     metrics_.queue_length_histogram->Increment(length_at_submit);
   }
-  if (token->metrics_.queue_length_histogram) {
-    token->metrics_.queue_length_histogram->Increment(length_at_submit);
-  }
 
   if (need_a_thread) {
     Status status = CreateThread();
