@@ -42,11 +42,11 @@ GFLAGS_VERSION=2.2.2
 GFLAGS_NAME=gflags-$GFLAGS_VERSION
 GFLAGS_SOURCE=$TP_SOURCE_DIR/$GFLAGS_NAME
 
-GLOG_VERSION=0.3.5
+GLOG_VERSION=0.6.0
 GLOG_NAME=glog-$GLOG_VERSION
 GLOG_SOURCE=$TP_SOURCE_DIR/$GLOG_NAME
 
-GMOCK_VERSION=1.10.0
+GMOCK_VERSION=1.12.1
 GMOCK_NAME=googletest-release-$GMOCK_VERSION
 GMOCK_SOURCE=$TP_SOURCE_DIR/$GMOCK_NAME
 
@@ -137,17 +137,17 @@ CRCUTIL_VERSION=2903870057d2f1f109b245650be29e856dc8b646
 CRCUTIL_NAME=crcutil-$CRCUTIL_VERSION
 CRCUTIL_SOURCE=$TP_SOURCE_DIR/$CRCUTIL_NAME
 
-LIBUNWIND_VERSION=1.5.0
+LIBUNWIND_VERSION=1.6.2
 LIBUNWIND_NAME=libunwind-$LIBUNWIND_VERSION
 LIBUNWIND_SOURCE=$TP_SOURCE_DIR/$LIBUNWIND_NAME
 
 # See package-llvm.sh for details on the LLVM tarball.
-LLVM_VERSION=9.0.0
+LLVM_VERSION=11.0.0
 LLVM_NAME=llvm-$LLVM_VERSION.src
 LLVM_SOURCE=$TP_SOURCE_DIR/$LLVM_NAME
 
 # The include-what-you-use is built along with LLVM in its source tree.
-IWYU_VERSION=0.13
+IWYU_VERSION=0.15
 
 # Python is required to build LLVM 3.6+ because it uses
 # llvm/utils/llvm-build/llvmbuild script. It is only built and installed if
@@ -162,7 +162,7 @@ PYTHON_SOURCE=$TP_SOURCE_DIR/$PYTHON_NAME
 #
 # The source can be found in the 'kudu' branch of https://github.com/cloudera/catapult
 # and built with "tracing/kudu-build.sh" included within the repository.
-TRACE_VIEWER_VERSION=21d76f8350fea2da2aa25cb6fd512703497d0c11
+TRACE_VIEWER_VERSION=99efe2f56191867ba7bb602c7c227dea6d576d2f
 TRACE_VIEWER_NAME=kudu-trace-viewer-$TRACE_VIEWER_VERSION
 TRACE_VIEWER_SOURCE=$TP_SOURCE_DIR/$TRACE_VIEWER_NAME
 
@@ -259,6 +259,10 @@ RANGER_VERSION=2.1.0
 RANGER_NAME=ranger-$RANGER_VERSION-admin
 RANGER_SOURCE=$TP_SOURCE_DIR/$RANGER_NAME
 
+RANGER_KMS_VERSION=2.1.0 # this probably should match the ranger version
+RANGER_KMS_NAME=ranger-$RANGER_KMS_VERSION-kms
+RANGER_KMS_SOURCE=$TP_SOURCE_DIR/$RANGER_KMS_NAME
+
 OATPP_VERSION=1.2.5
 OATPP_NAME=oatpp-$OATPP_VERSION
 OATPP_SOURCE=$TP_SOURCE_DIR/$OATPP_NAME
@@ -271,3 +275,7 @@ JWT_CPP_VERSION=3bd600762a70faccc7ec1c2dacb999cba6c6ef5e
 JWT_CPP=jwt-cpp
 JWT_CPP_NAME=$JWT_CPP-$JWT_CPP_VERSION
 JWT_CPP_SOURCE=$TP_SOURCE_DIR/$JWT_CPP_NAME
+
+ROCKSDB_VERSION=7.7.3
+ROCKSDB_NAME=rocksdb-$ROCKSDB_VERSION
+ROCKSDB_SOURCE=$TP_SOURCE_DIR/$ROCKSDB_NAME
